@@ -106,6 +106,8 @@ end
 
 type repr =
 | RInt of int
+| Rint64 of Int64.t
+| RFloat64 of float
 | RBlock of (int * int) (* tag × len *)
 | RString of string
 | RPointer of int
@@ -119,6 +121,8 @@ type data =
 
 type obj =
 | Struct of int * data array (* tag × data *)
+| Int64 of Int64.t (* Primitive integer *)
+| Float64 of float (* Primitive float *)
 | String of string
 
 module type Input =
@@ -255,6 +259,47 @@ let input_header64 chan =
   in
   (tag, len)
 
+let input_cstring chan : string =
+  let buff = Buffer.create 17 in
+  let rec loop () =
+    match input_char chan with
+    | '\o000' -> Buffer.contents buff
+    | c -> Buffer.add_char buff c |> loop
+  in loop ()
+
+let input_intL chan : int64 =
+  let i = input_byte chan in
+  let j = input_byte chan in
+  let k = input_byte chan in
+  let l = input_byte chan in
+  let m = input_byte chan in
+  let n = input_byte chan in
+  let o = input_byte chan in
+  let p = input_byte chan in
+  let ( lsl ) x y = Int64.(shift_left (of_int x) y) in
+  let ( lor ) = Int64.logor in
+  (i lsl 56) lor (j lsl 48) lor (k lsl 40) lor (l lsl 32) lor
+  (m lsl 24) lor (n lsl 16) lor (o lsl 8) lor (Int64.of_int p)
+
+let input_double_big chan : float =
+  Int64.float_of_bits (input_intL chan)
+
+let input_double_little chan : float =
+  let i = input_byte chan in
+  let j = input_byte chan in
+  let k = input_byte chan in
+  let l = input_byte chan in
+  let m = input_byte chan in
+  let n = input_byte chan in
+  let o = input_byte chan in
+  let p = input_byte chan in
+  let ( lsl ) x y = Int64.(shift_left (of_int x) y) in
+  let ( lor ) = Int64.logor in
+  let bits =
+    (p lsl 56) lor (o lsl 48) lor (n lsl 40) lor (m lsl 32) lor
+    (l lsl 24) lor (k lsl 16) lor (j lsl 8) lor (Int64.of_int i) in
+  Int64.float_of_bits bits
+
 let parse_object chan =
   let data = input_byte chan in
   if prefix_small_block <= data then
@@ -297,15 +342,21 @@ let parse_object chan =
     let addr = input_int32u chan in
     for _i = 0 to 15 do ignore (input_byte chan); done;
     RCode addr
+  | CODE_CUSTOM ->
+    begin match input_cstring chan with
+    | "_j" -> Rint64 (input_intL chan)
+    | s -> Printf.eprintf "Unhandled custom code: %s" s; assert false
+    end
+  | CODE_DOUBLE_BIG ->
+    RFloat64 (input_double_big chan)
+  | CODE_DOUBLE_LITTLE ->
+    RFloat64 (input_double_little chan)
   | CODE_DOUBLE_ARRAY32_LITTLE
-  | CODE_DOUBLE_BIG
-  | CODE_DOUBLE_LITTLE
   | CODE_DOUBLE_ARRAY8_BIG
   | CODE_DOUBLE_ARRAY8_LITTLE
   | CODE_DOUBLE_ARRAY32_BIG
   | CODE_INFIXPOINTER
-  | CODE_CUSTOM ->
-    Printf.eprintf "Unhandled code %04x\n%!" data; assert false
+    -> Printf.eprintf "Unhandled code %04x\n%!" data; assert false
 
 let parse chan =
   let (magic, len, _, _, size) = parse_header chan in
@@ -337,6 +388,16 @@ let parse chan =
   | RCode addr ->
     let data = Fun addr in
     data, None
+  | Rint64 i ->
+    let data = Ptr !current_object in
+    let () = LargeArray.set memory !current_object (Int64 i) in
+    let () = incr current_object in
+    data, None
+  | RFloat64 f ->
+    let data = Ptr !current_object in
+    let () = LargeArray.set memory !current_object (Float64 f) in
+    let () = incr current_object in
+    data, None
   in
 
   let rec fill block off accu =
@@ -362,8 +423,8 @@ end
 module IChannel =
 struct
   type t = in_channel
-  let input_byte = Pervasives.input_byte
-  let input_binary_int = Pervasives.input_binary_int
+  let input_byte = input_byte
+  let input_binary_int = input_binary_int
 end
 
 module IString =
@@ -396,10 +457,12 @@ let parse_string s = PString.parse (s, ref 0)
 let instantiate (p, mem) =
   let len = LargeArray.length mem in
   let ans = LargeArray.make len (Obj.repr 0) in
-  (** First pass: initialize the subobjects *)
+  (* First pass: initialize the subobjects *)
   for i = 0 to len - 1 do
     let obj = match LargeArray.get mem i with
     | Struct (tag, blk) -> Obj.new_block tag (Array.length blk)
+    | Int64 i -> Obj.repr i
+    | Float64 f -> Obj.repr f
     | String str -> Obj.repr str
     in
     LargeArray.set ans i obj
@@ -408,9 +471,9 @@ let instantiate (p, mem) =
   | Int n -> Obj.repr n
   | Ptr p -> LargeArray.get ans p
   | Atm tag -> Obj.new_block tag 0
-  | Fun _ -> assert false (** We shouldn't serialize closures *)
+  | Fun _ -> assert false (* We shouldn't serialize closures *)
   in
-  (** Second pass: set the pointers *)
+  (* Second pass: set the pointers *)
   for i = 0 to len - 1 do
     match LargeArray.get mem i with
     | Struct (_, blk) ->
@@ -418,6 +481,8 @@ let instantiate (p, mem) =
       for k = 0 to Array.length blk - 1 do
         Obj.set_field obj k (get_data blk.(k))
       done
+    | Int64 _
+    | Float64 _
     | String _ -> ()
   done;
   get_data p
