@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -10,6 +10,7 @@
 
 open CUnix
 open Names
+open Safe_typing
 
 type section_path = {
   dirpath : string list;
@@ -26,7 +27,7 @@ val default_root_prefix : DirPath.t
 
 val add_load_path : physical_path * logical_path -> unit
 
-val recheck_library :
+val recheck_library : safe_environment ->
   norec:object_file list ->
   admit:object_file list ->
-  check:object_file list -> unit
+  check:object_file list -> safe_environment
