@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -17,9 +17,7 @@ sig
   val equal : ('a -> 'a -> bool) -> 'a array -> 'a array -> bool
   val equal_norefl : ('a -> 'a -> bool) -> 'a array -> 'a array -> bool
   val is_empty : 'a array -> bool
-  val exists : ('a -> bool) -> 'a array -> bool
   val exists2 : ('a -> 'b -> bool) -> 'a array -> 'b array -> bool
-  val for_all : ('a -> bool) -> 'a array -> bool
   val for_all2 : ('a -> 'b -> bool) -> 'a array -> 'b array -> bool
   val for_all3 : ('a -> 'b -> 'c -> bool) ->
     'a array -> 'b array -> 'c array -> bool
@@ -37,6 +35,8 @@ sig
   val fold_left_i : (int -> 'a -> 'b -> 'a) -> 'a -> 'b array -> 'a
   val fold_right2 :
     ('a -> 'b -> 'c -> 'c) -> 'a array -> 'b array -> 'c -> 'c
+  val fold_right3 :
+    ('a -> 'b -> 'c -> 'd -> 'd) -> 'a array -> 'b array -> 'c array -> 'd -> 'd
   val fold_left2 :
     ('a -> 'b -> 'c -> 'a) -> 'a -> 'b array -> 'c array -> 'a
   val fold_left3 :
@@ -49,28 +49,18 @@ sig
   val map_to_list : ('a -> 'b) -> 'a array -> 'b list
   val map_of_list : ('a -> 'b) -> 'a list -> 'b array
   val chop : int -> 'a array -> 'a array * 'a array
-  val smartmap : ('a -> 'a) -> 'a array -> 'a array
-  [@@ocaml.deprecated "Same as [Smart.map]"]
-  val smartfoldmap : ('r -> 'a -> 'r * 'a) -> 'r -> 'a array -> 'r * 'a array
-  [@@ocaml.deprecated "Same as [Smart.fold_left_map]"]
-  val map2 : ('a -> 'b -> 'c) -> 'a array -> 'b array -> 'c array
   val map2_i : (int -> 'a -> 'b -> 'c) -> 'a array -> 'b array -> 'c array
   val map3 :
     ('a -> 'b -> 'c -> 'd) -> 'a array -> 'b array -> 'c array -> 'd array
+  val map3_i :
+    (int -> 'a -> 'b -> 'c -> 'd) -> 'a array -> 'b array -> 'c array -> 'd array
   val map_left : ('a -> 'b) -> 'a array -> 'b array
-  val iter2 : ('a -> 'b -> unit) -> 'a array -> 'b array -> unit
   val iter2_i : (int -> 'a -> 'b -> unit) -> 'a array -> 'b array -> unit
   val fold_left_map : ('a -> 'b -> 'a * 'c) -> 'a -> 'b array -> 'a * 'c array
   val fold_right_map : ('a -> 'c -> 'b * 'c) -> 'a array -> 'c -> 'b array * 'c
   val fold_left2_map : ('a -> 'b -> 'c -> 'a * 'd) -> 'a -> 'b array -> 'c array -> 'a * 'd array
+  val fold_left2_map_i : (int -> 'a -> 'b -> 'c -> 'a * 'd) -> 'a -> 'b array -> 'c array -> 'a * 'd array
   val fold_right2_map : ('a -> 'b -> 'c -> 'd * 'c) -> 'a array -> 'b array -> 'c -> 'd array * 'c
-  val fold_map : ('a -> 'b -> 'a * 'c) -> 'a -> 'b array -> 'a * 'c array
-  [@@ocaml.deprecated "Same as [fold_left_map]"]
-  val fold_map' : ('a -> 'c -> 'b * 'c) -> 'a array -> 'c -> 'b array * 'c
-  [@@ocaml.deprecated "Same as [fold_right_map]"]
-  val fold_map2' :
-    ('a -> 'b -> 'c -> 'd * 'c) -> 'a array -> 'b array -> 'c -> 'd array * 'c
-  [@@ocaml.deprecated "Same as [fold_right2_map]"]
   val distinct : 'a array -> bool
   val rev_of_list : 'a list -> 'a array
   val rev_to_list : 'a array -> 'a list
@@ -78,6 +68,7 @@ sig
   module Smart :
   sig
     val map : ('a -> 'a) -> 'a array -> 'a array
+    val map_i : (int -> 'a -> 'a) -> 'a array -> 'a array
     val map2 : ('a -> 'b -> 'b) -> 'a array -> 'b array -> 'b array
     val fold_left_map : ('a -> 'b -> 'a * 'b) -> 'a -> 'b array -> 'a * 'b array
     val fold_left2_map : ('a -> 'b -> 'c -> 'a * 'c) -> 'a -> 'b array -> 'c array -> 'a * 'c array
@@ -85,8 +76,6 @@ sig
   module Fun1 :
   sig
     val map : ('r -> 'a -> 'b) -> 'r -> 'a array -> 'b array
-    val smartmap : ('r -> 'a -> 'a) -> 'r -> 'a array -> 'a array
-    [@@ocaml.deprecated "Same as [Fun1.Smart.map]"]
     val iter : ('r -> 'a -> unit) -> 'r -> 'a array -> unit
     val iter2 : ('r -> 'a -> 'b -> unit) -> 'r -> 'a array -> 'b array -> unit
     module Smart :
@@ -134,17 +123,10 @@ let equal_norefl cmp t1 t2 =
 
 let equal cmp t1 t2 =
   if t1 == t2 then true else equal_norefl cmp t1 t2
-    
+
 
 let is_empty array = Int.equal (Array.length array) 0
 
-let exists f v =
-  let rec exrec = function
-    | -1 -> false
-    | n -> f (uget v n) || (exrec (n-1))
-  in
-  exrec ((Array.length v)-1)
-
 let exists2 f v1 v2 =
   let rec exrec = function
     | -1 -> false
@@ -153,15 +135,6 @@ let exists2 f v1 v2 =
   let lv1 = Array.length v1 in
   lv1 = Array.length v2 && exrec (lv1-1)
 
-let for_all f v =
-  let rec allrec = function
-    | -1 -> true
-    | n ->
-      let ans = f (uget v n) in
-      ans && (allrec (n-1))
-  in
-  allrec ((Array.length v)-1)
-
 let for_all2 f v1 v2 =
   let rec allrec = function
     | -1 -> true
@@ -284,6 +257,16 @@ let fold_left2_i f a v1 v2 =
   if Array.length v2 <> lv1 then invalid_arg "Array.fold_left2_i";
   fold a 0
 
+let fold_right3 f v1 v2 v3 a =
+  let lv1 = Array.length v1 in
+  let rec fold a n =
+    if n=0 then a
+    else
+      let k = n-1 in
+      fold (f (uget v1 k) (uget v2 k) (uget v3 k) a) k in
+  if Array.length v2 <> lv1 || Array.length v3 <> lv1 then invalid_arg "Array.fold_right3";
+  fold a lv1
+
 let fold_left3 f a v1 v2 v3 =
   let lv1 = Array.length v1 in
   let rec fold a n =
@@ -348,20 +331,6 @@ let chop n v =
   if n > vlen then failwith "Array.chop";
   (Array.sub v 0 n, Array.sub v n (vlen-n))
 
-let map2 f v1 v2 =
-  let len1 = Array.length v1 in
-  let len2 = Array.length v2 in
-  let () = if not (Int.equal len1 len2) then invalid_arg "Array.map2" in
-  if Int.equal len1 0 then
-    [| |]
-  else begin
-    let res = Array.make len1 (f (uget v1 0) (uget v2 0)) in
-    for i = 1 to pred len1 do
-      Array.unsafe_set res i (f (uget v1 i) (uget v2 i))
-    done;
-    res
-  end
-
 let map2_i f v1 v2 =
   let len1 = Array.length v1 in
   let len2 = Array.length v2 in
@@ -392,6 +361,21 @@ let map3 f v1 v2 v3 =
     res
   end
 
+let map3_i f v1 v2 v3 =
+  let len1 = Array.length v1 in
+  let len2 = Array.length v2 in
+  let len3 = Array.length v3 in
+  let () = if not (Int.equal len1 len2 && Int.equal len1 len3) then invalid_arg "Array.map3_i" in
+  if Int.equal len1 0 then
+    [| |]
+  else begin
+    let res = Array.make len1 (f 0 (uget v1 0) (uget v2 0) (uget v3 0)) in
+    for i = 1 to pred len1 do
+      Array.unsafe_set res i (f i (uget v1 i) (uget v2 i) (uget v3 i))
+    done;
+    res
+  end
+
 let map_left f a = (* Ocaml does not guarantee Array.map is LR *)
   let l = Array.length a in (* (even if so), then we rewrite it *)
   if Int.equal l 0 then [||] else begin
@@ -402,12 +386,6 @@ let map_left f a = (* Ocaml does not guarantee Array.map is LR *)
     r
   end
 
-let iter2 f v1 v2 =
-  let len1 = Array.length v1 in
-  let len2 = Array.length v2 in
-  let () = if not (Int.equal len2 len1) then invalid_arg "Array.iter2" in
-  for i = 0 to len1 - 1 do f (uget v1 i) (uget v2 i) done
-
 let iter2_i f v1 v2 =
   let len1 = Array.length v1 in
   let len2 = Array.length v2 in
@@ -428,15 +406,11 @@ else
   let v' = Array.map (fun x -> let (y,e) = f x !e' in e' := e; y) v in
   (v',!e')
 
-let fold_map' = fold_right_map
-
 let fold_left_map f e v =
   let e' = ref e in
   let v' = Array.map (fun x -> let (e,y) = f !e' x in e' := e; y) v in
   (!e',v')
 
-let fold_map = fold_left_map
-
 let fold_right2_map f v1 v2 e =
   let e' = ref e in
   let v' =
@@ -444,13 +418,16 @@ let fold_right2_map f v1 v2 e =
   in
   (v',!e')
 
-let fold_map2' = fold_right2_map
-
 let fold_left2_map f e v1 v2 =
   let e' = ref e in
   let v' = map2 (fun x1 x2 -> let (e,y) = f !e' x1 x2 in e' := e; y) v1 v2 in
   (!e',v')
 
+let fold_left2_map_i f e v1 v2 =
+  let e' = ref e in
+  let v' = map2_i (fun idx x1 x2 -> let (e,y) = f idx !e' x1 x2 in e' := e; y) v1 v2 in
+  (!e',v')
+
 let distinct v =
   let visited = Hashtbl.create 23 in
   try
@@ -492,7 +469,7 @@ struct
       end
     done;
     if !i < len then begin
-      (** The array is not the same as the original one *)
+      (* The array is not the same as the original one *)
       let ans : 'a array = Array.copy ar in
       let v = match !temp with None -> assert false | Some x -> x in
       Array.unsafe_set ans !i v;
@@ -506,6 +483,36 @@ struct
       ans
     end else ar
 
+  (* Same as map_i but smart *)
+  let map_i f (ar : 'a array) =
+    let len = Array.length ar in
+    let i = ref 0 in
+    let break = ref true in
+    let temp = ref None in
+    while !break && (!i < len) do
+      let v = Array.unsafe_get ar !i in
+      let v' = f !i v in
+      if v == v' then incr i
+      else begin
+        break := false;
+        temp := Some v';
+      end
+    done;
+    if !i < len then begin
+      (* The array is not the same as the original one *)
+      let ans : 'a array = Array.copy ar in
+      let v = match !temp with None -> assert false | Some x -> x in
+      Array.unsafe_set ans !i v;
+      incr i;
+      while !i < len do
+        let v = Array.unsafe_get ans !i in
+        let v' = f !i v in
+        if v != v' then Array.unsafe_set ans !i v';
+        incr i
+      done;
+      ans
+    end else ar
+
   let map2 f aux_ar ar =
     let len = Array.length ar in
     let aux_len = Array.length aux_ar in
@@ -524,7 +531,7 @@ struct
       end
     done;
     if !i < len then begin
-      (** The array is not the same as the original one *)
+      (* The array is not the same as the original one *)
       let ans : 'a array = Array.copy ar in
       let v = match !temp with None -> assert false | Some x -> x in
       Array.unsafe_set ans !i v;
@@ -545,7 +552,7 @@ struct
     let i = ref 0 in
     let break = ref true in
     let r = ref accu in
-    (** This variable is never accessed unset *)
+    (* This variable is never accessed unset *)
     let temp = ref None in
     while !break && (!i < len) do
       let v = Array.unsafe_get ar !i in
@@ -580,7 +587,7 @@ struct
     let i = ref 0 in
     let break = ref true in
     let r = ref accu in
-    (** This variable is never accessed unset *)
+    (* This variable is never accessed unset *)
     let temp = ref None in
     while !break && (!i < len) do
       let v = Array.unsafe_get ar !i in
@@ -611,10 +618,6 @@ struct
 
 end
 
-(* Deprecated aliases *)
-let smartmap = Smart.map
-let smartfoldmap = Smart.fold_left_map
-
 module Fun1 =
 struct
 
@@ -665,7 +668,7 @@ struct
         end
       done;
       if !i < len then begin
-        (** The array is not the same as the original one *)
+        (* The array is not the same as the original one *)
         let ans : 'a array = Array.copy ar in
         let v = match !temp with None -> assert false | Some x -> x in
         Array.unsafe_set ans !i v;
@@ -681,6 +684,4 @@ struct
 
   end
 
-  let smartmap = Smart.map
-
 end
