@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -48,6 +48,12 @@ sig
       Type names are hashed, so [create] may raise even if no type with
       the exact same name was registered due to a collision. *)
 
+  val anonymous : int -> 'a tag
+  (** [anonymous i] returns a tag describing an [i]-th anonymous type.
+      If [anonymous] is not used together with [create], [max_int] anonymous types
+      are available.
+      [anonymous] raises an exception if [i] is already registered. *)
+
   val eq : 'a tag -> 'b tag -> ('a, 'b) CSig.eq option
   (** [eq t1 t2] returns [Some witness] if [t1] is the same as [t2], [None] otherwise. *)
 
