@@ -8,6 +8,7 @@ export NJOBS
 
 if [ -n "${GITLAB_CI}" ];
 then
+    # Gitlab build, Coq installed into `_install_ci`
     export OCAMLPATH="$PWD/_install_ci/lib:$OCAMLPATH"
     export COQBIN="$PWD/_install_ci/bin"
     export CI_BRANCH="$CI_COMMIT_REF_NAME"
@@ -15,18 +16,22 @@ then
     then
         export CI_PULL_REQUEST="${CI_BRANCH#pr-}"
     fi
+elif [ -d "$PWD/_build/install/default/" ];
+then
+    # Dune build
+    export OCAMLPATH="$PWD/_build/install/default/lib/"
+    export COQBIN="$PWD/_build/install/default/bin"
+    export COQLIB="$PWD/_build/install/default/lib/coq"
+    CI_BRANCH="$(git rev-parse --abbrev-ref HEAD)"
+    export CI_BRANCH
 else
-    if [ -n "${TRAVIS}" ];
-    then
-        export CI_PULL_REQUEST="$TRAVIS_PULL_REQUEST"
-        export CI_BRANCH="$TRAVIS_BRANCH"
-    else # assume local
-        CI_BRANCH="$(git rev-parse --abbrev-ref HEAD)"
-        export CI_BRANCH
-    fi
+    # We assume we are in `-profile devel` build, thus `-local` is set
     export OCAMLPATH="$PWD:$OCAMLPATH"
     export COQBIN="$PWD/bin"
+    CI_BRANCH="$(git rev-parse --abbrev-ref HEAD)"
+    export CI_BRANCH
 fi
+
 export PATH="$COQBIN:$PATH"
 
 # Coq's tools need an ending slash :S, we should fix them.
@@ -41,8 +46,11 @@ for overlay in "${ci_dir}"/user-overlays/*.sh; do
     # shellcheck source=/dev/null
     . "${overlay}"
 done
+
+set +x
 # shellcheck source=ci-basic-overlay.sh
 . "${ci_dir}/ci-basic-overlay.sh"
+set -x
 
 # [git_download project] will download [project] and unpack it
 # in [$CI_BUILD_DIR/project] if the folder does not exist already;
@@ -54,27 +62,30 @@ git_download()
 {
   local PROJECT=$1
   local DEST="$CI_BUILD_DIR/$PROJECT"
+  local GITURL_VAR="${PROJECT}_CI_GITURL"
+  local GITURL="${!GITURL_VAR}"
+  local REF_VAR="${PROJECT}_CI_REF"
+  local REF="${!REF_VAR}"
 
   if [ -d "$DEST" ]; then
     echo "Warning: download and unpacking of $PROJECT skipped because $DEST already exists."
   elif [ "$FORCE_GIT" = "1" ] || [ "$CI" = "" ]; then
-    local GITURL_VAR="${PROJECT}_CI_GITURL"
-    local GITURL="${!GITURL_VAR}"
-    local REF_VAR="${PROJECT}_CI_REF"
-    local REF="${!REF_VAR}"
     git clone "$GITURL" "$DEST"
     cd "$DEST"
     git checkout "$REF"
   else # When possible, we download tarballs to reduce bandwidth and latency
     local ARCHIVEURL_VAR="${PROJECT}_CI_ARCHIVEURL"
     local ARCHIVEURL="${!ARCHIVEURL_VAR}"
-    local REF_VAR="${PROJECT}_CI_REF"
-    local REF="${!REF_VAR}"
     mkdir -p "$DEST"
     cd "$DEST"
-    wget "$ARCHIVEURL/$REF.tar.gz"
-    tar xvfz "$REF.tar.gz" --strip-components=1
-    rm -f "$REF.tar.gz"
+    local COMMIT=$(git ls-remote "$GITURL" "refs/heads/$REF" | cut -f 1)
+    if [[ "$COMMIT" == "" ]]; then
+      # $REF must have been a tag or hash, not a branch
+      COMMIT="$REF"
+    fi
+    wget "$ARCHIVEURL/$COMMIT.tar.gz"
+    tar xvfz "$COMMIT.tar.gz" --strip-components=1
+    rm -f "$COMMIT.tar.gz"
   fi
 }
 
