@@ -10,5 +10,9 @@ git_download fiat_crypto
 # building the executables.
 # c.f. https://github.com/coq/coq/pull/8313#issuecomment-416650241
 
+fiat_crypto_CI_TARGETS1="c-files printlite lite"
+fiat_crypto_CI_TARGETS2="coq"
+
 ( cd "${CI_BUILD_DIR}/fiat_crypto" && git submodule update --init --recursive && \
-  ulimit -s 32768 && make new-pipeline c-files )
+        ulimit -s 32768 && \
+        make ${fiat_crypto_CI_TARGETS1} && make -j 1 ${fiat_crypto_CI_TARGETS2} )
