@@ -25,6 +25,7 @@ let print_vfix_app () = print_string "vfix_app"
 let print_vswith () = print_string "switch"
 
 let ppsort = function
+  | SProp -> print_string "SProp"
   | Set -> print_string "Set"
   | Prop -> print_string "Prop"
   | Type u -> print_string "Type"
@@ -50,7 +51,7 @@ let rec ppzipper z =
       let n = nargs args in
       open_hbox ();
       for i = 0 to n-2 do
-	ppvalues (arg args i);print_string ";";print_space()
+        ppvalues (arg args i);print_string ";";print_space()
       done;
       if n-1 >= 0 then ppvalues (arg args (n-1));
       close_box()
@@ -82,6 +83,8 @@ and ppwhd whd =
   | Vcofix _ -> print_string "cofix"
   | Vconstr_const i -> print_string "C(";print_int i;print_string")"
   | Vconstr_block b -> ppvblock b
+  | Vint64 i -> printf "int64(%LiL)" i
+  | Vfloat64 f -> printf "float64(%.17g)" f
   | Vatom_stk(a,s) ->
       open_hbox();ppatom a;close_box();
       print_string"@";ppstack s
