@@ -27,6 +27,10 @@
 
 open Configwin_types
 
+let set_help_tip wev = function
+  | None -> ()
+  | Some help -> GtkBase.Widget.Tooltip.set_text wev#as_widget help
+
 let modifiers_to_string m =
   let rec iter m s =
     match m with
@@ -34,15 +38,15 @@ let modifiers_to_string m =
       | c :: m ->
           iter m ((
                     match c with
-			`CONTROL -> "<ctrl>"
-		      | `SHIFT -> "<shft>"
-		      | `LOCK -> "<lock>"
-		      | `MOD1 -> "<alt>"
-		      | `MOD2 -> "<mod2>"
-		      | `MOD3 -> "<mod3>"
-		      | `MOD4 -> "<mod4>"
-		      | `MOD5 -> "<mod5>"
-		      | _  -> raise Not_found
+                        `CONTROL -> "<ctrl>"
+                      | `SHIFT -> "<shft>"
+                      | `LOCK -> "<lock>"
+                      | `MOD1 -> "<alt>"
+                      | `MOD2 -> "<mod2>"
+                      | `MOD3 -> "<mod3>"
+                      | `MOD4 -> "<mod4>"
+                      | `MOD5 -> "<mod5>"
+                      | _  -> raise Not_found
                   ) ^ s)
   in
     iter m ""
@@ -55,7 +59,7 @@ class type widget =
 
 let debug = false
 let dbg s = if debug then Minilib.log s else ()
-
+(*
 (** This class builds a frame with a clist and two buttons :
    one to add items and one to remove the selected items.
    The class takes in parameter a function used to add items and
@@ -71,7 +75,6 @@ class ['a] list_selection_box
     f_color
     (eq : 'a -> 'a -> bool)
     add_function title editable
-    (tt:GData.tooltips)
     =
   let _ = dbg "list_selection_box" in
   let wev = GBin.event_box () in
@@ -86,20 +89,16 @@ class ['a] list_selection_box
   let wlist = match titles_opt with
     None ->
       GList.clist ~selection_mode: `MULTIPLE
-	~titles_show: false
-	~packing: wscroll#add ()
+        ~titles_show: false
+        ~packing: wscroll#add ()
   | Some l ->
       GList.clist ~selection_mode: `MULTIPLE
-	~titles: l
-	~titles_show: true
-	~packing: wscroll#add ()
+        ~titles: l
+        ~titles_show: true
+        ~packing: wscroll#add ()
   in
-  let _ =
-    match help_opt with
-      None -> ()
-    | Some help ->
-	tt#set_tip ~text: help ~privat: help wev#coerce
-  in  (* the vbox for the buttons *)
+  let _ = set_help_tip wev help_opt in
+  (* the vbox for the buttons *)
   let vbox_buttons = GPack.vbox () in
   let _ =
     if editable then
@@ -147,18 +146,18 @@ class ['a] list_selection_box
       wlist#freeze ();
       wlist#clear ();
       List.iter
-	(fun ele ->
-	  ignore (wlist#append (f_strings ele));
-	  match f_color ele with
-	    None -> ()
-	  | Some c ->
-	      try wlist#set_row ~foreground: (`NAME c) (wlist#rows - 1)
-	      with _ -> ()
-	)
-	!listref;
+        (fun ele ->
+          ignore (wlist#append (f_strings ele));
+          match f_color ele with
+            None -> ()
+          | Some c ->
+              try wlist#set_row ~foreground: (`NAME c) (wlist#rows - 1)
+              with _ -> ()
+        )
+        !listref;
 
       (match titles_opt with
-	None -> wlist#columns_autosize ()
+        None -> wlist#columns_autosize ()
       |	Some _ -> GToolbox.autosize_clist wlist);
       wlist#thaw ();
       (* the list of selectd elements is now empty *)
@@ -167,18 +166,18 @@ class ['a] list_selection_box
     (** Move up the selected rows. *)
     method up_selected =
       let rec iter n selrows l =
-	match selrows with
-	  [] -> (l, [])
-	| m :: qrows ->
-	    match l with
-	      [] -> ([],[])
-	    | [_] -> (l,[])
-	    | e1 :: e2 :: q when m = n + 1 ->
-		let newl, newrows = iter (n+1) qrows (e1 :: q) in
-		(e2 :: newl, n :: newrows)
-	    | e1 :: q ->
-		let newl, newrows = iter (n+1) selrows q in
-		(e1 ::  newl, newrows)
+        match selrows with
+          [] -> (l, [])
+        | m :: qrows ->
+            match l with
+              [] -> ([],[])
+            | [_] -> (l,[])
+            | e1 :: e2 :: q when m = n + 1 ->
+                let newl, newrows = iter (n+1) qrows (e1 :: q) in
+                (e2 :: newl, n :: newrows)
+            | e1 :: q ->
+                let newl, newrows = iter (n+1) selrows q in
+                (e1 ::  newl, newrows)
       in
       let sorted_select = List.sort compare list_select in
       let new_list, new_rows = iter 0 sorted_select !listref in
@@ -189,27 +188,28 @@ class ['a] list_selection_box
     method edit_selected f_edit =
       let sorted_select = List.sort compare list_select in
       match sorted_select with
-	[] -> ()
+        [] -> ()
       |	n :: _ ->
-	  try
-	    let ele = List.nth !listref n in
-	    let ele2 = f_edit ele in
-	    let rec iter m = function
-		[] -> []
-	      |	e :: q ->
-		  if n = m then
-		    ele2 :: q
-		  else
-		    e :: (iter (m+1) q)
-	    in
-	    self#update (iter 0 !listref);
-	    wlist#select n 0
-	  with
-	    Not_found ->
-	      ()
+          try
+            let ele = List.nth !listref n in
+            let ele2 = f_edit ele in
+            let rec iter m = function
+                [] -> []
+              |	e :: q ->
+                  if n = m then
+                    ele2 :: q
+                  else
+                    e :: (iter (m+1) q)
+            in
+            self#update (iter 0 !listref);
+            wlist#select n 0
+          with
+            Not_found ->
+              ()
 
     initializer
-      (** create the functions called when the buttons are clicked *)
+
+      (* create the functions called when the buttons are clicked *)
       let f_add () =
         (* get the files to add with the function provided *)
         let l = add_function () in
@@ -228,14 +228,14 @@ class ['a] list_selection_box
       in
       let f_remove () =
         (* remove the selected items from the listref and the clist *)
-	let rec iter n = function
-	    [] -> []
-	  | h :: q ->
-	      if List.mem n list_select then
-		iter (n+1) q
-	      else
-		h :: (iter (n+1) q)
-	in
+        let rec iter n = function
+            [] -> []
+          | h :: q ->
+              if List.mem n list_select then
+                iter (n+1) q
+              else
+                h :: (iter (n+1) q)
+        in
         let new_list = iter 0 !listref in
         self#update new_list
       in
@@ -248,10 +248,10 @@ class ['a] list_selection_box
       ignore (wb_up#connect#clicked ~callback:(fun () -> self#up_selected));
       (
        match f_edit_opt with
-	 None -> ()
+         None -> ()
        | Some f ->
-	   let _ = dbg "list_selection_box: connecting wb_edit" in
-	   ignore (wb_edit#connect#clicked ~callback:(fun () -> self#edit_selected f))
+           let _ = dbg "list_selection_box: connecting wb_edit" in
+           ignore (wb_edit#connect#clicked ~callback:(fun () -> self#edit_selected f))
       );
       (* connect the selection and deselection of items in the clist *)
       let f_select ~row ~column ~event =
@@ -278,10 +278,10 @@ class ['a] list_selection_box
       (* initialize the clist with the listref *)
       self#update !listref
   end;;
-
+*)
 
 (** This class is used to build a box for a string parameter.*)
-class string_param_box param (tt:GData.tooltips) =
+class string_param_box param =
   let _ = dbg "string_param_box" in
   let hbox = GPack.hbox () in
   let wev = GBin.event_box ~packing: (hbox#pack ~expand: false ~padding: 2) () in
@@ -291,84 +291,78 @@ class string_param_box param (tt:GData.tooltips) =
       ~packing: (hbox#pack ~expand: param.string_expand ~padding: 2)
       ()
   in
-  let _ =
-    match param.string_help with
-      None -> ()
-    | Some help ->
-	tt#set_tip ~text: help ~privat: help wev#coerce
-  in
+  let _ = set_help_tip wev param.string_help in
   let _ = we#set_text (param.string_to_string param.string_value) in
 
   object (self)
+
     (** This method returns the main box ready to be packed. *)
     method box = hbox#coerce
+
     (** This method applies the new value of the parameter. *)
     method apply =
       let new_value = param.string_of_string we#text in
       if new_value <> param.string_value then
-	let _ = param.string_f_apply new_value in
-	param.string_value <- new_value
+        let _ = param.string_f_apply new_value in
+        param.string_value <- new_value
       else
-	()
+        ()
   end ;;
 
 (** This class is used to build a box for a combo parameter.*)
-class combo_param_box param (tt:GData.tooltips) =
+class combo_param_box param =
     let _ = dbg "combo_param_box" in
     let hbox = GPack.hbox () in
     let wev = GBin.event_box ~packing: (hbox#pack ~expand: false ~padding: 2) () in
     let _wl = GMisc.label ~text: param.combo_label ~packing: wev#add () in
-    let _ =
-      match param.combo_help with
-	  None -> ()
-	| Some help ->
-	    tt#set_tip ~text: help ~privat: help wev#coerce
-    in
+    let _ = set_help_tip wev param.combo_help in
     let get_value = if not param.combo_new_allowed then
       let wc = GEdit.combo_box_text
-	~strings: param.combo_choices
-	?active:(let rec aux i = function
-		   |[] -> None
-		   |h::_ when h = param.combo_value -> Some i
-		   |_::t -> aux (succ i) t
-		 in aux 0 param.combo_choices)
-	~packing: (hbox#pack ~expand: param.combo_expand ~padding: 2)
-	()
+        ~strings: param.combo_choices
+        ?active:(let rec aux i = function
+                   |[] -> None
+                   |h::_ when h = param.combo_value -> Some i
+                   |_::t -> aux (succ i) t
+                 in aux 0 param.combo_choices)
+        ~packing: (hbox#pack ~expand: param.combo_expand ~padding: 2)
+        ()
       in
-	fun () -> match GEdit.text_combo_get_active wc with |None -> "" |Some s -> s
+        fun () -> match GEdit.text_combo_get_active wc with |None -> "" |Some s -> s
     else
       let (wc,_) = GEdit.combo_box_entry_text
-	~strings: param.combo_choices
-	~packing: (hbox#pack ~expand: param.combo_expand ~padding: 2)
-	()
+       ~strings: param.combo_choices
+       ~packing: (hbox#pack ~expand: param.combo_expand ~padding: 2)
+       ()
       in
       let _ = wc#entry#set_editable param.combo_editable in
       let _ = wc#entry#set_text param.combo_value in
-	fun () -> wc#entry#text
+       fun () -> wc#entry#text
     in
 object (self)
+
   (** This method returns the main box ready to be packed. *)
   method box = hbox#coerce
-    (** This method applies the new value of the parameter. *)
+
+  (** This method applies the new value of the parameter. *)
   method apply =
     let new_value = get_value () in
       if new_value <> param.combo_value then
-	let _ = param.combo_f_apply new_value in
-	  param.combo_value <- new_value
+        let _ = param.combo_f_apply new_value in
+          param.combo_value <- new_value
       else
-	()
+        ()
 end ;;
 
 (** Class used to pack a custom box. *)
-class custom_param_box param (tt:GData.tooltips) =
+class custom_param_box param =
   let _ = dbg "custom_param_box" in
   let top =
     match param.custom_framed with
       None -> param.custom_box#coerce
     | Some l ->
-	let wf = GBin.frame ~label: l () in
-	wf#add param.custom_box#coerce;
-	wf#coerce
+        let wf = GBin.frame ~label: l () in
+        wf#add param.custom_box#coerce;
+        wf#coerce
   in
   object (self)
     method box = top
@@ -376,7 +370,7 @@ class custom_param_box param (tt:GData.tooltips) =
   end
 
 (** This class is used to build a box for a text parameter.*)
-class text_param_box param (tt:GData.tooltips) =
+class text_param_box param =
   let _ = dbg "text_param_box" in
   let wf = GBin.frame ~label: param.string_label ~height: 100 () in
   let wev = GBin.event_box ~packing: wf#add () in
@@ -390,12 +384,7 @@ class text_param_box param (tt:GData.tooltips) =
       ~packing: wscroll#add
       ()
   in
-  let _ =
-    match param.string_help with
-      None -> ()
-    | Some help ->
-	tt#set_tip ~text: help ~privat: help wev#coerce
-  in
+  let _ = set_help_tip wev param.string_help in
   let _ = dbg "text_param_box: buffer creation" in
   let buffer = GText.buffer () in
 
@@ -404,47 +393,47 @@ class text_param_box param (tt:GData.tooltips) =
   let _ = dbg "text_param_box: object(self)" in
   object (self)
     val wview = wview
+
     (** This method returns the main box ready to be packed. *)
     method box = wf#coerce
+
     (** This method applies the new value of the parameter. *)
     method apply =
       let v = param.string_of_string (buffer#get_text ()) in
       if v <> param.string_value then
-	(
-	 dbg "apply new value!";
-	 let _ = param.string_f_apply v in
-	 param.string_value <- v
-	)
+        (
+         dbg "apply new value!";
+         let _ = param.string_f_apply v in
+         param.string_value <- v
+        )
       else
-	()
+        ()
   end ;;
 
 (** This class is used to build a box for a boolean parameter.*)
-class bool_param_box param (tt:GData.tooltips) =
+class bool_param_box param =
   let _ = dbg "bool_param_box" in
   let wchk = GButton.check_button
       ~label: param.bool_label
       ()
   in
-  let _ =
-    match param.bool_help with
-      None -> ()
-    | Some help -> tt#set_tip ~text: help ~privat: help wchk#coerce
-  in
+  let _ = set_help_tip wchk param.bool_help in
   let _ = wchk#set_active param.bool_value in
   let _ = wchk#misc#set_sensitive param.bool_editable in
 
   object (self)
+
     (** This method returns the check button ready to be packed. *)
     method box = wchk#coerce
+
     (** This method applies the new value of the parameter. *)
     method apply =
       let new_value = wchk#active in
       if new_value <> param.bool_value then
-	let _ = param.bool_f_apply new_value in
-	param.bool_value <- new_value
+        let _ = param.bool_f_apply new_value in
+        param.bool_value <- new_value
       else
-	()
+        ()
   end ;;
 
 class modifiers_param_box param =
@@ -462,29 +451,24 @@ class modifiers_param_box param =
                                  else value := List.filter ((<>) modifier) !value)))
             param.md_allow
   in
-  let _ =
-    match param.md_help with
-      None -> ()
-    | Some help ->
-       let tooltips = GData.tooltips () in
-       ignore (hbox#connect#destroy ~callback: tooltips#destroy);
-       tooltips#set_tip wev#coerce ~text: help ~privat: help
-  in
+  let _ = set_help_tip wev param.md_help in
   object (self)
+
     (** This method returns the main box ready to be packed. *)
     method box = hbox#coerce
+
     (** This method applies the new value of the parameter. *)
     method apply =
       let new_value = !value in
       if new_value <> param.md_value then
-	let _ = param.md_f_apply new_value in
-	param.md_value <- new_value
+        let _ = param.md_f_apply new_value in
+        param.md_value <- new_value
       else
-	()
+        ()
   end ;;
-
+(*
 (** This class is used to build a box for a parameter whose values are a list.*)
-class ['a] list_param_box (param : 'a list_param) (tt:GData.tooltips) =
+class ['a] list_param_box (param : 'a list_param) =
   let _ = dbg "list_param_box" in
   let listref = ref param.list_value in
   let frame_selection = new list_selection_box
@@ -500,16 +484,19 @@ class ['a] list_param_box (param : 'a list_param) (tt:GData.tooltips) =
   in
 
   object (self)
+
     (** This method returns the main box ready to be packed. *)
     method box = frame_selection#box#coerce
+
     (** This method applies the new value of the parameter. *)
     method apply =
       param.list_f_apply !listref ;
       param.list_value <- !listref
   end ;;
+*)
 
 (** This class creates a configuration box from a configuration structure *)
-class configuration_box (tt : GData.tooltips) conf_struct =
+class configuration_box conf_struct =
 
   let main_box = GPack.hbox () in
 
@@ -540,27 +527,27 @@ class configuration_box (tt : GData.tooltips) conf_struct =
 
   let make_param (main_box : #GPack.box) = function
   | String_param p ->
-    let box = new string_param_box p tt in
+    let box = new string_param_box p in
     let _ = main_box#pack ~expand: false ~padding: 2 box#box in
     box
   | Combo_param p ->
-    let box = new combo_param_box p tt in
+    let box = new combo_param_box p in
     let _ = main_box#pack ~expand: false ~padding: 2 box#box in
     box
   | Text_param p ->
-    let box = new text_param_box p tt in
+    let box = new text_param_box p in
     let _ = main_box#pack ~expand: p.string_expand ~padding: 2 box#box in
     box
   | Bool_param p ->
-    let box = new bool_param_box p tt in
+    let box = new bool_param_box p in
     let _ = main_box#pack ~expand: false ~padding: 2 box#box in
     box
   | List_param f ->
-    let box = f tt in
+    let box = f () in
     let _ = main_box#pack ~expand: true ~padding: 2 box#box in
     box
   | Custom_param p ->
-    let box = new custom_param_box p tt in
+    let box = new custom_param_box p in
     let _ = main_box#pack ~expand: p.custom_expand ~padding: 2 box#box in
     box
   | Modifiers_param p ->
@@ -662,19 +649,18 @@ class configuration_box (tt : GData.tooltips) conf_struct =
    to configure the various parameters. *)
 let edit ?(with_apply=true)
     ?(apply=(fun () -> ()))
-    title ?width ?height
+    title ?parent ?width ?height
     conf_struct =
   let dialog = GWindow.dialog
     ~position:`CENTER
     ~modal: true ~title: title
-    ?height ?width
+    ~type_hint:`DIALOG
+    ?parent ?height ?width
     ()
   in
-  let tooltips = GData.tooltips () in
+  let config_box = new configuration_box conf_struct in
 
-  let config_box = new configuration_box tooltips conf_struct in
-
-  let _ = dialog#vbox#add config_box#box#coerce in
+  let _ = dialog#vbox#pack ~expand:true config_box#box#coerce in
 
   if with_apply then
     dialog#add_button Configwin_messages.mApply `APPLY;
@@ -683,7 +669,6 @@ let edit ?(with_apply=true)
   dialog#add_button Configwin_messages.mCancel `CANCEL;
 
   let destroy () =
-    tooltips#destroy () ;
     dialog#destroy ();
   in
   let rec iter rep =
@@ -700,10 +685,12 @@ let edit ?(with_apply=true)
   in
     iter Return_cancel
 
+(*
 let edit_string l s =
   match GToolbox.input_string ~title: l ~text: s Configwin_messages.mValue with
     None -> s
   | Some s2 -> s2
+*)
 
 (** Create a string param. *)
 let string ?(editable=true) ?(expand=true) ?help ?(f=(fun _ -> ())) label v =
@@ -730,6 +717,7 @@ let bool ?(editable=true) ?help ?(f=(fun _ -> ())) label v =
       bool_f_apply = f ;
     }
 
+(*
 (** Create a list param. *)
 let list ?(editable=true) ?help
     ?(f=(fun (_:'a list) -> ()))
@@ -739,22 +727,21 @@ let list ?(editable=true) ?help
     ?titles ?(color=(fun (_:'a) -> (None : string option)))
     label (f_strings : 'a -> string list) v =
   List_param
-    (fun tt ->
-	new list_param_box
-	  {
-	    list_label = label ;
-	    list_help = help ;
-	    list_value = v ;
-	    list_editable = editable ;
-	    list_titles = titles;
-	    list_eq = eq ;
-	    list_strings = f_strings ;
-	    list_color = color ;
-	    list_f_edit = edit ;
-	    list_f_add = add ;
-	    list_f_apply = f ;
-	  }
-	  tt
+    (fun () ->
+        new list_param_box
+          {
+            list_label = label ;
+            list_help = help ;
+            list_value = v ;
+            list_editable = editable ;
+            list_titles = titles;
+            list_eq = eq ;
+            list_strings = f_strings ;
+            list_color = color ;
+            list_f_edit = edit ;
+            list_f_add = add ;
+            list_f_apply = f ;
+          }
     )
 
 (** Create a strings param. *)
@@ -763,6 +750,7 @@ let strings ?(editable=true) ?help
     ?(eq=Pervasives.(=))
     ?(add=(fun () -> [])) label v =
   list ~editable ?help ~f ~eq ~edit: (edit_string label) ~add label (fun s -> [s]) v
+*)
 
 (** Create a combo param. *)
 let combo ?(editable=true) ?(expand=true) ?help ?(f=(fun _ -> ()))
@@ -807,3 +795,40 @@ let custom ?label box f expand =
       custom_expand = expand ;
       custom_framed = label ;
     }
+
+(* Copying lablgtk question_box + forbidding hiding *)
+
+let question_box ~title ~buttons ?(default=1) ?icon ?parent message =
+  let button_nb = ref 0 in
+  let window = GWindow.dialog ~position:`CENTER ~modal:true ?parent ~type_hint:`DIALOG ~title () in
+  let hbox = GPack.hbox ~border_width:10 ~packing:window#vbox#add () in
+  let bbox = window#action_area in
+  begin match icon with
+    None -> ()
+  | Some i -> hbox#pack i#coerce ~padding:4
+  end;
+  ignore (GMisc.label ~text: message ~packing: hbox#add ());
+  (* the function called to create each button by iterating *)
+  let rec iter_buttons n = function
+      [] ->
+        ()
+    | button_label :: q ->
+        let b = GButton.button ~label: button_label
+            ~packing:(bbox#pack ~expand:true ~padding:4) ()
+        in
+        ignore (b#connect#clicked ~callback:
+          (fun () -> button_nb := n; window#destroy ()));
+        (* If it's the first button then give it the focus *)
+        if n = default then b#grab_default () else ();
+
+        iter_buttons (n+1) q
+  in
+  iter_buttons 1 buttons;
+  ignore (window#connect#destroy ~callback: GMain.Main.quit);
+  window#set_position `CENTER;
+  window#show ();
+  GMain.Main.main ();
+  !button_nb
+
+let message_box ~title ?icon ?parent ?(ok="Ok") message =
+  ignore (question_box ?icon ?parent ~title message ~buttons:[ ok ])
