@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -42,14 +42,11 @@ let version () =
       "The Coq Proof Assistant, version %s (%s)\
        \nArchitecture %s running %s operating system\
        \nGtk version is %s\
-       \nThis is %s (%s is the best one for this architecture and OS)\
-       \n"
+       \nThis is %s \n"
       ver date
       Coq_config.arch Sys.os_type
       (let x,y,z = GMain.Main.version in Printf.sprintf "%d.%d.%d" x y z)
       (Filename.basename Sys.executable_name)
-      Coq_config.best
-
 
 (** * Initial checks by launching test coqtop processes *)
 
@@ -59,7 +56,7 @@ let rec read_all_lines in_chan =
     let len = String.length arg  in
     let arg =
       if len > 0 && arg.[len - 1] = '\r' then
-	String.sub arg 0 (len - 1)
+        String.sub arg 0 (len - 1)
       else arg
     in
     arg::(read_all_lines in_chan)
@@ -105,7 +102,7 @@ let display_coqtop_answer cmd lines =
 
 let rec filter_coq_opts args =
   let argstr = String.concat " " (List.map Filename.quote args) in
-  let cmd = Filename.quote (coqtop_path ()) ^" -nois -filteropts " ^ argstr in
+  let cmd = Filename.quote (coqtop_path ()) ^" -nois -batch " ^ argstr in
   let cmd = requote cmd in
   let filtered_args = ref [] in
   let errlines = ref [] in
@@ -122,7 +119,7 @@ let rec filter_coq_opts args =
 
 and asks_for_coqtop args =
   let pb_mes = GWindow.message_dialog
-    ~message:"Failed to load coqtop. Reset the preference to default ?"
+    ~message:"Failed to load coqidetop. Reset the preference to default ?"
     ~message_type:`QUESTION ~buttons:GWindow.Buttons.yes_no () in
   match pb_mes#run () with
     | `YES ->
@@ -131,16 +128,15 @@ and asks_for_coqtop args =
       let () = pb_mes#destroy () in
       filter_coq_opts args
     | `DELETE_EVENT | `NO ->
-      let () = pb_mes#destroy () in
-      let cmd_sel = GWindow.file_selection
-	~title:"Coqtop to execute (edit your preference then)"
-	~filename:(coqtop_path ()) ~urgency_hint:true () in
-      match cmd_sel#run () with
-	| `OK ->
-	  let () = custom_coqtop := (Some cmd_sel#filename) in
-	  let () = cmd_sel#destroy () in
-	  filter_coq_opts args
-	| `CANCEL | `DELETE_EVENT | `HELP -> exit 0
+      let file = select_file_for_open
+        ~title:"coqidetop to execute (edit your preference then)"
+        ~filter:false
+        ~filename:(coqtop_path ()) () in
+      match file with
+      | Some _ ->
+          let () = custom_coqtop := file in
+          filter_coq_opts args
+      | None -> exit 0
 
 exception WrongExitStatus of string
 
@@ -337,8 +333,8 @@ let unsafe_handle_input handle feedback_processor state conds ~read_all =
     (* Parsing error at the end of s : we have only received a part of
         an xml answer. We store the current fragment for later *)
     let l_end = Lexing.lexeme_end lex in
-    (** Heuristic hack not to reimplement the lexer:  if ever the lexer dies
-        twice at the same place, then this is a non-recoverable error *)
+    (* Heuristic hack not to reimplement the lexer:  if ever the lexer dies
+       twice at the same place, then this is a non-recoverable error *)
     if state.lexerror = Some l_end then raise e;
     state.lexerror <- Some l_end
 
@@ -422,7 +418,7 @@ let rec respawn_coqtop ?(why=Unexpected) coqtop =
     let title = "Warning" in
     let icon = (warn_image ())#coerce in
     let buttons = ["Reset"; "Save all and quit"; "Quit without saving"] in
-    let ans = GToolbox.question_box ~title ~buttons ~icon "Coqtop died badly." in
+    let ans = GToolbox.question_box ~title ~buttons ~icon "coqidetop died badly." in
     if ans = 2 then (!save_all (); GtkMain.Main.quit ())
     else if ans = 3 then GtkMain.Main.quit ()
   | Planned -> ()
@@ -499,7 +495,7 @@ let init_coqtop coqtop task =
 type 'a query = 'a Interface.value task
 
 let eval_call call handle k =
-  (** Send messages to coqtop and prepare the decoding of the answer *)
+  (* Send messages to coqtop and prepare the decoding of the answer *)
   Minilib.log ("Start eval_call " ^ Xmlprotocol.pr_call call);
   assert (handle.alive && handle.waiting_for = None);
   handle.waiting_for <- Some (mk_ccb (call,k));
@@ -601,8 +597,8 @@ struct
     let opts = Hashtbl.fold mkopt current_state [] in
     eval_call (Xmlprotocol.set_options opts) h
       (function
-	| Interface.Good () -> k ()
-	| _ -> failwith "Cannot set options. Resetting coqtop")
+        | Interface.Good () -> k ()
+        | _ -> failwith "Cannot set options. Resetting coqtop")
 
 end
 
