@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -137,7 +137,6 @@ class type ops =
 object
   method go_to_insert : unit task
   method go_to_mark : GText.mark -> unit task
-  method tactic_wizard : string list -> unit task
   method process_next_phrase : unit task
   method process_until_end_or_error : unit task
   method handle_reset_initial : unit task
@@ -255,8 +254,8 @@ object(self)
       let sentence = Doc.find document find in
       let mark = sentence.start in
       let iter = script#buffer#get_iter_at_mark mark in
-      (** Sentence starts tend to be at the end of a line, so we rather choose
-          the first non-line-ending position. *)
+      (* Sentence starts tend to be at the end of a line, so we rather choose
+         the first non-line-ending position. *)
       let rec sentence_start iter =
         if iter#ends_line then sentence_start iter#forward_line
         else iter
@@ -281,7 +280,7 @@ object(self)
               List.exists (fun m ->
                 Gobject.get_oid m =
                 Gobject.get_oid (buffer#get_mark s.start)) marks in
-            try Doc.find document mem_marks 
+            try Doc.find document mem_marks
             with Not_found -> aux iter#backward_char in
         aux iter in
       let ss =
@@ -294,7 +293,7 @@ object(self)
       script#misc#set_tooltip_text ""; script#misc#set_has_tooltip true
     end;
     false
-    
+
   method destroy () =
     feedback_timer.Ideutils.kill ()
 
@@ -324,7 +323,7 @@ object(self)
 
   method private get_start_of_input =
     buffer#get_iter_at_mark (`NAME "start_of_input")
-  
+
   method private get_end_of_input =
     buffer#get_iter_at_mark (`NAME "stop_of_input")
 
@@ -698,7 +697,7 @@ object(self)
   method private find_id until =
     try
       Doc.find_id document (fun id { start;stop } -> until (Some id) start stop)
-    with Not_found -> initial_state, Doc.focused document 
+    with Not_found -> initial_state, Doc.focused document
 
   method private cleanup seg =
     if seg <> [] then begin
@@ -759,7 +758,7 @@ object(self)
                  (Doc.focused document && Doc.is_in_focus document safe_id))
     in
       undo to_id unfocus_needed)
-  
+
   method private backtrack_until ?move_insert until =
     self#backtrack_to_id ?move_insert (self#find_id until)
 
@@ -783,7 +782,7 @@ object(self)
 
   method backtrack_last_phrase =
     messages#default_route#clear;
-    try 
+    try
       let tgt = Doc.before_tip document in
       self#backtrack_to_id tgt
     with Not_found -> Coq.return (Coq.reset_coqtop _ct)
@@ -806,48 +805,6 @@ object(self)
     else Coq.seq (self#backtrack_to_iter ~move_insert:false point)
           (Coq.lift (fun () -> Sentence.tag_on_insert buffer)))
 
-  method tactic_wizard l =
-    let insert_phrase phrase tag =
-      let stop = self#get_start_of_input in
-      let phrase' = if stop#starts_line then phrase else "\n"^phrase in
-      buffer#insert ~iter:stop phrase';
-      Sentence.tag_on_insert buffer;
-      let start = self#get_start_of_input in
-      buffer#move_mark ~where:stop (`NAME "start_of_input");
-      buffer#apply_tag tag ~start ~stop;
-      if self#get_insert#compare stop <= 0 then
-        buffer#place_cursor ~where:stop;
-      let sentence =
-        mk_sentence
-          ~start:(`MARK (buffer#create_mark start))
-          ~stop:(`MARK (buffer#create_mark stop))
-          [] in
-      Doc.push document sentence;
-      messages#default_route#clear;
-      self#show_goals
-    in
-    let display_error (loc, s) =
-      messages#default_route#add (Ideutils.validate s) in
-    let try_phrase phrase stop more =
-      let action = log "Sending to coq now" in
-      let route_id = 0 in
-      let query = Coq.query (route_id,(phrase,Stateid.dummy)) in
-      let next = function
-      | Fail (_, l, str) -> (* FIXME: check *)
-        display_error (l, str);
-        messages#default_route#add (Pp.str ("Unsuccessfully tried: "^phrase));
-        more
-      | Good () -> stop Tags.Script.processed
-      in
-      Coq.bind (Coq.seq action query) next
-    in
-    let rec loop l = match l with
-      | [] -> Coq.return ()
-      | p :: l' ->
-        try_phrase ("progress "^p^".") (insert_phrase (p^".")) (loop l')
-    in
-    loop l
-
   method handle_reset_initial =
     let action () =
       (* clear the stack *)
