@@ -7,14 +7,14 @@ let list_items menu li =
   let tactic_item = function
     |[] -> Buffer.create 1
     |[s] -> let b = Buffer.create 16 in
-	    let () = Buffer.add_string b ("<menuitem action='"^menu^" "^(no_under s)^"' />\n") in
-	    b
+            let () = Buffer.add_string b ("<menuitem action='"^menu^" "^(no_under s)^"' />\n") in
+            b
     |s::_ as l -> let b = Buffer.create 50 in
-		  let () = (Buffer.add_string b ("<menu action='"^menu^" "^(String.make 1 s.[0])^"'>\n")) in
-		  let () = (List.iter
-			     (fun x -> Buffer.add_string b ("<menuitem action='"^menu^" "^(no_under x)^"' />\n")) l) in
-		  let () = Buffer.add_string b"</menu>\n" in
-		  b in
+                  let () = (Buffer.add_string b ("<menu action='"^menu^" "^(String.make 1 s.[0])^"'>\n")) in
+                  let () = (List.iter
+                             (fun x -> Buffer.add_string b ("<menuitem action='"^menu^" "^(no_under x)^"' />\n")) l) in
+                  let () = Buffer.add_string b"</menu>\n" in
+                  b in
   let () = List.iter (fun b -> Buffer.add_buffer res_buf (tactic_item b)) li in
   res_buf
 
@@ -100,18 +100,7 @@ let init () =
 \n    <menuitem action='Previous' />\
 \n    <menuitem action='Next' />\
 \n  </menu>\
-\n  <menu action='Try Tactics'>\
-\n    <menuitem action='auto' />\
-\n    <menuitem action='auto with *' />\
-\n    <menuitem action='eauto' />\
-\n    <menuitem action='eauto with *' />\
-\n    <menuitem action='intuition' />\
-\n    <menuitem action='omega' />\
-\n    <menuitem action='simpl' />\
-\n    <menuitem action='tauto' />\
-\n    <menuitem action='trivial' />\
-\n    <menuitem action='Wizard' />\
-\n    <separator />\
+\n  <menu action='Tactics'>\
 \n    %s\
 \n  </menu>\
 \n  <menu action='Templates'>\
@@ -140,6 +129,8 @@ let init () =
 \n    <menuitem action='Uncomment' />\
 \n    <separator />\
 \n    <menuitem action='Coqtop arguments' />\
+\n    <separator />\
+\n    <menuitem action='LaTeX-to-unicode' />\
 \n  </menu>\
 \n  <menu action='Compile'>\
 \n    <menuitem action='Compile buffer' />\
@@ -171,7 +162,6 @@ let init () =
 \n  <toolitem action='Interrupt' />\
 \n  <toolitem action='Previous' />\
 \n  <toolitem action='Next' />\
-\n  <toolitem action='Wizard' />\
 \n</toolbar>\
 \n</ui>"
     (if Coq_config.gtk_platform <> `QUARTZ then "<menuitem action='Quit' />" else "")
