@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -8,17 +8,18 @@
 (*         *     (see LICENSE file for the text of the license)         *)
 (************************************************************************)
 
-
 open Preferences
 
+let _ = GtkMain.Main.init ()
+
 let warn_image () =
   let img = GMisc.image () in
   img#set_stock `DIALOG_WARNING;
   img#set_icon_size `DIALOG;
   img
 
-let warning msg = 
-   GToolbox.message_box ~title:"Warning" ~icon:(warn_image ())#coerce msg 
+let warning msg =
+   GToolbox.message_box ~title:"Warning" ~icon:(warn_image ())#coerce msg
 
 let cb = GData.clipboard Gdk.Atom.primary
 
@@ -33,9 +34,38 @@ let push_info,pop_info,clear_info =
   (fun () -> decr size; status_context#pop ()),
   (fun () -> for _i = 1 to !size do status_context#pop () done; size := 0)
 
+type 'a mlist = Nil | Cons of { hd : 'a ; mutable tl : 'a mlist }
+
+let enqueue a x =
+  let rec aux x = match x with
+    | Nil -> assert false
+    | Cons p ->
+      match p.tl with
+      | Nil -> p.tl <- Cons { hd = a ; tl = Nil }
+      | _ -> aux p.tl in
+  match !x with
+  | Nil -> x := Cons { hd = a ; tl = Nil }
+  | _ -> aux !x
+
+let pop = function
+  | Cons p -> p.tl
+  | Nil -> assert false
+
 let flash_info =
+  let queue = ref Nil in
   let flash_context = status#new_context ~name:"Flash" in
-    (fun ?(delay=5000) s -> flash_context#flash ~delay s)
+  let rec process () = match !queue with
+    | Cons { hd = (delay,text) } ->
+       let msg = flash_context#push text in
+       ignore (Glib.Timeout.add ~ms:delay ~callback:(fun () ->
+                   flash_context#remove msg;
+                   queue := pop !queue;
+                   process (); false))
+    | Nil -> () in
+  fun ?(delay=5000) text ->
+    let processing = !queue <> Nil in
+    enqueue (delay,text) queue;
+    if not processing then process ()
 
 (* Note: Setting the same attribute with two separate tags appears to use
 the first value applied and not the second.  I saw this trying to set the background
@@ -43,10 +73,10 @@ color on Windows.  A clean fix, if ever needed, would be to combine the attribut
 of the tags into a single composite tag before applying.  This is left as an
 exercise for the reader. *)
 let insert_with_tags (buf : #GText.buffer_skel) mark rmark tags text =
-  (** FIXME: LablGTK2 does not export the C insert_with_tags function, so that
-      it has to reimplement its own helper function. Unluckily, it relies on
-      a slow algorithm, so that we have to have our own quicker version here.
-      Alas, it is still much slower than the native version... *)
+  (* FIXME: LablGTK2 does not export the C insert_with_tags function, so that
+     it has to reimplement its own helper function. Unluckily, it relies on
+     a slow algorithm, so that we have to have our own quicker version here.
+     Alas, it is still much slower than the native version... *)
   if CList.is_empty tags then buf#insert ~iter:(buf#get_iter_at_mark mark) text
   else
     let it = buf#get_iter_at_mark mark in
@@ -182,15 +212,15 @@ let try_export file_name s =
     try match encoding#get with
       |Eutf8 -> Minilib.log "UTF-8 is enforced" ; s
       |Elocale ->
-	let is_unicode,char_set = Glib.Convert.get_charset () in
-	if is_unicode then
-	  (Minilib.log "Locale is UTF-8" ; s)
-	else
-	  (Minilib.log ("Locale is "^char_set);
-	   Glib.Convert.convert_with_fallback
+        let is_unicode,char_set = Glib.Convert.get_charset () in
+        if is_unicode then
+          (Minilib.log "Locale is UTF-8" ; s)
+        else
+          (Minilib.log ("Locale is "^char_set);
+           Glib.Convert.convert_with_fallback
              ~from_codeset:"UTF-8" ~to_codeset:char_set s)
       |Emanual enc ->
-	(Minilib.log ("Manual charset is "^ enc);
+        (Minilib.log ("Manual charset is "^ enc);
          Glib.Convert.convert_with_fallback
            ~from_codeset:"UTF-8" ~to_codeset:enc s)
     with e ->
@@ -229,14 +259,17 @@ let current_dir () = match project_path#get with
 | None -> ""
 | Some dir -> dir
 
-let select_file_for_open ~title ?filename () =
+let select_file_for_open ~title ?(filter=true) ?parent ?filename () =
   let file_chooser =
-    GWindow.file_chooser_dialog ~action:`OPEN ~modal:true ~title ()
+    GWindow.file_chooser_dialog ~action:`OPEN ~modal:true ~title ?parent ()
   in
   file_chooser#add_button_stock `CANCEL `CANCEL ;
   file_chooser#add_select_button_stock `OPEN `OPEN ;
-  file_chooser#add_filter (filter_coq_files ());
-  file_chooser#add_filter (filter_all_files ());
+  if filter then
+    begin
+      file_chooser#add_filter (filter_coq_files ());
+      file_chooser#add_filter (filter_all_files ())
+    end;
   file_chooser#set_default_response `OPEN;
   let dir = match filename with
     | None -> current_dir ()
@@ -246,19 +279,19 @@ let select_file_for_open ~title ?filename () =
     match file_chooser#run () with
     | `OPEN ->
       begin
-	match file_chooser#filename with
-	  | None -> None
-	  | Some _ as f ->
+        match file_chooser#filename with
+          | None -> None
+          | Some _ as f ->
             project_path#set file_chooser#current_folder; f
       end
     | `DELETE_EVENT | `CANCEL -> None in
   file_chooser#destroy ();
   file
 
-let select_file_for_save ~title ?filename () =
+let select_file_for_save ~title ?parent ?filename () =
   let file = ref None in
   let file_chooser =
-    GWindow.file_chooser_dialog ~action:`SAVE ~modal:true ~title ()
+    GWindow.file_chooser_dialog ~action:`SAVE ~modal:true ~title ?parent ()
   in
   file_chooser#add_button_stock `CANCEL `CANCEL ;
   file_chooser#add_select_button_stock `SAVE `SAVE ;
@@ -328,15 +361,18 @@ let coqtop_path () =
       | None ->
         try
           let new_prog = System.get_toplevel_path "coqidetop" in
-            if Sys.file_exists new_prog then new_prog
-	    else
-	      let in_macos_bundle =
-		Filename.concat
-		  (Filename.dirname new_prog)
-		  (Filename.concat "../Resources/bin" (Filename.basename new_prog))
-	      in if Sys.file_exists in_macos_bundle then in_macos_bundle
-                 else "coqidetop"
-        with Not_found -> "coqidetop"
+            (* The file exists or it is to be found by path *)
+            if Sys.file_exists new_prog ||
+               CString.equal Filename.(basename new_prog) new_prog
+            then new_prog
+            else
+              let in_macos_bundle =
+                Filename.concat
+                  (Filename.dirname new_prog)
+                  (Filename.concat "../Resources/bin" (Filename.basename new_prog))
+              in if Sys.file_exists in_macos_bundle then in_macos_bundle
+                 else "coqidetop.opt"
+        with Not_found -> "coqidetop.opt"
   in file
 
 (* In win32, when a command-line is to be executed via cmd.exe
@@ -455,15 +491,6 @@ let browse prerr url =
   in
   run_command (fun _ -> ()) finally com
 
-let doc_url () =
-  if doc_url#get = use_default_doc_url || doc_url#get = ""
-  then
-    let addr = List.fold_left Filename.concat (Envars.docdir ())
-      ["html";"refman";"index.html"]
-    in
-    if Sys.file_exists addr then "file://"^addr else Coq_config.wwwrefman
-  else doc_url#get
-
 let url_for_keyword =
   let ht = Hashtbl.create 97 in
     lazy (
@@ -473,13 +500,7 @@ let url_for_keyword =
             (fun x -> Sys.file_exists (Filename.concat x "index_urls.txt"))
             (Minilib.coqide_data_dirs ())) "index_urls.txt" in
             open_in index_urls
-          with Not_found ->
-            let doc_url = doc_url () in
-            let n = String.length doc_url in
-              if n > 8 && String.sub doc_url 0 7 = "file://" then
-                open_in (String.sub doc_url 7 (n-7) ^ "index_urls.txt")
-              else
-                raise Exit
+          with Not_found -> raise Exit
         in
           try while true do
             let s = input_line cin in
@@ -500,7 +521,7 @@ let url_for_keyword =
 let browse_keyword prerr text =
   try
     let u = Lazy.force url_for_keyword text in
-    browse prerr (doc_url() ^ u)
+    browse prerr (Coq_config.wwwrefman ^ u)
   with Not_found -> prerr ("No documentation found for \""^text^"\".\n")
 
 let rec is_valid (s : Pp.t) = match Pp.repr s with
