@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -10,12 +10,10 @@
 
 open Configwin
 
-let pref_file = Filename.concat (Minilib.coqide_config_home ()) "coqiderc"
-let accel_file = Filename.concat (Minilib.coqide_config_home ()) "coqide.keys"
-let lang_manager = GSourceView2.source_language_manager ~default:true
+let lang_manager = GSourceView3.source_language_manager ~default:true
 let () = lang_manager#set_search_path
   ((Minilib.coqide_data_dirs ())@lang_manager#search_path)
-let style_manager = GSourceView2.source_style_scheme_manager ~default:true
+let style_manager = GSourceView3.source_style_scheme_manager ~default:true
 let () = style_manager#set_search_path
   ((Minilib.coqide_data_dirs ())@style_manager#search_path)
 
@@ -28,6 +26,7 @@ type tag = {
   tag_strikethrough : bool;
 }
 
+
 (** Generic preferences *)
 
 type obj = {
@@ -73,11 +72,11 @@ object (self)
   method default = default
 end
 
-let stick (pref : 'a preference) (obj : #GObj.widget as 'obj)
+let stick (pref : 'a preference) (obj : < connect : #GObj.widget_signals ; .. >)
   (cb : 'a -> unit) =
   let _ = cb pref#get in
   let p_id = pref#connect#changed ~callback:(fun v -> cb v) in
-  let _ = obj#misc#connect#destroy ~callback:(fun () -> pref#connect#disconnect p_id) in
+  let _ = obj#connect#destroy ~callback:(fun () -> pref#connect#disconnect p_id) in
   ()
 
 (** Useful marshallers *)
@@ -167,7 +166,7 @@ object
   method into l =
     try
       Some (CList.map (fun s ->
-        let split = CString.split sep s in
+        let split = String.split_on_char sep s in
         CList.nth split 0, CList.nth split 1) l)
     with Failure _ -> None
 end
@@ -234,24 +233,25 @@ end
 
 end
 
-let get_config_file name =
+let get_config_file dirs name =
   let find_config dir = Sys.file_exists (Filename.concat dir name) in
-  let config_dir = List.find find_config (Minilib.coqide_config_dirs ()) in
+  let config_dir = List.find find_config dirs in
   Filename.concat config_dir name
 
-(* Small hack to handle v8.3 to v8.4 change in configuration file *)
-let loaded_pref_file =
-  try get_config_file "coqiderc"
-  with Not_found -> Filename.concat (Option.default "" (Glib.get_home_dir ())) ".coqiderc"
+let get_unicode_bindings_local_file () =
+  try Some (get_config_file [Minilib.coqide_config_home ()] "coqide.bindings")
+  with Not_found -> None
 
-let loaded_accel_file =
-  try get_config_file "coqide.keys"
-  with Not_found -> Filename.concat (Option.default "" (Glib.get_home_dir ())) ".coqide.keys"
+let get_unicode_bindings_default_file () =
+  let name = "default.bindings" in
+  let chk d = Sys.file_exists (Filename.concat d name) in
+  try
+    let dir = List.find chk (Minilib.coqide_data_dirs ()) in
+    Some (Filename.concat dir name)
+  with Not_found -> None
 
 (** Hooks *)
 
-(** New style preferences *)
-
 let cmd_coqtop =
   new preference ~name:["cmd_coqtop"] ~init:None ~repr:Repr.(option string)
 
@@ -321,32 +321,49 @@ let attach_modifiers (pref : string preference) prefix =
   in
   pref#connect#changed ~callback:cb
 
+let select_arch m m_osx =
+  if Coq_config.arch = "Darwin" then m_osx else m
+
 let modifier_for_navigation =
-  new preference ~name:["modifier_for_navigation"] ~init:"<Control>" ~repr:Repr.(string)
+  new preference ~name:["modifier_for_navigation"]
+    ~init:(select_arch "<Control>" "<Control><Primary>") ~repr:Repr.(string)
 
 let modifier_for_templates =
   new preference ~name:["modifier_for_templates"] ~init:"<Control><Shift>" ~repr:Repr.(string)
- 
+
 let modifier_for_tactics =
-  new preference ~name:["modifier_for_tactics"] ~init:"<Control><Alt>" ~repr:Repr.(string)
+  new preference ~name:["modifier_for_tactics"]
+    ~init:(select_arch "<Control><Alt>" "<Control><Primary>") ~repr:Repr.(string)
 
 let modifier_for_display =
-  new preference ~name:["modifier_for_display"] ~init:"<Alt><Shift>" ~repr:Repr.(string)
+  new preference ~name:["modifier_for_display"]
+    ~init:(select_arch "<Alt><Shift>" "<Primary><Shift>")~repr:Repr.(string)
 
 let modifier_for_queries =
   new preference ~name:["modifier_for_queries"] ~init:"<Control><Shift>" ~repr:Repr.(string)
 
-let _ = attach_modifiers modifier_for_navigation "<Actions>/Navigation/"
-let _ = attach_modifiers modifier_for_templates "<Actions>/Templates/"
-let _ = attach_modifiers modifier_for_tactics "<Actions>/Tactics/"
-let _ = attach_modifiers modifier_for_display "<Actions>/View/"
-let _ = attach_modifiers modifier_for_queries "<Actions>/Queries/"
+let attach_modifiers_callback () =
+  (* Tell to propagate changes done in preference menu to accel map *)
+  (* To be done after the preferences are loaded *)
+  let _ = attach_modifiers modifier_for_navigation "<Actions>/Navigation/" in
+  let _ = attach_modifiers modifier_for_templates "<Actions>/Templates/" in
+  let _ = attach_modifiers modifier_for_tactics "<Actions>/Tactics/" in
+  let _ = attach_modifiers modifier_for_display "<Actions>/View/" in
+  let _ = attach_modifiers modifier_for_queries "<Actions>/Queries/" in
+  ()
 
 let modifiers_valid =
   new preference ~name:["modifiers_valid"] ~init:"<Alt><Control><Shift>" ~repr:Repr.(string)
 
+let browser_cmd_fmt =
+ try
+  let coq_netscape_remote_var = "COQREMOTEBROWSER" in
+  Sys.getenv coq_netscape_remote_var
+ with
+  Not_found -> Coq_config.browser
+
 let cmd_browse =
-  new preference ~name:["cmd_browse"] ~init:Flags.browser_cmd_fmt ~repr:Repr.(string)
+  new preference ~name:["cmd_browse"] ~init:browser_cmd_fmt ~repr:Repr.(string)
 
 let cmd_editor =
   let init = if Sys.os_type = "Win32" then "NOTEPAD %s" else "emacs %s" in
@@ -359,31 +376,9 @@ let text_font =
   in
   new preference ~name:["text_font"] ~init ~repr:Repr.(string)
 
-let doc_url =
-object
-  inherit [string] preference
-    ~name:["doc_url"] ~init:Coq_config.wwwrefman ~repr:Repr.(string)
-    as super
-
-  method! set v =
-    if not (Flags.is_standard_doc_url v) &&
-      v <> use_default_doc_url &&
-      (* Extra hack to support links to last released doc version *)
-      v <> Coq_config.wwwcoq ^ "doc" &&
-      v <> Coq_config.wwwcoq ^ "doc/"
-    then super#set v
-
-end
-
-let library_url =
-  new preference ~name:["library_url"] ~init:Coq_config.wwwstdlib ~repr:Repr.(string)
-
 let show_toolbar =
   new preference ~name:["show_toolbar"] ~init:true ~repr:Repr.(bool)
 
-let contextual_menus_on_goal =
-  new preference ~name:["contextual_menus_on_goal"] ~init:true ~repr:Repr.(bool)
-
 let window_width =
   new preference ~name:["window_width"] ~init:800 ~repr:Repr.(int)
 
@@ -393,6 +388,9 @@ let window_height =
 let auto_complete =
   new preference ~name:["auto_complete"] ~init:false ~repr:Repr.(bool)
 
+let auto_complete_delay =
+  new preference ~name:["auto_complete_delay"] ~init:250 ~repr:Repr.(int)
+
 let stop_before =
   new preference ~name:["stop_before"] ~init:true ~repr:Repr.(bool)
 
@@ -409,8 +407,8 @@ let vertical_tabs =
 let opposite_tabs =
   new preference ~name:["opposite_tabs"] ~init:false ~repr:Repr.(bool)
 
-let background_color =
-  new preference ~name:["background_color"] ~init:"cornsilk" ~repr:Repr.(string)
+(* let background_color = *)
+(*   new preference ~name:["background_color"] ~init:"cornsilk" ~repr:Repr.(string) *)
 
 let attach_tag (pref : string preference) (tag : GText.tag) f =
   tag#set_property (f pref#get);
@@ -425,8 +423,11 @@ let attach_fg (pref : string preference) (tag : GText.tag) =
 let processing_color =
   new preference ~name:["processing_color"] ~init:"light blue" ~repr:Repr.(string)
 
+let incompletely_processed_color =
+  new preference ~name:["incompletely_processed_color"] ~init:"light sky blue" ~repr:Repr.(string)
+
 let _ = attach_bg processing_color Tags.Script.to_process
-let _ = attach_bg processing_color Tags.Script.incomplete
+let _ = attach_bg incompletely_processed_color Tags.Script.incomplete
 
 let tags = ref Util.String.Map.empty
 
@@ -559,7 +560,8 @@ let tab_length =
 let highlight_current_line =
   new preference ~name:["highlight_current_line"] ~init:false ~repr:Repr.(bool)
 
-let nanoPG =
+let microPG =
+  (* Legacy name in preference is "nanoPG" *)
   new preference ~name:["nanoPG"] ~init:false ~repr:Repr.(bool)
 
 let user_queries =
@@ -587,7 +589,7 @@ object (self)
     | None -> set#set_active true
     | Some c ->
       set#set_active false;
-      but#set_color (Tags.color_of_string c)
+      but#set_color (Gdk.Color.color_parse c)
     in
     track tag.tag_bg_color bg_color bg_unset;
     track tag.tag_fg_color fg_color fg_unset;
@@ -599,7 +601,7 @@ object (self)
   method tag =
     let get but set =
       if set#active then None
-      else Some (Tags.string_of_color but#color)
+      else Some (Gdk.Color.color_to_string but#color)
     in
     {
       tag_bg_color = get bg_color bg_unset;
@@ -641,28 +643,73 @@ let tag_button () =
   let box = GPack.hbox () in
   new tag_button (Gobject.unsafe_cast box#as_widget)
 
-(** Old style preferences *)
+(** Loading/saving preferences *)
 
-let save_pref () =
+let pref_file = Filename.concat (Minilib.coqide_config_home ()) "coqiderc"
+let accel_file = Filename.concat (Minilib.coqide_config_home ()) "coqide.keys"
+
+let save_accel_pref () =
+  if not (Sys.file_exists (Minilib.coqide_config_home ()))
+  then Unix.mkdir (Minilib.coqide_config_home ()) 0o700;
+  GtkData.AccelMap.save accel_file
+
+let save_rc_pref () =
   if not (Sys.file_exists (Minilib.coqide_config_home ()))
   then Unix.mkdir (Minilib.coqide_config_home ()) 0o700;
-  let () = try GtkData.AccelMap.save accel_file with _ -> () in
   let add = Util.String.Map.add in
   let fold key obj accu = add key (obj.get ()) accu in
   let prefs = Util.String.Map.fold fold !preferences Util.String.Map.empty in
   let prefs = Util.String.Map.fold Util.String.Map.add !unknown_preferences prefs in
   Config_lexer.print_file pref_file prefs
 
-let load_pref () =
-  let () = try GtkData.AccelMap.load loaded_accel_file with _ -> () in
+let save_pref () =
+  save_accel_pref ();
+  save_rc_pref ()
 
-    let m = Config_lexer.load_file loaded_pref_file in
+let try_load_pref_file loader warn file =
+  try
+    loader file
+  with
+    e -> warn ~delay:5000 ("Could not load " ^ file ^ " ("^Printexc.to_string e^")")
+
+let load_pref_file loader warn name =
+  try
+    let user_file = get_config_file [Minilib.coqide_config_home ()] name in
+    warn ~delay:2000 ("Loading " ^ user_file);
+    try_load_pref_file loader warn user_file
+  with Not_found ->
+  try
+    let system_wide_file = get_config_file (Minilib.coqide_system_config_dirs ()) name in
+    warn ~delay:5000 ("No user " ^ name ^ ", using system wide configuration");
+    try_load_pref_file loader warn system_wide_file
+  with Not_found ->
+  (* Compatibility with oldest versions of CoqIDE (<= 8.4) *)
+  try
+    let old_user_file = get_config_file [Option.default "" (Glib.get_home_dir ())] ("."^name) in
+    warn ~delay:5000 ("No " ^ name ^ ", trying to recover from an older version of CoqIDE");
+    try_load_pref_file loader warn old_user_file
+  with Not_found ->
+  (* Built-in configuration *)
+    warn ~delay:5000 ("No " ^ name ^ ", using default internal configuration")
+
+let load_accel_pref ~warn =
+  load_pref_file GtkData.AccelMap.load warn "coqide.keys"
+
+let load_rc_pref ~warn =
+  let loader file =
+    let m = Config_lexer.load_file file in
     let iter name v =
       if Util.String.Map.mem name !preferences then
         try (Util.String.Map.find name !preferences).set v with _ -> ()
       else unknown_preferences := Util.String.Map.add name v !unknown_preferences
     in
-    Util.String.Map.iter iter m
+    Util.String.Map.iter iter m in
+  load_pref_file loader warn "coqiderc";
+  attach_modifiers_callback ()
+
+let load_pref ~warn =
+  load_rc_pref ~warn;
+  load_accel_pref ~warn
 
 let pstring name p = string ~f:p#set name p#get
 let pbool name p = bool ~f:p#set name p#get
@@ -673,15 +720,11 @@ let pmodifiers ?(all = false) name p = modifiers
   name
   (str_to_mod_list p#get)
 
-[@@@ocaml.warning "-3"]       (* String.uppercase_ascii since 4.03.0 GPR#124 *)
-let uppercase = String.uppercase
-[@@@ocaml.warning "+3"]
-
-let configure ?(apply=(fun () -> ())) () =
+let configure ?(apply=(fun () -> ())) parent =
   let cmd_coqtop =
     string
       ~f:(fun s -> cmd_coqtop#set (if s = "AUTO" then None else Some s))
-      "       coqtop" (match cmd_coqtop#get with |None -> "AUTO" | Some x -> x) in
+      "    coqidetop" (match cmd_coqtop#get with |None -> "AUTO" | Some x -> x) in
   let cmd_coqc = pstring "       coqc" cmd_coqc in
   let cmd_make = pstring "       make" cmd_make in
   let cmd_coqmakefile = pstring "coqmakefile" cmd_coqmakefile in
@@ -695,19 +738,19 @@ let configure ?(apply=(fun () -> ())) () =
       "Goal (∃n : nat, n ≤ 0)∧(∀x,y,z, x∈y⋃z↔x∈y∨x∈z).";
     box#pack ~expand:true w#coerce;
     ignore (w#misc#connect#realize
-	      ~callback:(fun () -> w#set_font_name text_font#get));
+              ~callback:(fun () -> w#set_font_name text_font#get));
     custom
       ~label:"Fonts for text"
       box
       (fun () ->
-	 let fd =  w#font_name in
-	 text_font#set fd)
+         let fd =  w#font_name in
+         text_font#set fd)
       true
   in
 
   let config_color =
     let box = GPack.vbox () in
-    let table = GPack.table
+    let grid = GPack.grid
       ~row_spacings:5
       ~col_spacings:5
       ~border_width:2
@@ -719,27 +762,28 @@ let configure ?(apply=(fun () -> ())) () =
     in
     let iter i (text, pref) =
       let label = GMisc.label
-        ~text ~packing:(table#attach ~expand:`X ~left:0 ~top:i) ()
+        ~text ~packing:(grid#attach (*~expand:`X*) ~left:0 ~top:i) ()
       in
       let () = label#set_xalign 0. in
       let button = GButton.color_button
-        ~color:(Tags.color_of_string pref#get)
-        ~packing:(table#attach ~left:1 ~top:i) ()
+        ~color:(Gdk.Color.color_parse pref#get)
+        ~packing:(grid#attach ~left:1 ~top:i) ()
       in
       let _ = button#connect#color_set ~callback:begin fun () ->
-        pref#set (Tags.string_of_color button#color)
+        pref#set (Gdk.Color.color_to_string button#color)
       end in
       let reset _ =
         pref#reset ();
-        button#set_color Tags.(color_of_string pref#get)
+        button#set_color (Gdk.Color.color_parse pref#get)
       in
       let _ = reset_button#connect#clicked ~callback:reset in
       ()
     in
     let () = Util.List.iteri iter [
-      ("Background color", background_color);
+(*       ("Background color", background_color); *)
       ("Background color of processed text", processed_color);
       ("Background color of text being processed", processing_color);
+      ("Background color of incompletely processed Qed", incompletely_processed_color);
       ("Background color of errors", error_color);
       ("Foreground color of errors", error_fg_color);
     ] in
@@ -756,7 +800,7 @@ let configure ?(apply=(fun () -> ())) () =
       ~packing:(box#pack ~expand:true)
       ()
     in
-    let table = GPack.table
+    let grid = GPack.grid
       ~row_spacings:5
       ~col_spacings:5
       ~border_width:2
@@ -766,13 +810,13 @@ let configure ?(apply=(fun () -> ())) () =
     let cb = ref [] in
     let iter text tag =
       let label = GMisc.label
-        ~text ~packing:(table#attach ~expand:`X ~left:0 ~top:!i) ()
+        ~text ~packing:(grid#attach (*~expand:`X*) ~left:0 ~top:!i) ()
       in
       let () = label#set_xalign 0. in
       let button = tag_button () in
       let callback () = tag#set button#tag in
       button#set_tag tag#get;
-      table#attach ~left:1 ~top:!i button#coerce;
+      grid#attach ~left:1 ~top:!i button#coerce;
       incr i;
       cb := callback :: !cb;
     in
@@ -790,52 +834,55 @@ let configure ?(apply=(fun () -> ())) () =
       let but = GButton.check_button ~label:text ~active ~packing:box#pack () in
       ignore (but#connect#toggled ~callback:(fun () -> pref#set but#active))
     in
+    let spin text ~min ~max (pref : int preference) =
+      let box = GPack.hbox ~packing:box#pack () in
+      let but = GEdit.spin_button
+        ~numeric:true ~update_policy:`IF_VALID ~digits:0
+        ~packing:box#pack ()
+      in
+      let _ = GMisc.label ~text:"Delay (ms)" ~packing:box#pack () in
+      let () = but#adjustment#set_bounds
+        ~lower:(float_of_int min) ~upper:(float_of_int max)
+        ~step_incr:1.
+        ()
+      in
+      let () = but#set_value (float_of_int pref#get) in
+      ignore (but#connect#value_changed ~callback:(fun () -> pref#set but#value_as_int))
+    in
     let () = button "Dynamic word wrap" dynamic_word_wrap in
     let () = button "Show line number" show_line_number in
     let () = button "Auto indentation" auto_indent in
     let () = button "Auto completion" auto_complete in
+    let () = spin "Auto completion delay" ~min:0 ~max:5000 auto_complete_delay in
     let () = button "Show spaces" show_spaces in
     let () = button "Show right margin" show_right_margin in
     let () = button "Show progress bar" show_progress_bar in
     let () = button "Insert spaces instead of tabs" spaces_instead_of_tabs in
     let () = button "Highlight current line" highlight_current_line in
-    let () = button "Emacs/PG keybindings (μPG mode)" nanoPG in
+    let () = button "Emacs/PG keybindings (μPG mode)" microPG in
     let callback () = () in
     custom ~label box callback true
   in
 
-(*
-  let show_toolbar =
-    bool
-      ~f:(fun s ->
-	    current.show_toolbar <- s;
-	    !show_toolbar s)
-      "Show toolbar" current.show_toolbar
-  in
   let window_height =
     string
-    ~f:(fun s -> current.window_height <- (try int_of_string s with _ -> 600);
-	  !resize_window ();
-       )
-      "Window height"
-      (string_of_int current.window_height)
+    ~f:(fun s -> try window_height#set (int_of_string s) with _ -> ())
+      "Default window height at starting time"
+      (string_of_int window_height#get)
   in
+
   let window_width =
     string
-    ~f:(fun s -> current.window_width <-
-	  (try int_of_string s with _ -> 800))
-      "Window width"
-      (string_of_int current.window_width)
+    ~f:(fun s -> try window_width#set (int_of_string s) with _ -> ())
+      "Default window width at starting time"
+      (string_of_int window_width#get)
   in
-*)
-(*
-  let config_appearance = [show_toolbar; window_width; window_height] in
-*)
+
   let global_auto_revert = pbool "Enable global auto revert" global_auto_revert in
   let global_auto_revert_delay =
     string
     ~f:(fun s -> global_auto_revert_delay#set
-	  (try int_of_string s with _ -> 10000))
+          (try int_of_string s with _ -> 10000))
       "Global auto revert delay (ms)"
       (string_of_int global_auto_revert_delay#get)
   in
@@ -844,7 +891,7 @@ let configure ?(apply=(fun () -> ())) () =
   let auto_save_delay =
     string
     ~f:(fun s -> auto_save_delay#set
-	  (try int_of_string s with _ -> 10000))
+          (try int_of_string s with _ -> 10000))
       "Auto save delay (ms)"
       (string_of_int auto_save_delay#get)
   in
@@ -863,8 +910,8 @@ let configure ?(apply=(fun () -> ())) () =
       ~f:(fun s -> encoding#set (inputenc_of_string s))
       ~new_allowed: true
       ("UTF-8"::"LOCALE":: match encoding#get with
-	|Emanual s -> [s]
-	|_ -> []
+        |Emanual s -> [s]
+        |_ -> []
       )
       (string_of_inputenc encoding#get)
   in
@@ -905,19 +952,19 @@ let configure ?(apply=(fun () -> ())) () =
   in
   let project_file_name = pstring "Default name for project file" project_file_name in
   let modifier_for_tactics =
-    pmodifiers "Modifiers for Tactics Menu" modifier_for_tactics
+    pmodifiers "Global change of modifiers for Tactics Menu" modifier_for_tactics
   in
   let modifier_for_templates =
-    pmodifiers "Modifiers for Templates Menu" modifier_for_templates
+    pmodifiers "Global change of modifiers for Templates Menu" modifier_for_templates
   in
   let modifier_for_navigation =
-    pmodifiers "Modifiers for Navigation Menu" modifier_for_navigation
+    pmodifiers "Global change of modifiers for Navigation Menu" modifier_for_navigation
   in
   let modifier_for_display =
-    pmodifiers "Modifiers for View Menu" modifier_for_display
+    pmodifiers "Global change of modifiers for View Menu" modifier_for_display
   in
   let modifier_for_queries =
-    pmodifiers "Modifiers for Queries Menu" modifier_for_queries
+    pmodifiers "Global change of modifiers for Queries Menu" modifier_for_queries
   in
   let modifiers_valid =
     pmodifiers ~all:true "Allowed modifiers" modifiers_valid
@@ -950,46 +997,11 @@ let configure ?(apply=(fun () -> ())) () =
                    else cmd_browse#get])
       cmd_browse#get
   in
-  let doc_url =
-    let predefined = [
-      "file://"^(List.fold_left Filename.concat (Envars.docdir ()) ["refman";"html"]);
-      Coq_config.wwwrefman;
-      use_default_doc_url
-    ] in
-    combo
-      "Manual URL"
-      ~f:doc_url#set
-      ~new_allowed: true
-      (predefined@[if List.mem doc_url#get predefined then ""
-                   else doc_url#get])
-      doc_url#get in
-  let library_url =
-    let predefined = [
-      "file://"^(List.fold_left Filename.concat (Envars.docdir ()) ["stdlib";"html"]);
-      Coq_config.wwwstdlib
-    ] in
-    combo
-      "Library URL"
-      ~f:(fun s -> library_url#set s)
-      ~new_allowed: true
-      (predefined@[if List.mem library_url#get predefined then ""
-                   else library_url#get])
-      library_url#get
-  in
-  let automatic_tactics =
-    strings
-      ~f:automatic_tactics#set
-      ~add:(fun () -> ["<edit me>"])
-      "Wizard tactics to try in order"
-      automatic_tactics#get
 
-  in
-
-  let contextual_menus_on_goal = pbool "Contextual menus on goal" contextual_menus_on_goal in
-
-  let misc = [contextual_menus_on_goal;stop_before;reset_on_tab_switch;
+  let misc = [stop_before;reset_on_tab_switch;
               vertical_tabs;opposite_tabs] in
 
+(*
   let add_user_query () =
     let input_string l v =
       match GToolbox.input_string ~title:l v with
@@ -1003,7 +1015,7 @@ let configure ?(apply=(fun () -> ())) () =
     let k =
       if Int.equal (CString.length k) 1 && Util.is_letter k.[0] then k
       else "" in
-    let k = uppercase k in
+    let k = String.uppercase_ascii k in
       [q, k]
   in
 
@@ -1019,45 +1031,41 @@ let configure ?(apply=(fun () -> ())) () =
       user_queries#get
 
   in
+*)
 
 (* ATTENTION !!!!! L'onglet Fonts doit etre en premier pour eviter un bug !!!!
    (shame on Benjamin) *)
   let cmds =
     [Section("Fonts", Some `SELECT_FONT,
-	     [config_font]);
+             [config_font]);
      Section("Colors", Some `SELECT_COLOR,
              [config_color; source_language; source_style]);
      Section("Tags", Some `SELECT_COLOR,
              [config_tags]);
      Section("Editor", Some `EDIT, [config_editor]);
      Section("Files", Some `DIRECTORY,
-	     [global_auto_revert;global_auto_revert_delay;
-	      auto_save; auto_save_delay; (* auto_save_name*)
-	      encodings; line_ending;
-	     ]);
+             [global_auto_revert;global_auto_revert_delay;
+              auto_save; auto_save_delay; (* auto_save_name*)
+              encodings; line_ending;
+             ]);
      Section("Project", Some (`STOCK "gtk-page-setup"),
-	     [project_file_name;read_project;
-	     ]);
-(*
-     Section("Appearance",
-	     config_appearance);
-*)
+             [project_file_name;read_project;
+             ]);
+     Section("Appearance", Some `PREFERENCES, [window_width; window_height]);
      Section("Externals", None,
-	     [cmd_coqtop;cmd_coqc;cmd_make;cmd_coqmakefile; cmd_coqdoc;
-	      cmd_print;cmd_editor;cmd_browse;doc_url;library_url]);
-     Section("Tactics Wizard", None,
-	     [automatic_tactics]);
+             [cmd_coqtop;cmd_coqc;cmd_make;cmd_coqmakefile; cmd_coqdoc;
+              cmd_print;cmd_editor;cmd_browse]);
      Section("Shortcuts", Some `PREFERENCES,
-	     [modifiers_valid; modifier_for_tactics;
+             [modifiers_valid; modifier_for_tactics;
         modifier_for_templates; modifier_for_display; modifier_for_navigation;
-        modifier_for_queries; user_queries]);
+        modifier_for_queries (*; user_queries *)]);
      Section("Misc", Some `ADD,
        misc)]
   in
 (*
   Format.printf "before edit: current.text_font = %s@." (Pango.Font.to_string current.text_font);
 *)
-  let x = edit ~apply "Customizations" cmds in
+  let x = edit ~apply "Customizations" ~parent cmds in
 (*
   Format.printf "after edit: current.text_font = %s@." (Pango.Font.to_string current.text_font);
 *)
