@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -8,8 +8,8 @@
 (*         *     (see LICENSE file for the text of the license)         *)
 (************************************************************************)
 
-val lang_manager : GSourceView2.source_language_manager
-val style_manager : GSourceView2.source_style_scheme_manager
+val lang_manager : GSourceView3.source_language_manager
+val style_manager : GSourceView3.source_style_scheme_manager
 
 type project_behavior = Ignore_args | Append_args | Subst_args
 type inputenc = Elocale | Eutf8 | Emanual of string
@@ -47,6 +47,10 @@ end
 
 val list_tags : unit -> tag preference Util.String.Map.t
 
+val get_unicode_bindings_local_file : unit -> string option
+val get_unicode_bindings_default_file : unit -> string option
+
+
 val cmd_coqtop : string option preference
 val cmd_coqc : string preference
 val cmd_make : string preference
@@ -74,19 +78,17 @@ val modifiers_valid : string preference
 val cmd_browse : string preference
 val cmd_editor : string preference
 val text_font : string preference
-val doc_url : string preference
-val library_url : string preference
 val show_toolbar : bool preference
-val contextual_menus_on_goal : bool preference
 val window_width : int preference
 val window_height : int preference
 val auto_complete : bool preference
+val auto_complete_delay : int preference
 val stop_before : bool preference
 val reset_on_tab_switch : bool preference
 val line_ending : line_ending preference
 val vertical_tabs : bool preference
 val opposite_tabs : bool preference
-val background_color : string preference
+(* val background_color : string preference *)
 val processing_color : string preference
 val processed_color : string preference
 val error_color : string preference
@@ -100,16 +102,16 @@ val show_progress_bar : bool preference
 val spaces_instead_of_tabs : bool preference
 val tab_length : int preference
 val highlight_current_line : bool preference
-val nanoPG : bool preference
+val microPG : bool preference
 val user_queries : (string * string) list preference
 val diffs : string preference
 
 val save_pref : unit -> unit
-val load_pref : unit -> unit
+val load_pref : warn:(delay:int -> string -> unit) -> unit
 
-val configure : ?apply:(unit -> unit) -> unit -> unit
+val configure : ?apply:(unit -> unit) -> GWindow.window -> unit
 
 val stick : 'a preference ->
-  (#GObj.widget as 'obj) -> ('a -> unit) -> unit
+  < connect : #GObj.widget_signals ; .. > -> ('a -> unit) -> unit
 
 val use_default_doc_url : string
