@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -47,7 +47,7 @@ type session = {
 }
 
 let create_buffer () =
-  let buffer = GSourceView2.source_buffer
+  let buffer = GSourceView3.source_buffer
     ~tag_table:Tags.Script.table
     ~highlight_matching_brackets:true
     ?language:(lang_manager#language source_language#get)
@@ -149,7 +149,7 @@ let set_buffer_handlers
     let rec aux old it =
       if it#is_start then None
       else if it#has_tag Tags.Script.processed then Some old
-      else if it#has_tag Tags.Script.error_bg then aux it it#backward_char 
+      else if it#has_tag Tags.Script.error_bg then aux it it#backward_char
       else None in
     aux it it in
   let insert_cb it s = if String.length s = 0 then () else begin
@@ -207,8 +207,8 @@ let set_buffer_handlers
          to a point indicated by coq. *)
       if !no_coq_action_required then begin
         let start, stop = get_start (), get_stop () in
-	List.iter (fun tag -> buffer#remove_tag tag ~start ~stop)
-		  Tags.Script.ephemere;
+        List.iter (fun tag -> buffer#remove_tag tag ~start ~stop)
+                  Tags.Script.ephemere;
         Sentence.tag_on_insert buffer
       end;
     end in
@@ -227,7 +227,7 @@ let set_buffer_handlers
     | Some s -> Minilib.log (s^" moved")
     | None -> ()
   in
-  (** Pluging callbacks *)
+  (* Pluging callbacks *)
   let _ = buffer#connect#insert_text ~callback:insert_cb in
   let _ = buffer#connect#delete_range ~callback:delete_cb in
   let _ = buffer#connect#begin_user_action ~callback:begin_action_cb in
@@ -257,9 +257,10 @@ let make_table_widget ?sort cd cb =
       ~model:store ~packing:frame#add () in
   let () = data#set_headers_visible true in
   let () = data#set_headers_clickable true in
-  let refresh clr = data#misc#modify_base [`NORMAL, `NAME clr] in
-  let _ = background_color#connect#changed ~callback:refresh in
-  let _ = data#misc#connect#realize ~callback:(fun () -> refresh background_color#get) in
+(* FIXME: handle this using CSS *)
+(*   let refresh clr = data#misc#modify_bg [`NORMAL, `NAME clr] in *)
+(*   let _ = background_color#connect#changed ~callback:refresh in *)
+(*   let _ = data#misc#connect#realize ~callback:(fun () -> refresh background_color#get) in *)
   let mk_rend c = GTree.cell_renderer_text [], ["text",c] in
   let cols =
     List.map2 (fun (_,c) (_,n,v) ->
@@ -274,9 +275,9 @@ let make_table_widget ?sort cd cb =
   let make_sorting i (_, c) =
     let sort (store : GTree.model) it1 it2 = match c with
     | `IntC c ->
-      Pervasives.compare (store#get ~row:it1 ~column:c) (store#get ~row:it2 ~column:c)
+      compare (store#get ~row:it1 ~column:c) (store#get ~row:it2 ~column:c)
     | `StringC c ->
-      Pervasives.compare (store#get ~row:it1 ~column:c) (store#get ~row:it2 ~column:c)
+      compare (store#get ~row:it1 ~column:c) (store#get ~row:it2 ~column:c)
     in
     store#set_sort_func i sort
   in
@@ -300,7 +301,7 @@ let create_errpage (script : Wg_ScriptView.script_view) : errpage =
         script#buffer#place_cursor ~where;
         script#misc#grab_focus ();
         ignore (script#scroll_to_iter
-		  ~use_align:false ~yalign:0.75 ~within_margin:0.25 where)) in
+                  ~use_align:false ~yalign:0.75 ~within_margin:0.25 where)) in
   let tip = GMisc.label ~text:"Double click to jump to error line" () in
   let box = GPack.vbox ~homogeneous:false () in
   let () = box#pack ~expand:true table#coerce in
@@ -312,10 +313,10 @@ let create_errpage (script : Wg_ScriptView.script_view) : errpage =
     method update errs =
       if !last_update = errs then ()
       else begin
-	last_update := errs;
-	access (fun _ store -> store#clear ());
+        last_update := errs;
+        access (fun _ store -> store#clear ());
         !callback errs;
-	List.iter (fun (lno, msg) -> access (fun columns store ->
+        List.iter (fun (lno, msg) -> access (fun columns store ->
           let line = store#append () in
           store#set ~row:line ~column:(find_int_col "Line" columns) lno;
           store#set ~row:line ~column:(find_string_col "Error message" columns) msg))
@@ -332,8 +333,8 @@ let create_jobpage coqtop coqops : jobpage =
       (fun columns store tp vc ->
         let row = store#get_iter tp in
         let w = store#get ~row ~column:(find_string_col "Worker" columns) in
-	let info () = Minilib.log ("Coq busy, discarding query") in
-	Coq.try_grab coqtop (coqops#stop_worker w) info
+        let info () = Minilib.log ("Coq busy, discarding query") in
+        Coq.try_grab coqtop (coqops#stop_worker w) info
       ) in
   let tip = GMisc.label ~text:"Double click to interrupt worker" () in
   let box = GPack.vbox ~homogeneous:false () in
@@ -346,10 +347,10 @@ let create_jobpage coqtop coqops : jobpage =
     method update jobs =
       if !last_update = jobs then ()
       else begin
-	last_update := jobs;
-	access (fun _ store -> store#clear ());
+        last_update := jobs;
+        access (fun _ store -> store#clear ());
         !callback jobs;
-	CString.Map.iter (fun id job -> access (fun columns store ->
+        CString.Map.iter (fun id job -> access (fun columns store ->
           let column = find_string_col "Worker" columns in
           if job = "Dead" then
             store#foreach (fun _ row ->
@@ -431,42 +432,46 @@ let kill (sn:session) =
   sn.script#destroy ();
   Coq.close_coqtop sn.coqtop
 
+let window_size = ref (window_width#get, window_height#get)
+
 let build_layout (sn:session) =
   let session_paned = GPack.paned `VERTICAL () in
   let session_box =
     GPack.vbox ~packing:(session_paned#pack1 ~shrink:false ~resize:true) ()
   in
 
-  (** Right part of the window. *)
+  (* Right part of the window. *)
 
   let eval_paned = GPack.paned `HORIZONTAL ~border_width:5
     ~packing:(session_box#pack ~expand:true) () in
   let script_frame = GBin.frame ~shadow_type:`IN
-    ~packing:eval_paned#add1 () in
+    ~packing:(eval_paned#pack1 ~shrink:false) () in
   let script_scroll = GBin.scrolled_window
     ~vpolicy:`AUTOMATIC ~hpolicy:`AUTOMATIC ~packing:script_frame#add () in
   let state_paned = GPack.paned `VERTICAL
-    ~packing:eval_paned#add2 () in
+    ~packing:(eval_paned#pack2 ~shrink:true) () in
 
-  (** Proof buffer. *)
+  (* Proof buffer. *)
 
   let title = Printf.sprintf "Proof (%s)" sn.tab_label#text in
   let proof_detachable = Wg_Detachable.detachable ~title () in
   let () = proof_detachable#button#misc#hide () in
   let () = proof_detachable#frame#set_shadow_type `IN in
-  let () = state_paned#add1 proof_detachable#coerce in
-  let callback _ = proof_detachable#show in
-  let () = proof_detachable#connect#attached ~callback in
-  let callback _ =
-    sn.proof#coerce#misc#set_size_request ~width:500 ~height:400 ()
-  in
-  let () = proof_detachable#connect#detached ~callback in
+  let () = state_paned#pack1 ~shrink:true proof_detachable#coerce in
   let proof_scroll = GBin.scrolled_window
     ~vpolicy:`AUTOMATIC ~hpolicy:`AUTOMATIC ~packing:proof_detachable#pack () in
+  let callback _ = proof_detachable#show;
+                   proof_scroll#coerce#misc#set_size_request ~width:0 ~height:0 ()
+  in
+  let () = proof_detachable#connect#attached ~callback in
+  let callback _ =
+    proof_scroll#coerce#misc#set_size_request ~width:500 ~height:400 ()
+  in
+  let () = proof_detachable#connect#detached ~callback in
 
-  (** Message buffer. *)
+  (* Message buffer. *)
 
-  let message_frame = GPack.notebook ~packing:state_paned#add () in
+  let message_frame = GPack.notebook ~packing:(state_paned#pack2 ~shrink:true) () in
   let add_msg_page pos name text (w : GObj.widget) =
     let detachable =
       Wg_Detachable.detachable ~title:(text^" ("^name^")") () in
@@ -499,23 +504,25 @@ let build_layout (sn:session) =
       ~callback:(fun () -> if sn.buffer#modified
         then img#set_stock `SAVE
         else img#set_stock `YES) in
-  let _ =
-    eval_paned#misc#connect#size_allocate
-      ~callback:
-      (let old_paned_width = ref 2 in
-       let old_paned_height = ref 2 in
-       fun {Gtk.width=paned_width;Gtk.height=paned_height} ->
-         if !old_paned_width <> paned_width ||
-            !old_paned_height <> paned_height
-         then begin
-           eval_paned#set_position
-             (eval_paned#position * paned_width / !old_paned_width);
-           state_paned#set_position
-             (state_paned#position * paned_height / !old_paned_height);
-           old_paned_width := paned_width;
-           old_paned_height := paned_height;
-	 end)
-  in
+  (* There was an issue in the previous implementation for setting the
+     position of the handle. It was using the size_allocate event but
+     there is an issue with size_allocate. G. Melquiond analyzed that
+     at starting time, the size_allocate event is only issued in
+     Layout phase of the gtk loop so that it is actually processed
+     only in the next iteration of the event-update-layout-paint loop,
+     after the user does something and trigger an effective new event
+     (see #10578). So we preventively enforce an estimated position
+     for the handles to be used in the very first initializing
+     iteration of the loop *)
+  let () =
+    (* 14 is the estimated size for vertical borders *)
+    let estimated_vertical_handle_position = (fst !window_size - 14) / 2 in
+    (* 169 is the estimated size for menus, command line, horizontal border *)
+    let estimated_horizontal_handle_position = (snd !window_size - 169) / 2 in
+    if estimated_vertical_handle_position > 0 then
+      eval_paned#set_position estimated_vertical_handle_position;
+    if estimated_horizontal_handle_position > 0 then
+      state_paned#set_position estimated_horizontal_handle_position in
   session_box#pack sn.finder#coerce;
   session_box#pack sn.segment#coerce;
   sn.command#pack_in (session_paned#pack2 ~shrink:false ~resize:false);
@@ -524,14 +531,14 @@ let build_layout (sn:session) =
   let detach, _ = add_msg_page 0 sn.tab_label#text "Messages" sn.messages#default_route#coerce in
   let _, label = add_msg_page 1 sn.tab_label#text "Errors" sn.errpage#coerce in
   let _, _ = add_msg_page 2 sn.tab_label#text "Jobs" sn.jobpage#coerce in
-  (** When a message is received, focus on the message pane *)
+  (* When a message is received, focus on the message pane *)
   let _ =
     sn.messages#default_route#connect#pushed ~callback:(fun _ _ ->
       let num = message_frame#page_num detach#coerce in
       if 0 <= num then message_frame#goto_page num
     )
   in
-  (** When an error occurs, paint the error label in red *)
+  (* When an error occurs, paint the error label in red *)
   let txt = label#text in
   let red s = "<span foreground=\"#FF0000\">" ^ s ^ "</span>" in
   sn.errpage#on_update ~callback:(fun l ->
@@ -539,8 +546,6 @@ let build_layout (sn:session) =
     else (label#set_text (red txt);label#set_use_markup true));
   session_tab#pack sn.tab_label#coerce;
   img#set_stock `YES;
-  eval_paned#set_position 1;
-  state_paned#set_position 1;
   let control =
     object
       method detach () = proof_detachable#detach ()
