@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -28,11 +28,10 @@ val create :
   ('a -> GObj.widget option * GObj.widget option * GObj.widget) ->
   ('a -> unit) ->
   ?enable_popup:bool ->
-  ?homogeneous_tabs:bool ->
+  ?group_name:string ->
   ?scrollable:bool ->
   ?show_border:bool ->
   ?show_tabs:bool ->
-  ?tab_border:int ->
   ?tab_pos:Gtk.Tags.position ->
   ?border_width:int ->
   ?width:int ->
