@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -14,7 +14,7 @@ type source_view = [ Gtk.text_view | `sourceview ] Gtk.obj
 
 class script_view : source_view -> Coq.coqtop ->
 object
-  inherit GSourceView2.source_view
+  inherit GSourceView3.source_view
   method undo : unit -> unit
   method redo : unit -> unit
   method clear_undo : unit -> unit
@@ -26,13 +26,14 @@ object
   method set_show_right_margin : bool -> unit
   method comment : unit -> unit
   method uncomment : unit -> unit
+  method apply_unicode_binding : unit -> unit
   method recenter_insert : unit
-  method complete_popup : Wg_Completion.complete_popup
+  method proposal : string option
 end
 
 val script_view : Coq.coqtop ->
-  ?source_buffer:GSourceView2.source_buffer ->
-  ?draw_spaces:SourceView2Enums.source_draw_spaces_flags list ->
+  ?source_buffer:GSourceView3.source_buffer ->
+  ?draw_spaces:SourceView3Enums.source_draw_spaces_flags list ->
   ?auto_indent:bool ->
   ?highlight_current_line:bool ->
   ?indent_on_tab:bool ->
@@ -42,7 +43,7 @@ val script_view : Coq.coqtop ->
   ?show_line_marks:bool ->
   ?show_line_numbers:bool ->
   ?show_right_margin:bool ->
-  ?smart_home_end:SourceView2Enums.source_smart_home_end_type ->
+  ?smart_home_end:SourceView3Enums.source_smart_home_end_type ->
   ?tab_width:int ->
   ?editable:bool ->
   ?cursor_visible:bool ->
