@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -28,5 +28,7 @@ exception ModuleInternalizationError of module_internalization_error
    kind is never ModAny, and it is equal to the input kind when this one
    isn't ModAny. *)
 
+type module_kind = Module | ModType | ModAny
+
 val interp_module_ast :
-  env -> Declaremods.module_kind -> module_ast -> module_struct_entry * Declaremods.module_kind * Univ.ContextSet.t
+  env -> module_kind -> module_ast -> module_struct_entry * module_kind * Univ.ContextSet.t
