@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -16,11 +16,12 @@ open Environ
 
 (** Should only be used for monomorphic terms *)
 val compile : fail_on_error:bool ->
-	      ?universes:int -> env -> constr -> (bytecodes * bytecodes * fv) option
+              ?universes:int -> env -> constr -> (bytecodes * bytecodes * fv) option
 (** init, fun, fv *)
 
 val compile_constant_body : fail_on_error:bool ->
-			    env -> constant_universes -> constant_def -> body_code option
+  env -> universes -> (Constr.t Mod_subst.substituted, 'opaque) constant_def ->
+  body_code option
 
 (** Shortcut of the previous function used during module strengthening *)
 
