@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -13,20 +13,25 @@ open Declarations
 
 (** {6 Cooking the constants. } *)
 
-type recipe = { from : constant_body; info : Opaqueproof.cooking_info }
+type recipe = { from : Opaqueproof.opaque constant_body; info : Opaqueproof.cooking_info }
 
 type inline = bool
 
-type result = {
-  cook_body : constant_def;
+type 'opaque result = {
+  cook_body : (constr Mod_subst.substituted, 'opaque) constant_def;
   cook_type : types;
-  cook_universes : constant_universes;
+  cook_universes : universes;
+  cook_relevance : Sorts.relevance;
   cook_inline : inline;
-  cook_context : Constr.named_context option;
+  cook_context : Names.Id.Set.t option;
 }
 
-val cook_constant : hcons:bool -> recipe -> result
-val cook_constr : Opaqueproof.cooking_info -> constr -> constr
+val cook_constant : recipe -> Opaqueproof.opaque result
+val cook_constr : Opaqueproof.cooking_info list ->
+  (constr * unit Opaqueproof.delayed_universes) -> (constr * unit Opaqueproof.delayed_universes)
+
+val cook_inductive :
+  Opaqueproof.cooking_info -> mutual_inductive_body -> Entries.mutual_inductive_entry
 
 (** {6 Utility functions used in module [Discharge]. } *)
 
