@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -24,7 +24,7 @@ open Constr
    is the term into which we should inline. *)
 
 type delta_hint =
-  | Inline of int * (Univ.AUContext.t * constr) option
+  | Inline of int * constr Univ.univ_abstracted option
   | Equiv of KerName.t
 
 (* NB: earlier constructor Prefix_equiv of ModPath.t
@@ -53,18 +53,25 @@ type delta_resolver = Deltamap.t
 
 let empty_delta_resolver = Deltamap.empty
 
-module Umap = struct
-  type 'a t = 'a MPmap.t * 'a MBImap.t
-  let empty = MPmap.empty, MBImap.empty
-  let is_empty (m1,m2) = MPmap.is_empty m1 && MBImap.is_empty m2
-  let add_mbi mbi x (m1,m2) = (m1,MBImap.add mbi x m2)
-  let add_mp mp x (m1,m2) = (MPmap.add mp x m1, m2)
-  let find_mp mp map = MPmap.find mp (fst map)
-  let find_mbi mbi map = MBImap.find mbi (snd map)
-  let iter_mbi f map = MBImap.iter f (snd map)
-  let fold fmp fmbi (m1,m2) i =
-    MPmap.fold fmp m1 (MBImap.fold fmbi m2 i)
-  let join map1 map2 = fold add_mp add_mbi map1 map2
+module Umap :
+  sig
+    type 'a t
+    val empty : 'a t
+    val is_empty : 'a t -> bool
+    val add_mbi : MBId.t -> 'a -> 'a t -> 'a t
+    val add_mp : ModPath.t -> 'a -> 'a t -> 'a t
+    val find : ModPath.t -> 'a t -> 'a
+    val join : 'a t -> 'a t -> 'a t
+    val fold : (ModPath.t -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
+  end = struct
+  type 'a t = 'a MPmap.t
+  let empty = MPmap.empty
+  let is_empty m = MPmap.is_empty m
+  let add_mbi mbi x m = MPmap.add (MPbound mbi) x m
+  let add_mp mp x m = MPmap.add mp x m
+  let find = MPmap.find
+  let fold = MPmap.fold
+  let join map1 map2 = fold add_mp map1 map2
 end
 
 type substitution = (ModPath.t * delta_resolver) Umap.t
@@ -93,8 +100,7 @@ let debug_string_of_delta resolve =
 let list_contents sub =
   let one_pair (mp,reso) = (ModPath.to_string mp,debug_string_of_delta reso) in
   let mp_one_pair mp0 p l = (ModPath.to_string mp0, one_pair p)::l in
-  let mbi_one_pair mbi p l = (MBId.debug_to_string mbi, one_pair p)::l in
-  Umap.fold mp_one_pair mbi_one_pair sub []
+  Umap.fold mp_one_pair sub []
 
 let debug_string_of_subst sub =
   let l = List.map (fun (s1,(s2,s3)) -> s1^"|->"^s2^"["^s3^"]")
@@ -108,7 +114,7 @@ let debug_pr_delta resolve =
 let debug_pr_subst sub =
   let l = list_contents sub in
   let f (s1,(s2,s3)) = hov 2 (str s1 ++ spc () ++ str "|-> " ++ str s2 ++
-			      spc () ++ str "[" ++ str s3 ++ str "]")
+                              spc () ++ str "[" ++ str s3 ++ str "]")
   in
   str "{" ++ hov 2 (prlist_with_sep pr_comma f l) ++ str "}"
 
@@ -150,15 +156,15 @@ let mp_of_delta resolve mp =
 let find_prefix resolve mp =
   let rec sub_mp = function
     | MPdot(mp,l) as mp_sup ->
-	(try Deltamap.find_mp mp_sup resolve
-	 with Not_found -> MPdot(sub_mp mp,l))
+        (try Deltamap.find_mp mp_sup resolve
+         with Not_found -> MPdot(sub_mp mp,l))
     | p -> Deltamap.find_mp p resolve
   in
   try sub_mp mp with Not_found -> mp
 
 (** Applying a resolver to a kernel name *)
 
-exception Change_equiv_to_inline of (int * (Univ.AUContext.t * constr))
+exception Change_equiv_to_inline of (int * constr Univ.univ_abstracted)
 
 let solve_delta_kn resolve kn =
   try
@@ -167,12 +173,12 @@ let solve_delta_kn resolve kn =
       | Inline (lev, Some c) ->	raise (Change_equiv_to_inline (lev,c))
       | Inline (_, None) -> raise Not_found
   with Not_found ->
-    let mp,dir,l = KerName.repr kn in
+    let mp,l = KerName.repr kn in
     let new_mp = find_prefix resolve mp in
     if mp == new_mp then
       kn
     else
-      KerName.make new_mp dir l
+      KerName.make new_mp l
 
 let kn_of_delta resolve kn =
   try solve_delta_kn resolve kn
@@ -201,9 +207,9 @@ let inline_of_delta inline resolver =
     | None -> []
     | Some inl_lev ->
       let extract kn hint l =
-	match hint with
-	  | Inline (lev,_) -> if lev <= inl_lev then (lev,kn)::l else l
-	  | _ -> l
+        match hint with
+          | Inline (lev,_) -> if lev <= inl_lev then (lev,kn)::l else l
+          | _ -> l
       in
       Deltamap.fold_kn extract resolver []
 
@@ -222,19 +228,14 @@ let search_delta_inline resolve kn1 kn2 =
 let subst_mp0 sub mp = (* 's like subst *)
  let rec aux mp =
   match mp with
-    | MPfile sid -> Umap.find_mp mp sub
-    | MPbound bid ->
-	begin
-	  try Umap.find_mbi bid sub
-	  with Not_found -> Umap.find_mp mp sub
-	end
+    | MPfile _ | MPbound _ -> Umap.find mp sub
     | MPdot (mp1,l) as mp2 ->
-	begin
-	  try Umap.find_mp mp2 sub
-	  with Not_found ->
-	    let mp1',resolve = aux mp1 in
-	    MPdot (mp1',l),resolve
-	end
+        begin
+          try Umap.find mp2 sub
+          with Not_found ->
+            let mp1',resolve = aux mp1 in
+            MPdot (mp1',l),resolve
+        end
  in
  try Some (aux mp) with Not_found -> None
 
@@ -244,18 +245,18 @@ let subst_mp sub mp =
   | Some (mp',_) -> mp'
 
 let subst_kn_delta sub kn =
- let mp,dir,l = KerName.repr kn in
+ let mp,l = KerName.repr kn in
   match subst_mp0 sub mp with
      Some (mp',resolve) ->
-      solve_delta_kn resolve (KerName.make mp' dir l)
+      solve_delta_kn resolve (KerName.make mp' l)
    | None -> kn
 
 
 let subst_kn sub kn =
- let mp,dir,l = KerName.repr kn in
+ let mp,l = KerName.repr kn in
   match subst_mp0 sub mp with
      Some (mp',_) ->
-      (KerName.make mp' dir l)
+      (KerName.make mp' l)
    | None -> kn
 
 exception No_subst
@@ -274,12 +275,12 @@ let progress f x ~orelse =
   if y != x then y else orelse
 
 let subst_mind sub mind =
-  let mpu,dir,l = MutInd.repr3 mind in
+  let mpu,l = MutInd.repr2 mind in
   let mpc = KerName.modpath (MutInd.canonical mind) in
   try
     let mpu,mpc,resolve,user = subst_dual_mp sub mpu mpc in
-    let knu = KerName.make mpu dir l in
-    let knc = if mpu == mpc then knu else KerName.make mpc dir l in
+    let knu = KerName.make mpu l in
+    let knc = if mpu == mpc then knu else KerName.make mpc l in
     let knc' =
       progress (kn_of_delta resolve) (if user then knu else knc) ~orelse:knc
     in
@@ -293,43 +294,34 @@ let subst_ind sub (ind,i as indi) =
 let subst_pind sub (ind,u) =
   (subst_ind sub ind, u)
 
-let subst_con0 sub (cst,u) =
-  let mpu,dir,l = Constant.repr3 cst in
+let subst_con0 sub cst =
+  let mpu,l = Constant.repr2 cst in
   let mpc = KerName.modpath (Constant.canonical cst) in
   let mpu,mpc,resolve,user = subst_dual_mp sub mpu mpc in
-  let knu = KerName.make mpu dir l in
-  let knc = if mpu == mpc then knu else KerName.make mpc dir l in
+  let knu = KerName.make mpu l in
+  let knc = if mpu == mpc then knu else KerName.make mpc l in
   match search_delta_inline resolve knu knc with
-    | Some (ctx, t) ->
+    | Some t ->
       (* In case of inlining, discard the canonical part (cf #2608) *)
-      let () = assert (Int.equal (Univ.AUContext.size ctx) (Univ.Instance.length u)) in
-      Constant.make1 knu, Vars.subst_instance_constr u t
+      Constant.make1 knu, Some t
     | None ->
       let knc' =
         progress (kn_of_delta resolve) (if user then knu else knc) ~orelse:knc
       in
       let cst' = Constant.make knu knc' in
-      cst', mkConstU (cst',u)
+      cst', None
 
 let subst_con sub cst =
   try subst_con0 sub cst
-  with No_subst -> fst cst, mkConstU cst
+  with No_subst -> cst, None
 
-let subst_con_kn sub con =
-  subst_con sub (con,Univ.Instance.empty)
-
-let subst_pcon sub (con,u as pcon) = 
-  try let con', can = subst_con0 sub pcon in 
-	con',u
+let subst_pcon sub (con,u as pcon) =
+  try let con', _can = subst_con0 sub con in
+        con',u
   with No_subst -> pcon
 
-let subst_pcon_term sub (con,u as pcon) = 
-  try let con', can = subst_con0 sub pcon in 
-	(con',u), can
-  with No_subst -> pcon, mkConstU pcon
-
 let subst_constant sub con =
-  try fst (subst_con0 sub (con,Univ.Instance.empty))
+  try fst (subst_con0 sub con)
   with No_subst -> con
 
 let subst_proj_repr sub p =
@@ -338,6 +330,16 @@ let subst_proj_repr sub p =
 let subst_proj sub p =
   Projection.map (subst_mind sub) p
 
+let subst_retro_action subst action =
+  let open Retroknowledge in
+  match action with
+  | Register_ind(prim,ind) ->
+    let ind' = subst_ind subst ind in
+    if ind == ind' then action else Register_ind(prim, ind')
+  | Register_type(prim,c) ->
+    let c' = subst_constant subst c in
+    if c == c' then action else Register_type(prim, c')
+
 (* Here the semantics is completely unclear.
    What does "Hint Unfold t" means when "t" is a parameter?
    Does the user mean "Unfold X.t" or does she mean "Unfold y"
@@ -350,97 +352,103 @@ let subst_evaluable_reference subst = function
 let rec map_kn f f' c =
   let func = map_kn f f' in
     match kind c with
-      | Const kn -> (try snd (f' kn) with No_subst -> c)
-      | Proj (p,t) -> 
+      | Const kn -> (try f' kn with No_subst -> c)
+      | Proj (p,t) ->
           let p' = Projection.map f p in
-	  let t' = func t in
-	    if p' == p && t' == t then c
-	    else mkProj (p', t')
+          let t' = func t in
+            if p' == p && t' == t then c
+            else mkProj (p', t')
       | Ind ((kn,i),u) ->
-	  let kn' = f kn in
-	  if kn'==kn then c else mkIndU ((kn',i),u)
+          let kn' = f kn in
+          if kn'==kn then c else mkIndU ((kn',i),u)
       | Construct (((kn,i),j),u) ->
-	  let kn' = f kn in
-	  if kn'==kn then c else mkConstructU (((kn',i),j),u)
+          let kn' = f kn in
+          if kn'==kn then c else mkConstructU (((kn',i),j),u)
       | Case (ci,p,ct,l) ->
-	  let ci_ind =
+          let ci_ind =
             let (kn,i) = ci.ci_ind in
-	    let kn' = f kn in
-	    if kn'==kn then ci.ci_ind else kn',i
-	  in
-	  let p' = func p in
-	  let ct' = func ct in
+            let kn' = f kn in
+            if kn'==kn then ci.ci_ind else kn',i
+          in
+          let p' = func p in
+          let ct' = func ct in
           let l' = Array.Smart.map func l in
-	    if (ci.ci_ind==ci_ind && p'==p
-		&& l'==l && ct'==ct)then c
-	    else
-	      mkCase ({ci with ci_ind = ci_ind},
-		      p',ct', l')
+            if (ci.ci_ind==ci_ind && p'==p
+                && l'==l && ct'==ct)then c
+            else
+              mkCase ({ci with ci_ind = ci_ind},
+                      p',ct', l')
       | Cast (ct,k,t) ->
-	  let ct' = func ct in
-	  let t'= func t in
-	    if (t'==t && ct'==ct) then c
-	    else mkCast (ct', k, t')
+          let ct' = func ct in
+          let t'= func t in
+            if (t'==t && ct'==ct) then c
+            else mkCast (ct', k, t')
       | Prod (na,t,ct) ->
-	  let ct' = func ct in
-	  let t'= func t in
-	    if (t'==t && ct'==ct) then c
-	    else mkProd (na, t', ct')
+          let ct' = func ct in
+          let t'= func t in
+            if (t'==t && ct'==ct) then c
+            else mkProd (na, t', ct')
       | Lambda (na,t,ct) ->
-	  let ct' = func ct in
-	  let t'= func t in
-	    if (t'==t && ct'==ct) then c
-	    else mkLambda (na, t', ct')
+          let ct' = func ct in
+          let t'= func t in
+            if (t'==t && ct'==ct) then c
+            else mkLambda (na, t', ct')
       | LetIn (na,b,t,ct) ->
-	  let ct' = func ct in
-	  let t'= func t in
-	  let b'= func b in
-	    if (t'==t && ct'==ct && b==b') then c
-	    else mkLetIn (na, b', t', ct')
+          let ct' = func ct in
+          let t'= func t in
+          let b'= func b in
+            if (t'==t && ct'==ct && b==b') then c
+            else mkLetIn (na, b', t', ct')
       | App (ct,l) ->
-	  let ct' = func ct in
+          let ct' = func ct in
           let l' = Array.Smart.map func l in
-	    if (ct'== ct && l'==l) then c
-	    else mkApp (ct',l')
+            if (ct'== ct && l'==l) then c
+            else mkApp (ct',l')
       | Evar (e,l) ->
           let l' = Array.Smart.map func l in
-	    if (l'==l) then c
-	    else mkEvar (e,l')
+            if (l'==l) then c
+            else mkEvar (e,l')
       | Fix (ln,(lna,tl,bl)) ->
           let tl' = Array.Smart.map func tl in
           let bl' = Array.Smart.map func bl in
-	    if (bl == bl'&& tl == tl') then c
-	    else mkFix (ln,(lna,tl',bl'))
+            if (bl == bl'&& tl == tl') then c
+            else mkFix (ln,(lna,tl',bl'))
       | CoFix(ln,(lna,tl,bl)) ->
           let tl' = Array.Smart.map func tl in
           let bl' = Array.Smart.map func bl in
-	    if (bl == bl'&& tl == tl') then c
-	    else mkCoFix (ln,(lna,tl',bl'))
+            if (bl == bl'&& tl == tl') then c
+            else mkCoFix (ln,(lna,tl',bl'))
       | _ -> c
 
 let subst_mps sub c =
+  let subst_pcon_term sub (con,u) =
+    let con', can = subst_con0 sub con in
+    match can with
+    | None -> mkConstU (con',u)
+    | Some t -> Vars.univ_instantiate_constr u t
+  in
   if is_empty_subst sub then c
-  else map_kn (subst_mind sub) (subst_con0 sub) c
+  else map_kn (subst_mind sub) (subst_pcon_term sub) c
 
 let rec replace_mp_in_mp mpfrom mpto mp =
   match mp with
     | _ when ModPath.equal mp mpfrom -> mpto
     | MPdot (mp1,l) ->
-	let mp1' = replace_mp_in_mp mpfrom mpto mp1 in
-	  if mp1 == mp1' then mp
-	  else MPdot (mp1',l)
+        let mp1' = replace_mp_in_mp mpfrom mpto mp1 in
+          if mp1 == mp1' then mp
+          else MPdot (mp1',l)
     | _ -> mp
 
 let replace_mp_in_kn mpfrom mpto kn =
- let mp,dir,l = KerName.repr kn in
+ let mp,l = KerName.repr kn in
   let mp'' = replace_mp_in_mp mpfrom mpto mp in
     if mp==mp'' then kn
-    else KerName.make mp'' dir l
+    else KerName.make mp'' l
 
 let rec mp_in_mp mp mp1 =
   match mp1 with
     | _ when ModPath.equal mp1 mp -> true
-    | MPdot (mp2,l) -> mp_in_mp mp mp2
+    | MPdot (mp2,_l) -> mp_in_mp mp mp2
     | _ -> false
 
 let subset_prefixed_by mp resolver =
@@ -451,7 +459,7 @@ let subset_prefixed_by mp resolver =
     match hint with
       | Inline _ -> rslv
       | Equiv _ ->
-	if mp_in_mp mp (KerName.modpath kn) then Deltamap.add_kn kn hint rslv else rslv
+        if mp_in_mp mp (KerName.modpath kn) then Deltamap.add_kn kn hint rslv else rslv
   in
   Deltamap.fold mp_prefix kn_prefix resolver empty_delta_resolver
 
@@ -471,7 +479,7 @@ let subst_mp_delta sub mp mkey =
       let mp1 = find_prefix resolve mp' in
       let resolve1 = subset_prefixed_by mp1 resolve in
       (subst_dom_delta_resolver
-	 (map_mp mp1 mkey empty_delta_resolver) resolve1),mp1
+         (map_mp mp1 mkey empty_delta_resolver) resolve1),mp1
 
 let gen_subst_delta_resolver dom subst resolver =
   let mp_apply_subst mkey mequ rslv =
@@ -483,9 +491,9 @@ let gen_subst_delta_resolver dom subst resolver =
     let kkey' = if dom then subst_kn subst kkey else kkey in
     let hint' = match hint with
       | Equiv kequ ->
-	  (try Equiv (subst_kn_delta subst kequ)
-	   with Change_equiv_to_inline (lev,c) -> Inline (lev,Some c))
-      | Inline (lev,Some (ctx, t)) -> Inline (lev,Some (ctx, subst_mps subst t))
+          (try Equiv (subst_kn_delta subst kequ)
+           with Change_equiv_to_inline (lev,c) -> Inline (lev,Some c))
+      | Inline (lev,Some t) -> Inline (lev,Some (Univ.map_univ_abstracted (subst_mps subst) t))
       | Inline (_,None) -> hint
     in
     Deltamap.add_kn kkey' hint' rslv
@@ -502,8 +510,8 @@ let update_delta_resolver resolver1 resolver2 =
   let kn_apply_rslv kkey hint1 rslv =
     let hint = match hint1 with
       | Equiv kequ ->
-	(try Equiv (solve_delta_kn resolver2 kequ)
-	 with Change_equiv_to_inline (lev,c) -> Inline (lev, Some c))
+        (try Equiv (solve_delta_kn resolver2 kequ)
+         with Change_equiv_to_inline (lev,c) -> Inline (lev, Some c))
       | Inline (_,Some _) -> hint1
       | Inline (_,None) ->
         (try Deltamap.find_kn kkey resolver2 with Not_found -> hint1)
@@ -525,46 +533,29 @@ let substition_prefixed_by k mp subst =
       Umap.add_mp new_key (mp_to,reso) sub
     else sub
   in
-  let mbi_prefixmp mbi _ sub = sub
-  in
-  Umap.fold mp_prefixmp mbi_prefixmp subst empty_subst
+  Umap.fold mp_prefixmp subst empty_subst
 
 let join subst1 subst2 =
   let apply_subst mpk add (mp,resolve) res =
     let mp',resolve' =
       match subst_mp0 subst2 mp with
-	| None -> mp, None
-	| Some (mp',resolve') ->  mp', Some resolve' in
+        | None -> mp, None
+        | Some (mp',resolve') ->  mp', Some resolve' in
     let resolve'' =
       match resolve' with
         | Some res ->
-	    add_delta_resolver
-	      (subst_dom_codom_delta_resolver subst2 resolve) res
-	| None ->
-	    subst_codom_delta_resolver subst2 resolve
+            add_delta_resolver
+              (subst_dom_codom_delta_resolver subst2 resolve) res
+        | None ->
+            subst_codom_delta_resolver subst2 resolve
     in
     let prefixed_subst = substition_prefixed_by mpk mp' subst2 in
     Umap.join prefixed_subst (add (mp',resolve'') res)
   in
   let mp_apply_subst mp = apply_subst mp (Umap.add_mp mp) in
-  let mbi_apply_subst mbi = apply_subst (MPbound mbi) (Umap.add_mbi mbi) in
-  let subst = Umap.fold mp_apply_subst mbi_apply_subst subst1 empty_subst in
+  let subst = Umap.fold mp_apply_subst subst1 empty_subst in
   Umap.join subst2 subst
 
-let rec occur_in_path mbi = function
-  | MPbound bid' -> MBId.equal mbi bid'
-  | MPdot (mp1,_) -> occur_in_path mbi mp1
-  | _ -> false
-
-let occur_mbid mbi sub =
-  let check_one mbi' (mp,_) =
-    if MBId.equal mbi mbi' || occur_in_path mbi mp then raise Exit
-  in
-  try
-    Umap.iter_mbi check_one sub;
-    false
-  with Exit -> true
-
 type 'a substituted = {
   mutable subst_value : 'a;
   mutable subst_subst : substitution list;
