@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -47,10 +47,10 @@ type signature_mismatch_error =
   | RecordFieldExpected of bool
   | RecordProjectionsExpected of Name.t list
   | NotEqualInductiveAliases
-  | IncompatibleInstances
   | IncompatibleUniverses of Univ.univ_inconsistency
   | IncompatiblePolymorphism of env * types * types
-  | IncompatibleConstraints of Univ.AUContext.t
+  | IncompatibleConstraints of { got : Univ.AUContext.t; expect : Univ.AUContext.t }
+  | IncompatibleVariance
 
 type module_typing_error =
   | SignatureMismatch of
@@ -138,7 +138,7 @@ let rec functor_smart_map fty f0 funct = match funct with
     let a' = f0 a in if a==a' then funct else NoFunctor a'
 
 let rec functor_iter fty f0 = function
-  |MoreFunctor (mbid,ty,e) -> fty ty; functor_iter fty f0 e
+  |MoreFunctor (_mbid,ty,e) -> fty ty; functor_iter fty f0 e
   |NoFunctor a -> f0 a
 
 (** {6 Misc operations } *)
@@ -171,7 +171,7 @@ let implem_iter fs fa impl = match impl with
 
 (** {6 Substitutions of modular structures } *)
 
-let id_delta x y = x
+let id_delta x _y = x
 
 let subst_with_body sub = function
   |WithMod(id,mp) as orig ->
@@ -198,9 +198,18 @@ let rec subst_structure sub do_delta sign =
   in
   List.Smart.map subst_body sign
 
+and subst_retro : type a. Mod_subst.substitution -> a module_retroknowledge -> a module_retroknowledge =
+  fun subst retro ->
+    match retro with
+    | ModTypeRK as r -> r
+    | ModBodyRK l as r ->
+      let l' = List.Smart.map (subst_retro_action subst) l in
+      if l == l' then r else ModBodyRK l
+
 and subst_body : 'a. _ -> _ -> (_ -> 'a -> 'a) -> _ -> 'a generic_module_body -> 'a generic_module_body =
   fun is_mod sub subst_impl do_delta mb ->
-  let { mod_mp=mp; mod_expr=me; mod_type=ty; mod_type_alg=aty } = mb in
+    let { mod_mp=mp; mod_expr=me; mod_type=ty; mod_type_alg=aty;
+          mod_retroknowledge=retro; _ } = mb in
   let mp' = subst_mp sub mp in
   let sub =
     if ModPath.equal mp mp' then sub
@@ -210,8 +219,10 @@ and subst_body : 'a. _ -> _ -> (_ -> 'a -> 'a) -> _ -> 'a generic_module_body ->
   let ty' = subst_signature sub do_delta ty in
   let me' = subst_impl sub me in
   let aty' = Option.Smart.map (subst_expression sub id_delta) aty in
+  let retro' = subst_retro sub retro in
   let delta' = do_delta mb.mod_delta sub in
-  if mp==mp' && me==me' && ty==ty' && aty==aty' && delta'==mb.mod_delta
+  if mp==mp' && me==me' && ty==ty' && aty==aty'
+     && retro==retro' && delta'==mb.mod_delta
   then mb
   else
     { mb with
@@ -219,7 +230,9 @@ and subst_body : 'a. _ -> _ -> (_ -> 'a -> 'a) -> _ -> 'a generic_module_body ->
       mod_expr = me';
       mod_type = ty';
       mod_type_alg = aty';
-      mod_delta = delta' }
+      mod_retroknowledge = retro';
+      mod_delta = delta';
+    }
 
 and subst_module sub do_delta mb =
   subst_body true sub subst_impl do_delta mb
@@ -260,43 +273,23 @@ let do_delta_dom_codom reso sub = subst_dom_codom_delta_resolver sub reso
 let subst_signature subst = subst_signature subst do_delta_codom
 let subst_structure subst = subst_structure subst do_delta_codom
 
-(** {6 Retroknowledge } *)
-
-(* spiwack: here comes the function which takes care of importing
-   the retroknowledge declared in the library *)
-(* lclrk : retroknowledge_action list, rkaction : retroknowledge action *)
-let add_retroknowledge =
-  let perform rkaction env = match rkaction with
-    | Retroknowledge.RKRegister (f, e) when (isConst e || isInd e) ->
-      Environ.register env f e
-    | _ ->
-      CErrors.anomaly ~label:"Modops.add_retroknowledge"
-        (Pp.str "had to import an unsupported kind of term.")
-  in
-  fun (ModBodyRK lclrk) env ->
-  (* The order of the declaration matters, for instance (and it's at the
-     time this comment is being written, the only relevent instance) the
-     int31 type registration absolutely needs int31 bits to be registered.
-     Since the local_retroknowledge is stored in reverse order (each new
-     registration is added at the top of the list) we need a fold_right
-     for things to go right (the pun is not intented). So we lose
-     tail recursivity, but the world will have exploded before any module
-     imports 10 000 retroknowledge registration.*)
-  List.fold_right perform lclrk env
-
 (** {6 Adding a module in the environment } *)
 
+let add_retroknowledge r env =
+  match r with
+  | ModBodyRK l -> List.fold_left Primred.add_retroknowledge env l
+
 let rec add_structure mp sign resolver linkinfo env =
   let add_one env (l,elem) = match elem with
     |SFBconst cb ->
-      let c = constant_of_delta_kn resolver (KerName.make2 mp l) in
+      let c = constant_of_delta_kn resolver (KerName.make mp l) in
       Environ.add_constant_key c cb linkinfo env
     |SFBmind mib ->
-      let mind = mind_of_delta_kn resolver (KerName.make2 mp l) in
-      let mib = 
-	if mib.mind_private != None then 
-	  { mib with mind_private = Some true }
-	else mib
+      let mind = mind_of_delta_kn resolver (KerName.make mp l) in
+      let mib =
+        if mib.mind_private != None then
+          { mib with mind_private = Some true }
+        else mib
       in
       Environ.add_mind_key mind (mib,ref linkinfo) env
     |SFBmodule mb -> add_module mb linkinfo env (* adds components as well *)
@@ -331,16 +324,12 @@ let strengthen_const mp_from l cb resolver =
   match cb.const_body with
   |Def _ -> cb
   |_ ->
-    let kn = KerName.make2 mp_from l in
+    let kn = KerName.make mp_from l in
     let con = constant_of_delta_kn resolver kn in
-    let u =
-      match cb.const_universes with
-      | Monomorphic_const _ -> Univ.Instance.empty
-      | Polymorphic_const ctx -> Univ.make_abstract_instance ctx
-    in
+    let u = Univ.make_abstract_instance (Declareops.constant_polymorphic_context cb) in
       { cb with
-	const_body = Def (Mod_subst.from_val (mkConstU (con,u)));
-	const_body_code = Some (Cemitcodes.from_val (Cbytegen.compile_alias con)) }
+        const_body = Def (Mod_subst.from_val (mkConstU (con,u)));
+        const_body_code = Some (Cemitcodes.from_val (Cbytegen.compile_alias con)) }
 
 let rec strengthen_mod mp_from mp_to mb =
   if mp_in_delta mb.mod_mp mb.mod_delta then mb
@@ -352,7 +341,7 @@ let rec strengthen_mod mp_from mp_to mb =
       mod_type = NoFunctor struc';
       mod_delta =
         add_mp_delta_resolver mp_from mp_to
-	  (add_delta_resolver mb.mod_delta reso) }
+          (add_delta_resolver mb.mod_delta reso) }
   |MoreFunctor _ -> mb
 
 and strengthen_sig mp_from struc mp_to reso = match struc with
@@ -371,7 +360,7 @@ and strengthen_sig mp_from struc mp_to reso = match struc with
     let item' = l,SFBmodule mb' in
     let reso',rest' = strengthen_sig mp_from rest mp_to reso in
     add_delta_resolver reso' mb.mod_delta, item':: rest'
-  |(l,SFBmodtype mty as item) :: rest ->
+  |(_l,SFBmodtype _mty as item) :: rest ->
     let reso',rest' = strengthen_sig mp_from rest mp_to reso in
     reso',item::rest'
 
@@ -385,7 +374,7 @@ let strengthen mtb mp =
       mod_type = NoFunctor struc';
       mod_delta =
         add_delta_resolver mtb.mod_delta
-	  (add_mp_delta_resolver mtb.mod_mp mp reso') }
+          (add_mp_delta_resolver mtb.mod_mp mp reso') }
   |MoreFunctor _ -> mtb
 
 let inline_delta_resolver env inl mp mbid mtb delta =
@@ -393,20 +382,21 @@ let inline_delta_resolver env inl mp mbid mtb delta =
   let rec make_inline delta = function
     | [] -> delta
     | (lev,kn)::r ->
-	let kn = replace_mp_in_kn (MPbound mbid) mp kn in
-	let con = constant_of_delta_kn delta kn in
-	try
-	  let constant = lookup_constant con env in
-	  let l = make_inline delta r in
-	  match constant.const_body with
-	    | Undef _ | OpaqueDef _ -> l
-	    | Def body ->
-	      let constr = Mod_subst.force_constr body in
+        let kn = replace_mp_in_kn (MPbound mbid) mp kn in
+        let con = constant_of_delta_kn delta kn in
+        try
+          let constant = lookup_constant con env in
+          let l = make_inline delta r in
+          match constant.const_body with
+            | Undef _ | OpaqueDef _ | Primitive _ -> l
+            | Def body ->
+              let constr = Mod_subst.force_constr body in
               let ctx = Declareops.constant_polymorphic_context constant in
-              add_inline_delta_resolver kn (lev, Some (ctx, constr)) l
-	with Not_found ->
-	  error_no_such_label_sub (Constant.label con)
-	    (ModPath.to_string (Constant.modpath con))
+              let constr = Univ.{univ_abstracted_value=constr; univ_abstracted_binder=ctx} in
+              add_inline_delta_resolver kn (lev, Some constr) l
+        with Not_found ->
+          error_no_such_label_sub (Constant.label con)
+            (ModPath.to_string (Constant.modpath con))
   in
   make_inline delta constants
 
@@ -417,14 +407,14 @@ let rec strengthen_and_subst_mod mb subst mp_from mp_to =
     if mb_is_an_alias then subst_module subst do_delta_dom mb
     else
       let reso',struc' =
-	strengthen_and_subst_struct struc subst
-	  mp_from mp_to false false mb.mod_delta
+        strengthen_and_subst_struct struc subst
+          mp_from mp_to false false mb.mod_delta
       in
       { mb with
-	mod_mp = mp_to;
-	mod_expr = Algebraic (NoFunctor (MEident mp_from));
-	mod_type = NoFunctor struc';
-	mod_delta = add_mp_delta_resolver mp_to mp_from reso' }
+        mod_mp = mp_to;
+        mod_expr = Algebraic (NoFunctor (MEident mp_from));
+        mod_type = NoFunctor struc';
+        mod_delta = add_mp_delta_resolver mp_to mp_from reso' }
   |MoreFunctor _ ->
     let subst = add_mp mb.mod_mp mp_to empty_delta_resolver subst in
     subst_module subst do_delta_dom mb
@@ -439,55 +429,55 @@ and strengthen_and_subst_struct str subst mp_from mp_to alias incl reso =
           else strengthen_const mp_from l cb' reso
         in
         let item' = if cb' == cb then item else (l, SFBconst cb') in
-	let reso',rest' =
-	  strengthen_and_subst_struct rest subst mp_from mp_to alias incl reso
+        let reso',rest' =
+          strengthen_and_subst_struct rest subst mp_from mp_to alias incl reso
         in
         let str' =
           if rest' == rest && item' == item then str
           else item' :: rest'
         in
-	if incl then
+        if incl then
           (* If we are performing an inclusion we need to add
              the fact that the constant mp_to.l is \Delta-equivalent
              to reso(mp_from.l) *)
-          let kn_from = KerName.make2 mp_from l in
-          let kn_to = KerName.make2 mp_to l in
+          let kn_from = KerName.make mp_from l in
+          let kn_to = KerName.make mp_to l in
           let old_name = kn_of_delta reso kn_from in
-	  add_kn_delta_resolver kn_to old_name reso', str'
-	else
-	  (* In this case the fact that the constant mp_to.l is
-	     \Delta-equivalent to resolver(mp_from.l) is already known
-	     because reso' contains mp_to maps to reso(mp_from) *)
-	  reso', str'
+          add_kn_delta_resolver kn_to old_name reso', str'
+        else
+          (* In this case the fact that the constant mp_to.l is
+             \Delta-equivalent to resolver(mp_from.l) is already known
+             because reso' contains mp_to maps to reso(mp_from) *)
+          reso', str'
     | (l,SFBmind mib) as item :: rest ->
         let mib' = subst_mind_body subst mib in
         let item' = if mib' == mib then item else (l, SFBmind mib') in
-	let reso',rest' =
-	  strengthen_and_subst_struct rest subst mp_from mp_to alias incl reso
+        let reso',rest' =
+          strengthen_and_subst_struct rest subst mp_from mp_to alias incl reso
         in
         let str' =
           if rest' == rest && item' == item then str
           else item' :: rest'
         in
-	(* Same as constant *)
-	if incl then
-	  let kn_from = KerName.make2 mp_from l in
-	  let kn_to = KerName.make2 mp_to l in
-	  let old_name = kn_of_delta reso kn_from in
+        (* Same as constant *)
+        if incl then
+          let kn_from = KerName.make mp_from l in
+          let kn_to = KerName.make mp_to l in
+          let old_name = kn_of_delta reso kn_from in
           add_kn_delta_resolver kn_to old_name reso', str'
-	else
+        else
           reso', str'
     | (l,SFBmodule mb) as item :: rest ->
-	let mp_from' = MPdot (mp_from,l) in
-	let mp_to' = MPdot (mp_to,l) in
-	let mb' = if alias then
-	  subst_module subst do_delta_dom mb
-	else
-	  strengthen_and_subst_mod mb subst mp_from' mp_to'
-	in
+        let mp_from' = MPdot (mp_from,l) in
+        let mp_to' = MPdot (mp_to,l) in
+        let mb' = if alias then
+          subst_module subst do_delta_dom mb
+        else
+          strengthen_and_subst_mod mb subst mp_from' mp_to'
+        in
         let item' = if mb' == mb then item else (l, SFBmodule mb') in
-	let reso',rest' =
-	  strengthen_and_subst_struct rest subst mp_from mp_to alias incl reso
+        let reso',rest' =
+          strengthen_and_subst_struct rest subst mp_from mp_to alias incl reso
         in
         let str' =
           if rest' == rest && item' == item then str
@@ -497,34 +487,34 @@ and strengthen_and_subst_struct str subst mp_from mp_to alias incl reso =
            on names, hence we add the fact that the functor can only
            be equivalent to itself. If we adopt an applicative
            semantic for functor this should be changed.*)
-	if is_functor mb'.mod_type then
+        if is_functor mb'.mod_type then
           add_mp_delta_resolver mp_to' mp_to' reso', str'
-	else
+        else
           add_delta_resolver reso' mb'.mod_delta, str'
     | (l,SFBmodtype mty) as item :: rest ->
-	let mp_from' = MPdot (mp_from,l) in
-	let mp_to' = MPdot(mp_to,l) in
-	let subst' = add_mp mp_from' mp_to' empty_delta_resolver subst in
-	let mty' = subst_modtype subst'
-	  (fun resolver _ -> subst_dom_codom_delta_resolver subst' resolver)
+        let mp_from' = MPdot (mp_from,l) in
+        let mp_to' = MPdot(mp_to,l) in
+        let subst' = add_mp mp_from' mp_to' empty_delta_resolver subst in
+        let mty' = subst_modtype subst'
+          (fun resolver _ -> subst_dom_codom_delta_resolver subst' resolver)
           mty
         in
         let item' = if mty' == mty then item else (l, SFBmodtype mty') in
-	let reso',rest' =
+        let reso',rest' =
           strengthen_and_subst_struct rest subst mp_from mp_to alias incl reso
         in
         let str' =
           if rest' == rest && item' == item then str
           else item' :: rest'
         in
-	add_mp_delta_resolver mp_to' mp_to' reso', str'
+        add_mp_delta_resolver mp_to' mp_to' reso', str'
 
 
 (** Let P be a module path when we write:
      "Module M:=P." or "Module M. Include P. End M."
     We need to perform two operations to compute the body of M.
     - The first one is applying the substitution {P <- M} on the type of P
-    - The second one is strenghtening. *)
+    - The second one is strengthening. *)
 
 let strengthen_and_subst_mb mb mp include_b = match mb.mod_type with
   |NoFunctor struc ->
@@ -535,12 +525,12 @@ let strengthen_and_subst_mb mb mp include_b = match mb.mod_type with
     let subst_resolver = map_mp mb.mod_mp mp empty_delta_resolver in
     let new_resolver =
       add_mp_delta_resolver mp mp_alias
-	(subst_dom_delta_resolver subst_resolver mb.mod_delta)
+        (subst_dom_delta_resolver subst_resolver mb.mod_delta)
     in
     let subst = map_mp mb.mod_mp mp new_resolver in
     let reso',struc' =
       strengthen_and_subst_struct struc subst
-	mb.mod_mp mp mb_is_an_alias include_b mb.mod_delta
+        mb.mod_mp mp mb_is_an_alias include_b mb.mod_delta
     in
     { mb with
       mod_mp = mp;
@@ -548,7 +538,7 @@ let strengthen_and_subst_mb mb mp include_b = match mb.mod_type with
       mod_expr = Algebraic (NoFunctor (MEident mb.mod_mp));
       mod_delta =
         if include_b then reso'
-	else add_delta_resolver new_resolver reso' }
+        else add_delta_resolver new_resolver reso' }
   |MoreFunctor _ ->
     let subst = map_mp mb.mod_mp mp empty_delta_resolver in
     subst_module subst do_delta_dom_codom mb
@@ -617,18 +607,14 @@ let clean_bounded_mod_expr sign =
 (** {6 Stm machinery } *)
 let join_constant_body except otab cb =
   match cb.const_body with
-  | OpaqueDef o ->
-      (match Opaqueproof.uuid_opaque otab o with
-      | Some uuid when not(Future.UUIDSet.mem uuid except) ->
-          Opaqueproof.join_opaque otab o
-      | _ -> ())
+  | OpaqueDef o -> Opaqueproof.join_opaque ~except otab o
   | _ -> ()
 
 let join_structure except otab s =
   let rec join_module : 'a. 'a generic_module_body -> unit = fun mb ->
     Option.iter join_expression mb.mod_type_alg;
     join_signature mb.mod_type
-  and join_field (l,body) = match body with
+  and join_field (_l,body) = match body with
     |SFBconst sb -> join_constant_body except otab sb
     |SFBmind _ -> ()
     |SFBmodule m ->
