@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -83,7 +83,7 @@ end
 module Name =
 struct
   type t = Anonymous     (** anonymous identifier *)
-	 | Name of Id.t  (** non-anonymous identifier *)
+         | Name of Id.t  (** non-anonymous identifier *)
 
   let mk_name id =
     Name id
@@ -207,7 +207,7 @@ struct
 
   let repr mbid = mbid
 
-  let to_string (i, s, p) =
+  let to_string (_i, s, p) =
     DirPath.to_string p ^ "." ^ s
 
   let debug_to_string (i, s, p) =
@@ -328,12 +328,12 @@ module ModPath = struct
   let rec dp = function
   | MPfile sl -> sl
   | MPbound (_,_,dp) -> dp
-  | MPdot (mp,l) -> dp mp
+  | MPdot (mp,_l) -> dp mp
 
   module Self_Hashcons = struct
     type t = module_path
     type u = (DirPath.t -> DirPath.t) * (MBId.t -> MBId.t) *
-	(string -> string)
+        (string -> string)
     let rec hashcons (hdir,huniqid,hstr as hfuns) = function
       | MPfile dir -> MPfile (hdir dir)
       | MPbound m -> MPbound (huniqid m)
@@ -356,6 +356,9 @@ module ModPath = struct
 
 end
 
+module DPset = Set.Make(DirPath)
+module DPmap = Map.Make(DirPath)
+
 module MPset = Set.Make(ModPath)
 module MPmap = CMap.Make(ModPath)
 
@@ -365,7 +368,6 @@ module KerName = struct
 
   type t = {
     modpath : ModPath.t;
-    dirpath : DirPath.t;
     knlabel : Label.t;
     mutable refhash : int;
     (** Lazily computed hash. If unset, it is set to negative values. *)
@@ -373,22 +375,15 @@ module KerName = struct
 
   type kernel_name = t
 
-  let make modpath dirpath knlabel =
-    { modpath; dirpath; knlabel; refhash = -1; }
-  let repr kn = (kn.modpath, kn.dirpath, kn.knlabel)
-
-  let make2 modpath knlabel =
-    { modpath; dirpath = DirPath.empty; knlabel; refhash = -1; }
+  let make modpath knlabel =
+    { modpath; knlabel; refhash = -1; }
+  let repr kn = (kn.modpath, kn.knlabel)
 
   let modpath kn = kn.modpath
   let label kn = kn.knlabel
 
   let to_string_gen mp_to_string kn =
-    let dp =
-      if DirPath.is_empty kn.dirpath then "."
-      else "#" ^ DirPath.to_string kn.dirpath ^ "#"
-    in
-    mp_to_string kn.modpath ^ dp ^ Label.to_string kn.knlabel
+    mp_to_string kn.modpath ^ "." ^ Label.to_string kn.knlabel
 
   let to_string kn = to_string_gen ModPath.to_string kn
 
@@ -396,15 +391,15 @@ module KerName = struct
 
   let print kn = str (to_string kn)
 
+  let debug_print kn = str (debug_to_string kn)
+
   let compare (kn1 : kernel_name) (kn2 : kernel_name) =
     if kn1 == kn2 then 0
     else
       let c = String.compare kn1.knlabel kn2.knlabel in
       if not (Int.equal c 0) then c
       else
-        let c = DirPath.compare kn1.dirpath kn2.dirpath in
-        if not (Int.equal c 0) then c
-        else ModPath.compare kn1.modpath kn2.modpath
+        ModPath.compare kn1.modpath kn2.modpath
 
   let equal kn1 kn2 =
     let h1 = kn1.refhash in
@@ -412,7 +407,6 @@ module KerName = struct
     if 0 <= h1 && 0 <= h2 && not (Int.equal h1 h2) then false
     else
       Label.equal kn1.knlabel kn2.knlabel &&
-      DirPath.equal kn1.dirpath kn2.dirpath &&
       ModPath.equal kn1.modpath kn2.modpath
 
   open Hashset.Combine
@@ -420,8 +414,8 @@ module KerName = struct
   let hash kn =
     let h = kn.refhash in
     if h < 0 then
-      let { modpath = mp; dirpath = dp; knlabel = lbl; } = kn in
-      let h = combine3 (ModPath.hash mp) (DirPath.hash dp) (Label.hash lbl) in
+      let { modpath = mp; knlabel = lbl; _ } = kn in
+      let h = combine (ModPath.hash mp) (Label.hash lbl) in
       (* Ensure positivity on all platforms. *)
       let h = h land 0x3FFFFFFF in
       let () = kn.refhash <- h in
@@ -432,12 +426,11 @@ module KerName = struct
     type t = kernel_name
     type u = (ModPath.t -> ModPath.t) * (DirPath.t -> DirPath.t)
         * (string -> string)
-    let hashcons (hmod,hdir,hstr) kn =
-      let { modpath = mp; dirpath = dp; knlabel = l; refhash; } = kn in
-      { modpath = hmod mp; dirpath = hdir dp; knlabel = hstr l; refhash; }
+    let hashcons (hmod,_hdir,hstr) kn =
+      let { modpath = mp; knlabel = l; refhash; } = kn in
+      { modpath = hmod mp; knlabel = hstr l; refhash; }
     let eq kn1 kn2 =
-      kn1.modpath == kn2.modpath && kn1.dirpath == kn2.dirpath &&
-        kn1.knlabel == kn2.knlabel
+      kn1.modpath == kn2.modpath && kn1.knlabel == kn2.knlabel
     let hash = hash
   end
 
@@ -492,21 +485,20 @@ module KerPair = struct
   let make knu knc = if KerName.equal knu knc then Same knc else Dual (knu,knc)
 
   let make1 = same
-  let make2 mp l = same (KerName.make2 mp l)
-  let make3 mp dir l = same (KerName.make mp dir l)
-  let repr3 kp = KerName.repr (user kp)
+  let make2 mp l = same (KerName.make mp l)
+  let repr2 kp = KerName.repr (user kp)
   let label kp = KerName.label (user kp)
   let modpath kp = KerName.modpath (user kp)
 
   let change_label kp lbl =
-    let (mp1,dp1,l1) = KerName.repr (user kp)
-    and (mp2,dp2,l2) = KerName.repr (canonical kp) in
-    assert (String.equal l1 l2 && DirPath.equal dp1 dp2);
+    let (mp1,l1) = KerName.repr (user kp)
+    and (mp2,l2) = KerName.repr (canonical kp) in
+    assert (String.equal l1 l2);
     if String.equal lbl l1 then kp
     else
-      let kn = KerName.make mp1 dp1 lbl in
+      let kn = KerName.make mp1 lbl in
       if mp1 == mp2 then same kn
-      else make kn (KerName.make mp2 dp2 lbl)
+      else make kn (KerName.make mp2 lbl)
 
   let to_string kp = KerName.to_string (user kp)
   let print kp = str (to_string kp)
@@ -623,8 +615,8 @@ let constr_modpath (ind,_) = ind_modpath ind
 
 let ith_mutual_inductive (mind, _) i = (mind, i)
 let ith_constructor_of_inductive ind i = (ind, i)
-let inductive_of_constructor (ind, i) = ind
-let index_of_constructor (ind, i) = i
+let inductive_of_constructor (ind, _i) = ind
+let index_of_constructor (_ind, i) = i
 
 let eq_ind (m1, i1) (m2, i2) = Int.equal i1 i2 && MutInd.equal m1 m2
 let eq_user_ind (m1, i1) (m2, i2) =
@@ -682,6 +674,8 @@ module InductiveOrdered_env = struct
   let compare = ind_user_ord
 end
 
+module Indset = Set.Make(InductiveOrdered)
+module Indset_env = Set.Make(InductiveOrdered_env)
 module Indmap = Map.Make(InductiveOrdered)
 module Indmap_env = Map.Make(InductiveOrdered_env)
 
@@ -695,6 +689,8 @@ module ConstructorOrdered_env = struct
   let compare = constructor_user_ord
 end
 
+module Constrset = Set.Make(ConstructorOrdered)
+module Constrset_env = Set.Make(ConstructorOrdered_env)
 module Constrmap = Map.Make(ConstructorOrdered)
 module Constrmap_env = Map.Make(ConstructorOrdered_env)
 
@@ -725,21 +721,14 @@ let hcons_construct = Hashcons.simple_hcons Hconstruct.generate Hconstruct.hcons
 
 (*****************)
 
-type transparent_state = Id.Pred.t * Cpred.t
-
-let empty_transparent_state = (Id.Pred.empty, Cpred.empty)
-let full_transparent_state = (Id.Pred.full, Cpred.full)
-let var_full_transparent_state = (Id.Pred.full, Cpred.empty)
-let cst_full_transparent_state = (Id.Pred.empty, Cpred.full)
-
 type 'a tableKey =
   | ConstKey of 'a
   | VarKey of Id.t
   | RelKey of Int.t
 
 type inv_rel_key = int (* index in the [rel_context] part of environment
-			  starting by the end, {\em inverse}
-			  of de Bruijn indice *)
+                          starting by the end, {\em inverse}
+                          of de Bruijn indice *)
 
 let eq_table_key f ik1 ik2 =
   if ik1 == ik2 then true
@@ -749,15 +738,12 @@ let eq_table_key f ik1 ik2 =
   | RelKey k1, RelKey k2 -> Int.equal k1 k2
   | _ -> false
 
-let eq_con_chk = Constant.UserOrd.equal
 let eq_mind_chk = MutInd.UserOrd.equal
 let eq_ind_chk (kn1,i1) (kn2,i2) = Int.equal i1 i2 && eq_mind_chk kn1 kn2
 
-
 (*******************************************************************)
 (** Compatibility layers *)
 
-type mod_bound_id = MBId.t
 let eq_constant_key = Constant.UserOrd.equal
 
 (** Compatibility layer for [ModPath] *)
@@ -885,6 +871,8 @@ struct
 
   let equal (c, b) (c', b') = Repr.equal c c' && b == b'
 
+  let repr_equal p p' = Repr.equal (repr p) (repr p')
+
   let hash (c, b) = (if b then 0 else 1) + Repr.hash c
 
   module SyntacticOrd = struct
@@ -933,9 +921,7 @@ struct
 
 end
 
-type projection = Projection.t
-
-module GlobRef = struct
+module GlobRefInternal = struct
 
   type t =
     | VarRef of variable           (** A reference to the section-context. *)
@@ -951,10 +937,79 @@ module GlobRef = struct
     | VarRef v1, VarRef v2 -> Id.equal v1 v2
     | (ConstRef _ | IndRef _ | ConstructRef _ | VarRef _), _ -> false
 
+  let global_eq_gen eq_cst eq_ind eq_cons x y =
+    x == y ||
+    match x, y with
+    | ConstRef cx, ConstRef cy -> eq_cst cx cy
+    | IndRef indx, IndRef indy -> eq_ind indx indy
+    | ConstructRef consx, ConstructRef consy -> eq_cons consx consy
+    | VarRef v1, VarRef v2 -> Id.equal v1 v2
+    | (VarRef _ | ConstRef _ | IndRef _ | ConstructRef _), _ -> false
+
+  let global_ord_gen ord_cst ord_ind ord_cons x y =
+    if x == y then 0
+    else match x, y with
+    | VarRef v1, VarRef v2 -> Id.compare v1 v2
+    | VarRef _, _ -> -1
+    | _, VarRef _ -> 1
+    | ConstRef cx, ConstRef cy -> ord_cst cx cy
+    | ConstRef _, _ -> -1
+    | _, ConstRef _ -> 1
+    | IndRef indx, IndRef indy -> ord_ind indx indy
+    | IndRef _, _ -> -1
+    | _ , IndRef _ -> 1
+    | ConstructRef consx, ConstructRef consy -> ord_cons consx consy
+
+  let global_hash_gen hash_cst hash_ind hash_cons gr =
+    let open Hashset.Combine in
+    match gr with
+    | ConstRef c -> combinesmall 1 (hash_cst c)
+    | IndRef i -> combinesmall 2 (hash_ind i)
+    | ConstructRef c -> combinesmall 3 (hash_cons c)
+    | VarRef id -> combinesmall 4 (Id.hash id)
+
 end
 
-type global_reference = GlobRef.t
-[@@ocaml.deprecated "Alias for [GlobRef.t]"]
+module GlobRef = struct
+
+  type t = GlobRefInternal.t =
+    | VarRef of variable           (** A reference to the section-context. *)
+    | ConstRef of Constant.t       (** A reference to the environment. *)
+    | IndRef of inductive          (** A reference to an inductive type. *)
+    | ConstructRef of constructor  (** A reference to a constructor of an inductive type. *)
+
+  let equal = GlobRefInternal.equal
+
+  (* By default, [global_reference] are ordered on their canonical part *)
+
+  module Ordered = struct
+    open Constant.CanOrd
+    type t = GlobRefInternal.t
+    let compare gr1 gr2 =
+      GlobRefInternal.global_ord_gen compare ind_ord constructor_ord gr1 gr2
+    let equal gr1 gr2 = GlobRefInternal.global_eq_gen equal eq_ind eq_constructor gr1 gr2
+    let hash gr = GlobRefInternal.global_hash_gen hash ind_hash constructor_hash gr
+  end
+
+  module Ordered_env = struct
+    open Constant.UserOrd
+    type t = GlobRefInternal.t
+    let compare gr1 gr2 =
+      GlobRefInternal.global_ord_gen compare ind_user_ord constructor_user_ord gr1 gr2
+    let equal gr1 gr2 =
+      GlobRefInternal.global_eq_gen equal eq_user_ind eq_user_constructor gr1 gr2
+    let hash gr = GlobRefInternal.global_hash_gen hash ind_user_hash constructor_user_hash gr
+  end
+
+  module Map = HMap.Make(Ordered)
+  module Set = Map.Set
+
+  (* Alternative sets and maps indexed by the user part of the kernel names *)
+
+  module Map_env = HMap.Make(Ordered_env)
+  module Set_env = Map_env.Set
+
+end
 
 type evaluable_global_reference =
   | EvalVarRef of Id.t
