@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -81,7 +81,7 @@ end
 module Name :
 sig
   type t = Anonymous     (** anonymous identifier *)
-	 | Name of Id.t  (** non-anonymous identifier *)
+         | Name of Id.t  (** non-anonymous identifier *)
 
   val mk_name : Id.t -> t
   (** constructor *)
@@ -149,18 +149,21 @@ sig
   val is_empty : t -> bool
   (** Test whether a directory path is empty. *)
 
-  val to_string : t -> string
-  (** Print directory paths as ["coq_root.module.submodule"] *)
-
   val initial : t
   (** Initial "seed" of the unique identifier generator *)
 
   val hcons : t -> t
   (** Hashconsing of directory paths. *)
 
+  val to_string : t -> string
+  (** Print non-empty directory paths as ["coq_root.module.submodule"] *)
+
   val print : t -> Pp.t
 end
 
+module DPset : Set.S with type elt = DirPath.t
+module DPmap : Map.ExtS with type key = DirPath.t and module Set := DPset
+
 (** {6 Names of structure elements } *)
 
 module Label :
@@ -180,15 +183,15 @@ sig
   val make : string -> t
   (** Create a label out of a string. *)
 
-  val to_string : t -> string
-  (** Conversion to string. *)
-
   val of_id : Id.t -> t
   (** Conversion from an identifier. *)
 
   val to_id : t -> Id.t
   (** Conversion to an identifier. *)
 
+  val to_string : t -> string
+  (** Conversion to string. *)
+
   val print : t -> Pp.t
   (** Pretty-printer. *)
 
@@ -227,10 +230,10 @@ sig
   (** Return the identifier contained in the argument. *)
 
   val to_string : t -> string
-  (** Conversion to a string. *)
+  (** Encode as a string (not to be used for user-facing messages). *)
 
   val debug_to_string : t -> string
-  (** Same as [to_string], but outputs information related to debug. *)
+  (** Same as [to_string], but outputs extra information related to debug. *)
 
 end
 
@@ -252,16 +255,17 @@ sig
 
   val is_bound : t -> bool
 
-  val to_string : t -> string
-
-  val debug_to_string : t -> string
-  (** Same as [to_string], but outputs information related to debug. *)
-
   val initial : t
   (** Name of the toplevel structure ([= MPfile initial_dir]) *)
 
   val dp : t -> DirPath.t
 
+  val to_string : t -> string
+  (** Encode as a string (not to be used for user-facing messages). *)
+
+  val debug_to_string : t -> string
+  (** Same as [to_string], but outputs extra information related to debug. *)
+
 end
 
 module MPset : Set.S with type elt = ModPath.t
@@ -274,21 +278,24 @@ sig
   type t
 
   (** Constructor and destructor *)
-  val make : ModPath.t -> DirPath.t -> Label.t -> t
-  val make2 : ModPath.t -> Label.t -> t
-  val repr : t -> ModPath.t * DirPath.t * Label.t
+  val make : ModPath.t -> Label.t -> t
+  val repr : t -> ModPath.t * Label.t
 
   (** Projections *)
   val modpath : t -> ModPath.t
   val label : t -> Label.t
 
-  (** Display *)
   val to_string : t -> string
-
-  val debug_to_string : t -> string
-  (** Same as [to_string], but outputs information related to debug. *)
+  (** Encode as a string (not to be used for user-facing messages). *)
 
   val print : t -> Pp.t
+  (** Print internal representation (not to be used for user-facing messages). *)
+
+  val debug_to_string : t -> string
+  (** Same as [to_string], but outputs extra information related to debug. *)
+
+  val debug_print : t -> Pp.t
+  (** Same as [print], but outputs extra information related to debug. *)
 
   (** Comparisons *)
   val compare : t -> t -> int
@@ -317,15 +324,12 @@ sig
   val make2 : ModPath.t -> Label.t -> t
   (** Shortcut for [(make1 (KerName.make2 ...))] *)
 
-  val make3 : ModPath.t -> DirPath.t -> Label.t -> t
-  (** Shortcut for [(make1 (KerName.make ...))] *)
-
   (** Projections *)
 
   val user : t -> KerName.t
   val canonical : t -> KerName.t
 
-  val repr3 : t -> ModPath.t * DirPath.t * Label.t
+  val repr2 : t -> ModPath.t * Label.t
   (** Shortcut for [KerName.repr (user ...)] *)
 
   val modpath : t -> ModPath.t
@@ -366,9 +370,16 @@ sig
   (** Displaying *)
 
   val to_string : t -> string
+  (** Encode as a string (not to be used for user-facing messages). *)
+
   val print : t -> Pp.t
+  (** Print internal representation (not to be used for user-facing messages). *)
+
   val debug_to_string : t -> string
+  (** Same as [to_string], but outputs extra information related to debug. *)
+
   val debug_print : t -> Pp.t
+  (** Same as [print], but outputs extra information related to debug. *)
 
 end
 
@@ -403,15 +414,12 @@ sig
   val make2 : ModPath.t -> Label.t -> t
   (** Shortcut for [(make1 (KerName.make2 ...))] *)
 
-  val make3 : ModPath.t -> DirPath.t -> Label.t -> t
-  (** Shortcut for [(make1 (KerName.make ...))] *)
-
   (** Projections *)
 
   val user : t -> KerName.t
   val canonical : t -> KerName.t
 
-  val repr3 : t -> ModPath.t * DirPath.t * Label.t
+  val repr2 : t -> ModPath.t * Label.t
   (** Shortcut for [KerName.repr (user ...)] *)
 
   val modpath : t -> ModPath.t
@@ -448,15 +456,22 @@ sig
   (** Displaying *)
 
   val to_string : t -> string
+  (** Encode as a string (not to be used for user-facing messages). *)
+
   val print : t -> Pp.t
+  (** Print internal representation (not to be used for user-facing messages). *)
+
   val debug_to_string : t -> string
+  (** Same as [to_string], but outputs extra information related to debug. *)
+
   val debug_print : t -> Pp.t
+  (** Same as [print], but outputs extra information related to debug. *)
 
 end
 
 module Mindset : CSig.SetS with type elt = MutInd.t
 module Mindmap : Map.ExtS with type key = MutInd.t and module Set := Mindset
-module Mindmap_env : CSig.MapS with type key = MutInd.t
+module Mindmap_env : CMap.ExtS with type key = MutInd.t
 
 (** Designation of a (particular) inductive type. *)
 type inductive = MutInd.t      (* the name of the inductive type *)
@@ -469,10 +484,14 @@ type constructor = inductive   (* designates the inductive type *)
                  * int         (* the index of the constructor
                                   BEWARE: indexing starts from 1. *)
 
-module Indmap : CSig.MapS with type key = inductive
-module Constrmap : CSig.MapS with type key = constructor
-module Indmap_env : CSig.MapS with type key = inductive
-module Constrmap_env : CSig.MapS with type key = constructor
+module Indset : CSet.S with type elt = inductive
+module Constrset : CSet.S with type elt = constructor
+module Indset_env : CSet.S with type elt = inductive
+module Constrset_env : CSet.S with type elt = constructor
+module Indmap : CMap.ExtS with type key = inductive and module Set := Indset
+module Constrmap : CMap.ExtS with type key = constructor and module Set := Constrset
+module Indmap_env : CMap.ExtS with type key = inductive and module Set := Indset_env
+module Constrmap_env : CMap.ExtS with type key = constructor and module Set := Constrset_env
 
 val ind_modpath : inductive -> ModPath.t
 val constr_modpath : constructor -> ModPath.t
@@ -514,32 +533,17 @@ type 'a tableKey =
   | VarKey of Id.t
   | RelKey of Int.t
 
-(** Sets of names *)
-type transparent_state = Id.Pred.t * Cpred.t
-
-val empty_transparent_state : transparent_state
-val full_transparent_state : transparent_state
-val var_full_transparent_state : transparent_state
-val cst_full_transparent_state : transparent_state
-
 type inv_rel_key = int (** index in the [rel_context] part of environment
-			  starting by the end, {e inverse}
-			  of de Bruijn indice *)
+                          starting by the end, {e inverse}
+                          of de Bruijn indice *)
 
 val eq_table_key : ('a -> 'a -> bool) -> 'a tableKey -> 'a tableKey -> bool
 val eq_constant_key : Constant.t -> Constant.t -> bool
 
 (** equalities on constant and inductive names (for the checker) *)
 
-val eq_con_chk : Constant.t -> Constant.t -> bool
-[@@ocaml.deprecated "Same as [Constant.UserOrd.equal]."]
-
 val eq_ind_chk : inductive -> inductive -> bool
 
-(** {6 Deprecated functions. For backward compatibility.} *)
-
-type mod_bound_id = MBId.t
-[@@ocaml.deprecated "Same as [MBId.t]."]
 (** {5 Module paths} *)
 
 type module_path = ModPath.t =
@@ -586,8 +590,12 @@ module Projection : sig
     val map : (MutInd.t -> MutInd.t) -> t -> t
     val map_npars : (MutInd.t -> int -> MutInd.t * int) -> t -> t
 
-    val print : t -> Pp.t
     val to_string : t -> string
+    (** Encode as a string (not to be used for user-facing messages). *)
+
+    val print : t -> Pp.t
+    (** Print internal representation (not to be used for user-facing messages). *)
+
   end
   type t (* = Repr.t * bool *)
 
@@ -619,19 +627,22 @@ module Projection : sig
   val hcons : t -> t
   (** Hashconsing of projections. *)
 
+  val repr_equal : t -> t -> bool
+  (** Ignoring the unfolding boolean. *)
+
   val compare : t -> t -> int
 
   val map : (MutInd.t -> MutInd.t) -> t -> t
   val map_npars : (MutInd.t -> int -> MutInd.t * int) -> t -> t
 
   val to_string : t -> string
+  (** Encode as a string (not to be used for user-facing messages). *)
+
   val print : t -> Pp.t
+  (** Print internal representation (not to be used for user-facing messages). *)
 
 end
 
-type projection = Projection.t
-[@@ocaml.deprecated "Alias for [Projection.t]"]
-
 (** {6 Global reference is a kernel side type for all references together } *)
 
 (* XXX: Should we define GlobRefCan GlobRefUser? *)
@@ -645,10 +656,29 @@ module GlobRef : sig
 
   val equal : t -> t -> bool
 
-end
+  module Ordered : sig
+    type nonrec t = t
+    val compare : t -> t -> int
+    val equal : t -> t -> bool
+    val hash : t -> int
+  end
 
-type global_reference = GlobRef.t
-[@@ocaml.deprecated "Alias for [GlobRef.t]"]
+  module Ordered_env : sig
+    type nonrec t = t
+    val compare : t -> t -> int
+    val equal : t -> t -> bool
+    val hash : t -> int
+  end
+
+  module Set_env : CSig.SetS with type elt = t
+  module Map_env : Map.ExtS
+    with type key = t and module Set := Set_env
+
+  module Set : CSig.SetS with type elt = t
+  module Map : Map.ExtS
+    with type key = t and module Set := Set
+
+end
 
 (** Better to have it here that in Closure, since required in grammar.cma *)
 (* XXX: Move to a module *)
