@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -36,10 +36,27 @@ open Util
 module RawLevel =
 struct
   open Names
+
+  module UGlobal = struct
+    type t = DirPath.t * int
+
+    let make dp i = (DirPath.hcons dp,i)
+
+    let equal (d, i) (d', i') = DirPath.equal d d' && Int.equal i i'
+
+    let hash (d,i) = Hashset.Combine.combine i (DirPath.hash d)
+
+    let compare (d, i) (d', i') =
+      let c = Int.compare i i' in
+      if Int.equal c 0 then DirPath.compare d d'
+      else c
+  end
+
   type t =
+    | SProp
     | Prop
     | Set
-    | Level of int * DirPath.t
+    | Level of UGlobal.t
     | Var of int
 
   (* Hash-consing *)
@@ -47,22 +64,25 @@ struct
   let equal x y =
     x == y ||
       match x, y with
+      | SProp, SProp -> true
       | Prop, Prop -> true
       | Set, Set -> true
-      | Level (n,d), Level (n',d') ->
-        Int.equal n n' && DirPath.equal d d'
+      | Level l, Level l' -> UGlobal.equal l l'
       | Var n, Var n' -> Int.equal n n'
       | _ -> false
 
   let compare u v =
     match u, v with
+    | SProp, SProp -> 0
+    | SProp, _ -> -1
+    | _, SProp -> 1
     | Prop,Prop -> 0
     | Prop, _ -> -1
     | _, Prop -> 1
     | Set, Set -> 0
     | Set, _ -> -1
     | _, Set -> 1
-    | Level (i1, dp1), Level (i2, dp2) ->
+    | Level (dp1, i1), Level (dp2, i2) ->
       if i1 < i2 then -1
       else if i1 > i2 then 1
       else DirPath.compare dp1 dp2
@@ -73,6 +93,7 @@ struct
   let hequal x y =
     x == y ||
       match x, y with
+      | SProp, SProp -> true
       | Prop, Prop -> true
       | Set, Set -> true
       | Level (n,d), Level (n',d') ->
@@ -81,39 +102,42 @@ struct
       | _ -> false
 
   let hcons = function
+    | SProp as x -> x
     | Prop as x -> x
     | Set as x -> x
-    | Level (n,d) as x -> 
+    | Level (d,n) as x ->
       let d' = Names.DirPath.hcons d in
-        if d' == d then x else Level (n,d')
-    | Var n as x -> x
+        if d' == d then x else Level (d',n)
+    | Var _n as x -> x
 
   open Hashset.Combine
 
   let hash = function
-    | Prop -> combinesmall 1 0
-    | Set -> combinesmall 1 1
+    | SProp -> combinesmall 1 0
+    | Prop -> combinesmall 1 1
+    | Set -> combinesmall 1 2
     | Var n -> combinesmall 2 n
-    | Level (n, d) -> combinesmall 3 (combine n (Names.DirPath.hash d))
+    | Level (d, n) -> combinesmall 3 (combine n (Names.DirPath.hash d))
 
 end
 
 module Level = struct
 
-  open Names
+  module UGlobal = RawLevel.UGlobal
 
   type raw_level = RawLevel.t =
+  | SProp
   | Prop
   | Set
-  | Level of int * DirPath.t
+  | Level of UGlobal.t
   | Var of int
 
   (** Embed levels with their hash value *)
-  type t = { 
+  type t = {
     hash : int;
     data : RawLevel.t }
 
-  let equal x y = 
+  let equal x y =
     x == y || Int.equal x.hash y.hash && RawLevel.equal x.data y.data
 
   let hash x = x.hash
@@ -140,14 +164,16 @@ module Level = struct
 
   let set = make Set
   let prop = make Prop
+  let sprop = make SProp
 
-  let is_small x = 
+  let is_small x =
     match data x with
     | Level _ -> false
     | Var _ -> false
+    | SProp -> true
     | Prop -> true
     | Set -> true
- 
+
   let is_prop x =
     match data x with
     | Prop -> true
@@ -158,69 +184,66 @@ module Level = struct
     | Set -> true
     | _ -> false
 
+  let is_sprop x =
+    match data x with
+    | SProp -> true
+    | _ -> false
+
   let compare u v =
-    if u == v then 0
-    else
-      let c = Int.compare (hash u) (hash v) in
-	if c == 0 then RawLevel.compare (data u) (data v)
-	else c
-
-  let natural_compare u v =
     if u == v then 0
     else RawLevel.compare (data u) (data v)
-	    
-  let to_string x = 
+
+  let to_string x =
     match data x with
+    | SProp -> "SProp"
     | Prop -> "Prop"
     | Set -> "Set"
-    | Level (n,d) -> Names.DirPath.to_string d^"."^string_of_int n
+    | Level (d,n) -> Names.DirPath.to_string d^"."^string_of_int n
     | Var n -> "Var(" ^ string_of_int n ^ ")"
 
   let pr u = str (to_string u)
 
   let apart u v =
     match data u, data v with
+    | SProp, _ | _, SProp
     | Prop, Set | Set, Prop -> true
     | _ -> false
 
   let vars = Array.init 20 (fun i -> make (Var i))
 
-  let var n = 
+  let var n =
     if n < 20 then vars.(n) else make (Var n)
 
   let var_index u =
     match data u with
     | Var n -> Some n | _ -> None
 
-  let make m n = make (Level (n, Names.DirPath.hcons m))
+  let make qid = make (Level qid)
 
   let name u =
     match data u with
-    | Level (n, d) -> Some (d, n)
+    | Level (d, n) -> Some (d, n)
     | _ -> None
 end
 
 (** Level maps *)
-module LMap = struct 
+module LMap = struct
   module M = HMap.Make (Level)
   include M
 
-  let union l r = 
-    merge (fun k l r -> 
-      match l, r with
-      | Some _, _ -> l
-      | _, _ -> r) l r
+  let lunion l r =
+    union (fun _k l _r -> Some l) l r
 
-  let subst_union l r = 
-    merge (fun k l r -> 
+  let subst_union l r =
+    union (fun _k l r ->
       match l, r with
-      | Some (Some _), _ -> l
-      | Some None, None -> l
-      | _, _ -> r) l r
+      | Some _, _ -> Some l
+      | None, None -> Some l
+      | _, _ -> Some r) l r
 
   let diff ext orig =
-    fold (fun u v acc -> 
-      if mem u orig then acc 
+    fold (fun u v acc ->
+      if mem u orig then acc
       else add u v acc)
       ext empty
 
@@ -265,22 +288,22 @@ module Universe =
 struct
   (* Invariants: non empty, sorted and without duplicates *)
 
-  module Expr = 
+  module Expr =
   struct
     type t = Level.t * int
 
     (* Hashing of expressions *)
-    module ExprHash = 
+    module ExprHash =
     struct
       type t = Level.t * int
       type u = Level.t -> Level.t
-      let hashcons hdir (b,n as x) = 
-	let b' = hdir b in 
-	  if b' == b then x else (b',n)
+      let hashcons hdir (b,n as x) =
+        let b' = hdir b in
+          if b' == b then x else (b',n)
       let eq l1 l2 =
-        l1 == l2 || 
+        l1 == l2 ||
         match l1,l2 with
-	| (b,n), (b',n') -> b == b' && n == n'
+        | (b,n), (b',n') -> b == b' && n == n'
 
       let hash (x, n) = n + Level.hash x
 
@@ -295,11 +318,12 @@ struct
 
     let compare u v =
       if u == v then 0
-      else 
-	let (x, n) = u and (x', n') = v in
-	  if Int.equal n n' then Level.compare x x'
-	  else n - n'
+      else
+        let (x, n) = u and (x', n') = v in
+          if Int.equal n n' then Level.compare x x'
+          else n - n'
 
+    let sprop = hcons (Level.sprop, 0)
     let prop = hcons (Level.prop, 0)
     let set = hcons (Level.set, 0)
     let type1 = hcons (Level.set, 1)
@@ -310,29 +334,29 @@ struct
 
     let equal x y = x == y ||
       (let (u,n) = x and (v,n') = y in
-	 Int.equal n n' && Level.equal u v)
+         Int.equal n n' && Level.equal u v)
 
     let hash = ExprHash.hash
 
     let leq (u,n) (v,n') =
       let cmp = Level.compare u v in
-	if Int.equal cmp 0 then n <= n'
-	else if n <= n' then 
-	  (Level.is_prop u && Level.is_small v)
-	else false
+        if Int.equal cmp 0 then n <= n'
+        else if n <= n' then
+          (Level.is_prop u && not (Level.is_sprop v))
+        else false
 
-    let successor (u,n) =
-      if Level.is_prop u then type1
+    let successor (u,n as e) =
+      if is_small e then type1
       else (u, n + 1)
 
-    let addn k (u,n as x) = 
-      if k = 0 then x 
-      else if Level.is_prop u then
-	(Level.set,n+k)
+    let addn k (u,n as x) =
+      if k = 0 then x
+      else if Level.is_small u then
+        (Level.set,n+k)
       else (u,n+k)
 
     type super_result =
-	SuperSame of bool
+        SuperSame of bool
         (* The level expressions are in cumulativity relation. boolean
            indicates if left is smaller than right?  *)
       | SuperDiff of int
@@ -345,13 +369,16 @@ struct
        left expression is "smaller" than the right one in both cases. *)
     let super (u,n) (v,n') =
       let cmp = Level.compare u v in
-	if Int.equal cmp 0 then SuperSame (n < n')
-	else
+        if Int.equal cmp 0 then SuperSame (n < n')
+        else
           let open RawLevel in
           match Level.data u, n, Level.data v, n' with
-          | Prop, _, Prop, _ -> SuperSame (n < n')
-          | Prop, 0, _, _ -> SuperSame true
-          | _, _, Prop, 0 -> SuperSame false
+          | SProp, _, SProp, _ | Prop, _, Prop, _ -> SuperSame (n < n')
+          | SProp, 0, Prop, 0 -> SuperSame true
+          | Prop, 0, SProp, 0 -> SuperSame false
+          | (SProp | Prop), 0, _, _ -> SuperSame true
+          | _, _, (SProp | Prop), 0 -> SuperSame false
+
           | _, _, _, _ -> SuperDiff cmp
 
     let to_string (v, n) =
@@ -360,26 +387,26 @@ struct
 
     let pr x = str(to_string x)
 
-    let pr_with f (v, n) = 
+    let pr_with f (v, n) =
       if Int.equal n 0 then f v
       else f v ++ str"+" ++ int n
 
     let is_level = function
-      | (v, 0) -> true
+      | (_v, 0) -> true
       | _ -> false
 
     let level = function
       | (v,0) -> Some v
       | _ -> None
-	
-    let get_level (v,n) = v
 
-    let map f (v, n as x) = 
-      let v' = f v in 
-	if v' == v then x
-	else if Level.is_prop v' && n != 0 then
-	  (Level.set, n)
-	else (v', n)
+    let get_level (v,_n) = v
+
+    let map f (v, n as x) =
+      let v' = f v in
+        if v' == v then x
+        else if Level.is_prop v' && n != 0 then
+          (Level.set, n)
+        else (v', n)
 
   end
 
@@ -405,16 +432,16 @@ struct
 
   let pr l = match l with
     | [u] -> Expr.pr u
-    | _ -> 
+    | _ ->
       str "max(" ++ hov 0
-	(prlist_with_sep pr_comma Expr.pr l) ++
+        (prlist_with_sep pr_comma Expr.pr l) ++
         str ")"
 
   let pr_with f l = match l with
     | [u] -> Expr.pr_with f u
-    | _ -> 
+    | _ ->
       str "max(" ++ hov 0
-	(prlist_with_sep pr_comma (Expr.pr_with f) l) ++
+        (prlist_with_sep pr_comma (Expr.pr_with f) l) ++
         str ")"
 
   let is_level l = match l with
@@ -429,14 +456,16 @@ struct
     | [l] -> Expr.level l
     | _ -> None
 
-  let levels l = 
+  let levels l =
     List.fold_left (fun acc x -> LSet.add (Expr.get_level x) acc) LSet.empty l
 
-  let is_small u = 
+  let is_small u =
     match u with
     | [l] -> Expr.is_small l
     | _ -> false
 
+  let sprop = tip Expr.sprop
+
   (* The lower predicative level of the hierarchy that contains (impredicative)
      Prop and singleton inductive types *)
   let type0m = tip Expr.prop
@@ -445,15 +474,16 @@ struct
   let type0 = tip Expr.set
 
   (* When typing [Prop] and [Set], there is no constraint on the level,
-     hence the definition of [type1_univ], the type of [Prop] *)    
-  let type1 = tip (Expr.successor Expr.set)
+     hence the definition of [type1_univ], the type of [Prop] *)
+  let type1 = tip Expr.type1
 
+  let is_sprop x = equal sprop x
   let is_type0m x = equal type0m x
   let is_type0 x = equal type0 x
 
   (* Returns the formal universe that lies just above the universe variable u.
      Used to type the sort u. *)
-  let super l = 
+  let super l =
     if is_small l then type1
     else
       List.Smart.map (fun x -> Expr.successor x) l
@@ -468,26 +498,26 @@ struct
     | h1 :: t1, h2 :: t2 ->
        let open Expr in
        (match super h1 h2 with
-	| SuperSame true (* h1 < h2 *) -> merge_univs t1 l2
-	| SuperSame false -> merge_univs l1 t2
-	| SuperDiff c ->
+        | SuperSame true (* h1 < h2 *) -> merge_univs t1 l2
+        | SuperSame false -> merge_univs l1 t2
+        | SuperDiff c ->
            if c <= 0 (* h1 < h2 is name order *)
-	   then cons h1 (merge_univs t1 l2)
-	   else cons h2 (merge_univs l1 t2))
+           then cons h1 (merge_univs t1 l2)
+           else cons h2 (merge_univs l1 t2))
 
   let sort u =
-    let rec aux a l = 
+    let rec aux a l =
       match l with
       | b :: l' ->
-	let open Expr in
+        let open Expr in
         (match super a b with
-	 | SuperSame false -> aux a l'
-	 | SuperSame true -> l
-	 | SuperDiff c ->
-	    if c <= 0 then cons a l
-	    else cons b (aux a l'))
+         | SuperSame false -> aux a l'
+         | SuperSame true -> l
+         | SuperDiff c ->
+            if c <= 0 then cons a l
+            else cons b (aux a l'))
       | [] -> cons a l
-    in 
+    in
       List.fold_right (fun a acc -> aux a acc) u []
 
   (* Returns the formal universe that is greater than the universes u and v.
@@ -525,9 +555,9 @@ open Universe
 let universe_level = Universe.level
 
 
-type constraint_type = Lt | Le | Eq
+type constraint_type = AcyclicGraph.constraint_type = Lt | Le | Eq
 
-type explanation = (constraint_type * universe) list
+type explanation = (constraint_type * Level.t) list
 
 let constraint_type_ord c1 c2 = match c1, c2 with
 | Lt, Lt -> 0
@@ -548,11 +578,11 @@ exception UniverseInconsistency of univ_inconsistency
 let error_inconsistency o u v p =
   raise (UniverseInconsistency (o,make u,make v,p))
 
-(* Constraints and sets of constraints. *)    
+(* Constraints and sets of constraints. *)
 
 type univ_constraint = Level.t * constraint_type * Level.t
 
-let pr_constraint_type op = 
+let pr_constraint_type op =
   let op_str = match op with
     | Lt -> " < "
     | Le -> " <= "
@@ -571,21 +601,18 @@ struct
       else Level.compare v v'
 end
 
-module Constraint = 
-struct 
+module Constraint =
+struct
   module S = Set.Make(UConstraintOrd)
   include S
 
   let pr prl c =
-    fold (fun (u1,op,u2) pp_std ->
-      pp_std ++ prl u1 ++ pr_constraint_type op ++
-	prl u2 ++ fnl () )  c (str "")
+    v 0 (prlist_with_sep spc (fun (u1,op,u2) ->
+      hov 0 (prl u1 ++ pr_constraint_type op ++ prl u2))
+       (elements c))
 
-  let universes_of c =
-    fold (fun (u1, op, u2) unvs -> LSet.add u2 (LSet.add u1 unvs)) c LSet.empty
 end
 
-let universes_of_constraints = Constraint.universes_of
 let empty_constraint = Constraint.empty
 let union_constraint = Constraint.union
 let eq_constraint = Constraint.equal
@@ -599,7 +626,7 @@ module Hconstraint =
       type u = universe_level -> universe_level
       let hashcons hul (l1,k,l2) = (hul l1, k, hul l2)
       let eq (l1,k,l2) (l1',k',l2') =
-	l1 == l1' && k == k' && l2 == l2'
+        l1 == l1' && k == k' && l2 == l2'
       let hash = Hashtbl.hash
     end)
 
@@ -609,11 +636,11 @@ module Hconstraints =
       type t = constraints
       type u = univ_constraint -> univ_constraint
       let hashcons huc s =
-	Constraint.fold (fun x -> Constraint.add (huc x)) s Constraint.empty
+        Constraint.fold (fun x -> Constraint.add (huc x)) s Constraint.empty
       let eq s s' =
-	List.for_all2eq (==)
-	  (Constraint.elements s)
-	  (Constraint.elements s')
+        List.for_all2eq (==)
+          (Constraint.elements s)
+          (Constraint.elements s')
       let hash = Hashtbl.hash
     end)
 
@@ -632,7 +659,7 @@ type 'a constraint_function = 'a -> 'a -> constraints -> constraints
 
 let enforce_eq_level u v c =
   (* We discard trivial constraints like u=u *)
-  if Level.equal u v then c 
+  if Level.equal u v then c
   else if Level.apart u v then
     error_inconsistency Eq u v None
   else Constraint.add (u,Eq,v) c
@@ -651,30 +678,35 @@ let enforce_eq u v c =
 let constraint_add_leq v u c =
   (* We just discard trivial constraints like u<=u *)
   if Expr.equal v u then c
-  else 
+  else
     match v, u with
-    | (x,n), (y,m) -> 
+    | (x,n), (y,m) ->
     let j = m - n in
       if j = -1 (* n = m+1, v+1 <= u <-> v < u *) then
-	Constraint.add (x,Lt,y) c
+        Constraint.add (x,Lt,y) c
       else if j <= -1 (* n = m+k, v+k <= u <-> v+(k-1) < u *) then
-	if Level.equal x y then (* u+(k+1) <= u *)
-	  raise (UniverseInconsistency (Le, Universe.tip v, Universe.tip u, None))
-	else anomaly (Pp.str"Unable to handle arbitrary u+k <= v constraints.")
+        if Level.equal x y then (* u+(k+1) <= u *)
+          raise (UniverseInconsistency (Le, Universe.tip v, Universe.tip u, None))
+        else anomaly (Pp.str"Unable to handle arbitrary u+k <= v constraints.")
       else if j = 0 then
-	Constraint.add (x,Le,y) c
+        Constraint.add (x,Le,y) c
       else (* j >= 1 *) (* m = n + k, u <= v+k *)
-	if Level.equal x y then c (* u <= u+k, trivial *)
-	else if Level.is_small x then c (* Prop,Set <= u+S k, trivial *)
+        if Level.equal x y then c (* u <= u+k, trivial *)
+        else if Level.is_small x then c (* Prop,Set <= u+S k, trivial *)
         else Constraint.add (x,Le,y) c (* u <= v implies u <= v+k *)
-	  
+
 let check_univ_leq_one u v = Universe.exists (Expr.leq u) v
 
-let check_univ_leq u v = 
+let check_univ_leq u v =
   Universe.for_all (fun u -> check_univ_leq_one u v) u
 
 let enforce_leq u v c =
-  List.fold_left (fun c v -> (List.fold_left (fun c u -> constraint_add_leq u v c) c u)) c v
+  match is_sprop u, is_sprop v with
+  | true, true -> c
+  | true, false | false, true ->
+    raise (UniverseInconsistency (Le, u, v, None))
+  | false, false ->
+    List.fold_left (fun c v -> (List.fold_left (fun c u -> constraint_add_leq u v c) c u)) c v
 
 let enforce_leq u v c =
   if check_univ_leq u v then c
@@ -689,12 +721,12 @@ let enforce_leq_level u v c =
 let univ_level_mem u v =
   List.exists (fun (l, n) -> Int.equal n 0 && Level.equal u l) v
 
-let univ_level_rem u v min = 
+let univ_level_rem u v min =
   match Universe.level v with
   | Some u' -> if Level.equal u u' then min else v
   | None -> List.filter (fun (l, n) -> not (Int.equal n 0 && Level.equal u l)) v
 
-(* Is u mentionned in v (or equals to v) ? *)
+(* Is u mentioned in v (or equals to v) ? *)
 
 
 (**********************************************************************)
@@ -762,7 +794,7 @@ module Instance : sig
 
     val empty : t
     val is_empty : t -> bool
-      
+
     val of_array : Level.t array -> t
     val to_array : t -> Level.t array
 
@@ -776,10 +808,10 @@ module Instance : sig
     val share : t -> t * int
 
     val subst_fn : universe_level_subst_fn -> t -> t
-    
+
     val pr : (Level.t -> Pp.t) -> ?variance:Variance.t array -> t -> Pp.t
     val levels : t -> LSet.t
-end = 
+end =
 struct
   type t = Level.t array
 
@@ -790,64 +822,64 @@ struct
     type nonrec t = t
     type u = Level.t -> Level.t
 
-    let hashcons huniv a = 
+    let hashcons huniv a =
       let len = Array.length a in
-	if Int.equal len 0 then empty
-	else begin
-	  for i = 0 to len - 1 do
-	    let x = Array.unsafe_get a i in
-	    let x' = huniv x in
-	      if x == x' then ()
-	      else Array.unsafe_set a i x'
-	  done;
-	  a
-	end
+        if Int.equal len 0 then empty
+        else begin
+          for i = 0 to len - 1 do
+            let x = Array.unsafe_get a i in
+            let x' = huniv x in
+              if x == x' then ()
+              else Array.unsafe_set a i x'
+          done;
+          a
+        end
 
     let eq t1 t2 =
       t1 == t2 ||
-	(Int.equal (Array.length t1) (Array.length t2) &&
-	   let rec aux i =
-	     (Int.equal i (Array.length t1)) || (t1.(i) == t2.(i) && aux (i + 1))
-	   in aux 0)
-	
-    let hash a = 
+        (Int.equal (Array.length t1) (Array.length t2) &&
+           let rec aux i =
+             (Int.equal i (Array.length t1)) || (t1.(i) == t2.(i) && aux (i + 1))
+           in aux 0)
+
+    let hash a =
       let accu = ref 0 in
-	for i = 0 to Array.length a - 1 do
-	  let l = Array.unsafe_get a i in
-	  let h = Level.hash l in
-	    accu := Hashset.Combine.combine !accu h;
-	done;
-	(* [h] must be positive. *)
-	let h = !accu land 0x3FFFFFFF in
-	  h
+        for i = 0 to Array.length a - 1 do
+          let l = Array.unsafe_get a i in
+          let h = Level.hash l in
+            accu := Hashset.Combine.combine !accu h;
+        done;
+        (* [h] must be positive. *)
+        let h = !accu land 0x3FFFFFFF in
+          h
   end
 
   module HInstance = Hashcons.Make(HInstancestruct)
 
   let hcons = Hashcons.simple_hcons HInstance.generate HInstance.hcons Level.hcons
-    
+
   let hash = HInstancestruct.hash
-    
+
   let share a = (hcons a, hash a)
-	      
+
   let empty = hcons [||]
 
   let is_empty x = Int.equal (Array.length x) 0
 
   let append x y =
     if Array.length x = 0 then y
-    else if Array.length y = 0 then x 
+    else if Array.length y = 0 then x
     else Array.append x y
 
   let of_array a =
-    assert(Array.for_all (fun x -> not (Level.is_prop x)) a);
+    assert(Array.for_all (fun x -> not (Level.is_prop x || Level.is_sprop x)) a);
     a
 
   let to_array a = a
 
   let length a = Array.length a
 
-  let subst_fn fn t = 
+  let subst_fn fn t =
     let t' = CArray.Smart.map fn t in
       if t' == t then t else of_array t'
 
@@ -860,20 +892,20 @@ struct
     in
     prvecti_with_sep spc ppu
 
-  let equal t u = 
+  let equal t u =
     t == u ||
       (Array.is_empty t && Array.is_empty u) ||
-      (CArray.for_all2 Level.equal t u 
-	 (* Necessary as universe instances might come from different modules and 
-	    unmarshalling doesn't preserve sharing *))
+      (CArray.for_all2 Level.equal t u
+         (* Necessary as universe instances might come from different modules and
+            unmarshalling doesn't preserve sharing *))
 
 end
 
-let enforce_eq_instances x y = 
+let enforce_eq_instances x y =
   let ax = Instance.to_array x and ay = Instance.to_array y in
     if Array.length ax != Array.length ay then
       anomaly (Pp.(++) (Pp.str "Invalid argument: enforce_eq_instances called with")
-		 (Pp.str " instances of different lengths."));
+                 (Pp.str " instances of different lengths."));
     CArray.fold_right2 enforce_eq_level ax ay
 
 let enforce_eq_variance_instances = Variance.eq_constraints
@@ -881,10 +913,10 @@ let enforce_leq_variance_instances = Variance.leq_constraints
 
 let subst_instance_level s l =
   match l.Level.data with
-  | Level.Var n -> s.(n) 
+  | Level.Var n -> s.(n)
   | _ -> l
 
-let subst_instance_instance s i = 
+let subst_instance_instance s i =
   Array.Smart.map (fun l -> subst_instance_level s l) i
 
 let subst_instance_universe s u =
@@ -900,14 +932,12 @@ let subst_instance_constraint s (u,d,v as c) =
     else (u',d,v')
 
 let subst_instance_constraints s csts =
-  Constraint.fold 
+  Constraint.fold
     (fun c csts -> Constraint.add (subst_instance_constraint s c) csts)
-    csts Constraint.empty 
-
-type universe_instance = Instance.t
+    csts Constraint.empty
 
 type 'a puniverses = 'a * Instance.t
-let out_punivs (x, y) = x
+let out_punivs (x, _y) = x
 let in_punivs x = (x, Instance.empty)
 let eq_puniverses f (x, u) (y, u') =
   f x y && Instance.equal u u'
@@ -932,8 +962,8 @@ struct
   let hcons (univs, cst) =
     (Instance.hcons univs, hcons_constraints cst)
 
-  let instance (univs, cst) = univs
-  let constraints (univs, cst) = cst
+  let instance (univs, _cst) = univs
+  let constraints (_univs, cst) = cst
 
   let union (univs, cst) (univs', cst') =
     Instance.append univs univs', Constraint.union cst cst'
@@ -949,69 +979,45 @@ let hcons_universe_context = UContext.hcons
 
 module AUContext =
 struct
-  include UContext
+  type t = Names.Name.t array constrained
 
   let repr (inst, cst) =
-    (Array.mapi (fun i l -> Level.var i) inst, cst)
+    (Array.init (Array.length inst) (fun i -> Level.var i), cst)
+
+  let pr f ?variance ctx = UContext.pr f ?variance (repr ctx)
 
   let instantiate inst (u, cst) =
     assert (Array.length u = Array.length inst);
     subst_instance_constraints inst cst
 
+  let names (nas, _) = nas
+
+  let hcons (univs, cst) =
+    (Array.map Names.Name.hcons univs, hcons_constraints cst)
+
+  let empty = ([||], Constraint.empty)
+
+  let is_empty (nas, cst) = Array.is_empty nas && Constraint.is_empty cst
+
+  let union (nas, cst) (nas', cst') = (Array.append nas nas', Constraint.union cst cst')
+
+  let size (nas, _) = Array.length nas
+
 end
 
-type abstract_universe_context = AUContext.t
+type 'a univ_abstracted = {
+  univ_abstracted_value : 'a;
+  univ_abstracted_binder : AUContext.t;
+}
+
+let map_univ_abstracted f {univ_abstracted_value;univ_abstracted_binder} =
+  let univ_abstracted_value = f univ_abstracted_value in
+  {univ_abstracted_value;univ_abstracted_binder}
+
 let hcons_abstract_universe_context = AUContext.hcons
 
-(** Universe info for cumulative inductive types: A context of
-   universe levels with universe constraints, representing local
-   universe variables and constraints, together with an array of
-   Variance.t.
-
-    This data structure maintains the invariant that the variance
-   array has the same length as the universe instance. *)
-module CumulativityInfo =
-struct
-  type t = universe_context * Variance.t array
-
-  let make x =
-    if (Instance.length (UContext.instance (fst x))) =
-       (Array.length (snd x)) then x
-    else anomaly (Pp.str "Invalid subtyping information encountered!")
-
-  let empty = (UContext.empty, [||])
-  let is_empty (univs, variance) = UContext.is_empty univs && Array.is_empty variance
-
-  let pr prl (univs, variance) =
-    UContext.pr prl ~variance univs
-
-  let hcons (univs, variance) = (* should variance be hconsed? *)
-    (UContext.hcons univs, variance)
-
-  let univ_context (univs, subtypcst) = univs
-  let variance (univs, variance) = variance
-
-  (** This function takes a universe context representing constraints
-     of an inductive and produces a CumulativityInfo.t with the
-     trivial subtyping relation. *)
-  let from_universe_context univs =
-    (univs, Array.init (UContext.size univs) (fun _ -> Variance.Invariant))
-
-  let leq_constraints (_,variance) u u' csts = Variance.leq_constraints variance u u' csts
-  let eq_constraints (_,variance) u u' csts = Variance.eq_constraints variance u u' csts
-
-end
-
-type cumulativity_info = CumulativityInfo.t
-let hcons_cumulativity_info = CumulativityInfo.hcons
-
-module ACumulativityInfo = CumulativityInfo
-
-type abstract_cumulativity_info = ACumulativityInfo.t
-let hcons_abstract_cumulativity_info = ACumulativityInfo.hcons
-
 (** A set of universes with universe constraints.
-    We linearize the set to a list after typechecking. 
+    We linearize the set to a list after typechecking.
     Beware, representation could change.
 *)
 
@@ -1024,7 +1030,7 @@ struct
 
   let equal (univs, cst as x) (univs', cst' as y) =
     x == y || (LSet.equal univs univs' && Constraint.equal cst cst')
-									
+
   let of_set s = (s, Constraint.empty)
   let singleton l = of_set (LSet.singleton l)
   let of_instance i = of_set (Instance.levels i)
@@ -1053,8 +1059,8 @@ struct
     let univs = Array.fold_left fold univs v in
     (univs, cst)
 
-  let sort_levels a = 
-    Array.sort Level.natural_compare a; a
+  let sort_levels a =
+    Array.sort Level.compare a; a
 
   let to_context (ctx, cst) =
     (Instance.of_array (sort_levels (Array.of_list (LSet.elements ctx))), cst)
@@ -1066,8 +1072,8 @@ struct
     if is_empty ctx then mt() else
       h 0 (LSet.pr prl univs ++ str " |= ") ++ h 0 (v 0 (Constraint.pr prl cst))
 
-  let constraints (univs, cst) = cst
-  let levels (univs, cst) = univs
+  let constraints (_univs, cst) = cst
+  let levels (univs, _cst) = univs
 
   let size (univs,_) = LSet.cardinal univs
 end
@@ -1078,6 +1084,9 @@ type universe_context_set = ContextSet.t
 type 'a in_universe_context = 'a * universe_context
 type 'a in_universe_context_set = 'a * universe_context_set
 
+let extend_in_context_set (a, ctx) ctx' =
+  (a, ContextSet.union ctx ctx')
+
 (** Substitutions. *)
 
 let empty_subst = LMap.empty
@@ -1103,17 +1112,17 @@ let subst_univs_level_instance subst i =
   let i' = Instance.subst_fn (subst_univs_level_level subst) i in
     if i == i' then i
     else i'
-	
+
 let subst_univs_level_constraint subst (u,d,v) =
-  let u' = subst_univs_level_level subst u 
+  let u' = subst_univs_level_level subst u
   and v' = subst_univs_level_level subst v in
     if d != Lt && Level.equal u' v' then None
     else Some (u',d,v')
 
 let subst_univs_level_constraints subst csts =
-  Constraint.fold 
+  Constraint.fold
     (fun c -> Option.fold_right Constraint.add (subst_univs_level_constraint subst c))
-    csts Constraint.empty 
+    csts Constraint.empty
 
 let subst_univs_level_abstract_universe_context subst (inst, csts) =
   inst, subst_univs_level_constraints subst csts
@@ -1127,49 +1136,46 @@ let subst_univs_expr_opt fn (l,n) =
   Universe.addn n (fn l)
 
 let subst_univs_universe fn ul =
-  let subst, nosubst = 
-    List.fold_right (fun u (subst,nosubst) -> 
+  let subst, nosubst =
+    List.fold_right (fun u (subst,nosubst) ->
       try let a' = subst_univs_expr_opt fn u in
-	    (a' :: subst, nosubst)
+            (a' :: subst, nosubst)
       with Not_found -> (subst, u :: nosubst))
       ul ([], [])
-  in 
+  in
     if CList.is_empty subst then ul
-    else 
-      let substs = 
-	List.fold_left Universe.merge_univs Universe.empty subst
+    else
+      let substs =
+        List.fold_left Universe.merge_univs Universe.empty subst
       in
-	List.fold_left (fun acc u -> Universe.merge_univs acc (Universe.tip u))
-	  substs nosubst
+        List.fold_left (fun acc u -> Universe.merge_univs acc (Universe.tip u))
+          substs nosubst
 
-let make_instance_subst i = 
+let make_instance_subst i =
   let arr = Instance.to_array i in
     Array.fold_left_i (fun i acc l ->
       LMap.add l (Level.var i) acc)
       LMap.empty arr
 
-let make_inverse_instance_subst i = 
+let make_inverse_instance_subst i =
   let arr = Instance.to_array i in
     Array.fold_left_i (fun i acc l ->
       LMap.add (Level.var i) l acc)
       LMap.empty arr
 
-let make_abstract_instance (ctx, _) = 
-  Array.mapi (fun i l -> Level.var i) ctx
+let make_abstract_instance (ctx, _) =
+  Array.init (Array.length ctx) (fun i -> Level.var i)
 
-let abstract_universes ctx =
+let abstract_universes nas ctx =
   let instance = UContext.instance ctx in
+  let () = assert (Int.equal (Array.length nas) (Instance.length instance)) in
   let subst = make_instance_subst instance in
-  let cstrs = subst_univs_level_constraints subst 
+  let cstrs = subst_univs_level_constraints subst
       (UContext.constraints ctx)
   in
-  let ctx = UContext.make (instance, cstrs) in
+  let ctx = (nas, cstrs) in
   instance, ctx
 
-let abstract_cumulativity_info (univs, variance) =
-  let subst, univs = abstract_universes univs in
-  subst, (univs, variance)
-
 let rec compact_univ s vars i u =
   match u with
   | [] -> (s, List.rev vars)
@@ -1190,44 +1196,40 @@ let pr_constraints prl = Constraint.pr prl
 
 let pr_universe_context = UContext.pr
 
-let pr_cumulativity_info = CumulativityInfo.pr
-
 let pr_abstract_universe_context = AUContext.pr
 
-let pr_abstract_cumulativity_info = ACumulativityInfo.pr
-
 let pr_universe_context_set = ContextSet.pr
 
-let pr_universe_subst = 
+let pr_universe_subst =
   LMap.pr (fun u -> str" := " ++ Universe.pr u ++ spc ())
 
-let pr_universe_level_subst = 
+let pr_universe_level_subst =
   LMap.pr (fun u -> str" := " ++ Level.pr u ++ spc ())
 
-module Huniverse_set = 
+module Huniverse_set =
   Hashcons.Make(
     struct
       type t = universe_set
       type u = universe_level -> universe_level
       let hashcons huc s =
-	LSet.fold (fun x -> LSet.add (huc x)) s LSet.empty
+        LSet.fold (fun x -> LSet.add (huc x)) s LSet.empty
       let eq s s' =
-	LSet.equal s s'
+        LSet.equal s s'
       let hash = Hashtbl.hash
     end)
 
-let hcons_universe_set = 
+let hcons_universe_set =
   Hashcons.simple_hcons Huniverse_set.generate Huniverse_set.hcons Level.hcons
 
-let hcons_universe_context_set (v, c) = 
+let hcons_universe_context_set (v, c) =
   (hcons_universe_set v, hcons_constraints c)
 
 let hcons_univ x = Universe.hcons x
 
-let explain_universe_inconsistency prl (o,u,v,p) =
+let explain_universe_inconsistency prl (o,u,v,p : univ_inconsistency) =
   let pr_uni = Universe.pr_with prl in
   let pr_rel = function
-    | Eq -> str"=" | Lt -> str"<" | Le -> str"<=" 
+    | Eq -> str"=" | Lt -> str"<" | Le -> str"<="
   in
   let reason = match p with
     | None -> mt()
@@ -1236,14 +1238,10 @@ let explain_universe_inconsistency prl (o,u,v,p) =
       if p = [] then mt ()
       else
         str " because" ++ spc() ++ pr_uni v ++
-	prlist (fun (r,v) -> spc() ++ pr_rel r ++ str" " ++ pr_uni v)
+        prlist (fun (r,v) -> spc() ++ pr_rel r ++ str" " ++ prl v)
           p ++
-	(if Universe.equal (snd (List.last p)) u then mt() else
+        (if Universe.equal (Universe.make (snd (List.last p))) u then mt() else
            (spc() ++ str "= " ++ pr_uni u))
   in
     str "Cannot enforce" ++ spc() ++ pr_uni u ++ spc() ++
       pr_rel o ++ spc() ++ pr_uni v ++ reason
-
-let compare_levels = Level.compare
-let eq_levels = Level.equal
-let equal_universes = Universe.equal
