@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -13,7 +13,7 @@
 (** The route of a generic argument, from parsing to evaluation.
 In the following diagram, "object" can be tactic_expr, constr, tactic_arg, etc.
 
-{% \begin{%}verbatim{% }%}
+{% \begin{verbatim} %}
              parsing          in_raw                            out_raw
    char stream ---> raw_object ---> raw_object generic_argument -------+
                           encapsulation                          decaps|
@@ -36,7 +36,7 @@ In the following diagram, "object" can be tactic_expr, constr, tactic_arg, etc.
     |
     V
 effective use
-{% \end{%}verbatim{% }%}
+{% \end{verbatim} %}
 
 To distinguish between the uninterpreted, globalized and
 interpreted worlds, we annotate the type [generic_argument] by a
