@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -42,9 +42,6 @@ type doc_view =
    internal representation opaque here. *)
 type t = doc_view
 
-type std_ppcmds = t
-[@@ocaml.deprecated "alias of Pp.t"]
-
 let repr x = x
 let unrepr x = x
 
@@ -200,9 +197,9 @@ let pp_with ft pp =
     | Ppcmd_print_break(m,n)  -> pp_print_break ft m n
     | Ppcmd_force_newline     -> pp_force_newline ft ()
     | Ppcmd_comment coms      -> List.iter (pr_com ft) coms
-    | Ppcmd_tag(tag, s)       -> pp_open_tag  ft tag;
+    | Ppcmd_tag(tag, s)       -> pp_open_tag  ft tag [@warning "-3"];
                                  pp_cmd s;
-                                 pp_close_tag ft ()
+                                 pp_close_tag ft () [@warning "-3"]
   in
   try pp_cmd pp
   with reraise ->
@@ -268,7 +265,7 @@ let prlist_sep_lastsep no_empty sep_thunk lastsep_thunk elem l =
     | h::t   -> h ++ sep ++ insert_seps t
   in
   insert_seps filtered_elems
-  
+
 let prlist_strict pr l = prlist_sep_lastsep true mt mt pr l
 (* [prlist_with_sep sep pr [a ; ... ; c]] outputs
    [pr a ++ sep() ++ ... ++ sep() ++ pr c] *)
