@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -18,7 +18,7 @@
 (* to interpret.                                                        *)
 (*                                                                      *)
 (* The datatype has a public view to allow serialization or advanced    *)
-(* uses, however regular users are _strongly_ warned againt its use,    *)
+(* uses, however regular users are _strongly_ warned against its use,   *)
 (* they should instead rely on the available functions below.           *)
 (*                                                                      *)
 (* Box order and number is indeed an important factor. Try to create    *)
@@ -42,9 +42,6 @@ type pp_tag = string
    internal representation opaque here. *)
 type t
 
-type std_ppcmds = t
-[@@ocaml.deprecated "alias of Pp.t"]
-
 type block_type =
   | Pp_hbox   of int
   | Pp_vbox   of int
@@ -152,8 +149,8 @@ val prlist_strict :  ('a -> t) -> 'a list -> t
 val prlist_with_sep :
    (unit -> t) -> ('a -> t) -> 'a list -> t
 (** [prlist_with_sep sep pr [a ; ... ; c]] outputs
-    [pr a ++ sep () ++ ... ++ sep () ++ pr c]. 
-    where the thunk sep is memoized, rather than being called each place 
+    [pr a ++ sep () ++ ... ++ sep () ++ pr c].
+    where the thunk sep is memoized, rather than being called each place
      its result is used.
 *)
 
