@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -102,9 +102,10 @@ type time
 
 val get_time : unit -> time
 val time_difference : time -> time -> float (** in seconds *)
+
 val fmt_time_difference : time -> time -> Pp.t
 
-val with_time : batch:bool -> ('a -> 'b) -> 'a -> 'b
+val with_time : batch:bool -> header:Pp.t -> ('a -> 'b) -> 'a -> 'b
 
 (** [get_toplevel_path program] builds a complete path to the
    executable denoted by [program]. This involves:
@@ -122,4 +123,4 @@ val with_time : batch:bool -> ('a -> 'b) -> 'a -> 'b
  the right name you want you execution to fail rather than fall into
  choosing some random binary from the system-wide installation of
  Coq. *)
-val get_toplevel_path : string -> string
+val get_toplevel_path : ?byte:bool -> string -> string
