@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -8,13 +8,20 @@
 (*         *     (see LICENSE file for the text of the license)         *)
 (************************************************************************)
 
+type 'a pervasives_ref = 'a ref
+let pervasives_ref = ref
+let pervasives_compare = compare
+let (!) = (!)
+let (+) = (+)
+let (-) = (-)
+
 (* Mapping under pairs *)
 
 let on_fst f (a,b) = (f a,b)
 let on_snd f (a,b) = (a,f b)
 let map_pair f (a,b) = (f a,f b)
 
-(* Mapping under pairs *)
+(* Mapping under triplets *)
 
 let on_pi1 f (a,b,c) = (f a,b,c)
 let on_pi2 f (a,b,c) = (a,f b,c)
@@ -90,7 +97,7 @@ let matrix_transpose mat =
 let identity x = x
 
 (** Left-to-right function composition:
-    
+
     [f1 %> f2] is [fun x -> f2 (f1 x)].
 
     [f1 %> f2 %> f3] is [fun x -> f3 (f2 (f1 x))].
