@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -42,23 +42,22 @@ let error_undeclared_key key =
 (****************************************************************************)
 (* 1- Tables                                                                *)
 
-class type ['a] table_of_A =
-object
-  method add : 'a -> unit
-  method remove : 'a -> unit
-  method mem : 'a -> unit
-  method print : unit
-end
+type 'a table_of_A =  {
+  add : Environ.env -> 'a -> unit;
+  remove : Environ.env -> 'a -> unit;
+  mem : Environ.env -> 'a -> unit;
+  print : unit -> unit;
+}
 
 module MakeTable =
   functor
    (A : sig
-	  type t
-	  type key
-	  val compare : t -> t -> int
-	  val table : (string * key table_of_A) list ref
-	  val encode : key -> t
-	  val subst : substitution -> t -> t
+          type t
+          type key
+          module Set : CSig.SetS with type elt = t
+          val table : (string * key table_of_A) list ref
+          val encode : Environ.env -> key -> t
+          val subst : substitution -> t -> t
           val printer : t -> Pp.t
           val key : option_name
           val title : string
@@ -73,9 +72,9 @@ module MakeTable =
 
     let _ =
       if String.List.mem_assoc nick !A.table then
-	user_err Pp.(str "Sorry, this table name is already used.")
+        user_err Pp.(str "Sorry, this table name (" ++ str nick ++ str ") is already used.")
 
-    module MySet = Set.Make (struct type t = A.t let compare = A.compare end)
+    module MySet = A.Set
 
     let t = Summary.ref MySet.empty ~name:nick
 
@@ -84,45 +83,45 @@ module MakeTable =
           | GOadd -> t := MySet.add p !t
           | GOrmv -> t := MySet.remove p !t in
         let load_options i o = if Int.equal i 1 then cache_options o in
-	let subst_options (subst,(f,p as obj)) = 
-	  let p' = A.subst subst p in
-	    if p' == p then obj else
-	      (f,p')
-	in
+        let subst_options (subst,(f,p as obj)) =
+          let p' = A.subst subst p in
+            if p' == p then obj else
+              (f,p')
+        in
         let inGo : option_mark * A.t -> obj =
           Libobject.declare_object {(Libobject.default_object nick) with
                 Libobject.load_function = load_options;
-		Libobject.open_function = load_options;
-		Libobject.cache_function = cache_options;
-		Libobject.subst_function = subst_options;
-		Libobject.classify_function = (fun x -> Substitute x)}
-	in
+                Libobject.open_function = load_options;
+                Libobject.cache_function = cache_options;
+                Libobject.subst_function = subst_options;
+                Libobject.classify_function = (fun x -> Substitute x)}
+        in
         ((fun c -> Lib.add_anonymous_leaf (inGo (GOadd, c))),
          (fun c -> Lib.add_anonymous_leaf (inGo (GOrmv, c))))
 
     let print_table table_name printer table =
       Feedback.msg_notice
         (str table_name ++
-	   (hov 0
-	      (if MySet.is_empty table then str " None" ++ fnl ()
+           (hov 0
+              (if MySet.is_empty table then str " None" ++ fnl ()
                else MySet.fold
-		 (fun a b -> spc () ++ printer a ++ b)
-		 table (mt ()) ++ str "." ++ fnl ())))
+                 (fun a b -> spc () ++ printer a ++ b)
+                 table (mt ()) ++ str "." ++ fnl ())))
 
-    class table_of_A () =
-    object
-      method add x = add_option (A.encode x)
-      method remove x = remove_option (A.encode x)
-      method mem x =
-	let y = A.encode x in
+    let table_of_A = {
+       add = (fun env x -> add_option (A.encode env x));
+       remove = (fun env x -> remove_option (A.encode env x));
+       mem = (fun env x ->
+        let y = A.encode env x in
         let answer = MySet.mem y !t in
-        Feedback.msg_info (A.member_message y answer)
-      method print = print_table A.title A.printer !t
-    end
+        Feedback.msg_info (A.member_message y answer));
+       print = (fun () -> print_table A.title A.printer !t);
+     }
 
-    let _ = A.table := (nick,new table_of_A ())::!A.table
-    let active c = MySet.mem c !t
-    let elements () = MySet.elements !t
+    let _ = A.table := (nick, table_of_A)::!A.table
+
+    let v () = !t
+    let active x = A.Set.mem x !t
   end
 
 let string_table = ref []
@@ -140,9 +139,9 @@ module StringConvert = functor (A : StringConvertArg) ->
 struct
   type t = string
   type key = string
-  let compare = String.compare
+  module Set = CString.Set
   let table = string_table
-  let encode x = x
+  let encode _env x = x
   let subst _ x = x
   let printer = str
   let key = A.key
@@ -160,8 +159,8 @@ let get_ref_table k = String.List.assoc (nickname k) !ref_table
 module type RefConvertArg =
 sig
   type t
-  val compare : t -> t -> int
-  val encode : qualid -> t
+  module Set : CSig.SetS with type elt = t
+  val encode : Environ.env -> qualid -> t
   val subst : substitution -> t -> t
   val printer : t -> Pp.t
   val key : option_name
@@ -173,7 +172,7 @@ module RefConvert = functor (A : RefConvertArg) ->
 struct
   type t = A.t
   type key = qualid
-  let compare = A.compare
+  module Set = A.Set
   let table = ref_table
   let encode = A.encode
   let subst = A.subst
@@ -216,11 +215,11 @@ let get_option key = OptionMap.find key !value_tab
 
 let check_key key = try
   let _ = get_option key in
-  user_err Pp.(str "Sorry, this option name is already used.")
+  user_err Pp.(str "Sorry, this option name ("++ str (nickname key) ++ str ") is already used.")
 with Not_found ->
   if String.List.mem_assoc (nickname key) !string_table
     || String.List.mem_assoc (nickname key) !ref_table
-  then user_err Pp.(str "Sorry, this option name is already used.")
+  then user_err Pp.(str "Sorry, this option name (" ++ str (nickname key) ++ str ") is already used.")
 
 open Libobject
 
@@ -235,7 +234,7 @@ let declare_option cast uncast append ?(preprocess = fun x -> x)
   let default = read() in
   let change =
       let _ = Summary.declare_summary (nickname key)
-        { Summary.freeze_function = (fun _ -> read ());
+        { Summary.freeze_function = (fun ~marshallable -> read ());
           Summary.unfreeze_function = write;
           Summary.init_function = (fun () -> write default) } in
       let cache_options (_,(l,m,v)) =
@@ -246,14 +245,14 @@ let declare_option cast uncast append ?(preprocess = fun x -> x)
       | OptGlobal -> cache_options o
       | OptExport -> ()
       | OptLocal | OptDefault ->
-        (** Ruled out by classify_function *)
+        (* Ruled out by classify_function *)
         assert false
       in
       let open_options i  (_, (l, _, _) as o) = match l with
       | OptExport -> if Int.equal i 1 then cache_options o
       | OptGlobal -> ()
       | OptLocal | OptDefault ->
-        (** Ruled out by classify_function *)
+        (* Ruled out by classify_function *)
         assert false
       in
       let subst_options (subst,obj) = obj in
@@ -276,10 +275,7 @@ let declare_option cast uncast append ?(preprocess = fun x -> x)
   let cread () = cast (read ()) in
   let cwrite l v = warn (); change l OptSet (uncast v) in
   let cappend l v = warn (); change l OptAppend (uncast v) in
-  value_tab := OptionMap.add key (name, depr, (cread,cwrite,cappend)) !value_tab;
-  write
-
-type 'a write_function = 'a -> unit
+  value_tab := OptionMap.add key (name, depr, (cread,cwrite,cappend)) !value_tab
 
 let declare_int_option =
   declare_option
@@ -302,6 +298,18 @@ let declare_stringopt_option =
     (function StringOptValue v -> v | _ -> anomaly (Pp.str "async_option."))
     (fun _ _ -> anomaly (Pp.str "async_option."))
 
+let declare_bool_option_and_ref ~depr ~name ~key ~(value:bool) =
+  let r_opt = ref value in
+  let optwrite v = r_opt := v in
+  let optread () = !r_opt in
+  let _ = declare_bool_option {
+      optdepr = depr;
+      optname = name;
+      optkey = key;
+      optread; optwrite
+    } in
+  optread
+
 (* 3- User accessible commands *)
 
 (* Setting values of options *)
@@ -390,9 +398,9 @@ let print_option_value key =
   let s = read () in
   match s with
     | BoolValue b ->
-	Feedback.msg_info (str "The " ++ str name ++ str " mode is " ++ str (if b then "on" else "off"))
+        Feedback.msg_notice (str "The " ++ str name ++ str " mode is " ++ str (if b then "on" else "off"))
     | _ ->
-	Feedback.msg_info (str "Current value of " ++ str name ++ str " is " ++ msg_option_value (name, s))
+        Feedback.msg_notice (str "Current value of " ++ str name ++ str " is " ++ msg_option_value (name, s))
 
 let get_tables () =
   let tables = !value_tab in
@@ -425,6 +433,3 @@ let print_tables () =
       (fun (nickkey,_) p -> p ++ str "  " ++ str nickkey ++ fnl ())
       !ref_table (mt ()) ++
     fnl ()
-
-
-
