@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -15,6 +15,31 @@ open Names
 open Libnames
 open Globnames
 
+type object_prefix = {
+  obj_dir : DirPath.t;
+  obj_mp  : ModPath.t;
+}
+
+let eq_op op1 op2 =
+  DirPath.equal op1.obj_dir op2.obj_dir &&
+  ModPath.equal op1.obj_mp  op2.obj_mp
+
+(* to this type are mapped DirPath.t's in the nametab *)
+module GlobDirRef = struct
+  type t =
+    | DirOpenModule of object_prefix
+    | DirOpenModtype of object_prefix
+    | DirOpenSection of object_prefix
+    | DirModule of object_prefix
+
+  let equal r1 r2 = match r1, r2 with
+    | DirOpenModule op1, DirOpenModule op2 -> eq_op op1 op2
+    | DirOpenModtype op1, DirOpenModtype op2 -> eq_op op1 op2
+    | DirOpenSection op1, DirOpenSection op2 -> eq_op op1 op2
+    | DirModule op1, DirModule op2 -> eq_op op1 op2
+    | _ -> false
+
+end
 
 exception GlobalizationError of qualid
 
@@ -30,6 +55,9 @@ let error_global_not_found qid =
 *)
 type visibility = Until of int | Exactly of int
 
+let map_visibility f = function
+  | Until i -> Until (f i)
+  | Exactly i -> Exactly (f i)
 
 (* Data structure for nametabs *******************************************)
 
@@ -71,6 +99,9 @@ module type NAMETREE = sig
   val user_name : qualid -> t -> user_name
   val shortest_qualid : ?loc:Loc.t -> Id.Set.t -> user_name -> t -> qualid
   val find_prefixes : qualid -> t -> elt list
+
+  (** Matches a prefix of [qualid], useful for completion *)
+  val match_prefixes : qualid -> t -> elt list
 end
 
 module Make (U : UserName) (E : EqualityType) : NAMETREE
@@ -97,7 +128,7 @@ struct
   (* Dictionaries of short names *)
   type nametree =
       { path : path_status;
-	map : nametree ModIdmap.t }
+        map : nametree ModIdmap.t }
 
   let mktree p m = { path=p; map=m }
   let empty_tree = mktree Nothing ModIdmap.empty
@@ -118,34 +149,34 @@ struct
             let ptab = modify () empty_tree in
             ModIdmap.add modid ptab tree.map
         in
-	let this =
+        let this =
           if level <= 0 then
-	    match tree.path with
-	      | Absolute (n,_) ->
-		  (* This is an absolute name, we must keep it
-		     otherwise it may become unaccessible forever *)
+            match tree.path with
+              | Absolute (n,_) ->
+                  (* This is an absolute name, we must keep it
+                     otherwise it may become unaccessible forever *)
                 warn_masking_absolute n; tree.path
-	      | Nothing
-	      | Relative _ -> Relative (uname,o)
+              | Nothing
+              | Relative _ -> Relative (uname,o)
           else tree.path
-	in
-	mktree this map
+        in
+        mktree this map
     | [] ->
-	match tree.path with
-	  | Absolute (uname',o') ->
-	      if E.equal o' o then begin
-		assert (U.equal uname uname');
-		tree
-		  (* we are putting the same thing for the second time :) *)
-	      end
-	      else
-		(* This is an absolute name, we must keep it otherwise it may
-		   become unaccessible forever *)
-		(* But ours is also absolute! This is an error! *)
-		user_err Pp.(str @@ "Cannot mask the absolute name \""
+        match tree.path with
+          | Absolute (uname',o') ->
+              if E.equal o' o then begin
+                assert (U.equal uname uname');
+                tree
+                  (* we are putting the same thing for the second time :) *)
+              end
+              else
+                (* This is an absolute name, we must keep it otherwise it may
+                   become unaccessible forever *)
+                (* But ours is also absolute! This is an error! *)
+                user_err Pp.(str @@ "Cannot mask the absolute name \""
                                    ^ U.to_string uname' ^ "\"!")
-	  | Nothing
-	  | Relative _ -> mktree (Absolute (uname,o)) tree.map
+          | Nothing
+          | Relative _ -> mktree (Absolute (uname,o)) tree.map
 
 let rec push_exactly uname o level tree = function
 | [] ->
@@ -210,7 +241,7 @@ let user_name qid tab =
 let find uname tab =
   let id,l = U.repr uname in
     match search (Id.Map.find id tab) l with
-	Absolute (_,o) -> o
+        Absolute (_,o) -> o
       | _ -> raise Not_found
 
 let exists uname tab =
@@ -229,9 +260,9 @@ let shortest_qualid ?loc ctx uname tab =
     | Absolute (u,_) | Relative (u,_)
           when U.equal u uname && not (is_empty && hidden) -> List.rev pos
     | _ ->
-	match dir with
-	    [] -> raise Not_found
-	  | id::dir -> find_uname (id::pos) dir (ModIdmap.find id tree.map)
+        match dir with
+            [] -> raise Not_found
+          | id::dir -> find_uname (id::pos) dir (ModIdmap.find id tree.map)
   in
   let ptab = Id.Map.find id tab in
   let found_dir = find_uname [] dir ptab in
@@ -256,10 +287,20 @@ let find_prefixes qid tab =
     search_prefixes (Id.Map.find id tab) (DirPath.repr dir)
   with Not_found -> []
 
+let match_prefixes =
+  let cprefix x y = CString.(compare x (sub y 0 (min (length x) (length y)))) in
+  fun qid tab ->
+    try
+      let (dir,id) = repr_qualid qid in
+      let id_prefix = cprefix Id.(to_string id) in
+      let matches = Id.Map.filter_range (fun x -> id_prefix Id.(to_string x)) tab in
+      let matches = Id.Map.mapi (fun _key tab -> search_prefixes tab (DirPath.repr dir)) matches in
+      (* Coq's flatten is "magical", so this is not so bad perf-wise *)
+      CList.flatten @@ Id.Map.(fold (fun _ r l -> r :: l) matches [])
+    with Not_found -> []
+
 end
 
-
-
 (* Global name tables *************************************************)
 
 module FullPath =
@@ -279,10 +320,10 @@ module ExtRefTab = Make(FullPath)(ExtRefEqual)
 module MPTab = Make(FullPath)(MPEqual)
 
 type ccitab = ExtRefTab.t
-let the_ccitab = ref (ExtRefTab.empty : ccitab)
+let the_ccitab = Summary.ref ~name:"ccitab" (ExtRefTab.empty : ccitab)
 
 type mptab = MPTab.t
-let the_modtypetab = ref (MPTab.empty : mptab)
+let the_modtypetab = Summary.ref ~name:"modtypetab" (MPTab.empty : mptab)
 
 module DirPath' =
 struct
@@ -292,29 +333,21 @@ struct
     | id :: l -> (id, l)
 end
 
-module GlobDir =
-struct
-  type t = global_dir_reference
-  let equal = eq_global_dir_reference
-end
-
-module DirTab = Make(DirPath')(GlobDir)
+module DirTab = Make(DirPath')(GlobDirRef)
 
 (* If we have a (closed) module M having a submodule N, than N does not
    have the entry in [the_dirtab]. *)
 type dirtab = DirTab.t
-let the_dirtab = ref (DirTab.empty : dirtab)
-
-type universe_id = DirPath.t * int
+let the_dirtab = Summary.ref ~name:"dirtab" (DirTab.empty : dirtab)
 
 module UnivIdEqual =
 struct
-  type t = universe_id
-  let equal (d, i) (d', i') = DirPath.equal d d' && Int.equal i i'
+  type t = Univ.Level.UGlobal.t
+  let equal = Univ.Level.UGlobal.equal
 end
 module UnivTab = Make(FullPath)(UnivIdEqual)
 type univtab = UnivTab.t
-let the_univtab = ref (UnivTab.empty : univtab)
+let the_univtab = Summary.ref ~name:"univtab" (UnivTab.empty : univtab)
 
 (* Reversed name tables ***************************************************)
 
@@ -322,29 +355,26 @@ let the_univtab = ref (UnivTab.empty : univtab)
 module Globrevtab = HMap.Make(ExtRefOrdered)
 
 type globrevtab = full_path Globrevtab.t
-let the_globrevtab = ref (Globrevtab.empty : globrevtab)
+let the_globrevtab = Summary.ref ~name:"globrevtab" (Globrevtab.empty : globrevtab)
 
 
 type mprevtab = DirPath.t MPmap.t
-let the_modrevtab = ref (MPmap.empty : mprevtab)
+let the_modrevtab = Summary.ref ~name:"modrevtab" (MPmap.empty : mprevtab)
 
 type mptrevtab = full_path MPmap.t
-let the_modtyperevtab = ref (MPmap.empty : mptrevtab)
+let the_modtyperevtab = Summary.ref ~name:"modtyperevtab" (MPmap.empty : mptrevtab)
 
 module UnivIdOrdered =
 struct
-  type t = universe_id
-  let hash (d, i) = i + DirPath.hash d
-  let compare (d, i) (d', i') =
-    let c = Int.compare i i' in
-    if Int.equal c 0 then DirPath.compare d d'
-    else c
+  type t = Univ.Level.UGlobal.t
+  let hash = Univ.Level.UGlobal.hash
+  let compare = Univ.Level.UGlobal.compare
 end
 
 module UnivIdMap = HMap.Make(UnivIdOrdered)
 
 type univrevtab = full_path UnivIdMap.t
-let the_univrevtab = ref (UnivIdMap.empty : univrevtab)
+let the_univrevtab = Summary.ref ~name:"univrevtab" (UnivIdMap.empty : univrevtab)
 
 (* Push functions *********************************************************)
 
@@ -355,24 +385,25 @@ let the_univrevtab = ref (UnivIdMap.empty : univrevtab)
 let push_xref visibility sp xref =
   match visibility with
     | Until _ ->
-	the_ccitab := ExtRefTab.push visibility sp xref !the_ccitab;
-	the_globrevtab := Globrevtab.add xref sp !the_globrevtab
+        the_ccitab := ExtRefTab.push visibility sp xref !the_ccitab;
+        the_globrevtab := Globrevtab.add xref sp !the_globrevtab
     | _ ->
-	begin
-	  if ExtRefTab.exists sp !the_ccitab then
-	    match ExtRefTab.find sp !the_ccitab with
-	      | TrueGlobal( ConstRef _) | TrueGlobal( IndRef _) |
-		    TrueGlobal( ConstructRef _) as xref ->
-		  the_ccitab := ExtRefTab.push visibility sp xref !the_ccitab;
-	      | _ -> 
-		  the_ccitab := ExtRefTab.push visibility sp xref !the_ccitab;
-	    else
-	      the_ccitab := ExtRefTab.push visibility sp xref !the_ccitab;
-	end
+        begin
+          if ExtRefTab.exists sp !the_ccitab then
+            let open GlobRef in
+            match ExtRefTab.find sp !the_ccitab with
+              | TrueGlobal( ConstRef _) | TrueGlobal( IndRef _) |
+                    TrueGlobal( ConstructRef _) as xref ->
+                  the_ccitab := ExtRefTab.push visibility sp xref !the_ccitab;
+              | _ ->
+                  the_ccitab := ExtRefTab.push visibility sp xref !the_ccitab;
+            else
+              the_ccitab := ExtRefTab.push visibility sp xref !the_ccitab;
+        end
 
 let push_cci visibility sp ref =
   push_xref visibility sp (TrueGlobal ref)
-    
+
 (* This is for Syntactic Definitions *)
 let push_syndef visibility sp kn =
   push_xref visibility sp (SynDef kn)
@@ -387,7 +418,7 @@ let push_modtype vis sp kn =
 let push_dir vis dir dir_ref =
   the_dirtab := DirTab.push vis dir dir_ref !the_dirtab;
   match dir_ref with
-  | DirModule { obj_mp; _ } -> the_modrevtab := MPmap.add obj_mp dir !the_modrevtab
+  | GlobDirRef.DirModule { obj_mp; _ } -> the_modrevtab := MPmap.add obj_mp dir !the_modrevtab
   | _ -> ()
 
 (* This is for global universe names *)
@@ -421,17 +452,17 @@ let locate_dir qid = DirTab.locate qid !the_dirtab
 
 let locate_module qid =
   match locate_dir qid with
-    | DirModule { obj_mp ; _} -> obj_mp
+    | GlobDirRef.DirModule { obj_mp ; _} -> obj_mp
     | _ -> raise Not_found
 
 let full_name_module qid =
   match locate_dir qid with
-    | DirModule { obj_dir ; _} -> obj_dir
+    | GlobDirRef.DirModule { obj_dir ; _} -> obj_dir
     | _ -> raise Not_found
 
 let locate_section qid =
   match locate_dir qid with
-    | DirOpenSection { obj_dir; _ } -> obj_dir
+    | GlobDirRef.DirOpenSection { obj_dir; _ } -> obj_dir
     | _ -> raise Not_found
 
 let locate_all qid =
@@ -444,9 +475,14 @@ let locate_extended_all_dir qid = DirTab.find_prefixes qid !the_dirtab
 
 let locate_extended_all_modtype qid = MPTab.find_prefixes qid !the_modtypetab
 
+(* Completion *)
+let completion_canditates qualid =
+  ExtRefTab.match_prefixes qualid !the_ccitab
+
 (* Derived functions *)
 
 let locate_constant qid =
+  let open GlobRef in
   match locate_extended qid with
     | TrueGlobal (ConstRef kn) -> kn
     | _ -> raise Not_found
@@ -474,10 +510,6 @@ let exists_cci sp = ExtRefTab.exists sp !the_ccitab
 
 let exists_dir dir = DirTab.exists dir !the_dirtab
 
-let exists_section = exists_dir
-
-let exists_module = exists_dir
-
 let exists_modtype sp = MPTab.exists sp !the_modtypetab
 
 let exists_universe kn = UnivTab.exists kn !the_univtab
@@ -485,6 +517,7 @@ let exists_universe kn = UnivTab.exists kn !the_univtab
 (* Reverse locate functions ***********************************************)
 
 let path_of_global ref =
+  let open GlobRef in
   match ref with
     | VarRef id -> make_path DirPath.empty id
     | _ -> Globrevtab.find (TrueGlobal ref) !the_globrevtab
@@ -510,6 +543,7 @@ let path_of_universe mp =
 (* Shortest qualid functions **********************************************)
 
 let shortest_qualid_of_global ?loc ctx ref =
+  let open GlobRef in
   match ref with
     | VarRef id -> make_qualid ?loc DirPath.empty id
     | _ ->
@@ -538,47 +572,9 @@ let pr_global_env env ref =
     if !Flags.debug then Feedback.msg_debug (Pp.str "pr_global_env not found"); raise e
 
 let global_inductive qid =
+  let open GlobRef in
   match global qid with
   | IndRef ind -> ind
   | ref ->
       user_err ?loc:qid.CAst.loc ~hdr:"global_inductive"
         (pr_qualid qid ++ spc () ++ str "is not an inductive type")
-
-(********************************************************************)
-
-(********************************************************************)
-(* Registration of tables as a global table and rollback            *)
-
-type frozen = ccitab * dirtab * mptab * univtab
-    * globrevtab * mprevtab * mptrevtab * univrevtab
-
-let freeze _ : frozen =
-  !the_ccitab,
-  !the_dirtab,
-  !the_modtypetab,
-  !the_univtab,
-  !the_globrevtab,
-  !the_modrevtab,
-  !the_modtyperevtab,
-  !the_univrevtab
-
-let unfreeze (ccit,dirt,mtyt,univt,globr,modr,mtyr,univr) =
-  the_ccitab := ccit;
-  the_dirtab := dirt;
-  the_modtypetab := mtyt;
-  the_univtab := univt;
-  the_globrevtab := globr;
-  the_modrevtab := modr;
-  the_modtyperevtab := mtyr;
-  the_univrevtab := univr
-
-let _ =
-  Summary.declare_summary "names"
-    { Summary.freeze_function = freeze;
-      Summary.unfreeze_function = unfreeze;
-      Summary.init_function = Summary.nop }
-
-(* Deprecated synonyms *)
-
-let extended_locate = locate_extended
-let absolute_reference = global_of_path
