@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -15,15 +15,14 @@
   freezing the states of both [Lib] and [Summary]. We provide functions
   to write and restore state to and from a given file. *)
 
-val intern_state : string -> unit
-val extern_state : string -> unit
-
 type state
-val freeze : marshallable:Summary.marshallable -> state
+val freeze : marshallable:bool -> state
 val unfreeze : state -> unit
 
 val summary_of_state : state -> Summary.frozen
+val lib_of_state : state -> Lib.frozen
 val replace_summary : state -> Summary.frozen -> state
+val replace_lib : state -> Lib.frozen -> state
 
 (** {6 Rollback } *)
 
