@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -10,19 +10,7 @@
 
 open Constr
 
-let contrib_name = "btauto"
-
-let init_constant dir s =
-  let find_constant contrib dir s =
-    UnivGen.constr_of_global (Coqlib.find_reference contrib dir s)
-  in
-  find_constant contrib_name dir s
-
-let get_constant dir s = lazy (UnivGen.constr_of_global @@ Coqlib.coq_reference contrib_name dir s)
-
-let get_inductive dir s =
-  let glob_ref () = Coqlib.find_reference contrib_name ("Coq" :: dir) s in
-  Lazy.from_fun (fun () -> Globnames.destIndRef (glob_ref ()))
+let bt_lib_constr n = lazy (UnivGen.constr_of_monomorphic_global @@ Coqlib.lib_ref n)
 
 let decomp_term sigma (c : Constr.t) =
   Constr.kind (EConstr.Unsafe.to_constr (Termops.strip_outer_cast sigma (EConstr.of_constr c)))
@@ -31,27 +19,23 @@ let lapp c v  = Constr.mkApp (Lazy.force c, v)
 
 let (===) = Constr.equal
 
+
 module CoqList = struct
-  let path = ["Init"; "Datatypes"]
-  let typ = get_constant path "list"
-  let _nil = get_constant path "nil"
-  let _cons = get_constant path "cons"
+  let _nil =  bt_lib_constr "core.list.nil"
+  let _cons = bt_lib_constr "core.list.cons"
 
   let cons ty h t = lapp _cons [|ty; h ; t|]
   let nil ty = lapp _nil [|ty|]
   let rec of_list ty = function
     | [] -> nil ty
     | t::q -> cons ty t (of_list ty q)
-  let type_of_list ty = lapp typ [|ty|]
 
 end
 
 module CoqPositive = struct
-  let path = ["Numbers"; "BinNums"]
-  let typ = get_constant path "positive"
-  let _xH = get_constant path "xH"
-  let _xO = get_constant path "xO"
-  let _xI = get_constant path "xI"
+  let _xH = bt_lib_constr "num.pos.xH"
+  let _xO = bt_lib_constr "num.pos.xO"
+  let _xI = bt_lib_constr "num.pos.xI"
 
   (* A coq nat from an int *)
   let rec of_int n =
@@ -70,10 +54,10 @@ module Env = struct
   type t = (int ConstrHashtbl.t * int ref)
 
   let add (tbl, off) (t : Constr.t) =
-    try ConstrHashtbl.find tbl t 
+    try ConstrHashtbl.find tbl t
     with
-    | Not_found -> 
-      let i = !off in 
+    | Not_found ->
+      let i = !off in
       let () = ConstrHashtbl.add tbl t i in
       let () = incr off in
       i
@@ -91,14 +75,14 @@ end
 
 module Bool = struct
 
-  let typ = get_constant ["Init"; "Datatypes"] "bool"
-  let ind = get_inductive ["Init"; "Datatypes"] "bool"
-  let trueb = get_constant ["Init"; "Datatypes"] "true"
-  let falseb = get_constant ["Init"; "Datatypes"] "false"
-  let andb = get_constant ["Init"; "Datatypes"] "andb"
-  let orb = get_constant ["Init"; "Datatypes"] "orb"
-  let xorb = get_constant ["Init"; "Datatypes"] "xorb"
-  let negb = get_constant ["Init"; "Datatypes"] "negb"
+  let ind    = lazy (Globnames.destIndRef (Coqlib.lib_ref "core.bool.type"))
+  let typ    = bt_lib_constr "core.bool.type"
+  let trueb  = bt_lib_constr "core.bool.true"
+  let falseb = bt_lib_constr "core.bool.false"
+  let andb   = bt_lib_constr "core.bool.andb"
+  let orb    = bt_lib_constr "core.bool.orb"
+  let xorb   = bt_lib_constr "core.bool.xorb"
+  let negb   = bt_lib_constr "core.bool.negb"
 
   type t =
   | Var of int
@@ -150,21 +134,20 @@ module Btauto = struct
 
   open Pp
 
-  let eq = get_constant ["Init"; "Logic"]  "eq"
+  let eq = bt_lib_constr "core.eq.type"
 
-  let f_var = get_constant ["btauto"; "Reflect"] "formula_var"
-  let f_btm = get_constant ["btauto"; "Reflect"] "formula_btm"
-  let f_top = get_constant ["btauto"; "Reflect"] "formula_top"
-  let f_cnj = get_constant ["btauto"; "Reflect"] "formula_cnj"
-  let f_dsj = get_constant ["btauto"; "Reflect"] "formula_dsj"
-  let f_neg = get_constant ["btauto"; "Reflect"] "formula_neg"
-  let f_xor = get_constant ["btauto"; "Reflect"] "formula_xor"
-  let f_ifb = get_constant ["btauto"; "Reflect"] "formula_ifb"
+  let f_var = bt_lib_constr "plugins.btauto.f_var"
+  let f_btm = bt_lib_constr "plugins.btauto.f_btm"
+  let f_top = bt_lib_constr "plugins.btauto.f_top"
+  let f_cnj = bt_lib_constr "plugins.btauto.f_cnj"
+  let f_dsj = bt_lib_constr "plugins.btauto.f_dsj"
+  let f_neg = bt_lib_constr "plugins.btauto.f_neg"
+  let f_xor = bt_lib_constr "plugins.btauto.f_xor"
+  let f_ifb = bt_lib_constr "plugins.btauto.f_ifb"
 
-  let eval = get_constant ["btauto"; "Reflect"] "formula_eval"
-  let witness = get_constant ["btauto"; "Reflect"] "boolean_witness"
-
-  let soundness = get_constant ["btauto"; "Reflect"] "reduce_poly_of_formula_sound_alt"
+  let eval = bt_lib_constr "plugins.btauto.eval"
+  let witness = bt_lib_constr "plugins.btauto.witness"
+  let soundness = bt_lib_constr "plugins.btauto.soundness"
 
   let rec convert = function
   | Bool.Var n -> lapp f_var [|CoqPositive.of_int n|]
@@ -176,16 +159,17 @@ module Btauto = struct
   | Bool.Xorb (b1, b2) -> lapp f_xor [|convert b1; convert b2|]
   | Bool.Ifb (b1, b2, b3) -> lapp f_ifb [|convert b1; convert b2; convert b3|]
 
-  let convert_env env : Constr.t = 
+  let convert_env env : Constr.t =
     CoqList.of_list (Lazy.force Bool.typ) env
 
   let reify env t = lapp eval [|convert_env env; convert t|]
 
-  let print_counterexample p env gl =
+  let print_counterexample p penv gl =
     let var = lapp witness [|p|] in
     let var = EConstr.of_constr var in
     (* Compute an assignment that dissatisfies the goal *)
-    let _, var = Tacmach.pf_reduction_of_red_expr gl (Genredexpr.CbvVm None) var in
+    let redfun, _ = Redexpr.reduction_of_red_expr (Refiner.pf_env gl) Genredexpr.(CbvVm None) in
+    let _, var = redfun Refiner.(pf_env gl) Refiner.(project gl) var in
     let var = EConstr.Unsafe.to_constr var in
     let rec to_list l = match decomp_term (Tacmach.project gl) l with
     | App (c, _)
@@ -209,10 +193,10 @@ module Btauto = struct
     let msg =
       try
         let var = to_list var in
-        let assign = List.combine env var in
+        let assign = List.combine penv var in
         let map_msg (key, v) =
           let b = if v then str "true" else str "false" in
-          let sigma, env = Pfedit.get_current_context () in
+          let sigma, env = Tacmach.project gl, Tacmach.pf_env gl in
           let term = Printer.pr_constr_env env sigma key in
           term ++ spc () ++ str ":=" ++ spc () ++ b
         in
@@ -224,7 +208,7 @@ module Btauto = struct
     Tacticals.tclFAIL 0 msg gl
 
   let try_unification env =
-    Proofview.Goal.nf_enter begin fun gl ->
+    Proofview.Goal.enter begin fun gl ->
       let concl = Proofview.Goal.concl gl in
       let eq = Lazy.force eq in
       let concl = EConstr.Unsafe.to_constr concl in
@@ -240,7 +224,7 @@ module Btauto = struct
     end
 
   let tac =
-    Proofview.Goal.nf_enter begin fun gl ->
+    Proofview.Goal.enter begin fun gl ->
       let concl = Proofview.Goal.concl gl in
       let concl = EConstr.Unsafe.to_constr concl in
       let sigma = Tacmach.New.project gl in
