@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -15,7 +15,6 @@ open ModPath
 open Namegen
 open Nameops
 open Libnames
-open Globnames
 open Table
 open Miniml
 open Mlutil
@@ -112,17 +111,12 @@ let pseudo_qualify = qualify "__"
 let is_upper s = match s.[0] with 'A' .. 'Z' -> true | _ -> false
 let is_lower s = match s.[0] with 'a' .. 'z' | '_' -> true | _ -> false
 
-[@@@ocaml.warning "-3"]       (* String.(un)capitalize_ascii since 4.03.0 GPR#124 *)
-let capitalize = String.capitalize
-let uncapitalize = String.uncapitalize
-[@@@ocaml.warning "+3"]
-
-let lowercase_id id = Id.of_string (uncapitalize (ascii_of_id id))
+let lowercase_id id = Id.of_string (String.uncapitalize_ascii (ascii_of_id id))
 let uppercase_id id =
   let s = ascii_of_id id in
   assert (not (String.is_empty s));
   if s.[0] == '_' then Id.of_string ("Coq_"^s)
-  else Id.of_string (capitalize s)
+  else Id.of_string (String.capitalize_ascii s)
 
 type kind = Term | Type | Cons | Mod
 
@@ -130,7 +124,7 @@ module KOrd =
 struct
   type t = kind * string
   let compare (k1, s1) (k2, s2) =
-    let c = Pervasives.compare k1 k2 (** OK *) in
+    let c = pervasives_compare k1 k2 (* OK *) in
     if c = 0 then String.compare s1 s2
     else c
 end
@@ -170,9 +164,9 @@ let rename_tvars avoid l =
   let rec rename avoid = function
     | [] -> [],avoid
     | id :: idl ->
-	let id = rename_id (lowercase_id id) avoid in
-	let idl, avoid = rename (Id.Set.add id avoid) idl in
-	(id :: idl, avoid) in
+        let id = rename_id (lowercase_id id) avoid in
+        let idl, avoid = rename (Id.Set.add id avoid) idl in
+        (id :: idl, avoid) in
   fst (rename avoid l)
 
 let push_vars ids (db,avoid) =
@@ -277,8 +271,8 @@ let params_ren_add, params_ren_mem =
 *)
 
 type visible_layer = { mp : ModPath.t;
-		       params : ModPath.t list;
-		       mutable content : Label.t KMap.t; }
+                       params : ModPath.t list;
+                       mutable content : Label.t KMap.t; }
 
 let pop_visible, push_visible, get_visible =
   let vis = ref [] in
@@ -287,10 +281,10 @@ let pop_visible, push_visible, get_visible =
     match !vis with
       | [] -> assert false
       | v :: vl ->
-	  vis := vl;
-	  (* we save the 1st-level-content of MPfile for later use *)
-	  if get_phase () == Impl && modular () && is_modfile v.mp
-	  then add_mpfiles_content v.mp v.content
+          vis := vl;
+          (* we save the 1st-level-content of MPfile for later use *)
+          if get_phase () == Impl && modular () && is_modfile v.mp
+          then add_mpfiles_content v.mp v.content
   and push mp mps =
     vis := { mp = mp; params = mps; content = KMap.empty } :: !vis
   and get () = !vis
@@ -362,9 +356,9 @@ let modfstlev_rename =
     with Not_found ->
       let s = ascii_of_id id in
       if is_lower s || begins_with_CoqXX s then
-	(add_index id 1; "Coq_"^s)
+        (add_index id 1; "Coq_"^s)
       else
-	(add_index id 0; s)
+        (add_index id 0; s)
 
 (*s Creating renaming for a [module_path] : first, the real function ... *)
 
@@ -454,13 +448,13 @@ let visible_clash mp0 ks =
     | [] -> false
     | v :: _ when ModPath.equal v.mp mp0 -> false
     | v :: vis ->
-	let b = KMap.mem ks v.content in
-	if b && not (is_mp_bound mp0) then true
-	else begin
-	  if b then params_ren_add mp0;
-	  if params_lookup mp0 ks v.params then false
-	  else clash vis
-	end
+        let b = KMap.mem ks v.content in
+        if b && not (is_mp_bound mp0) then true
+        else begin
+          if b then params_ren_add mp0;
+          if params_lookup mp0 ks v.params then false
+          else clash vis
+        end
   in clash (get_visible ())
 
 (* Same, but with verbose output (and mp0 shouldn't be a MPbound) *)
@@ -470,10 +464,10 @@ let visible_clash_dbg mp0 ks =
     | [] -> None
     | v :: _ when ModPath.equal v.mp mp0 -> None
     | v :: vis ->
-	try Some (v.mp,KMap.find ks v.content)
-	with Not_found ->
-	  if params_lookup mp0 ks v.params then None
-	  else clash vis
+        try Some (v.mp,KMap.find ks v.content)
+        with Not_found ->
+          if params_lookup mp0 ks v.params then None
+          else clash vis
   in clash (get_visible ())
 
 (* After the 1st pass, we can decide which modules will be opened initially *)
@@ -489,9 +483,9 @@ let opened_libraries () =
        after such an open, there's no unambiguous way to refer to objects of B. *)
     let to_open =
       List.filter
-	(fun mp ->
-	   not (List.exists (fun k -> KMap.mem k (get_mpfiles_content mp)) used_ks))
-	used_files
+        (fun mp ->
+           not (List.exists (fun k -> KMap.mem k (get_mpfiles_content mp)) used_ks))
+        used_files
     in
     mpfiles_clear ();
     List.iter mpfiles_add to_open;
@@ -555,18 +549,18 @@ let pp_ocaml_extern k base rls = match rls with
   | [] -> assert false
   | base_s :: rls' ->
       if (not (modular ())) (* Pseudo qualification with "" *)
-	|| (List.is_empty rls')  (* Case of a file A.v used as a module later *)
-	|| (not (mpfiles_mem base)) (* Module not opened *)
-	|| (mpfiles_clash base (fstlev_ks k rls')) (* Conflict in opened files *)
-	|| (visible_clash base (fstlev_ks k rls')) (* Local conflict *)
+        || (List.is_empty rls')  (* Case of a file A.v used as a module later *)
+        || (not (mpfiles_mem base)) (* Module not opened *)
+        || (mpfiles_clash base (fstlev_ks k rls')) (* Conflict in opened files *)
+        || (visible_clash base (fstlev_ks k rls')) (* Local conflict *)
       then
-	(* We need to fully qualify. Last clash situation is unsupported *)
-	match visible_clash_dbg base (Mod,base_s) with
-	  | None -> dottify rls
-	  | Some (mp,l) -> error_module_clash base (MPdot (mp,l))
+        (* We need to fully qualify. Last clash situation is unsupported *)
+        match visible_clash_dbg base (Mod,base_s) with
+          | None -> dottify rls
+          | Some (mp,l) -> error_module_clash base (MPdot (mp,l))
       else
-	(* Standard situation : object in an opened file *)
-	dottify rls'
+        (* Standard situation : object in an opened file *)
+        dottify rls'
 
 (* [pp_ocaml_gen] : choosing between [pp_ocaml_local] or [pp_ocaml_extern] *)
 
@@ -574,11 +568,11 @@ let pp_ocaml_gen k mp rls olab =
   match common_prefix_from_list mp (get_visible_mps ()) with
     | Some prefix -> pp_ocaml_local k prefix mp rls olab
     | None ->
-	let base = base_mp mp in
-	if is_mp_bound base then pp_ocaml_bound base rls
-	else pp_ocaml_extern k base rls
+        let base = base_mp mp in
+        if is_mp_bound base then pp_ocaml_bound base rls
+        else pp_ocaml_extern k base rls
 
-(* For Haskell, things are simplier: we have removed (almost) all structures *)
+(* For Haskell, things are simpler: we have removed (almost) all structures *)
 
 let pp_haskell_gen k mp rls = match rls with
   | [] -> assert false
@@ -593,9 +587,9 @@ let pp_global k r =
   let ls = ref_renaming (k,r) in
   assert (List.length ls > 1);
   let s = List.hd ls in
-  let mp,_,l = repr_of_r r in
+  let mp,l = repr_of_r r in
   if ModPath.equal mp (top_visible_mp ()) then
-    (* simpliest situation: definition of r (or use in the same context) *)
+    (* simplest situation: definition of r (or use in the same context) *)
     (* we update the visible environment *)
     (add_visible (k,s) l; unquote s)
   else
@@ -612,10 +606,10 @@ let pp_module mp =
   let ls = mp_renaming mp in
   match mp with
     | MPdot (mp0,l) when ModPath.equal mp0 (top_visible_mp ()) ->
-	(* simpliest situation: definition of mp (or use in the same context) *)
-	(* we update the visible environment *)
-	let s = List.hd ls in
-	add_visible (Mod,s) l; s
+        (* simplest situation: definition of mp (or use in the same context) *)
+        (* we update the visible environment *)
+        let s = List.hd ls in
+        add_visible (Mod,s) l; s
     | _ -> pp_ocaml_gen Mod mp (List.rev ls) None
 
 (** Special hack for constants of type Ascii.ascii : if an
@@ -634,21 +628,21 @@ let check_extract_ascii () =
       | Haskell -> "Prelude.Char"
       | _ -> raise Not_found
     in
-    String.equal (find_custom (IndRef (ind_ascii, 0))) (char_type)
+    String.equal (find_custom (GlobRef.IndRef (ind_ascii, 0))) (char_type)
   with Not_found -> false
 
 let is_list_cons l =
- List.for_all (function MLcons (_,ConstructRef(_,_),[]) -> true | _ -> false) l
+ List.for_all (function MLcons (_,GlobRef.ConstructRef(_,_),[]) -> true | _ -> false) l
 
 let is_native_char = function
-  | MLcons(_,ConstructRef ((kn,0),1),l) ->
+  | MLcons(_,GlobRef.ConstructRef ((kn,0),1),l) ->
     MutInd.equal kn ind_ascii && check_extract_ascii () && is_list_cons l
   | _ -> false
 
 let get_native_char c =
   let rec cumul = function
     | [] -> 0
-    | MLcons(_,ConstructRef(_,j),[])::l -> (2-j) + 2 * (cumul l)
+    | MLcons(_,GlobRef.ConstructRef(_,j),[])::l -> (2-j) + 2 * (cumul l)
     | _ -> assert false
   in
   let l = match c with MLcons(_,_,l) -> l | _ -> assert false in
