@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -14,7 +14,6 @@ open Declarations
 open Names
 open ModPath
 open Libnames
-open Globnames
 open Pp
 open CErrors
 open Util
@@ -29,24 +28,27 @@ open Common
 
 let toplevel_env () =
   let get_reference = function
-    | (_,kn), Lib.Leaf o ->
-	let mp,_,l = KerName.repr kn in
-	begin match Libobject.object_tag o with
-	  | "CONSTANT" ->
-            let constant = Global.lookup_constant (Constant.make1 kn) in
-            Some (l, SFBconst constant)
-	  | "INDUCTIVE" ->
-            let inductive = Global.lookup_mind (MutInd.make1 kn) in
-            Some (l, SFBmind inductive)
-	  | "MODULE" ->
-            let modl = Global.lookup_module (MPdot (mp, l)) in
-            Some (l, SFBmodule modl)
-	  | "MODULE TYPE" ->
-            let modtype = Global.lookup_modtype (MPdot (mp, l)) in
-            Some (l, SFBmodtype modtype)
-          | "INCLUDE" -> user_err Pp.(str "No extraction of toplevel Include yet.")
-	  | _ -> None
-        end
+    | (_,kn), Lib.Leaf Libobject.AtomicObject o ->
+      let mp,l = KerName.repr kn in
+            begin match Libobject.object_tag o with
+            | "CONSTANT" ->
+        let constant = Global.lookup_constant (Constant.make1 kn) in
+        Some (l, SFBconst constant)
+            | "INDUCTIVE" ->
+        let inductive = Global.lookup_mind (MutInd.make1 kn) in
+        Some (l, SFBmind inductive)
+            | _ -> None
+      end
+    | (_,kn), Lib.Leaf Libobject.ModuleObject _ ->
+      let mp,l = KerName.repr kn in
+      let modl = Global.lookup_module (MPdot (mp, l)) in
+      Some (l, SFBmodule modl)
+    | (_,kn), Lib.Leaf Libobject.ModuleTypeObject _ ->
+      let mp,l = KerName.repr kn in
+      let modtype = Global.lookup_modtype (MPdot (mp, l)) in
+      Some (l, SFBmodtype modtype)
+    | (_,kn), Lib.Leaf Libobject.IncludeObject _ ->
+      user_err Pp.(str "No extraction of toplevel Include yet.")
     | _ -> None
   in
   List.rev (List.map_filter get_reference (Lib.contents ()))
@@ -115,7 +117,7 @@ module Visit : VISIT = struct
     v.mp <- MPset.union (prefixes_mp mp) v.mp;
     v.mp_all <- MPset.add mp v.mp_all
   let add_kn kn = v.kn <- KNset.add kn v.kn; add_mp (KerName.modpath kn)
-  let add_ref = function
+  let add_ref = let open GlobRef in function
     | ConstRef c -> add_kn (Constant.user c)
     | IndRef (ind,_) | ConstructRef ((ind,_),_) -> add_kn (MutInd.user ind)
     | VarRef _ -> assert false
@@ -124,7 +126,7 @@ module Visit : VISIT = struct
 end
 
 let add_field_label mp = function
-  | (lab, (SFBconst _|SFBmind _)) -> Visit.add_kn (KerName.make2 mp lab)
+  | (lab, (SFBconst _|SFBmind _)) -> Visit.add_kn (KerName.make mp lab)
   | (lab, (SFBmodule _|SFBmodtype _)) -> Visit.add_mp_all (MPdot (mp,lab))
 
 let rec add_labels mp = function
@@ -145,12 +147,12 @@ let check_fix env sg cb i =
     | Def lbody ->
         (match EConstr.kind sg (get_body lbody) with
           | Fix ((_,j),recd) when Int.equal i j -> check_arity env cb; (true,recd)
-	  | CoFix (j,recd) when Int.equal i j -> check_arity env cb; (false,recd)
-	  | _ -> raise Impossible)
-    | Undef _ | OpaqueDef _ -> raise Impossible
+          | CoFix (j,recd) when Int.equal i j -> check_arity env cb; (false,recd)
+          | _ -> raise Impossible)
+    | Undef _ | OpaqueDef _ | Primitive _ -> raise Impossible
 
 let prec_declaration_equal sg (na1, ca1, ta1) (na2, ca2, ta2) =
-  Array.equal Name.equal na1 na2 &&
+  Array.equal (Context.eq_annot Name.equal) na1 na2 &&
   Array.equal (EConstr.eq_constr sg) ca1 ca2 &&
   Array.equal (EConstr.eq_constr sg) ta1 ta2
 
@@ -164,14 +166,14 @@ let factor_fix env sg l cb msb =
     let labels = Array.make n l in
     List.iteri
       (fun j ->
-	 function
-	   | (l,SFBconst cb') ->
+         function
+           | (l,SFBconst cb') ->
               let check' = check_fix env sg cb' (j+1) in
               if not ((fst check : bool) == (fst check') &&
                         prec_declaration_equal sg (snd check) (snd check'))
-	       then raise Impossible;
-	       labels.(j+1) <- l;
-	   | _ -> raise Impossible) msb';
+               then raise Impossible;
+               labels.(j+1) <- l;
+           | _ -> raise Impossible) msb';
     labels, recd, msb''
   end
 
@@ -208,10 +210,10 @@ let env_for_mtb_with_def env mp me reso idl =
   Modops.add_structure mp before reso env
 
 let make_cst resolver mp l =
-  Mod_subst.constant_of_delta_kn resolver (KerName.make2 mp l)
+  Mod_subst.constant_of_delta_kn resolver (KerName.make mp l)
 
 let make_mind resolver mp l =
-  Mod_subst.mind_of_delta_kn resolver (KerName.make2 mp l)
+  Mod_subst.mind_of_delta_kn resolver (KerName.make mp l)
 
 (* From a [structure_body] (i.e. a list of [structure_field_body])
    to specifications. *)
@@ -254,8 +256,8 @@ and extract_mexpr_spec env mp1 (me_struct_o,me_alg) = match me_alg with
       let sg = Evd.from_env env in
       (match extract_with_type env' sg (EConstr.of_constr c) with
        (* cb may contain some kn *)
-	 | None -> mt
-	 | Some (vl,typ) ->
+         | None -> mt
+         | Some (vl,typ) ->
             type_iter_references Visit.add_ref typ;
             MTwith(mt,ML_With_type(idl,vl,typ)))
   | MEwith(me',WithMod(idl,mp))->
@@ -269,8 +271,8 @@ and extract_mexpr_spec env mp1 (me_struct_o,me_alg) = match me_alg with
 and extract_mexpression_spec env mp1 (me_struct,me_alg) = match me_alg with
   | MoreFunctor (mbid, mtb, me_alg') ->
       let me_struct' = match me_struct with
-	| MoreFunctor (mbid',_,me') when MBId.equal mbid' mbid -> me'
-	| _ -> assert false
+        | MoreFunctor (mbid',_,me') when MBId.equal mbid' mbid -> me'
+        | _ -> assert false
       in
       let mp = MPbound mbid in
       let env' = Modops.add_module_type mp mtb env in
@@ -286,7 +288,7 @@ and extract_msignature_spec env mp1 reso = function
       let mp = MPbound mbid in
       let env' = Modops.add_module_type mp mtb env in
       MTfunsig (mbid, extract_mbody_spec env mp mtb,
-		extract_msignature_spec env' mp1 reso me)
+                extract_msignature_spec env' mp1 reso me)
 
 and extract_mbody_spec : 'a. _ -> _ -> 'a generic_module_body -> _ =
   fun env mp mb -> match mb.mod_type_alg with
@@ -306,38 +308,38 @@ let rec extract_structure env mp reso ~all = function
       (try
          let sg = Evd.from_env env in
          let vl,recd,struc = factor_fix env sg l cb struc in
-	 let vc = Array.map (make_cst reso mp) vl in
-	 let ms = extract_structure env mp reso ~all struc in
-	 let b = Array.exists Visit.needed_cst vc in
-	 if all || b then
+         let vc = Array.map (make_cst reso mp) vl in
+         let ms = extract_structure env mp reso ~all struc in
+         let b = Array.exists Visit.needed_cst vc in
+         if all || b then
            let d = extract_fixpoint env sg vc recd in
-	   if (not b) && (logical_decl d) then ms
-	   else begin Visit.add_decl_deps d; (l,SEdecl d) :: ms end
-	 else ms
+           if (not b) && (logical_decl d) then ms
+           else begin Visit.add_decl_deps d; (l,SEdecl d) :: ms end
+         else ms
        with Impossible ->
-	 let ms = extract_structure env mp reso ~all struc in
-	 let c = make_cst reso mp l in
-	 let b = Visit.needed_cst c in
-	 if all || b then
-	   let d = extract_constant env c cb in
-	   if (not b) && (logical_decl d) then ms
-	   else begin Visit.add_decl_deps d; (l,SEdecl d) :: ms end
-	 else ms)
+         let ms = extract_structure env mp reso ~all struc in
+         let c = make_cst reso mp l in
+         let b = Visit.needed_cst c in
+         if all || b then
+           let d = extract_constant env c cb in
+           if (not b) && (logical_decl d) then ms
+           else begin Visit.add_decl_deps d; (l,SEdecl d) :: ms end
+         else ms)
   | (l,SFBmind mib) :: struc ->
       let ms = extract_structure env mp reso ~all struc in
       let mind = make_mind reso mp l in
       let b = Visit.needed_ind mind in
       if all || b then
-	let d = Dind (mind, extract_inductive env mind) in
-	if (not b) && (logical_decl d) then ms
-	else begin Visit.add_decl_deps d; (l,SEdecl d) :: ms end
+        let d = Dind (mind, extract_inductive env mind) in
+        if (not b) && (logical_decl d) then ms
+        else begin Visit.add_decl_deps d; (l,SEdecl d) :: ms end
       else ms
   | (l,SFBmodule mb) :: struc ->
       let ms = extract_structure env mp reso ~all struc in
       let mp = MPdot (mp,l) in
       let all' = all || Visit.needed_mp_all mp in
       if all' || Visit.needed_mp mp then
-	(l,SEmodule (extract_module env mp ~all:all' mb)) :: ms
+        (l,SEmodule (extract_module env mp ~all:all' mb)) :: ms
       else ms
   | (l,SFBmodtype mtb) :: struc ->
       let ms = extract_structure env mp reso ~all struc in
@@ -361,7 +363,7 @@ and extract_mexpr env mp = function
       Visit.add_mp_all mp; Miniml.MEident mp
   | MEapply (me, arg) ->
       Miniml.MEapply (extract_mexpr env mp me,
-	              extract_mexpr env mp (MEident arg))
+                      extract_mexpr env mp (MEident arg))
 
 and extract_mexpression env mp = function
   | NoFunctor me -> extract_mexpr env mp me
@@ -371,7 +373,7 @@ and extract_mexpression env mp = function
       Miniml.MEfunctor
         (mbid,
          extract_mbody_spec env mp1 mtb,
-	 extract_mexpression env' mp me)
+         extract_mexpression env' mp me)
 
 and extract_msignature env mp reso ~all = function
   | NoFunctor struc ->
@@ -383,7 +385,7 @@ and extract_msignature env mp reso ~all = function
       Miniml.MEfunctor
         (mbid,
          extract_mbody_spec env mp1 mtb,
-	 extract_msignature env' mp reso ~all me)
+         extract_msignature env' mp reso ~all me)
 
 and extract_module env mp ~all mb =
   (* A module has an empty [mod_expr] when :
@@ -445,19 +447,19 @@ let mono_filename f =
   match f with
     | None -> None, None, default_id
     | Some f ->
-	let f =
-	  if Filename.check_suffix f d.file_suffix then
-	    Filename.chop_suffix f d.file_suffix
-	  else f
-	in
-	let id =
-	  if lang () != Haskell then default_id
-	  else
+        let f =
+          if Filename.check_suffix f d.file_suffix then
+            Filename.chop_suffix f d.file_suffix
+          else f
+        in
+        let id =
+          if lang () != Haskell then default_id
+          else
             try Id.of_string (Filename.basename f)
-	    with UserError _ ->
+            with UserError _ ->
               user_err Pp.(str "Extraction: provided filename is not a valid identifier")
-	in
-	Some (f^d.file_suffix), Option.map ((^) f) d.sig_suffix, id
+        in
+        Some (f^d.file_suffix), Option.map ((^) f) d.sig_suffix, id
 
 (* Builds a suitable filename from a module id *)
 
@@ -492,8 +494,8 @@ let formatter dry file =
     if dry then Format.make_formatter (fun _ _ _ -> ()) (fun _ -> ())
     else
       match file with
-	| Some f -> Topfmt.with_output_to f
-	| None -> Format.formatter_of_buffer buf
+        | Some f -> Topfmt.with_output_to f
+        | None -> Format.formatter_of_buffer buf
   in
   (* XXX: Fixme, this shouldn't depend on Topfmt *)
   (* We never want to see ellipsis ... in extracted code *)
@@ -552,14 +554,14 @@ let print_structure_to_file (fn,si,mo) dry struc =
        let cout = open_out si in
        let ft = formatter false (Some cout) in
        begin try
-	 set_phase Intf;
-	 pp_with ft (d.sig_preamble mo comment opened unsafe_needs);
-	 pp_with ft (d.pp_sig (signature_of_structure struc));
+         set_phase Intf;
+         pp_with ft (d.sig_preamble mo comment opened unsafe_needs);
+         pp_with ft (d.pp_sig (signature_of_structure struc));
          Format.pp_print_flush ft ();
-	 close_out cout;
+         close_out cout;
        with reraise ->
          Format.pp_print_flush ft ();
-	 close_out cout; raise reraise
+         close_out cout; raise reraise
        end;
        info_file si)
     (if dry then None else si);
@@ -604,9 +606,9 @@ let rec locate_ref = function
       in
       match mpo, ro with
         | None, None -> Nametab.error_global_not_found qid
-	| None, Some r -> let refs,mps = locate_ref l in r::refs,mps
-	| Some mp, None -> let refs,mps = locate_ref l in refs,mp::mps
-	| Some mp, Some r ->
+        | None, Some r -> let refs,mps = locate_ref l in r::refs,mps
+        | Some mp, None -> let refs,mps = locate_ref l in refs,mp::mps
+        | Some mp, Some r ->
            warning_ambiguous_name (qid,mp,r);
            let refs,mps = locate_ref l in refs,mp::mps
 
@@ -635,7 +637,7 @@ let separate_extraction lr =
   warns ();
   let print = function
     | (MPfile dir as mp, sel) as e ->
-	print_structure_to_file (module_filename mp) false [e]
+        print_structure_to_file (module_filename mp) false [e]
     | _ -> assert false
   in
   List.iter print struc;
@@ -645,7 +647,6 @@ let separate_extraction lr =
     is \verb!Extraction! [qualid]. *)
 
 let simple_extraction r =
-  Vernacentries.dump_global CAst.(make (Constrexpr.AN r));
   match locate_ref [r] with
   | ([], [mp]) as p -> full_extr None p
   | [r],[] ->
@@ -685,8 +686,8 @@ let extraction_library is_rec m =
   warns ();
   let print = function
     | (MPfile dir as mp, sel) as e ->
-	let dry = not is_rec && not (DirPath.equal dir dir_m) in
-	print_structure_to_file (module_filename mp) dry [e]
+        let dry = not is_rec && not (DirPath.equal dir dir_m) in
+        print_structure_to_file (module_filename mp) dry [e]
     | _ -> assert false
   in
   List.iter print struc;
@@ -710,10 +711,10 @@ let structure_for_compute env sg c =
   init false false ~compute:true;
   let ast, mlt = Extraction.extract_constr env sg c in
   let ast = Mlutil.normalize ast in
-  let refs = ref Refset.empty in
-  let add_ref r = refs := Refset.add r !refs in
+  let refs = ref GlobRef.Set.empty in
+  let add_ref r = refs := GlobRef.Set.add r !refs in
   let () = ast_iter_references add_ref add_ref add_ref ast in
-  let refs = Refset.elements !refs in
+  let refs = GlobRef.Set.elements !refs in
   let struc = optimize_struct (refs,[]) (mono_environment refs []) in
   (flatten_structure struc), ast, mlt
 
@@ -750,17 +751,16 @@ let extract_and_compile l =
   Feedback.msg_notice (str "Extracted code successfully compiled")
 
 (* Show the extraction of the current ongoing proof *)
-
-let show_extraction () =
+let show_extraction ~pstate =
   init ~inner:true false false;
-  let prf = Proof_global.give_me_the_proof () in
-  let sigma, env = Pfedit.get_current_context () in
+  let prf = Proof_global.get_proof pstate in
+  let sigma, env = Pfedit.get_current_context pstate in
   let trms = Proof.partial_proof prf in
   let extr_term t =
     let ast, ty = extract_constr env sigma t in
     let mp = Lib.current_mp () in
-    let l = Label.of_id (Proof_global.get_current_proof_name ()) in
-    let fake_ref = ConstRef (Constant.make2 mp l) in
+    let l = Label.of_id (Proof_global.get_proof_name pstate) in
+    let fake_ref = GlobRef.ConstRef (Constant.make2 mp l) in
     let decl = Dterm (fake_ref, ast, ty) in
     print_one_decl [] mp decl
   in
