@@ -1,7 +1,6 @@
 open Pp
 open Util
 open Names
-open Globnames
 open Table
 open Miniml
 open Mlutil
@@ -17,7 +16,10 @@ let json_bool b =
   if b then str "true" else str "false"
 
 let json_global typ ref =
-  json_str (Common.pp_global typ ref)
+  if is_custom ref then
+    json_str (find_custom ref)
+  else
+    json_str (Common.pp_global typ ref)
 
 let json_id id =
   json_str (Id.to_string id)
@@ -155,6 +157,14 @@ let rec json_expr env = function
       ("value", json_expr env a)
     ]
   | MLaxiom -> json_dict [("what", json_str "expr:axiom")]
+  | MLuint i -> json_dict [
+      ("what", json_str "expr:int");
+      ("int", json_str (Uint63.to_string i))
+    ]
+  | MLfloat f -> json_dict [
+      ("what", json_str "expr:float");
+      ("float", json_str (Float64.to_string f))
+    ]
 
 and json_one_pat env (ids,p,t) =
   let ids', env' = push_vars (List.rev_map id_of_mlid ids) env in json_dict [
@@ -196,10 +206,10 @@ and json_function env t =
 
 let json_ind ip pl cv = json_dict [
     ("what", json_str "decl:ind");
-    ("name", json_global Type (IndRef ip));
+    ("name", json_global Type (GlobRef.IndRef ip));
     ("argnames", json_list (List.map json_id pl));
     ("constructors", json_listarr (Array.mapi (fun idx c -> json_dict [
-        ("name", json_global Cons (ConstructRef (ip, idx+1)));
+        ("name", json_global Cons (GlobRef.ConstructRef (ip, idx+1)));
         ("argtypes", json_list (List.map (json_type pl) c))
       ]) cv))
   ]
