@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -10,7 +10,6 @@
 
 open Names
 open ModPath
-open Globnames
 open CErrors
 open Util
 open Miniml
@@ -37,19 +36,19 @@ let se_iter do_decl do_spec do_mp =
     | MTident mp -> do_mp mp
     | MTfunsig (_,mt,mt') -> mt_iter mt; mt_iter mt'
     | MTwith (mt,ML_With_type(idl,l,t))->
-	let mp_mt = msid_of_mt mt in
-	let l',idl' = List.sep_last idl in
-	let mp_w =
-	  List.fold_left (fun mp l -> MPdot(mp,Label.of_id l)) mp_mt idl'
-	in
-	let r = ConstRef (Constant.make2 mp_w (Label.of_id l')) in
+        let mp_mt = msid_of_mt mt in
+        let l',idl' = List.sep_last idl in
+        let mp_w =
+          List.fold_left (fun mp l -> MPdot(mp,Label.of_id l)) mp_mt idl'
+        in
+        let r = GlobRef.ConstRef (Constant.make2 mp_w (Label.of_id l')) in
         mt_iter mt; do_spec (Stype(r,l,Some t))
     | MTwith (mt,ML_With_module(idl,mp))->
         let mp_mt = msid_of_mt mt in
         let mp_w =
-	  List.fold_left (fun mp l -> MPdot(mp,Label.of_id l)) mp_mt idl
-	in
-	mt_iter mt; do_mp mp_w; do_mp mp
+          List.fold_left (fun mp l -> MPdot(mp,Label.of_id l)) mp_mt idl
+        in
+        mt_iter mt; do_mp mp_w; do_mp mp
     | MTsig (_, sign) -> List.iter spec_iter sign
   and spec_iter = function
     | (_,Spec s) -> do_spec s
@@ -59,7 +58,7 @@ let se_iter do_decl do_spec do_mp =
   let rec se_iter = function
     | (_,SEdecl d) -> do_decl d
     | (_,SEmodule m) ->
-	me_iter m.ml_mod_expr; mt_iter m.ml_mod_type
+        me_iter m.ml_mod_expr; mt_iter m.ml_mod_type
     | (_,SEmodtype m) -> mt_iter m
   and me_iter = function
     | MEident mp -> do_mp mp
@@ -104,22 +103,22 @@ let ast_iter_references do_term do_cons do_type a =
       | MLglob r -> do_term r
       | MLcons (_,r,_) -> do_cons r
       | MLcase (ty,_,v) ->
-	type_iter_references do_type ty;
-	Array.iter (fun (_,p,_) -> patt_iter_references do_cons p) v
+        type_iter_references do_type ty;
+        Array.iter (fun (_,p,_) -> patt_iter_references do_cons p) v
 
       | MLrel _ | MLlam _ | MLapp _ | MLletin _ | MLtuple _ | MLfix _ | MLexn _
-      | MLdummy _ | MLaxiom | MLmagic _ -> ()
+      | MLdummy _ | MLaxiom | MLmagic _ | MLuint _ | MLfloat _ -> ()
   in iter a
 
 let ind_iter_references do_term do_cons do_type kn ind =
   let type_iter = type_iter_references do_type in
-  let cons_iter cp l = do_cons (ConstructRef cp); List.iter type_iter l in
+  let cons_iter cp l = do_cons (GlobRef.ConstructRef cp); List.iter type_iter l in
   let packet_iter ip p =
-    do_type (IndRef ip);
+    do_type (GlobRef.IndRef ip);
     if lang () == Ocaml then
       (match ind.ind_equiv with
-	 | Miniml.Equiv kne -> do_type (IndRef (MutInd.make1 kne, snd ip));
-	 | _ -> ());
+         | Miniml.Equiv kne -> do_type (GlobRef.IndRef (MutInd.make1 kne, snd ip));
+         | _ -> ());
     Array.iteri (fun j -> cons_iter (ip,j+1)) p.ip_types
   in
   if lang () == Ocaml then record_iter_references do_term ind.ind_kind;
@@ -133,7 +132,7 @@ let decl_iter_references do_term do_cons do_type =
     | Dtype (r,_,t) -> do_type r; type_iter t
     | Dterm (r,a,t) -> do_term r; ast_iter a; type_iter t
     | Dfix(rv,c,t) ->
-	Array.iter do_term rv; Array.iter ast_iter c; Array.iter type_iter t
+        Array.iter do_term rv; Array.iter ast_iter c; Array.iter type_iter t
 
 let spec_iter_references do_term do_cons do_type = function
   | Sind (kn,ind) -> ind_iter_references do_term do_cons do_type kn ind
@@ -164,7 +163,7 @@ let rec type_search f = function
 let decl_type_search f = function
   | Dind (_,{ind_packets=p})  ->
       Array.iter
-	(fun {ip_types=v} -> Array.iter (List.iter (type_search f)) v) p
+        (fun {ip_types=v} -> Array.iter (List.iter (type_search f)) v) p
   | Dterm (_,_,u) -> type_search f u
   | Dfix (_,_,v) -> Array.iter (type_search f) v
   | Dtype (_,_,u) -> type_search f u
@@ -172,7 +171,7 @@ let decl_type_search f = function
 let spec_type_search f = function
   | Sind (_,{ind_packets=p}) ->
       Array.iter
-	(fun {ip_types=v} -> Array.iter (List.iter (type_search f)) v) p
+        (fun {ip_types=v} -> Array.iter (List.iter (type_search f)) v) p
   | Stype (_,_,ot) -> Option.iter (type_search f) ot
   | Sval (_,u) -> type_search f u
 
@@ -196,7 +195,7 @@ let rec msig_of_ms = function
   | (l,SEdecl (Dfix (rv,_,tv))) :: ms ->
       let msig = ref (msig_of_ms ms) in
       for i = Array.length rv - 1 downto 0 do
-	msig := (l,Spec (Sval (rv.(i),tv.(i))))::!msig
+        msig := (l,Spec (Sval (rv.(i),tv.(i))))::!msig
       done;
       !msig
   | (l,SEmodule m) :: ms -> (l,Smodule m.ml_mod_type) :: (msig_of_ms ms)
@@ -230,13 +229,13 @@ let get_decl_in_structure r struc =
     let rec go ll sel = match ll with
       | [] -> assert false
       | l :: ll ->
-	  match search_structure l (not (List.is_empty ll)) sel with
-	    | SEdecl d -> d
-	    | SEmodtype m -> assert false
-	    | SEmodule m ->
-		match m.ml_mod_expr with
-		  | MEstruct (_,sel) -> go ll sel
-		  | _ -> error_not_visible r
+          match search_structure l (not (List.is_empty ll)) sel with
+            | SEdecl d -> d
+            | SEmodtype m -> assert false
+            | SEmodule m ->
+                match m.ml_mod_expr with
+                  | MEstruct (_,sel) -> go ll sel
+                  | _ -> error_not_visible r
     in go ll sel
   with Not_found ->
     anomaly (Pp.str "reference not found in extracted structure.")
@@ -258,8 +257,8 @@ let dfix_to_mlfix rv av i =
   let s = make_subst (Array.length rv - 1) Refmap'.empty
   in
   let rec subst n t = match t with
-    | MLglob ((ConstRef kn) as refe) ->
-	(try MLrel (n + (Refmap'.find refe s)) with Not_found -> t)
+    | MLglob ((GlobRef.ConstRef kn) as refe) ->
+        (try MLrel (n + (Refmap'.find refe s)) with Not_found -> t)
     | _ -> ast_map_lift subst n t
   in
   let ids = Array.map (fun r -> Label.to_id (label_of_r r)) rv in
@@ -278,9 +277,9 @@ let rec optim_se top to_appear s = function
       let i = inline r a in
       if i then s := Refmap'.add r a !s;
       let d = match dump_unused_vars (optimize_fix a) with
-	| MLfix (0, _, [|c|]) ->
-	  Dfix ([|r|], [|ast_subst (MLglob r) c|], [|t|])
-	| a -> Dterm (r, a, t)
+        | MLfix (0, _, [|c|]) ->
+          Dfix ([|r|], [|ast_subst (MLglob r) c|], [|t|])
+        | a -> Dterm (r, a, t)
       in
       (l,SEdecl d) :: (optim_se top to_appear s lse)
   | (l,SEdecl (Dfix (rv,av,tv))) :: lse ->
@@ -288,8 +287,8 @@ let rec optim_se top to_appear s = function
       (* This fake body ensures that no fixpoint will be auto-inlined. *)
       let fake_body = MLfix (0,[||],[||]) in
       for i = 0 to Array.length rv - 1 do
-	if inline rv.(i) fake_body
-	then s := Refmap'.add rv.(i) (dfix_to_mlfix rv av i) !s
+        if inline rv.(i) fake_body
+        then s := Refmap'.add rv.(i) (dfix_to_mlfix rv av i) !s
       done;
       let av' = Array.map dump_unused_vars av in
       (l,SEdecl (Dfix (rv, av', tv))) :: (optim_se top to_appear s lse)
@@ -309,7 +308,7 @@ and optim_me to_appear s = function
    For non-library extraction, we recompute a minimal set of dependencies
    for first-level definitions (no module pruning yet). *)
 
-let base_r = function
+let base_r = let open GlobRef in function
   | ConstRef c as r -> r
   | IndRef (kn,_) -> IndRef (kn,0)
   | ConstructRef ((kn,_),_) -> IndRef (kn,0)
@@ -327,7 +326,7 @@ let reset_needed, add_needed, add_needed_mp, found_needed, is_needed =
      Refset'.mem r !needed || MPset.mem (modpath_of_r r) !needed_mps))
 
 let declared_refs = function
-  | Dind (kn,_) -> [IndRef (kn,0)]
+  | Dind (kn,_) -> [GlobRef.IndRef (kn,0)]
   | Dtype (r,_,_) -> [r]
   | Dterm (r,_,_) -> [r]
   | Dfix (rv,_,_) -> Array.to_list rv
@@ -344,7 +343,7 @@ let compute_deps_decl = function
   | Dterm (r,u,t) ->
       type_iter_references add_needed t;
       if not (is_custom r) then
-	ast_iter_references add_needed add_needed add_needed u
+        ast_iter_references add_needed add_needed add_needed u
   | Dfix _ as d ->
       decl_iter_references add_needed add_needed add_needed d
 
@@ -371,10 +370,10 @@ let rec depcheck_se = function
       List.iter found_needed refs';
       (* Hack to avoid extracting unused part of a Dfix *)
       match d with
-	| Dfix (rv,trms,tys) when (List.for_all is_custom refs') ->
-	  let trms' =  Array.make (Array.length rv) (MLexn "UNUSED") in
-	  ((l,SEdecl (Dfix (rv,trms',tys))) :: se')
-	| _ -> (compute_deps_decl d; t::se')
+        | Dfix (rv,trms,tys) when (List.for_all is_custom refs') ->
+          let trms' =  Array.make (Array.length rv) (MLexn "UNUSED") in
+          ((l,SEdecl (Dfix (rv,trms',tys))) :: se')
+        | _ -> (compute_deps_decl d; t::se')
     end
   | t :: se ->
     let se' = depcheck_se se in
