@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -144,9 +144,19 @@ let coerce_to_constr_context v =
     out_gen (topwit wit_constr_context) v
   else raise (CannotCoerceTo "a term context")
 
+let is_intro_pattern v =
+  if has_type v (topwit wit_intro_pattern) then
+    Some (out_gen (topwit wit_intro_pattern) v).CAst.v
+  else
+    None
+
 (* Interprets an identifier which must be fresh *)
 let coerce_var_to_ident fresh env sigma v =
   let fail () = raise (CannotCoerceTo "a fresh identifier") in
+  match is_intro_pattern v with
+  | Some (IntroNaming (IntroIdentifier id)) -> id
+  | Some _ -> fail ()
+  | None ->
   if has_type v (topwit wit_intro_pattern) then
     match out_gen (topwit wit_intro_pattern) v with
     | { CAst.v=IntroNaming (IntroIdentifier id)} -> id
@@ -170,18 +180,18 @@ let id_of_name = function
   | Name.Anonymous -> Id.of_string "x"
   | Name.Name x -> x in
   let fail () = raise (CannotCoerceTo "an identifier") in
-  if has_type v (topwit wit_intro_pattern) then
-    match out_gen (topwit wit_intro_pattern) v with
-    | {CAst.v=IntroNaming (IntroIdentifier id)} -> id
-    | _ -> fail ()
-  else if has_type v (topwit wit_var) then
+  match is_intro_pattern v with
+  | Some (IntroNaming (IntroIdentifier id)) -> id
+  | Some _ -> fail ()
+  | None ->
+  if has_type v (topwit wit_var) then
     out_gen (topwit wit_var) v
   else
     match Value.to_constr v with
     | None -> fail ()
     | Some c ->
        match EConstr.kind sigma c with
-       | Var id -> id 
+       | Var id -> id
        | Meta m -> id_of_name (Evd.meta_name sigma m)
        | Evar (kn,_) ->
         begin match Evd.evar_ident kn sigma with
@@ -190,16 +200,17 @@ let id_of_name = function
         end
        | Const (cst,_) -> Label.to_id (Constant.label cst)
        | Construct (cstr,_) ->
-	  let ref = Globnames.ConstructRef cstr in
-	  let basename = Nametab.basename_of_global ref in
-	  basename
+          let ref = GlobRef.ConstructRef cstr in
+          let basename = Nametab.basename_of_global ref in
+          basename
        | Ind (ind,_) ->
-	  let ref = Globnames.IndRef ind in
-	  let basename = Nametab.basename_of_global ref in
-	  basename
+          let ref = GlobRef.IndRef ind in
+          let basename = Nametab.basename_of_global ref in
+          basename
        | Sort s ->
           begin
-	    match ESorts.kind sigma s with
+            match ESorts.kind sigma s with
+            | Sorts.SProp -> Label.to_id (Label.make "SProp")
             | Sorts.Prop -> Label.to_id (Label.make "Prop")
             | Sorts.Set -> Label.to_id (Label.make "Set")
             | Sorts.Type _ -> Label.to_id (Label.make "Type")
@@ -208,9 +219,10 @@ let id_of_name = function
 
 
 let coerce_to_intro_pattern sigma v =
-  if has_type v (topwit wit_intro_pattern) then
-    (out_gen (topwit wit_intro_pattern) v).CAst.v
-  else if has_type v (topwit wit_var) then
+  match is_intro_pattern v with
+  | Some pat -> pat
+  | None ->
+  if has_type v (topwit wit_var) then
     let id = out_gen (topwit wit_var) v in
     IntroNaming (IntroIdentifier id)
   else match Value.to_constr v with
@@ -226,11 +238,9 @@ let coerce_to_intro_pattern_naming sigma v =
   | _ -> raise (CannotCoerceTo "a naming introduction pattern")
 
 let coerce_to_hint_base v =
-  if has_type v (topwit wit_intro_pattern) then
-    match out_gen (topwit wit_intro_pattern) v with
-    | {CAst.v=IntroNaming (IntroIdentifier id)} -> Id.to_string id
-    | _ -> raise (CannotCoerceTo "a hint base name")
-  else raise (CannotCoerceTo "a hint base name")
+  match is_intro_pattern v with
+  | Some (IntroNaming (IntroIdentifier id)) -> Id.to_string id
+  | Some _ | None -> raise (CannotCoerceTo "a hint base name")
 
 let coerce_to_int v =
   if has_type v (topwit wit_int) then
@@ -239,12 +249,12 @@ let coerce_to_int v =
 
 let coerce_to_constr env v =
   let fail () = raise (CannotCoerceTo "a term") in
-  if has_type v (topwit wit_intro_pattern) then
-    match out_gen (topwit wit_intro_pattern) v with
-    | {CAst.v=IntroNaming (IntroIdentifier id)} ->
+  match is_intro_pattern v with
+  | Some (IntroNaming (IntroIdentifier id)) ->
       (try ([], constr_of_id env id) with Not_found -> fail ())
-    | _ -> fail ()
-  else if has_type v (topwit wit_constr) then
+  | Some _ -> fail ()
+  | None ->
+  if has_type v (topwit wit_constr) then
     let c = out_gen (topwit wit_constr) v in
     ([], c)
   else if has_type v (topwit wit_constr_under_binders) then
@@ -268,16 +278,16 @@ let coerce_to_closed_constr env v =
 let coerce_to_evaluable_ref env sigma v =
   let fail () = raise (CannotCoerceTo "an evaluable reference") in
   let ev =
-  if has_type v (topwit wit_intro_pattern) then
-    match out_gen (topwit wit_intro_pattern) v with
-    | {CAst.v=IntroNaming (IntroIdentifier id)} when is_variable env id -> EvalVarRef id
-    | _ -> fail ()
-  else if has_type v (topwit wit_var) then
+  match is_intro_pattern v with
+  | Some (IntroNaming (IntroIdentifier id)) when is_variable env id -> EvalVarRef id
+  | Some _ -> fail ()
+  | None ->
+  if has_type v (topwit wit_var) then
     let id = out_gen (topwit wit_var) v in
     if Id.List.mem id (Termops.ids_of_context env) then EvalVarRef id
     else fail ()
   else if has_type v (topwit wit_ref) then
-    let open Globnames in
+    let open GlobRef in
     let r = out_gen (topwit wit_ref) v in
     match r with
     | VarRef var -> EvalVarRef var
@@ -307,11 +317,11 @@ let coerce_to_intro_pattern_list ?loc sigma v =
 
 let coerce_to_hyp env sigma v =
   let fail () = raise (CannotCoerceTo "a variable") in
-  if has_type v (topwit wit_intro_pattern) then
-    match out_gen (topwit wit_intro_pattern) v with
-    | {CAst.v=IntroNaming (IntroIdentifier id)} when is_variable env id -> id
-    | _ -> fail ()
-  else if has_type v (topwit wit_var) then
+  match is_intro_pattern v with
+  | Some (IntroNaming (IntroIdentifier id)) when is_variable env id -> id
+  | Some _ -> fail ()
+  | None ->
+  if has_type v (topwit wit_var) then
     let id = out_gen (topwit wit_var) v in
     if is_variable env id then id else fail ()
   else match Value.to_constr v with
@@ -339,12 +349,11 @@ let coerce_to_reference sigma v =
 (* Quantified named or numbered hypothesis or hypothesis in context *)
 (* (as in Inversion) *)
 let coerce_to_quantified_hypothesis sigma v =
-  if has_type v (topwit wit_intro_pattern) then
-    let v = out_gen (topwit wit_intro_pattern) v in
-    match v with
-    | {CAst.v=IntroNaming (IntroIdentifier id)} -> NamedHyp id
-    | _ -> raise (CannotCoerceTo "a quantified hypothesis")
-  else if has_type v (topwit wit_var) then
+  match is_intro_pattern v with
+  | Some (IntroNaming (IntroIdentifier id)) -> NamedHyp id
+  | Some _ -> raise (CannotCoerceTo "a quantified hypothesis")
+  | None ->
+  if has_type v (topwit wit_var) then
     let id = out_gen (topwit wit_var) v in
     NamedHyp id
   else if has_type v (topwit wit_int) then
