@@ -1,4 +1,9 @@
 
+type __ = Obj.t
+
+type unit0 =
+| Tt
+
 (** val negb : bool -> bool **)
 
 let negb = function
@@ -9,6 +14,20 @@ type nat =
 | O
 | S of nat
 
+type ('a, 'b) sum =
+| Inl of 'a
+| Inr of 'b
+
+(** val fst : ('a1 * 'a2) -> 'a1 **)
+
+let fst = function
+| x,_ -> x
+
+(** val snd : ('a1 * 'a2) -> 'a2 **)
+
+let snd = function
+| _,y -> y
+
 (** val app : 'a1 list -> 'a1 list -> 'a1 list **)
 
 let rec app l m =
@@ -37,6 +56,43 @@ module Coq__1 = struct
 end
 include Coq__1
 
+(** val nth : nat -> 'a1 list -> 'a1 -> 'a1 **)
+
+let rec nth n0 l default =
+  match n0 with
+  | O -> (match l with
+          | [] -> default
+          | x::_ -> x)
+  | S m -> (match l with
+            | [] -> default
+            | _::t0 -> nth m t0 default)
+
+(** val rev_append : 'a1 list -> 'a1 list -> 'a1 list **)
+
+let rec rev_append l l' =
+  match l with
+  | [] -> l'
+  | a::l0 -> rev_append l0 (a::l')
+
+(** val map : ('a1 -> 'a2) -> 'a1 list -> 'a2 list **)
+
+let rec map f = function
+| [] -> []
+| a::t0 -> (f a)::(map f t0)
+
+(** val fold_left : ('a1 -> 'a2 -> 'a1) -> 'a2 list -> 'a1 -> 'a1 **)
+
+let rec fold_left f l a0 =
+  match l with
+  | [] -> a0
+  | b::t0 -> fold_left f t0 (f a0 b)
+
+(** val fold_right : ('a2 -> 'a1 -> 'a1) -> 'a1 -> 'a2 list -> 'a1 **)
+
+let rec fold_right f a0 = function
+| [] -> a0
+| b::t0 -> f b (fold_right f a0 t0)
+
 type positive =
 | XI of positive
 | XO of positive
@@ -188,6 +244,13 @@ module Coq_Pos =
     | XO p -> XO (mul p y)
     | XH -> y
 
+  (** val iter : ('a1 -> 'a1) -> 'a1 -> positive -> 'a1 **)
+
+  let rec iter f x = function
+  | XI n' -> f (iter f (iter f x n') n')
+  | XO n' -> iter f (iter f x n') n'
+  | XH -> f x
+
   (** val size_nat : positive -> nat **)
 
   let rec size_nat = function
@@ -218,6 +281,20 @@ module Coq_Pos =
   let compare =
     compare_cont Eq
 
+  (** val max : positive -> positive -> positive **)
+
+  let max p p' =
+    match compare p p' with
+    | Gt -> p
+    | _ -> p'
+
+  (** val leb : positive -> positive -> bool **)
+
+  let leb x y =
+    match compare x y with
+    | Gt -> false
+    | _ -> true
+
   (** val gcdn : nat -> positive -> positive -> positive **)
 
   let rec gcdn n0 a b =
@@ -269,29 +346,6 @@ let rec pow_pos rmul x = function
 | XO i0 -> let p = pow_pos rmul x i0 in rmul p p
 | XH -> x
 
-(** val nth : nat -> 'a1 list -> 'a1 -> 'a1 **)
-
-let rec nth n0 l default =
-  match n0 with
-  | O -> (match l with
-          | [] -> default
-          | x::_ -> x)
-  | S m -> (match l with
-            | [] -> default
-            | _::t0 -> nth m t0 default)
-
-(** val map : ('a1 -> 'a2) -> 'a1 list -> 'a2 list **)
-
-let rec map f = function
-| [] -> []
-| a::t0 -> (f a)::(map f t0)
-
-(** val fold_right : ('a2 -> 'a1 -> 'a1) -> 'a1 -> 'a2 list -> 'a1 **)
-
-let rec fold_right f a0 = function
-| [] -> a0
-| b::t0 -> f b (fold_right f a0 t0)
-
 module Z =
  struct
   (** val double : z -> z **)
@@ -379,6 +433,18 @@ module Z =
        | Zpos y' -> Zneg (Coq_Pos.mul x' y')
        | Zneg y' -> Zpos (Coq_Pos.mul x' y'))
 
+  (** val pow_pos : z -> positive -> z **)
+
+  let pow_pos z0 =
+    Coq_Pos.iter (mul z0) (Zpos XH)
+
+  (** val pow : z -> z -> z **)
+
+  let pow x = function
+  | Z0 -> Zpos XH
+  | Zpos p -> pow_pos x p
+  | Zneg _ -> Z0
+
   (** val compare : z -> z -> comparison **)
 
   let compare x y =
@@ -435,6 +501,18 @@ module Z =
   | Zpos p -> Npos p
   | _ -> N0
 
+  (** val of_nat : nat -> z **)
+
+  let of_nat = function
+  | O -> Z0
+  | S n1 -> Zpos (Coq_Pos.of_succ_nat n1)
+
+  (** val of_N : n -> z **)
+
+  let of_N = function
+  | N0 -> Z0
+  | Npos p -> Zpos p
+
   (** val pos_div_eucl : positive -> z -> z * z **)
 
   let rec pos_div_eucl a b =
@@ -506,6 +584,15 @@ let zeq_bool x y =
   | Eq -> true
   | _ -> false
 
+type 'c pExpr =
+| PEc of 'c
+| PEX of positive
+| PEadd of 'c pExpr * 'c pExpr
+| PEsub of 'c pExpr * 'c pExpr
+| PEmul of 'c pExpr * 'c pExpr
+| PEopp of 'c pExpr
+| PEpow of 'c pExpr * n
+
 type 'c pol =
 | Pc of 'c
 | Pinj of positive * 'c pol
@@ -818,15 +905,6 @@ let rec psquare cO cI cadd cmul ceqb = function
   let p3 = psquare cO cI cadd cmul ceqb p2 in
   mkPX cO ceqb (padd cO cadd ceqb (mkPX cO ceqb p3 i (p0 cO)) twoPQ) i q2
 
-type 'c pExpr =
-| PEc of 'c
-| PEX of positive
-| PEadd of 'c pExpr * 'c pExpr
-| PEsub of 'c pExpr * 'c pExpr
-| PEmul of 'c pExpr * 'c pExpr
-| PEopp of 'c pExpr
-| PEpow of 'c pExpr * n
-
 (** val mk_X : 'a1 -> 'a1 -> positive -> 'a1 pol **)
 
 let mk_X cO cI j =
@@ -889,53 +967,105 @@ let rec norm_aux cO cI cadd cmul csub copp ceqb = function
   ppow_N cO cI cadd cmul ceqb (fun p -> p)
     (norm_aux cO cI cadd cmul csub copp ceqb pe1) n0
 
-type 'a bFormula =
+type ('tA, 'tX, 'aA, 'aF) gFormula =
 | TT
 | FF
-| X
-| A of 'a
-| Cj of 'a bFormula * 'a bFormula
-| D of 'a bFormula * 'a bFormula
-| N of 'a bFormula
-| I of 'a bFormula * 'a bFormula
+| X of 'tX
+| A of 'tA * 'aA
+| Cj of ('tA, 'tX, 'aA, 'aF) gFormula * ('tA, 'tX, 'aA, 'aF) gFormula
+| D of ('tA, 'tX, 'aA, 'aF) gFormula * ('tA, 'tX, 'aA, 'aF) gFormula
+| N of ('tA, 'tX, 'aA, 'aF) gFormula
+| I of ('tA, 'tX, 'aA, 'aF) gFormula * 'aF option
+   * ('tA, 'tX, 'aA, 'aF) gFormula
 
-(** val map_bformula : ('a1 -> 'a2) -> 'a1 bFormula -> 'a2 bFormula **)
+(** val mapX :
+    ('a2 -> 'a2) -> ('a1, 'a2, 'a3, 'a4) gFormula -> ('a1, 'a2, 'a3, 'a4)
+    gFormula **)
+
+let rec mapX f = function
+| X x -> X (f x)
+| Cj (f1, f2) -> Cj ((mapX f f1), (mapX f f2))
+| D (f1, f2) -> D ((mapX f f1), (mapX f f2))
+| N f1 -> N (mapX f f1)
+| I (f1, o, f2) -> I ((mapX f f1), o, (mapX f f2))
+| x -> x
+
+(** val foldA :
+    ('a5 -> 'a3 -> 'a5) -> ('a1, 'a2, 'a3, 'a4) gFormula -> 'a5 -> 'a5 **)
+
+let rec foldA f f0 acc =
+  match f0 with
+  | A (_, an) -> f acc an
+  | Cj (f1, f2) -> foldA f f1 (foldA f f2 acc)
+  | D (f1, f2) -> foldA f f1 (foldA f f2 acc)
+  | N f1 -> foldA f f1 acc
+  | I (f1, _, f2) -> foldA f f1 (foldA f f2 acc)
+  | _ -> acc
+
+(** val cons_id : 'a1 option -> 'a1 list -> 'a1 list **)
+
+let cons_id id l =
+  match id with
+  | Some id0 -> id0::l
+  | None -> l
+
+(** val ids_of_formula : ('a1, 'a2, 'a3, 'a4) gFormula -> 'a4 list **)
+
+let rec ids_of_formula = function
+| I (_, id, f') -> cons_id id (ids_of_formula f')
+| _ -> []
+
+(** val collect_annot : ('a1, 'a2, 'a3, 'a4) gFormula -> 'a3 list **)
+
+let rec collect_annot = function
+| A (_, a) -> a::[]
+| Cj (f1, f2) -> app (collect_annot f1) (collect_annot f2)
+| D (f1, f2) -> app (collect_annot f1) (collect_annot f2)
+| N f0 -> collect_annot f0
+| I (f1, _, f2) -> app (collect_annot f1) (collect_annot f2)
+| _ -> []
+
+type 'a bFormula = ('a, __, unit0, unit0) gFormula
+
+(** val map_bformula :
+    ('a1 -> 'a2) -> ('a1, 'a3, 'a4, 'a5) gFormula -> ('a2, 'a3, 'a4, 'a5)
+    gFormula **)
 
 let rec map_bformula fct = function
 | TT -> TT
 | FF -> FF
-| X -> X
-| A a -> A (fct a)
+| X p -> X p
+| A (a, t0) -> A ((fct a), t0)
 | Cj (f1, f2) -> Cj ((map_bformula fct f1), (map_bformula fct f2))
 | D (f1, f2) -> D ((map_bformula fct f1), (map_bformula fct f2))
 | N f0 -> N (map_bformula fct f0)
-| I (f1, f2) -> I ((map_bformula fct f1), (map_bformula fct f2))
+| I (f1, a, f2) -> I ((map_bformula fct f1), a, (map_bformula fct f2))
 
-type 'x clause = 'x list
+type ('x, 'annot) clause = ('x * 'annot) list
 
-type 'x cnf = 'x clause list
+type ('x, 'annot) cnf = ('x, 'annot) clause list
 
-(** val tt : 'a1 cnf **)
+(** val cnf_tt : ('a1, 'a2) cnf **)
 
-let tt =
+let cnf_tt =
   []
 
-(** val ff : 'a1 cnf **)
+(** val cnf_ff : ('a1, 'a2) cnf **)
 
-let ff =
+let cnf_ff =
   []::[]
 
 (** val add_term :
-    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> 'a1 -> 'a1 clause -> 'a1
-    clause option **)
+    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> ('a1 * 'a2) -> ('a1, 'a2)
+    clause -> ('a1, 'a2) clause option **)
 
 let rec add_term unsat deduce t0 = function
 | [] ->
-  (match deduce t0 t0 with
+  (match deduce (fst t0) (fst t0) with
    | Some u -> if unsat u then None else Some (t0::[])
    | None -> Some (t0::[]))
 | t'::cl0 ->
-  (match deduce t0 t' with
+  (match deduce (fst t0) (fst t') with
    | Some u ->
      if unsat u
      then None
@@ -948,8 +1078,8 @@ let rec add_term unsat deduce t0 = function
       | None -> None))
 
 (** val or_clause :
-    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> 'a1 clause -> 'a1 clause
-    -> 'a1 clause option **)
+    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> ('a1, 'a2) clause -> ('a1,
+    'a2) clause -> ('a1, 'a2) clause option **)
 
 let rec or_clause unsat deduce cl1 cl2 =
   match cl1 with
@@ -959,63 +1089,326 @@ let rec or_clause unsat deduce cl1 cl2 =
      | Some cl' -> or_clause unsat deduce cl cl'
      | None -> None)
 
-(** val or_clause_cnf :
-    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> 'a1 clause -> 'a1 cnf ->
-    'a1 cnf **)
+(** val xor_clause_cnf :
+    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> ('a1, 'a2) clause -> ('a1,
+    'a2) cnf -> ('a1, 'a2) cnf **)
 
-let or_clause_cnf unsat deduce t0 f =
-  fold_right (fun e acc ->
+let xor_clause_cnf unsat deduce t0 f =
+  fold_left (fun acc e ->
     match or_clause unsat deduce t0 e with
     | Some cl -> cl::acc
-    | None -> acc) [] f
+    | None -> acc) f []
+
+(** val or_clause_cnf :
+    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> ('a1, 'a2) clause -> ('a1,
+    'a2) cnf -> ('a1, 'a2) cnf **)
+
+let or_clause_cnf unsat deduce t0 f =
+  match t0 with
+  | [] -> f
+  | _::_ -> xor_clause_cnf unsat deduce t0 f
 
 (** val or_cnf :
-    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> 'a1 cnf -> 'a1 cnf -> 'a1
-    cnf **)
+    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> ('a1, 'a2) cnf -> ('a1,
+    'a2) cnf -> ('a1, 'a2) cnf **)
 
 let rec or_cnf unsat deduce f f' =
   match f with
-  | [] -> tt
+  | [] -> cnf_tt
   | e::rst ->
-    app (or_cnf unsat deduce rst f') (or_clause_cnf unsat deduce e f')
+    rev_append (or_cnf unsat deduce rst f') (or_clause_cnf unsat deduce e f')
 
-(** val and_cnf : 'a1 cnf -> 'a1 cnf -> 'a1 cnf **)
+(** val and_cnf : ('a1, 'a2) cnf -> ('a1, 'a2) cnf -> ('a1, 'a2) cnf **)
 
 let and_cnf =
-  app
+  rev_append
+
+type ('term, 'annot, 'tX, 'aF) tFormula = ('term, 'tX, 'annot, 'aF) gFormula
+
+(** val is_cnf_tt : ('a1, 'a2) cnf -> bool **)
+
+let is_cnf_tt = function
+| [] -> true
+| _::_ -> false
+
+(** val is_cnf_ff : ('a1, 'a2) cnf -> bool **)
+
+let is_cnf_ff = function
+| [] -> false
+| c0::l ->
+  (match c0 with
+   | [] -> (match l with
+            | [] -> true
+            | _::_ -> false)
+   | _::_ -> false)
+
+(** val and_cnf_opt : ('a1, 'a2) cnf -> ('a1, 'a2) cnf -> ('a1, 'a2) cnf **)
+
+let and_cnf_opt f1 f2 =
+  if if is_cnf_ff f1 then true else is_cnf_ff f2
+  then cnf_ff
+  else and_cnf f1 f2
+
+(** val or_cnf_opt :
+    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> ('a1, 'a2) cnf -> ('a1,
+    'a2) cnf -> ('a1, 'a2) cnf **)
+
+let or_cnf_opt unsat deduce f1 f2 =
+  if if is_cnf_tt f1 then true else is_cnf_tt f2
+  then cnf_tt
+  else if is_cnf_ff f2 then f1 else or_cnf unsat deduce f1 f2
 
 (** val xcnf :
-    ('a2 -> bool) -> ('a2 -> 'a2 -> 'a2 option) -> ('a1 -> 'a2 cnf) -> ('a1
-    -> 'a2 cnf) -> bool -> 'a1 bFormula -> 'a2 cnf **)
+    ('a2 -> bool) -> ('a2 -> 'a2 -> 'a2 option) -> ('a1 -> 'a3 -> ('a2, 'a3)
+    cnf) -> ('a1 -> 'a3 -> ('a2, 'a3) cnf) -> bool -> ('a1, 'a3, 'a4, 'a5)
+    tFormula -> ('a2, 'a3) cnf **)
 
-let rec xcnf unsat deduce normalise0 negate0 pol0 = function
-| TT -> if pol0 then tt else ff
-| FF -> if pol0 then ff else tt
-| X -> ff
-| A x -> if pol0 then normalise0 x else negate0 x
+let rec xcnf unsat deduce normalise1 negate0 pol0 = function
+| TT -> if pol0 then cnf_tt else cnf_ff
+| FF -> if pol0 then cnf_ff else cnf_tt
+| X _ -> cnf_ff
+| A (x, t0) -> if pol0 then normalise1 x t0 else negate0 x t0
 | Cj (e1, e2) ->
   if pol0
-  then and_cnf (xcnf unsat deduce normalise0 negate0 pol0 e1)
-         (xcnf unsat deduce normalise0 negate0 pol0 e2)
-  else or_cnf unsat deduce (xcnf unsat deduce normalise0 negate0 pol0 e1)
-         (xcnf unsat deduce normalise0 negate0 pol0 e2)
+  then and_cnf_opt (xcnf unsat deduce normalise1 negate0 pol0 e1)
+         (xcnf unsat deduce normalise1 negate0 pol0 e2)
+  else or_cnf_opt unsat deduce (xcnf unsat deduce normalise1 negate0 pol0 e1)
+         (xcnf unsat deduce normalise1 negate0 pol0 e2)
 | D (e1, e2) ->
   if pol0
-  then or_cnf unsat deduce (xcnf unsat deduce normalise0 negate0 pol0 e1)
-         (xcnf unsat deduce normalise0 negate0 pol0 e2)
-  else and_cnf (xcnf unsat deduce normalise0 negate0 pol0 e1)
-         (xcnf unsat deduce normalise0 negate0 pol0 e2)
-| N e -> xcnf unsat deduce normalise0 negate0 (negb pol0) e
-| I (e1, e2) ->
+  then or_cnf_opt unsat deduce (xcnf unsat deduce normalise1 negate0 pol0 e1)
+         (xcnf unsat deduce normalise1 negate0 pol0 e2)
+  else and_cnf_opt (xcnf unsat deduce normalise1 negate0 pol0 e1)
+         (xcnf unsat deduce normalise1 negate0 pol0 e2)
+| N e -> xcnf unsat deduce normalise1 negate0 (negb pol0) e
+| I (e1, _, e2) ->
   if pol0
-  then or_cnf unsat deduce
-         (xcnf unsat deduce normalise0 negate0 (negb pol0) e1)
-         (xcnf unsat deduce normalise0 negate0 pol0 e2)
-  else and_cnf (xcnf unsat deduce normalise0 negate0 (negb pol0) e1)
-         (xcnf unsat deduce normalise0 negate0 pol0 e2)
+  then or_cnf_opt unsat deduce
+         (xcnf unsat deduce normalise1 negate0 (negb pol0) e1)
+         (xcnf unsat deduce normalise1 negate0 pol0 e2)
+  else and_cnf_opt (xcnf unsat deduce normalise1 negate0 (negb pol0) e1)
+         (xcnf unsat deduce normalise1 negate0 pol0 e2)
+
+(** val radd_term :
+    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> ('a1 * 'a2) -> ('a1, 'a2)
+    clause -> (('a1, 'a2) clause, 'a2 list) sum **)
+
+let rec radd_term unsat deduce t0 = function
+| [] ->
+  (match deduce (fst t0) (fst t0) with
+   | Some u -> if unsat u then Inr ((snd t0)::[]) else Inl (t0::[])
+   | None -> Inl (t0::[]))
+| t'::cl0 ->
+  (match deduce (fst t0) (fst t') with
+   | Some u ->
+     if unsat u
+     then Inr ((snd t0)::((snd t')::[]))
+     else (match radd_term unsat deduce t0 cl0 with
+           | Inl cl' -> Inl (t'::cl')
+           | Inr l -> Inr l)
+   | None ->
+     (match radd_term unsat deduce t0 cl0 with
+      | Inl cl' -> Inl (t'::cl')
+      | Inr l -> Inr l))
+
+(** val ror_clause :
+    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> ('a1 * 'a2) list -> ('a1,
+    'a2) clause -> (('a1, 'a2) clause, 'a2 list) sum **)
+
+let rec ror_clause unsat deduce cl1 cl2 =
+  match cl1 with
+  | [] -> Inl cl2
+  | t0::cl ->
+    (match radd_term unsat deduce t0 cl2 with
+     | Inl cl' -> ror_clause unsat deduce cl cl'
+     | Inr l -> Inr l)
+
+(** val xror_clause_cnf :
+    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> ('a1 * 'a2) list -> ('a1,
+    'a2) clause list -> ('a1, 'a2) clause list * 'a2 list **)
+
+let xror_clause_cnf unsat deduce t0 f =
+  fold_left (fun pat e ->
+    let acc,tg = pat in
+    (match ror_clause unsat deduce t0 e with
+     | Inl cl -> (cl::acc),tg
+     | Inr l -> acc,(rev_append tg l))) f ([],[])
+
+(** val ror_clause_cnf :
+    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> ('a1 * 'a2) list -> ('a1,
+    'a2) clause list -> ('a1, 'a2) clause list * 'a2 list **)
+
+let ror_clause_cnf unsat deduce t0 f =
+  match t0 with
+  | [] -> f,[]
+  | _::_ -> xror_clause_cnf unsat deduce t0 f
+
+(** val ror_cnf :
+    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> ('a1, 'a2) clause list ->
+    ('a1, 'a2) clause list -> ('a1, 'a2) cnf * 'a2 list **)
+
+let rec ror_cnf unsat deduce f f' =
+  match f with
+  | [] -> cnf_tt,[]
+  | e::rst ->
+    let rst_f',t0 = ror_cnf unsat deduce rst f' in
+    let e_f',t' = ror_clause_cnf unsat deduce e f' in
+    (rev_append rst_f' e_f'),(rev_append t0 t')
+
+(** val ror_cnf_opt :
+    ('a1 -> bool) -> ('a1 -> 'a1 -> 'a1 option) -> ('a1, 'a2) cnf -> ('a1,
+    'a2) cnf -> ('a1, 'a2) cnf * 'a2 list **)
+
+let ror_cnf_opt unsat deduce f1 f2 =
+  if is_cnf_tt f1
+  then cnf_tt,[]
+  else if is_cnf_tt f2
+       then cnf_tt,[]
+       else if is_cnf_ff f2 then f1,[] else ror_cnf unsat deduce f1 f2
+
+(** val ratom : ('a1, 'a2) cnf -> 'a2 -> ('a1, 'a2) cnf * 'a2 list **)
+
+let ratom c a =
+  if if is_cnf_ff c then true else is_cnf_tt c then c,(a::[]) else c,[]
+
+(** val rxcnf :
+    ('a2 -> bool) -> ('a2 -> 'a2 -> 'a2 option) -> ('a1 -> 'a3 -> ('a2, 'a3)
+    cnf) -> ('a1 -> 'a3 -> ('a2, 'a3) cnf) -> bool -> ('a1, 'a3, 'a4, 'a5)
+    tFormula -> ('a2, 'a3) cnf * 'a3 list **)
+
+let rec rxcnf unsat deduce normalise1 negate0 polarity = function
+| TT -> if polarity then cnf_tt,[] else cnf_ff,[]
+| FF -> if polarity then cnf_ff,[] else cnf_tt,[]
+| X _ -> cnf_ff,[]
+| A (x, t0) -> ratom (if polarity then normalise1 x t0 else negate0 x t0) t0
+| Cj (e1, e2) ->
+  let e3,t1 = rxcnf unsat deduce normalise1 negate0 polarity e1 in
+  let e4,t2 = rxcnf unsat deduce normalise1 negate0 polarity e2 in
+  if polarity
+  then (and_cnf_opt e3 e4),(rev_append t1 t2)
+  else let f',t' = ror_cnf_opt unsat deduce e3 e4 in
+       f',(rev_append t1 (rev_append t2 t'))
+| D (e1, e2) ->
+  let e3,t1 = rxcnf unsat deduce normalise1 negate0 polarity e1 in
+  let e4,t2 = rxcnf unsat deduce normalise1 negate0 polarity e2 in
+  if polarity
+  then let f',t' = ror_cnf_opt unsat deduce e3 e4 in
+       f',(rev_append t1 (rev_append t2 t'))
+  else (and_cnf_opt e3 e4),(rev_append t1 t2)
+| N e -> rxcnf unsat deduce normalise1 negate0 (negb polarity) e
+| I (e1, _, e2) ->
+  let e3,t1 = rxcnf unsat deduce normalise1 negate0 (negb polarity) e1 in
+  if polarity
+  then if is_cnf_ff e3
+       then rxcnf unsat deduce normalise1 negate0 polarity e2
+       else let e4,t2 = rxcnf unsat deduce normalise1 negate0 polarity e2 in
+            let f',t' = ror_cnf_opt unsat deduce e3 e4 in
+            f',(rev_append t1 (rev_append t2 t'))
+  else let e4,t2 = rxcnf unsat deduce normalise1 negate0 polarity e2 in
+       (and_cnf_opt e3 e4),(rev_append t1 t2)
+
+type ('term, 'annot, 'tX) to_constrT = { mkTT : 'tX; mkFF : 'tX;
+                                         mkA : ('term -> 'annot -> 'tX);
+                                         mkCj : ('tX -> 'tX -> 'tX);
+                                         mkD : ('tX -> 'tX -> 'tX);
+                                         mkI : ('tX -> 'tX -> 'tX);
+                                         mkN : ('tX -> 'tX) }
+
+(** val aformula :
+    ('a1, 'a2, 'a3) to_constrT -> ('a1, 'a2, 'a3, 'a4) tFormula -> 'a3 **)
+
+let rec aformula to_constr = function
+| TT -> to_constr.mkTT
+| FF -> to_constr.mkFF
+| X p -> p
+| A (x, t0) -> to_constr.mkA x t0
+| Cj (f1, f2) ->
+  to_constr.mkCj (aformula to_constr f1) (aformula to_constr f2)
+| D (f1, f2) -> to_constr.mkD (aformula to_constr f1) (aformula to_constr f2)
+| N f0 -> to_constr.mkN (aformula to_constr f0)
+| I (f1, _, f2) ->
+  to_constr.mkI (aformula to_constr f1) (aformula to_constr f2)
+
+(** val is_X : ('a1, 'a2, 'a3, 'a4) tFormula -> 'a3 option **)
+
+let is_X = function
+| X p -> Some p
+| _ -> None
+
+(** val abs_and :
+    ('a1, 'a2, 'a3) to_constrT -> ('a1, 'a2, 'a3, 'a4) tFormula -> ('a1, 'a2,
+    'a3, 'a4) tFormula -> (('a1, 'a2, 'a3, 'a4) tFormula -> ('a1, 'a2, 'a3,
+    'a4) tFormula -> ('a1, 'a2, 'a3, 'a4) tFormula) -> ('a1, 'a3, 'a2, 'a4)
+    gFormula **)
+
+let abs_and to_constr f1 f2 c =
+  match is_X f1 with
+  | Some _ -> X (aformula to_constr (c f1 f2))
+  | None ->
+    (match is_X f2 with
+     | Some _ -> X (aformula to_constr (c f1 f2))
+     | None -> c f1 f2)
+
+(** val abs_or :
+    ('a1, 'a2, 'a3) to_constrT -> ('a1, 'a2, 'a3, 'a4) tFormula -> ('a1, 'a2,
+    'a3, 'a4) tFormula -> (('a1, 'a2, 'a3, 'a4) tFormula -> ('a1, 'a2, 'a3,
+    'a4) tFormula -> ('a1, 'a2, 'a3, 'a4) tFormula) -> ('a1, 'a3, 'a2, 'a4)
+    gFormula **)
+
+let abs_or to_constr f1 f2 c =
+  match is_X f1 with
+  | Some _ ->
+    (match is_X f2 with
+     | Some _ -> X (aformula to_constr (c f1 f2))
+     | None -> c f1 f2)
+  | None -> c f1 f2
+
+(** val mk_arrow :
+    'a4 option -> ('a1, 'a2, 'a3, 'a4) tFormula -> ('a1, 'a2, 'a3, 'a4)
+    tFormula -> ('a1, 'a2, 'a3, 'a4) tFormula **)
+
+let mk_arrow o f1 f2 =
+  match o with
+  | Some _ -> (match is_X f1 with
+               | Some _ -> f2
+               | None -> I (f1, o, f2))
+  | None -> I (f1, None, f2)
+
+(** val abst_form :
+    ('a1, 'a2, 'a3) to_constrT -> ('a2 -> bool) -> bool -> ('a1, 'a2, 'a3,
+    'a4) tFormula -> ('a1, 'a3, 'a2, 'a4) gFormula **)
+
+let rec abst_form to_constr needA pol0 = function
+| TT -> if pol0 then TT else X to_constr.mkTT
+| FF -> if pol0 then X to_constr.mkFF else FF
+| X p -> X p
+| A (x, t0) -> if needA t0 then A (x, t0) else X (to_constr.mkA x t0)
+| Cj (f1, f2) ->
+  let f3 = abst_form to_constr needA pol0 f1 in
+  let f4 = abst_form to_constr needA pol0 f2 in
+  if pol0
+  then abs_and to_constr f3 f4 (fun x x0 -> Cj (x, x0))
+  else abs_or to_constr f3 f4 (fun x x0 -> Cj (x, x0))
+| D (f1, f2) ->
+  let f3 = abst_form to_constr needA pol0 f1 in
+  let f4 = abst_form to_constr needA pol0 f2 in
+  if pol0
+  then abs_or to_constr f3 f4 (fun x x0 -> D (x, x0))
+  else abs_and to_constr f3 f4 (fun x x0 -> D (x, x0))
+| N f0 ->
+  let f1 = abst_form to_constr needA (negb pol0) f0 in
+  (match is_X f1 with
+   | Some a -> X (to_constr.mkN a)
+   | None -> N f1)
+| I (f1, o, f2) ->
+  let f3 = abst_form to_constr needA (negb pol0) f1 in
+  let f4 = abst_form to_constr needA pol0 f2 in
+  if pol0
+  then abs_or to_constr f3 f4 (mk_arrow o)
+  else abs_and to_constr f3 f4 (mk_arrow o)
 
 (** val cnf_checker :
-    ('a1 list -> 'a2 -> bool) -> 'a1 cnf -> 'a2 list -> bool **)
+    (('a1 * 'a2) list -> 'a3 -> bool) -> ('a1, 'a2) cnf -> 'a3 list -> bool **)
 
 let rec cnf_checker checker f l =
   match f with
@@ -1026,12 +1419,12 @@ let rec cnf_checker checker f l =
      | c::l0 -> if checker e c then cnf_checker checker f0 l0 else false)
 
 (** val tauto_checker :
-    ('a2 -> bool) -> ('a2 -> 'a2 -> 'a2 option) -> ('a1 -> 'a2 cnf) -> ('a1
-    -> 'a2 cnf) -> ('a2 list -> 'a3 -> bool) -> 'a1 bFormula -> 'a3 list ->
-    bool **)
+    ('a2 -> bool) -> ('a2 -> 'a2 -> 'a2 option) -> ('a1 -> 'a3 -> ('a2, 'a3)
+    cnf) -> ('a1 -> 'a3 -> ('a2, 'a3) cnf) -> (('a2 * 'a3) list -> 'a4 ->
+    bool) -> ('a1, __, 'a3, unit0) gFormula -> 'a4 list -> bool **)
 
-let tauto_checker unsat deduce normalise0 negate0 checker f w =
-  cnf_checker checker (xcnf unsat deduce normalise0 negate0 true f) w
+let tauto_checker unsat deduce normalise1 negate0 checker f w =
+  cnf_checker checker (xcnf unsat deduce normalise1 negate0 true f) w
 
 (** val cneqb : ('a1 -> 'a1 -> bool) -> 'a1 -> 'a1 -> bool **)
 
@@ -1221,61 +1614,76 @@ let psub0 =
 let padd0 =
   padd
 
-(** val xnormalise :
+(** val popp0 : ('a1 -> 'a1) -> 'a1 pol -> 'a1 pol **)
+
+let popp0 =
+  popp
+
+(** val normalise :
     'a1 -> 'a1 -> ('a1 -> 'a1 -> 'a1) -> ('a1 -> 'a1 -> 'a1) -> ('a1 -> 'a1
     -> 'a1) -> ('a1 -> 'a1) -> ('a1 -> 'a1 -> bool) -> 'a1 formula -> 'a1
-    nFormula list **)
+    nFormula **)
 
-let xnormalise cO cI cplus ctimes cminus copp ceqb t0 =
-  let { flhs = lhs; fop = o; frhs = rhs } = t0 in
+let normalise cO cI cplus ctimes cminus copp ceqb f =
+  let { flhs = lhs; fop = op; frhs = rhs } = f in
   let lhs0 = norm cO cI cplus ctimes cminus copp ceqb lhs in
   let rhs0 = norm cO cI cplus ctimes cminus copp ceqb rhs in
+  (match op with
+   | OpEq -> (psub0 cO cplus cminus copp ceqb lhs0 rhs0),Equal
+   | OpNEq -> (psub0 cO cplus cminus copp ceqb lhs0 rhs0),NonEqual
+   | OpLe -> (psub0 cO cplus cminus copp ceqb rhs0 lhs0),NonStrict
+   | OpGe -> (psub0 cO cplus cminus copp ceqb lhs0 rhs0),NonStrict
+   | OpLt -> (psub0 cO cplus cminus copp ceqb rhs0 lhs0),Strict
+   | OpGt -> (psub0 cO cplus cminus copp ceqb lhs0 rhs0),Strict)
+
+(** val xnormalise : ('a1 -> 'a1) -> 'a1 nFormula -> 'a1 nFormula list **)
+
+let xnormalise copp = function
+| e,o ->
   (match o with
-   | OpEq ->
-     ((psub0 cO cplus cminus copp ceqb lhs0 rhs0),Strict)::(((psub0 cO cplus
-                                                               cminus copp
-                                                               ceqb rhs0 lhs0),Strict)::[])
-   | OpNEq -> ((psub0 cO cplus cminus copp ceqb lhs0 rhs0),Equal)::[]
-   | OpLe -> ((psub0 cO cplus cminus copp ceqb lhs0 rhs0),Strict)::[]
-   | OpGe -> ((psub0 cO cplus cminus copp ceqb rhs0 lhs0),Strict)::[]
-   | OpLt -> ((psub0 cO cplus cminus copp ceqb lhs0 rhs0),NonStrict)::[]
-   | OpGt -> ((psub0 cO cplus cminus copp ceqb rhs0 lhs0),NonStrict)::[])
+   | Equal -> (e,Strict)::(((popp0 copp e),Strict)::[])
+   | NonEqual -> (e,Equal)::[]
+   | Strict -> ((popp0 copp e),NonStrict)::[]
+   | NonStrict -> ((popp0 copp e),Strict)::[])
+
+(** val xnegate : ('a1 -> 'a1) -> 'a1 nFormula -> 'a1 nFormula list **)
+
+let xnegate copp = function
+| e,o ->
+  (match o with
+   | NonEqual -> (e,Strict)::(((popp0 copp e),Strict)::[])
+   | x -> (e,x)::[])
+
+(** val cnf_of_list :
+    'a1 -> ('a1 -> 'a1 -> bool) -> ('a1 -> 'a1 -> bool) -> 'a1 nFormula list
+    -> 'a2 -> ('a1 nFormula, 'a2) cnf **)
+
+let cnf_of_list cO ceqb cleb l tg =
+  fold_right (fun x acc ->
+    if check_inconsistent cO ceqb cleb x then acc else ((x,tg)::[])::acc)
+    cnf_tt l
 
 (** val cnf_normalise :
     'a1 -> 'a1 -> ('a1 -> 'a1 -> 'a1) -> ('a1 -> 'a1 -> 'a1) -> ('a1 -> 'a1
-    -> 'a1) -> ('a1 -> 'a1) -> ('a1 -> 'a1 -> bool) -> 'a1 formula -> 'a1
-    nFormula cnf **)
+    -> 'a1) -> ('a1 -> 'a1) -> ('a1 -> 'a1 -> bool) -> ('a1 -> 'a1 -> bool)
+    -> 'a1 formula -> 'a2 -> ('a1 nFormula, 'a2) cnf **)
 
-let cnf_normalise cO cI cplus ctimes cminus copp ceqb t0 =
-  map (fun x -> x::[]) (xnormalise cO cI cplus ctimes cminus copp ceqb t0)
-
-(** val xnegate :
-    'a1 -> 'a1 -> ('a1 -> 'a1 -> 'a1) -> ('a1 -> 'a1 -> 'a1) -> ('a1 -> 'a1
-    -> 'a1) -> ('a1 -> 'a1) -> ('a1 -> 'a1 -> bool) -> 'a1 formula -> 'a1
-    nFormula list **)
-
-let xnegate cO cI cplus ctimes cminus copp ceqb t0 =
-  let { flhs = lhs; fop = o; frhs = rhs } = t0 in
-  let lhs0 = norm cO cI cplus ctimes cminus copp ceqb lhs in
-  let rhs0 = norm cO cI cplus ctimes cminus copp ceqb rhs in
-  (match o with
-   | OpEq -> ((psub0 cO cplus cminus copp ceqb lhs0 rhs0),Equal)::[]
-   | OpNEq ->
-     ((psub0 cO cplus cminus copp ceqb lhs0 rhs0),Strict)::(((psub0 cO cplus
-                                                               cminus copp
-                                                               ceqb rhs0 lhs0),Strict)::[])
-   | OpLe -> ((psub0 cO cplus cminus copp ceqb rhs0 lhs0),NonStrict)::[]
-   | OpGe -> ((psub0 cO cplus cminus copp ceqb lhs0 rhs0),NonStrict)::[]
-   | OpLt -> ((psub0 cO cplus cminus copp ceqb rhs0 lhs0),Strict)::[]
-   | OpGt -> ((psub0 cO cplus cminus copp ceqb lhs0 rhs0),Strict)::[])
+let cnf_normalise cO cI cplus ctimes cminus copp ceqb cleb t0 tg =
+  let f = normalise cO cI cplus ctimes cminus copp ceqb t0 in
+  if check_inconsistent cO ceqb cleb f
+  then cnf_ff
+  else cnf_of_list cO ceqb cleb (xnormalise copp f) tg
 
 (** val cnf_negate :
     'a1 -> 'a1 -> ('a1 -> 'a1 -> 'a1) -> ('a1 -> 'a1 -> 'a1) -> ('a1 -> 'a1
-    -> 'a1) -> ('a1 -> 'a1) -> ('a1 -> 'a1 -> bool) -> 'a1 formula -> 'a1
-    nFormula cnf **)
+    -> 'a1) -> ('a1 -> 'a1) -> ('a1 -> 'a1 -> bool) -> ('a1 -> 'a1 -> bool)
+    -> 'a1 formula -> 'a2 -> ('a1 nFormula, 'a2) cnf **)
 
-let cnf_negate cO cI cplus ctimes cminus copp ceqb t0 =
-  map (fun x -> x::[]) (xnegate cO cI cplus ctimes cminus copp ceqb t0)
+let cnf_negate cO cI cplus ctimes cminus copp ceqb cleb t0 tg =
+  let f = normalise cO cI cplus ctimes cminus copp ceqb t0 in
+  if check_inconsistent cO ceqb cleb f
+  then cnf_tt
+  else cnf_of_list cO ceqb cleb (xnegate copp f) tg
 
 (** val xdenorm : positive -> 'a1 pol -> 'a1 pExpr **)
 
@@ -1368,14 +1776,6 @@ let simpl_cone cO cI ctimes ceqb e = match e with
 
 type q = { qnum : z; qden : positive }
 
-(** val qnum : q -> z **)
-
-let qnum x = x.qnum
-
-(** val qden : q -> positive **)
-
-let qden x = x.qden
-
 (** val qeq_bool : q -> q -> bool **)
 
 let qeq_bool x y =
@@ -1429,16 +1829,16 @@ let qpower q0 = function
 
 type 'a t =
 | Empty
-| Leaf of 'a
-| Node of 'a t * 'a * 'a t
+| Elt of 'a
+| Branch of 'a t * 'a * 'a t
 
 (** val find : 'a1 -> 'a1 t -> positive -> 'a1 **)
 
 let rec find default vm p =
   match vm with
   | Empty -> default
-  | Leaf i -> i
-  | Node (l, e, r) ->
+  | Elt i -> i
+  | Branch (l, e, r) ->
     (match p with
      | XI p2 -> find default r p2
      | XO p2 -> find default l p2
@@ -1448,24 +1848,39 @@ let rec find default vm p =
 
 let rec singleton default x v =
   match x with
-  | XI p -> Node (Empty, default, (singleton default p v))
-  | XO p -> Node ((singleton default p v), default, Empty)
-  | XH -> Leaf v
+  | XI p -> Branch (Empty, default, (singleton default p v))
+  | XO p -> Branch ((singleton default p v), default, Empty)
+  | XH -> Elt v
 
 (** val vm_add : 'a1 -> positive -> 'a1 -> 'a1 t -> 'a1 t **)
 
 let rec vm_add default x v = function
 | Empty -> singleton default x v
-| Leaf vl ->
+| Elt vl ->
   (match x with
-   | XI p -> Node (Empty, vl, (singleton default p v))
-   | XO p -> Node ((singleton default p v), vl, Empty)
-   | XH -> Leaf v)
-| Node (l, o, r) ->
+   | XI p -> Branch (Empty, vl, (singleton default p v))
+   | XO p -> Branch ((singleton default p v), vl, Empty)
+   | XH -> Elt v)
+| Branch (l, o, r) ->
   (match x with
-   | XI p -> Node (l, o, (vm_add default p v r))
-   | XO p -> Node ((vm_add default p v l), o, r)
-   | XH -> Node (l, v, r))
+   | XI p -> Branch (l, o, (vm_add default p v r))
+   | XO p -> Branch ((vm_add default p v l), o, r)
+   | XH -> Branch (l, v, r))
+
+(** val zeval_const : z pExpr -> z option **)
+
+let rec zeval_const = function
+| PEc c -> Some c
+| PEX _ -> None
+| PEadd (e1, e2) ->
+  map_option2 (fun x y -> Some (Z.add x y)) (zeval_const e1) (zeval_const e2)
+| PEsub (e1, e2) ->
+  map_option2 (fun x y -> Some (Z.sub x y)) (zeval_const e1) (zeval_const e2)
+| PEmul (e1, e2) ->
+  map_option2 (fun x y -> Some (Z.mul x y)) (zeval_const e1) (zeval_const e2)
+| PEopp e0 -> map_option (fun x -> Some (Z.opp x)) (zeval_const e0)
+| PEpow (e1, n0) ->
+  map_option (fun x -> Some (Z.pow x (Z.of_N n0))) (zeval_const e1)
 
 type zWitness = z psatz
 
@@ -1484,57 +1899,11 @@ let psub1 =
 let padd1 =
   padd0 Z0 Z.add zeq_bool
 
-(** val norm0 : z pExpr -> z pol **)
+(** val normZ : z pExpr -> z pol **)
 
-let norm0 =
+let normZ =
   norm Z0 (Zpos XH) Z.add Z.mul Z.sub Z.opp zeq_bool
 
-(** val xnormalise0 : z formula -> z nFormula list **)
-
-let xnormalise0 t0 =
-  let { flhs = lhs; fop = o; frhs = rhs } = t0 in
-  let lhs0 = norm0 lhs in
-  let rhs0 = norm0 rhs in
-  (match o with
-   | OpEq ->
-     ((psub1 lhs0 (padd1 rhs0 (Pc (Zpos XH)))),NonStrict)::(((psub1 rhs0
-                                                               (padd1 lhs0
-                                                                 (Pc (Zpos
-                                                                 XH)))),NonStrict)::[])
-   | OpNEq -> ((psub1 lhs0 rhs0),Equal)::[]
-   | OpLe -> ((psub1 lhs0 (padd1 rhs0 (Pc (Zpos XH)))),NonStrict)::[]
-   | OpGe -> ((psub1 rhs0 (padd1 lhs0 (Pc (Zpos XH)))),NonStrict)::[]
-   | OpLt -> ((psub1 lhs0 rhs0),NonStrict)::[]
-   | OpGt -> ((psub1 rhs0 lhs0),NonStrict)::[])
-
-(** val normalise : z formula -> z nFormula cnf **)
-
-let normalise t0 =
-  map (fun x -> x::[]) (xnormalise0 t0)
-
-(** val xnegate0 : z formula -> z nFormula list **)
-
-let xnegate0 t0 =
-  let { flhs = lhs; fop = o; frhs = rhs } = t0 in
-  let lhs0 = norm0 lhs in
-  let rhs0 = norm0 rhs in
-  (match o with
-   | OpEq -> ((psub1 lhs0 rhs0),Equal)::[]
-   | OpNEq ->
-     ((psub1 lhs0 (padd1 rhs0 (Pc (Zpos XH)))),NonStrict)::(((psub1 rhs0
-                                                               (padd1 lhs0
-                                                                 (Pc (Zpos
-                                                                 XH)))),NonStrict)::[])
-   | OpLe -> ((psub1 rhs0 lhs0),NonStrict)::[]
-   | OpGe -> ((psub1 lhs0 rhs0),NonStrict)::[]
-   | OpLt -> ((psub1 rhs0 (padd1 lhs0 (Pc (Zpos XH)))),NonStrict)::[]
-   | OpGt -> ((psub1 lhs0 (padd1 rhs0 (Pc (Zpos XH)))),NonStrict)::[])
-
-(** val negate : z formula -> z nFormula cnf **)
-
-let negate t0 =
-  map (fun x -> x::[]) (xnegate0 t0)
-
 (** val zunsat : z nFormula -> bool **)
 
 let zunsat =
@@ -1545,6 +1914,66 @@ let zunsat =
 let zdeduce =
   nformula_plus_nformula Z0 Z.add zeq_bool
 
+(** val xnnormalise : z formula -> z nFormula **)
+
+let xnnormalise t0 =
+  let { flhs = lhs; fop = o; frhs = rhs } = t0 in
+  let lhs0 = normZ lhs in
+  let rhs0 = normZ rhs in
+  (match o with
+   | OpEq -> (psub1 rhs0 lhs0),Equal
+   | OpNEq -> (psub1 rhs0 lhs0),NonEqual
+   | OpLe -> (psub1 rhs0 lhs0),NonStrict
+   | OpGe -> (psub1 lhs0 rhs0),NonStrict
+   | OpLt -> (psub1 rhs0 lhs0),Strict
+   | OpGt -> (psub1 lhs0 rhs0),Strict)
+
+(** val xnormalise0 : z nFormula -> z nFormula list **)
+
+let xnormalise0 = function
+| e,o ->
+  (match o with
+   | Equal ->
+     ((psub1 e (Pc (Zpos XH))),NonStrict)::(((psub1 (Pc (Zneg XH)) e),NonStrict)::[])
+   | NonEqual -> (e,Equal)::[]
+   | Strict -> ((psub1 (Pc Z0) e),NonStrict)::[]
+   | NonStrict -> ((psub1 (Pc (Zneg XH)) e),NonStrict)::[])
+
+(** val cnf_of_list0 :
+    'a1 -> z nFormula list -> (z nFormula * 'a1) list list **)
+
+let cnf_of_list0 tg l =
+  fold_right (fun x acc -> if zunsat x then acc else ((x,tg)::[])::acc)
+    cnf_tt l
+
+(** val normalise0 : z formula -> 'a1 -> (z nFormula, 'a1) cnf **)
+
+let normalise0 t0 tg =
+  let f = xnnormalise t0 in
+  if zunsat f then cnf_ff else cnf_of_list0 tg (xnormalise0 f)
+
+(** val xnegate0 : z nFormula -> z nFormula list **)
+
+let xnegate0 = function
+| e,o ->
+  (match o with
+   | NonEqual ->
+     ((psub1 e (Pc (Zpos XH))),NonStrict)::(((psub1 (Pc (Zneg XH)) e),NonStrict)::[])
+   | Strict -> ((psub1 e (Pc (Zpos XH))),NonStrict)::[]
+   | x -> (e,x)::[])
+
+(** val negate : z formula -> 'a1 -> (z nFormula, 'a1) cnf **)
+
+let negate t0 tg =
+  let f = xnnormalise t0 in
+  if zunsat f then cnf_tt else cnf_of_list0 tg (xnegate0 f)
+
+(** val cnfZ :
+    (z formula, 'a1, 'a2, 'a3) tFormula -> (z nFormula, 'a1) cnf * 'a1 list **)
+
+let cnfZ f =
+  rxcnf zunsat zdeduce normalise0 negate true f
+
 (** val ceiling : z -> z -> z **)
 
 let ceiling a b =
@@ -1558,6 +1987,7 @@ type zArithProof =
 | RatProof of zWitness * zArithProof
 | CutProof of zWitness * zArithProof
 | EnumProof of zWitness * zWitness * zArithProof list
+| ExProof of positive * zArithProof
 
 (** val zgcdM : z -> z -> z **)
 
@@ -1629,6 +2059,29 @@ let valid_cut_sign = function
 | NonStrict -> true
 | _ -> false
 
+(** val bound_var : positive -> z formula **)
+
+let bound_var v =
+  { flhs = (PEX v); fop = OpGe; frhs = (PEc Z0) }
+
+(** val mk_eq_pos : positive -> positive -> positive -> z formula **)
+
+let mk_eq_pos x y t0 =
+  { flhs = (PEX x); fop = OpEq; frhs = (PEsub ((PEX y), (PEX t0))) }
+
+(** val max_var : positive -> z pol -> positive **)
+
+let rec max_var jmp = function
+| Pc _ -> jmp
+| Pinj (j, p2) -> max_var (Coq_Pos.add j jmp) p2
+| PX (p2, _, q0) ->
+  Coq_Pos.max (max_var jmp p2) (max_var (Coq_Pos.succ jmp) q0)
+
+(** val max_var_nformulae : z nFormula list -> positive **)
+
+let max_var_nformulae l =
+  fold_left (fun acc f -> Coq_Pos.max acc (max_var XH (fst f))) l XH
+
 (** val zChecker : z nFormula list -> zArithProof -> bool **)
 
 let rec zChecker l = function
@@ -1671,11 +2124,22 @@ let rec zChecker l = function
          | None -> true)
       | None -> false)
    | None -> false)
+| ExProof (x, prf) ->
+  let fr = max_var_nformulae l in
+  if Coq_Pos.leb x fr
+  then let z0 = Coq_Pos.succ fr in
+       let t0 = Coq_Pos.succ z0 in
+       let nfx = xnnormalise (mk_eq_pos x z0 t0) in
+       let posz = xnnormalise (bound_var z0) in
+       let post = xnnormalise (bound_var t0) in
+       zChecker (nfx::(posz::(post::l))) prf
+  else false
 
 (** val zTautoChecker : z formula bFormula -> zArithProof list -> bool **)
 
 let zTautoChecker f w =
-  tauto_checker zunsat zdeduce normalise negate zChecker f w
+  tauto_checker zunsat zdeduce normalise0 negate (fun cl ->
+    zChecker (map fst cl)) f w
 
 type qWitness = q psatz
 
@@ -1685,17 +2149,17 @@ let qWeakChecker =
   check_normalised_formulas { qnum = Z0; qden = XH } { qnum = (Zpos XH);
     qden = XH } qplus qmult qeq_bool qle_bool
 
-(** val qnormalise : q formula -> q nFormula cnf **)
+(** val qnormalise : q formula -> 'a1 -> (q nFormula, 'a1) cnf **)
 
-let qnormalise =
+let qnormalise t0 tg =
   cnf_normalise { qnum = Z0; qden = XH } { qnum = (Zpos XH); qden = XH }
-    qplus qmult qminus qopp qeq_bool
+    qplus qmult qminus qopp qeq_bool qle_bool t0 tg
 
-(** val qnegate : q formula -> q nFormula cnf **)
+(** val qnegate : q formula -> 'a1 -> (q nFormula, 'a1) cnf **)
 
-let qnegate =
+let qnegate t0 tg =
   cnf_negate { qnum = Z0; qden = XH } { qnum = (Zpos XH); qden = XH } qplus
-    qmult qminus qopp qeq_bool
+    qmult qminus qopp qeq_bool qle_bool t0 tg
 
 (** val qunsat : q nFormula -> bool **)
 
@@ -1707,10 +2171,23 @@ let qunsat =
 let qdeduce =
   nformula_plus_nformula { qnum = Z0; qden = XH } qplus qeq_bool
 
+(** val normQ : q pExpr -> q pol **)
+
+let normQ =
+  norm { qnum = Z0; qden = XH } { qnum = (Zpos XH); qden = XH } qplus qmult
+    qminus qopp qeq_bool
+
+(** val cnfQ :
+    (q formula, 'a1, 'a2, 'a3) tFormula -> (q nFormula, 'a1) cnf * 'a1 list **)
+
+let cnfQ f =
+  rxcnf qunsat qdeduce qnormalise qnegate true f
+
 (** val qTautoChecker : q formula bFormula -> qWitness list -> bool **)
 
 let qTautoChecker f w =
-  tauto_checker qunsat qdeduce qnormalise qnegate qWeakChecker f w
+  tauto_checker qunsat qdeduce qnormalise qnegate (fun cl ->
+    qWeakChecker (map fst cl)) f w
 
 type rcst =
 | C0
@@ -1720,9 +2197,16 @@ type rcst =
 | CPlus of rcst * rcst
 | CMinus of rcst * rcst
 | CMult of rcst * rcst
+| CPow of rcst * (z, nat) sum
 | CInv of rcst
 | COpp of rcst
 
+(** val z_of_exp : (z, nat) sum -> z **)
+
+let z_of_exp = function
+| Inl z1 -> z1
+| Inr n0 -> Z.of_nat n0
+
 (** val q_of_Rcst : rcst -> q **)
 
 let rec q_of_Rcst = function
@@ -1733,6 +2217,7 @@ let rec q_of_Rcst = function
 | CPlus (r1, r2) -> qplus (q_of_Rcst r1) (q_of_Rcst r2)
 | CMinus (r1, r2) -> qminus (q_of_Rcst r1) (q_of_Rcst r2)
 | CMult (r1, r2) -> qmult (q_of_Rcst r1) (q_of_Rcst r2)
+| CPow (r1, z0) -> qpower (q_of_Rcst r1) (z_of_exp z0)
 | CInv r0 -> qinv (q_of_Rcst r0)
 | COpp r0 -> qopp (q_of_Rcst r0)
 
@@ -1744,17 +2229,17 @@ let rWeakChecker =
   check_normalised_formulas { qnum = Z0; qden = XH } { qnum = (Zpos XH);
     qden = XH } qplus qmult qeq_bool qle_bool
 
-(** val rnormalise : q formula -> q nFormula cnf **)
+(** val rnormalise : q formula -> 'a1 -> (q nFormula, 'a1) cnf **)
 
-let rnormalise =
+let rnormalise t0 tg =
   cnf_normalise { qnum = Z0; qden = XH } { qnum = (Zpos XH); qden = XH }
-    qplus qmult qminus qopp qeq_bool
+    qplus qmult qminus qopp qeq_bool qle_bool t0 tg
 
-(** val rnegate : q formula -> q nFormula cnf **)
+(** val rnegate : q formula -> 'a1 -> (q nFormula, 'a1) cnf **)
 
-let rnegate =
+let rnegate t0 tg =
   cnf_negate { qnum = Z0; qden = XH } { qnum = (Zpos XH); qden = XH } qplus
-    qmult qminus qopp qeq_bool
+    qmult qminus qopp qeq_bool qle_bool t0 tg
 
 (** val runsat : q nFormula -> bool **)
 
@@ -1769,5 +2254,5 @@ let rdeduce =
 (** val rTautoChecker : rcst formula bFormula -> rWitness list -> bool **)
 
 let rTautoChecker f w =
-  tauto_checker runsat rdeduce rnormalise rnegate rWeakChecker
-    (map_bformula (map_Formula q_of_Rcst) f) w
+  tauto_checker runsat rdeduce rnormalise rnegate (fun cl ->
+    rWeakChecker (map fst cl)) (map_bformula (map_Formula q_of_Rcst) f) w
