@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -8,6 +8,7 @@
 (*         *     (see LICENSE file for the text of the license)         *)
 (************************************************************************)
 
+module CVars = Vars
 open Ltac_plugin
 open Pp
 open Util
@@ -17,7 +18,6 @@ open EConstr
 open Vars
 open CClosure
 open Environ
-open Libnames
 open Globnames
 open Glob_term
 open Locus
@@ -28,7 +28,6 @@ open Tacinterp
 open Libobject
 open Printer
 open Declare
-open Decl_kinds
 open Entries
 open Newring_ast
 open Proofview.Notations
@@ -42,14 +41,14 @@ type protect_flag = Eval|Prot|Rec
 
 type protection = Evd.evar_map -> EConstr.t -> GlobRef.t -> (Int.t -> protect_flag) option
 
-let global_head_of_constr sigma c = 
+let global_head_of_constr sigma c =
   let f, args = decompose_app sigma c in
     try fst (Termops.global_of_constr sigma f)
     with Not_found -> CErrors.anomaly (str "global_head_of_constr.")
 
-let global_of_constr_nofail c = 
+let global_of_constr_nofail c =
   try global_of_constr c
-  with Not_found -> VarRef (Id.of_string "dummy")
+  with Not_found -> GlobRef.VarRef (Id.of_string "dummy")
 
 let rec mk_clos_but f_map n t =
   let (f, args) = Constr.decompose_appvect t in
@@ -88,10 +87,10 @@ let protect_red map env sigma c0 =
   EConstr.of_constr (eval 0 c)
 
 let protect_tac map =
-  Tactics.reduct_option (protect_red map,DEFAULTcast) None
+  Tactics.reduct_option ~check:false (protect_red map,DEFAULTcast) None
 
 let protect_tac_in map id =
-  Tactics.reduct_option (protect_red map,DEFAULTcast) (Some(id, Locus.InHyp))
+  Tactics.reduct_option ~check:false (protect_red map,DEFAULTcast) (Some(id, Locus.InHyp))
 
 
 (****************************************************************************)
@@ -99,7 +98,7 @@ let protect_tac_in map id =
 let rec closed_under sigma cset t =
   try
     let (gr, _) = Termops.global_of_constr sigma t in
-    Refset_env.mem gr cset
+    GlobRef.Set_env.mem gr cset
   with Not_found ->
     match EConstr.kind sigma t with
     | Cast(c,_,_) -> closed_under sigma cset c
@@ -111,7 +110,7 @@ let closed_term args _ = match args with
   let t = Option.get (Value.to_constr t) in
   let l = List.map (fun c -> Value.cast (Genarg.topwit Stdarg.wit_ref) c) (Option.get (Value.to_list l)) in
   Proofview.tclEVARMAP >>= fun sigma ->
-  let cs = List.fold_right Refset_env.add l Refset_env.empty in
+  let cs = List.fold_right GlobRef.Set_env.add l GlobRef.Set_env.empty in
   if closed_under sigma cs t then Proofview.tclUNIT () else Tacticals.New.tclFAIL 0 (mt())
 | _ -> assert false
 
@@ -128,7 +127,7 @@ let closed_term_ast =
   fun l ->
   let l = List.map (fun gr -> ArgArg(Loc.tag gr)) l in
   TacFun([Name(Id.of_string"t")],
-  TacML(Loc.tag (tacname,
+  TacML(CAst.make (tacname,
   [TacGeneric (Genarg.in_gen (Genarg.glbwit Stdarg.wit_constr) (DAst.make @@ GVar(Id.of_string"t"),None));
    TacGeneric (Genarg.in_gen (Genarg.glbwit (Genarg.wit_list Stdarg.wit_ref)) l)])))
 (*
@@ -140,8 +139,8 @@ let _ = add_tacdef false ((Loc.ghost,Id.of_string"ring_closed_term"
 let ic c =
   let env = Global.env() in
   let sigma = Evd.from_env env in
-  let sigma, c = Constrintern.interp_open_constr env sigma c in
-  (sigma, c)
+  let c, uctx = Constrintern.interp_constr env sigma c in
+  (Evd.from_ctx uctx, c)
 
 let ic_unsafe c = (*FIXME remove *)
   let env = Global.env() in
@@ -150,20 +149,22 @@ let ic_unsafe c = (*FIXME remove *)
 
 let decl_constant na univs c =
   let open Constr in
-  let vars = Univops.universes_of_constr c in
-  let univs = Univops.restrict_universe_context univs vars in
-  let univs = Monomorphic_const_entry univs in
-  mkConst(declare_constant (Id.of_string na) 
-            (DefinitionEntry (definition_entry ~opaque:true ~univs c),
-	     IsProof Lemma))
+  let vars = CVars.universes_of_constr c in
+  let univs = UState.restrict_universe_context ~lbound:(Global.universes_lbound ()) univs vars in
+  let () = Declare.declare_universe_context ~poly:false univs in
+  let types = (Typeops.infer (Global.env ()) c).uj_type in
+  let univs = Monomorphic_entry Univ.ContextSet.empty in
+  mkConst(declare_constant ~name:(Id.of_string na)
+            ~kind:Decls.(IsProof Lemma)
+            (DefinitionEntry (definition_entry ~opaque:true ~types ~univs c)))
 
 (* Calling a global tactic *)
 let ltac_call tac (args:glob_tactic_arg list) =
-  TacArg(Loc.tag @@ TacCall (Loc.tag (ArgArg(Loc.tag @@ Lazy.force tac),args)))
+  TacArg(CAst.make @@ TacCall (CAst.make (ArgArg(Loc.tag @@ Lazy.force tac),args)))
 
 let dummy_goal env sigma =
-  let (gl,_,sigma) = 
-    Goal.V82.mk_goal sigma (named_context_val env) EConstr.mkProp Evd.Store.empty in
+  let (gl,_,sigma) =
+    Goal.V82.mk_goal sigma (named_context_val env) EConstr.mkProp in
   {Evd.it = gl; Evd.sigma = sigma}
 
 let constr_of evd v = match Value.to_constr v with
@@ -193,37 +194,28 @@ let exec_tactic env evd n f args =
   in
   let (_, args, lfun) = List.fold_right fold args (0, [], Id.Map.empty) in
   let ist = { (Tacinterp.default_ist ()) with Tacinterp.lfun = lfun; } in
-  (** Build the getter *)
+  (* Build the getter *)
   let lid = List.init n (fun i -> Id.of_string("x"^string_of_int i)) in
   let n = Genarg.in_gen (Genarg.glbwit Stdarg.wit_int) n in
-  let get_res = TacML (Loc.tag (get_res, [TacGeneric n])) in
+  let get_res = TacML (CAst.make (get_res, [TacGeneric n])) in
   let getter = Tacexp (TacFun (List.map (fun n -> Name n) lid, get_res)) in
-  (** Evaluate the whole result *)
+  (* Evaluate the whole result *)
   let gl = dummy_goal env evd in
   let gls = Proofview.V82.of_tactic (Tacinterp.eval_tactic_ist ist (ltac_call f (args@[getter]))) gl in
   let evd = Evd.minimize_universes (Refiner.project gls) in
   let nf c = constr_of evd c in
   Array.map nf !tactic_res, Evd.universe_context_set evd
 
-let stdlib_modules =
-  [["Coq";"Setoids";"Setoid"];
-   ["Coq";"Lists";"List"];
-   ["Coq";"Init";"Datatypes"];
-   ["Coq";"Init";"Logic"];
-  ]
+let gen_constant n = lazy (EConstr.of_constr (UnivGen.constr_of_monomorphic_global (Coqlib.lib_ref n)))
+let gen_reference n = lazy (Coqlib.lib_ref n)
 
-let coq_constant c =
-  lazy (EConstr.of_constr (UnivGen.constr_of_global @@ Coqlib.gen_reference_in_modules "Ring" stdlib_modules c))
-let coq_reference c =
-  lazy (Coqlib.gen_reference_in_modules "Ring" stdlib_modules c)
+let coq_mk_Setoid = gen_constant "plugins.setoid_ring.Build_Setoid_Theory"
+let coq_None = gen_reference "core.option.None"
+let coq_Some = gen_reference "core.option.Some"
+let coq_eq = gen_constant "core.eq.type"
 
-let coq_mk_Setoid = coq_constant "Build_Setoid_Theory"
-let coq_None = coq_reference "None"
-let coq_Some = coq_reference "Some"
-let coq_eq = coq_constant "eq"
-
-let coq_cons = coq_reference "cons"
-let coq_nil = coq_reference "nil"
+let coq_cons = gen_reference "core.list.cons"
+let coq_nil = gen_reference "core.list.nil"
 
 let lapp f args = mkApp(Lazy.force f,args)
 
@@ -259,17 +251,19 @@ let plugin_modules =
     ]
 
 let my_constant c =
-  lazy (EConstr.of_constr (UnivGen.constr_of_global @@ Coqlib.gen_reference_in_modules "Ring" plugin_modules c))
+  lazy (EConstr.of_constr (UnivGen.constr_of_monomorphic_global @@ Coqlib.gen_reference_in_modules "Ring" plugin_modules c))
+    [@@ocaml.warning "-3"]
 let my_reference c =
   lazy (Coqlib.gen_reference_in_modules "Ring" plugin_modules c)
+    [@@ocaml.warning "-3"]
 
 let znew_ring_path =
   DirPath.make (List.map Id.of_string ["InitialRing";plugin_dir;"Coq"])
 let zltac s =
-  lazy(KerName.make (ModPath.MPfile znew_ring_path) DirPath.empty (Label.make s))
+  lazy(KerName.make (ModPath.MPfile znew_ring_path) (Label.make s))
 
-let mk_cst l s = lazy (Coqlib.coq_reference "newring" l s);;
-let pol_cst s = mk_cst [plugin_dir;"Ring_polynom"] s ;;
+let mk_cst l s = lazy (Coqlib.coq_reference "newring" l s) [@@ocaml.warning "-3"]
+let pol_cst s = mk_cst [plugin_dir;"Ring_polynom"] s
 
 (* Ring theory *)
 
@@ -331,7 +325,18 @@ let _ = add_map "ring"
 module Cmap = Map.Make(Constr)
 
 let from_carrier = Summary.ref Cmap.empty ~name:"ring-tac-carrier-table"
-let from_name = Summary.ref Spmap.empty ~name:"ring-tac-name-table"
+
+let print_rings () =
+  Feedback.msg_notice (strbrk "The following ring structures have been declared:");
+  Cmap.iter (fun _carrier ring ->
+      let env = Global.env () in
+      let sigma = Evd.from_env env in
+      Feedback.msg_notice
+        (hov 2
+           (Ppconstr.pr_id ring.ring_name ++ spc() ++
+            str"with carrier "++ pr_constr_env env sigma ring.ring_carrier++spc()++
+            str"and equivalence relation "++ pr_constr_env env sigma ring.ring_req))
+    ) !from_carrier
 
 let ring_for_carrier r = Cmap.find r !from_carrier
 
@@ -354,9 +359,7 @@ let find_ring_structure env sigma l =
     | [] -> assert false
 
 let add_entry (sp,_kn) e =
-  from_carrier := Cmap.add e.ring_carrier e !from_carrier;
-  from_name := Spmap.add sp e !from_name
-
+  from_carrier := Cmap.add e.ring_carrier e !from_carrier
 
 let subst_th (subst,th) =
   let c' = subst_mps subst th.ring_carrier in
@@ -384,7 +387,8 @@ let subst_th (subst,th) =
      pretac' == th.ring_pre_tac &&
      posttac' == th.ring_post_tac then th
   else
-    { ring_carrier = c';
+    { ring_name = th.ring_name;
+      ring_carrier = c';
       ring_req = eq';
       ring_setoid = set';
       ring_ext = ext';
@@ -400,13 +404,9 @@ let subst_th (subst,th) =
 
 let theory_to_obj : ring_info -> obj =
   let cache_th (name,th) = add_entry name th in
-  declare_object
-    {(default_object "tactic-new-ring-theory") with
-      open_function = (fun i o -> if Int.equal i 1 then cache_th o);
-      cache_function = cache_th;
-      subst_function = subst_th;
-      classify_function = (fun x -> Substitute x)}
-
+  declare_object @@ global_object_nodischarge "tactic-new-ring-theory"
+    ~cache:cache_th
+    ~subst:(Some subst_th)
 
 let setoid_of_relation env evd a r =
   try
@@ -425,65 +425,12 @@ let op_morph r add mul opp req m1 m2 m3 =
 let op_smorph r add mul req m1 m2 =
   lapp coq_mk_seqe [| r; add; mul; req; m1; m2 |]
 
-(* let default_ring_equality (r,add,mul,opp,req) = *)
-(*   let is_setoid = function *)
-(*       {rel_refl=Some _; rel_sym=Some _;rel_trans=Some _;rel_aeq=rel} -> *)
-(*         eq_constr_nounivs req rel (\* Qu: use conversion ? *\) *)
-(*     | _ -> false in *)
-(*   match default_relation_for_carrier ~filter:is_setoid r with *)
-(*       Leibniz _ -> *)
-(*         let setoid = lapp coq_eq_setoid [|r|] in *)
-(*         let op_morph = *)
-(*           match opp with *)
-(*               Some opp -> lapp coq_eq_morph [|r;add;mul;opp|] *)
-(*             | None -> lapp coq_eq_smorph [|r;add;mul|] in *)
-(*         (setoid,op_morph) *)
-(*     | Relation rel -> *)
-(*         let setoid = setoid_of_relation rel in *)
-(*         let is_endomorphism = function *)
-(*             { args=args } -> List.for_all *)
-(*                 (function (var,Relation rel) -> *)
-(*                   var=None && eq_constr_nounivs req rel *)
-(*                   | _ -> false) args in *)
-(*         let add_m = *)
-(*           try default_morphism ~filter:is_endomorphism add *)
-(*           with Not_found -> *)
-(*             error "ring addition should be declared as a morphism" in *)
-(*         let mul_m = *)
-(*           try default_morphism ~filter:is_endomorphism mul *)
-(*           with Not_found -> *)
-(*             error "ring multiplication should be declared as a morphism" in *)
-(*         let op_morph = *)
-(*           match opp with *)
-(*             | Some opp -> *)
-(*             (let opp_m = *)
-(*               try default_morphism ~filter:is_endomorphism opp *)
-(*               with Not_found -> *)
-(*                 error "ring opposite should be declared as a morphism" in *)
-(*              let op_morph = *)
-(*                op_morph r add mul opp req add_m.lem mul_m.lem opp_m.lem in *)
-(*              msgnl *)
-(*                (str"Using setoid \""++pr_constr rel.rel_aeq++str"\""++spc()++   *)
-(*                str"and morphisms \""++pr_constr add_m.morphism_theory++ *)
-(*                str"\","++spc()++ str"\""++pr_constr mul_m.morphism_theory++ *)
-(*                str"\""++spc()++str"and \""++pr_constr opp_m.morphism_theory++ *)
-(*                str"\""); *)
-(*              op_morph) *)
-(*             | None -> *)
-(*             (msgnl *)
-(*               (str"Using setoid \""++pr_constr rel.rel_aeq++str"\"" ++ spc() ++   *)
-(*                str"and morphisms \""++pr_constr add_m.morphism_theory++ *)
-(*                str"\""++spc()++str"and \""++ *)
-(*                pr_constr mul_m.morphism_theory++str"\""); *)
-(*             op_smorph r add mul req add_m.lem mul_m.lem) in *)
-(*         (setoid,op_morph) *)
-
 let ring_equality env evd (r,add,mul,opp,req) =
   match EConstr.kind !evd req with
     | App (f, [| _ |]) when eq_constr_nounivs !evd f (Lazy.force coq_eq) ->
-	let setoid = plapp evd coq_eq_setoid [|r|] in
-	let op_morph =
-	  match opp with
+        let setoid = plapp evd coq_eq_setoid [|r|] in
+        let op_morph =
+          match opp with
               Some opp -> plapp evd coq_eq_morph [|r;add;mul;opp|]
           | None -> plapp evd coq_eq_smorph [|r;add;mul|] in
         let sigma = !evd in
@@ -492,41 +439,41 @@ let ring_equality env evd (r,add,mul,opp,req) =
         evd := sigma;
         (setoid,op_morph)
     | _ ->
-	let setoid = setoid_of_relation (Global.env ()) evd r req in
-	let signature = [Some (r,Some req);Some (r,Some req)],Some(r,Some req) in
-	let add_m, add_m_lem =
-	  try Rewrite.default_morphism signature add
+        let setoid = setoid_of_relation (Global.env ()) evd r req in
+        let signature = [Some (r,Some req);Some (r,Some req)],Some(r,Some req) in
+        let add_m, add_m_lem =
+          try Rewrite.default_morphism signature add
           with Not_found ->
             error "ring addition should be declared as a morphism" in
-	let mul_m, mul_m_lem =
+        let mul_m, mul_m_lem =
           try Rewrite.default_morphism signature mul
           with Not_found ->
             error "ring multiplication should be declared as a morphism" in
         let op_morph =
           match opp with
             | Some opp ->
-		(let opp_m,opp_m_lem =
-		  try Rewrite.default_morphism ([Some(r,Some req)],Some(r,Some req)) opp
-		  with Not_found ->
+                (let opp_m,opp_m_lem =
+                  try Rewrite.default_morphism ([Some(r,Some req)],Some(r,Some req)) opp
+                  with Not_found ->
                     error "ring opposite should be declared as a morphism" in
-		let op_morph =
-		  op_morph r add mul opp req add_m_lem mul_m_lem opp_m_lem in
-		  Flags.if_verbose
-		    Feedback.msg_info
+                let op_morph =
+                  op_morph r add mul opp req add_m_lem mul_m_lem opp_m_lem in
+                  Flags.if_verbose
+                    Feedback.msg_info
                     (str"Using setoid \""++ pr_econstr_env env !evd req++str"\""++spc()++
                         str"and morphisms \""++pr_econstr_env env !evd add_m_lem ++
                         str"\","++spc()++ str"\""++pr_econstr_env env !evd mul_m_lem++
                         str"\""++spc()++str"and \""++pr_econstr_env env !evd opp_m_lem++
-			str"\"");
-		  op_morph)
+                        str"\"");
+                  op_morph)
             | None ->
-		(Flags.if_verbose
-		    Feedback.msg_info
+                (Flags.if_verbose
+                    Feedback.msg_info
                     (str"Using setoid \""++pr_econstr_env env !evd req ++str"\"" ++ spc() ++
                         str"and morphisms \""++pr_econstr_env env !evd add_m_lem ++
-			str"\""++spc()++str"and \""++
+                        str"\""++spc()++str"and \""++
                         pr_econstr_env env !evd mul_m_lem++str"\"");
-		 op_smorph r add mul req add_m_lem mul_m_lem) in
+                 op_smorph r add mul req add_m_lem mul_m_lem) in
           (setoid,op_morph)
 
 let build_setoid_params env evd r add mul opp req eqth =
@@ -563,7 +510,7 @@ let interp_cst_tac env sigma rk kind (zero,one,add,mul,opp) cst_tac =
         closed_term_ast (List.map Smartlocate.global_with_alias lc)
     | None ->
         let t = ArgArg(Loc.tag @@ Lazy.force ltac_inv_morph_nothing) in
-              TacArg(Loc.tag (TacCall(Loc.tag (t,[]))))
+              TacArg(CAst.make (TacCall(CAst.make (t,[]))))
 
 let make_hyp env evd c =
   let t = Retyping.get_type_of env !evd c in
@@ -572,11 +519,11 @@ let make_hyp env evd c =
 let make_hyp_list env evdref lH =
   let evd, carrier = Evarutil.new_global !evdref (Lazy.force coq_hypo) in
   evdref := evd;
-  let l = 
+  let l =
     List.fold_right
       (fun c l -> plapp evdref coq_cons [|carrier; (make_hyp env evdref c); l|]) lH
       (plapp evdref coq_nil [|carrier|])
-  in 
+  in
   let sigma, l' = Typing.solve_evars env !evdref l in
   evdref := sigma;
   let l' = EConstr.Unsafe.to_constr l' in
@@ -588,7 +535,7 @@ let interp_power env evdref pow =
   match pow with
   | None ->
       let t = ArgArg(Loc.tag (Lazy.force ltac_inv_morph_nothing)) in
-      (TacArg(Loc.tag (TacCall(Loc.tag (t,[])))), plapp evdref coq_None [|carrier|])
+      (TacArg(CAst.make (TacCall(CAst.make (t,[])))), plapp evdref coq_None [|carrier|])
   | Some (tac, spec) ->
       let tac =
         match tac with
@@ -654,14 +601,15 @@ let add_theory0 name (sigma, rth) eqth morphth cst_tac (pre,post) power sign div
   let _ =
     Lib.add_leaf name
       (theory_to_obj
-        { ring_carrier = r;
+        { ring_name = name;
+          ring_carrier = r;
           ring_req = req;
           ring_setoid = sth;
           ring_ext = params.(1);
           ring_morph = params.(2);
           ring_th = params.(0);
           ring_cst_tac = cst_tac;
-	  ring_pow_tac = pow_tac;
+          ring_pow_tac = pow_tac;
           ring_lemma1 = lemma1;
           ring_lemma2 = lemma2;
           ring_pre_tac = pretac;
@@ -760,7 +708,7 @@ let new_field_path =
   DirPath.make (List.map Id.of_string ["Field_tac";plugin_dir;"Coq"])
 
 let field_ltac s =
-  lazy(KerName.make (ModPath.MPfile new_field_path) DirPath.empty (Label.make s))
+  lazy(KerName.make (ModPath.MPfile new_field_path) (Label.make s))
 
 
 let _ = add_map "field"
@@ -832,7 +780,18 @@ let dest_field env evd th_spec =
     | _ -> error "bad field structure"
 
 let field_from_carrier = Summary.ref Cmap.empty ~name:"field-tac-carrier-table"
-let field_from_name = Summary.ref Spmap.empty ~name:"field-tac-name-table"
+
+let print_fields () =
+  Feedback.msg_notice (strbrk "The following field structures have been declared:");
+  Cmap.iter (fun _carrier fi ->
+      let env = Global.env () in
+      let sigma = Evd.from_env env in
+      Feedback.msg_notice
+        (hov 2
+           (Id.print fi.field_name ++ spc() ++
+            str"with carrier "++ pr_constr_env env sigma fi.field_carrier++spc()++
+            str"and equivalence relation "++ pr_constr_env env sigma fi.field_req))
+    ) !field_from_carrier
 
 let field_for_carrier r = Cmap.find r !field_from_carrier
 
@@ -856,8 +815,7 @@ let find_field_structure env sigma l =
     | [] -> assert false
 
 let add_field_entry (sp,_kn) e =
-  field_from_carrier := Cmap.add e.field_carrier e !field_from_carrier;
-  field_from_name := Spmap.add sp e !field_from_name
+  field_from_carrier := Cmap.add e.field_carrier e !field_from_carrier
 
 let subst_th (subst,th) =
   let c' = subst_mps subst th.field_carrier in
@@ -883,7 +841,8 @@ let subst_th (subst,th) =
      pretac' == th.field_pre_tac &&
      posttac' == th.field_post_tac then th
   else
-    { field_carrier = c';
+    { field_name = th.field_name;
+      field_carrier = c';
       field_req = eq';
       field_cst_tac = tac';
       field_pow_tac = pow_tac';
@@ -897,27 +856,24 @@ let subst_th (subst,th) =
 
 let ftheory_to_obj : field_info -> obj =
   let cache_th (name,th) = add_field_entry name th in
-  declare_object
-    {(default_object "tactic-new-field-theory") with
-      open_function = (fun i o -> if Int.equal i 1 then cache_th o);
-      cache_function = cache_th;
-      subst_function = subst_th;
-      classify_function = (fun x -> Substitute x) }
+  declare_object @@ global_object_nodischarge "tactic-new-field-theory"
+    ~cache:cache_th
+    ~subst:(Some subst_th)
 
 let field_equality evd r inv req =
   match EConstr.kind !evd req with
     | App (f, [| _ |]) when eq_constr_nounivs !evd f (Lazy.force coq_eq) ->
-        let c = UnivGen.constr_of_global (Coqlib.build_coq_eq_data()).congr in
+        let c = UnivGen.constr_of_monomorphic_global Coqlib.(lib_ref "core.eq.congr") in
         let c = EConstr.of_constr c in
         mkApp(c,[|r;r;inv|])
     | _ ->
-	let _setoid = setoid_of_relation (Global.env ()) evd r req in
-	let signature = [Some (r,Some req)],Some(r,Some req) in
-	let inv_m, inv_m_lem =
-	  try Rewrite.default_morphism signature inv
+        let _setoid = setoid_of_relation (Global.env ()) evd r req in
+        let signature = [Some (r,Some req)],Some(r,Some req) in
+        let inv_m, inv_m_lem =
+          try Rewrite.default_morphism signature inv
           with Not_found ->
             error "field inverse should be declared as a morphism" in
-	  inv_m_lem
+          inv_m_lem
 
 let add_field_theory0 name fth eqth morphth cst_tac inj (pre,post) power sign odiv =
   let open Constr in
@@ -948,13 +904,13 @@ let add_field_theory0 name fth eqth morphth cst_tac inj (pre,post) power sign od
       | None -> params.(7) in
   let lemma1 = decl_constant (Id.to_string name^"_field_lemma1")
     ctx lemma1 in
-  let lemma2 = decl_constant (Id.to_string name^"_field_lemma2") 
+  let lemma2 = decl_constant (Id.to_string name^"_field_lemma2")
     ctx lemma2 in
-  let lemma3 = decl_constant (Id.to_string name^"_field_lemma3") 
+  let lemma3 = decl_constant (Id.to_string name^"_field_lemma3")
     ctx lemma3 in
-  let lemma4 = decl_constant (Id.to_string name^"_field_lemma4") 
+  let lemma4 = decl_constant (Id.to_string name^"_field_lemma4")
     ctx lemma4 in
-  let cond_lemma = decl_constant (Id.to_string name^"_lemma5") 
+  let cond_lemma = decl_constant (Id.to_string name^"_lemma5")
     ctx cond_lemma in
   let cst_tac =
     interp_cst_tac env sigma morphth kind (zero,one,add,mul,opp) cst_tac in
@@ -971,7 +927,8 @@ let add_field_theory0 name fth eqth morphth cst_tac inj (pre,post) power sign od
   let _ =
     Lib.add_leaf name
       (ftheory_to_obj
-        { field_carrier = r;
+        { field_name = name;
+          field_carrier = r;
           field_req = req;
           field_cst_tac = cst_tac;
           field_pow_tac = pow_tac;
