@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -14,15 +14,13 @@ open Goal
 open Environ
 open Evd
 open Constr
-
-open Ltac_plugin
-open Tacexpr
+open Genintern
 
 (** ******** Small Scale Reflection pattern matching facilities ************* *)
 
 (** Pattern parsing *)
 
-(** The type of context patterns, the patterns of the [set] tactic and 
+(** The type of context patterns, the patterns of the [set] tactic and
     [:] tactical. These are patterns that identify a precise subterm. *)
 type cpattern
 val pr_cpattern : cpattern -> Pp.t
@@ -48,7 +46,7 @@ type ('ident, 'term) ssrpattern =
   | E_As_X_In_T of 'term * 'ident * 'term
 
 type pattern = evar_map * (constr, constr) ssrpattern
-val pp_pattern : pattern -> Pp.t
+val pp_pattern : env -> pattern -> Pp.t
 
 (** Extracts the redex and applies to it the substitution part of the pattern.
   @raise Anomaly if called on [In_T] or [In_X_In_T] *)
@@ -80,10 +78,10 @@ type occ = (bool * int list) option
 type subst = env -> constr -> constr -> int -> constr
 
 (** [eval_pattern b env sigma t pat occ subst] maps [t] calling [subst] on every
-    [occ] occurrence of [pat]. The [int] argument is the number of 
+    [occ] occurrence of [pat]. The [int] argument is the number of
     binders traversed. If [pat] is [None] then then subst is called on [t].
-    [t] must live in [env] and [sigma], [pat] must have been interpreted in 
-    (an extension of) [sigma]. 
+    [t] must live in [env] and [sigma], [pat] must have been interpreted in
+    (an extension of) [sigma].
   @raise NoMatch if [pat] has no occurrence and [b] is [true] (default [false])
   @return [t] where all [occ] occurrences of [pat] have been mapped using
     [subst] *)
@@ -93,12 +91,12 @@ val eval_pattern :
   pattern option -> occ -> subst ->
     constr
 
-(** [fill_occ_pattern b env sigma t pat occ h] is a simplified version of 
-    [eval_pattern]. 
-    It replaces all [occ] occurrences of [pat] in [t] with Rel [h]. 
-    [t] must live in [env] and [sigma], [pat] must have been interpreted in 
-    (an extension of) [sigma]. 
-  @raise NoMatch if [pat] has no occurrence and [b] is [true] (default [false]) 
+(** [fill_occ_pattern b env sigma t pat occ h] is a simplified version of
+    [eval_pattern].
+    It replaces all [occ] occurrences of [pat] in [t] with Rel [h].
+    [t] must live in [env] and [sigma], [pat] must have been interpreted in
+    (an extension of) [sigma].
+  @raise NoMatch if [pat] has no occurrence and [b] is [true] (default [false])
   @return the instance of the redex of [pat] that was matched and [t]
     transformed as described above. *)
 val fill_occ_pattern :
@@ -109,7 +107,7 @@ val fill_occ_pattern :
 
 (** *************************** Low level APIs ****************************** *)
 
-(* The primitive matching facility. It matches of a term with holes, like 
+(* The primitive matching facility. It matches of a term with holes, like
    the T pattern above, and calls a continuation on its occurrences. *)
 
 type ssrdir = L2R | R2L
@@ -118,7 +116,7 @@ val pr_dir_side : ssrdir -> Pp.t
 (** a pattern for a term with wildcards *)
 type tpattern
 
-(** [mk_tpattern env sigma0 sigma_p ok p_origin dir t] compiles a term [t] 
+(** [mk_tpattern env sigma0 sigma_p ok p_origin dir t] compiles a term [t]
     living in [env] [sigma] (an extension of [sigma0]) intro a [tpattern].
     The [tpattern] can hold a (proof) term [p] and a diction [dir]. The [ok]
     callback is used to filter occurrences.
@@ -132,31 +130,31 @@ val mk_tpattern :
   ssrdir -> constr ->
     evar_map * tpattern
 
-(** [findP env t i k] is a stateful function that finds the next occurrence 
-    of a tpattern and calls the callback [k] to map the subterm matched. 
-    The [int] argument passed to [k] is the number of binders traversed so far 
-    plus the initial value [i]. 
-  @return [t] where the subterms identified by the selected occurrences of 
+(** [findP env t i k] is a stateful function that finds the next occurrence
+    of a tpattern and calls the callback [k] to map the subterm matched.
+    The [int] argument passed to [k] is the number of binders traversed so far
+    plus the initial value [i].
+  @return [t] where the subterms identified by the selected occurrences of
     the patter have been mapped using [k]
   @raise NoMatch if the raise_NoMatch flag given to [mk_tpattern_matcher] is
-    [true] and if the pattern did not match 
+    [true] and if the pattern did not match
   @raise UserEerror if the raise_NoMatch flag given to [mk_tpattern_matcher] is
     [false] and if the pattern did not match *)
 type find_P =
   env -> constr -> int -> k:subst -> constr
 
-(** [conclude ()] asserts that all mentioned ocurrences have been visited.
+(** [conclude ()] asserts that all mentioned occurrences have been visited.
   @return the instance of the pattern, the evarmap after the pattern
     instantiation, the proof term and the ssrdit stored in the tpattern
   @raise UserEerror if too many occurrences were specified *)
 type conclude =
   unit -> constr * ssrdir * (evar_map * UState.t * constr)
 
-(** [mk_tpattern_matcher b o sigma0 occ sigma_tplist] creates a pair 
+(** [mk_tpattern_matcher b o sigma0 occ sigma_tplist] creates a pair
     a function [find_P] and [conclude] with the behaviour explained above.
     The flag [b] (default [false]) changes the error reporting behaviour
     of [find_P] if none of the [tpattern] matches. The argument [o] can
-    be passed to tune the [UserError] eventually raised (useful if the 
+    be passed to tune the [UserError] eventually raised (useful if the
     pattern is coming from the LHS/RHS of an equation) *)
 val mk_tpattern_matcher :
   ?all_instances:bool ->
@@ -165,24 +163,24 @@ val mk_tpattern_matcher :
   evar_map -> occ -> evar_map * tpattern list ->
     find_P * conclude
 
-(** Example of [mk_tpattern_matcher] to implement 
+(** Example of [mk_tpattern_matcher] to implement
     [rewrite \{occ\}\[in t\]rules].
-    It first matches "in t" (called [pat]), then in all matched subterms 
+    It first matches "in t" (called [pat]), then in all matched subterms
     it matches the LHS of the rules using [find_R].
     [concl0] is the initial goal, [concl] will be the goal where some terms
     are replaced by a De Bruijn index. The [rw_progress] extra check
     selects only occurrences that are not rewritten to themselves (e.g.
-    an occurrence "x + x" rewritten with the commutativity law of addition 
+    an occurrence "x + x" rewritten with the commutativity law of addition
     is skipped) {[
   let find_R, conclude = match pat with
   | Some (_, In_T _) ->
       let aux (sigma, pats) (d, r, lhs, rhs) =
-        let sigma, pat = 
+        let sigma, pat =
           mk_tpattern env0 sigma0 (sigma, r) (rw_progress rhs) d lhs in
         sigma, pats @ [pat] in
       let rpats = List.fold_left aux (r_sigma, []) rules in
       let find_R, end_R = mk_tpattern_matcher sigma0 occ rpats in
-      find_R ~k:(fun _ _ h -> mkRel h), 
+      find_R ~k:(fun _ _ h -> mkRel h),
       fun cl -> let rdx, d, r = end_R () in (d,r),rdx
   | _ -> ... in
   let concl = eval_pattern env0 sigma0 concl0 pat occ find_R in
@@ -196,7 +194,7 @@ val pf_fill_occ_term : goal sigma -> occ -> evar_map * EConstr.t -> EConstr.t *
 (* It may be handy to inject a simple term into the first form of cpattern *)
 val cpattern_of_term : char * glob_constr_and_expr -> Geninterp.interp_sign -> cpattern
 
-(** Helpers to make stateful closures. Example: a [find_P] function may be 
+(** Helpers to make stateful closures. Example: a [find_P] function may be
     called many times, but the pattern instantiation phase is performed only the
     first time. The corresponding [conclude] has to return the instantiated
     pattern redex. Since it is up to [find_P] to raise [NoMatch] if the pattern
@@ -205,12 +203,13 @@ val cpattern_of_term : char * glob_constr_and_expr -> Geninterp.interp_sign -> c
 
 (** [do_once r f] calls [f] and updates the ref only once *)
 val do_once : 'a option ref -> (unit -> 'a) -> unit
+
 (** [assert_done r] return the content of r. @raise Anomaly is r is [None] *)
 val assert_done : 'a option ref -> 'a
 
 (** Very low level APIs.
     these are calls to evarconv's [the_conv_x] followed by
-    [solve_unif_constraints_with_heuristics] and [resolve_typeclasses].
+    [solve_unif_constraints_with_heuristics].
     In case of failure they raise [NoMatch] *)
 
 val unify_HO : env -> evar_map -> EConstr.constr -> EConstr.constr -> evar_map
@@ -223,17 +222,14 @@ val loc_of_cpattern : cpattern -> Loc.t option
 val id_of_pattern : pattern -> Names.Id.t option
 val is_wildcard : cpattern -> bool
 val cpattern_of_id : Names.Id.t -> cpattern
-val pr_constr_pat : constr -> Pp.t
+val pr_constr_pat : env -> evar_map -> constr -> Pp.t
+val pr_econstr_pat : env -> evar_map -> econstr -> Pp.t
 val pf_merge_uc : UState.t -> goal Evd.sigma -> goal Evd.sigma
 val pf_unsafe_merge_uc : UState.t -> goal Evd.sigma -> goal Evd.sigma
 
 (* One can also "Set SsrMatchingDebug" from a .v *)
 val debug : bool -> unit
 
-(* One should delimit a snippet with "Set SsrMatchingProfiling" and
- * "Unset SsrMatchingProfiling" to get timings *)
-val profile : bool -> unit
-
 val ssrinstancesof : cpattern -> Tacmach.tactic
 
 (** Functions used for grammar extensions. Do not use. *)
@@ -243,7 +239,7 @@ sig
   val wit_rpatternty : (rpattern, rpattern, rpattern) Genarg.genarg_type
   val glob_rpattern : Genintern.glob_sign -> rpattern -> rpattern
   val subst_rpattern : Mod_subst.substitution -> rpattern -> rpattern
-  val interp_rpattern : Geninterp.interp_sign -> Proof_type.goal Evd.sigma -> rpattern -> Evd.evar_map * rpattern
+  val interp_rpattern : Geninterp.interp_sign -> Goal.goal Evd.sigma -> rpattern -> Evd.evar_map * rpattern
   val pr_rpattern : rpattern -> Pp.t
   val mk_rpattern : (cpattern, cpattern) ssrpattern -> rpattern
   val mk_lterm : Constrexpr.constr_expr -> Geninterp.interp_sign option -> cpattern
@@ -251,7 +247,7 @@ sig
 
   val glob_cpattern : Genintern.glob_sign -> cpattern -> cpattern
   val subst_ssrterm : Mod_subst.substitution -> cpattern -> cpattern
-  val interp_ssrterm : Geninterp.interp_sign -> Proof_type.goal Evd.sigma -> cpattern -> Evd.evar_map * cpattern
+  val interp_ssrterm : Geninterp.interp_sign -> Goal.goal Evd.sigma -> cpattern -> Evd.evar_map * cpattern
   val pr_ssrterm : cpattern -> Pp.t
 end
 
