@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -15,7 +15,6 @@ open Environ
 open EConstr
 open Inductiveops
 open Glob_term
-open Ltac_pretype
 open Evardefine
 
 (** {5 Compilation of pattern-matching } *)
@@ -41,25 +40,25 @@ val irrefutable : env -> cases_pattern -> bool
 (** {6 Compilation primitive. } *)
 
 val compile_cases :
-  ?loc:Loc.t -> case_style ->
-  (type_constraint -> env -> evar_map ref -> ltac_var_map -> glob_constr -> unsafe_judgment) * evar_map ref ->
+  ?loc:Loc.t -> program_mode:bool -> case_style ->
+  (type_constraint -> GlobEnv.t -> evar_map -> glob_constr -> evar_map * unsafe_judgment) * evar_map ->
   type_constraint ->
-  env -> ltac_var_map -> glob_constr option * tomatch_tuples * cases_clauses ->
-  unsafe_judgment
+  GlobEnv.t -> glob_constr option * tomatch_tuples * cases_clauses ->
+  evar_map * unsafe_judgment
 
-val constr_of_pat : 
+val constr_of_pat :
            Environ.env ->
-           Evd.evar_map ref ->
+           Evd.evar_map ->
            rel_context ->
            Glob_term.cases_pattern ->
            Names.Id.Set.t ->
-           Glob_term.cases_pattern *
+           Evd.evar_map * Glob_term.cases_pattern *
            (rel_context * constr *
             (types * constr list) * Glob_term.cases_pattern) *
            Names.Id.Set.t
 
 type 'a rhs =
-    { rhs_env    : env;
+    { rhs_env    : GlobEnv.t;
       rhs_vars   : Id.Set.t;
       avoid_ids  : Id.Set.t;
       it         : 'a option}
@@ -103,30 +102,26 @@ and pattern_continuation =
   | Result of cases_pattern list
 
 type 'a pattern_matching_problem =
-    { env       : env;
-      lvar      : Ltac_pretype.ltac_var_map;
-      evdref    : evar_map ref;
+    { env       : GlobEnv.t;
       pred      : constr;
       tomatch   : tomatch_stack;
       history   : pattern_continuation;
       mat       : 'a matrix;
       caseloc   : Loc.t option;
       casestyle : case_style;
-      typing_function: type_constraint -> env -> evar_map ref -> 'a option -> unsafe_judgment }
+      typing_function: type_constraint -> GlobEnv.t -> evar_map -> 'a option -> evar_map * unsafe_judgment }
 
+val compile : program_mode:bool -> evar_map -> 'a pattern_matching_problem -> evar_map * unsafe_judgment
 
-val compile : 'a pattern_matching_problem -> unsafe_judgment
-
-val prepare_predicate : ?loc:Loc.t ->
+val prepare_predicate : ?loc:Loc.t -> program_mode:bool ->
            (type_constraint ->
-            Environ.env -> Evd.evar_map ref -> ltac_var_map -> glob_constr -> unsafe_judgment) ->
-           Environ.env ->
+            GlobEnv.t -> Evd.evar_map -> glob_constr -> Evd.evar_map * unsafe_judgment) ->
+           GlobEnv.t ->
            Evd.evar_map ->
-           Ltac_pretype.ltac_var_map ->
            (types * tomatch_type) list ->
-           (rel_context * rel_context) list ->
+           rel_context list ->
            constr option ->
-           glob_constr option -> (Evd.evar_map * Name.t list * constr) list
+           glob_constr option -> (Evd.evar_map * (Name.t * Name.t list) list * constr) list
 
-val make_return_predicate_ltac_lvar : Evd.evar_map -> Name.t ->
-  Glob_term.glob_constr -> constr -> Ltac_pretype.ltac_var_map -> ltac_var_map
+val make_return_predicate_ltac_lvar : GlobEnv.t -> Evd.evar_map -> Name.t ->
+  Glob_term.glob_constr -> constr -> GlobEnv.t
