@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -13,6 +13,7 @@ open Util
 open Pp
 open Names
 open Constr
+open Context
 open Termops
 open EConstr
 open Vars
@@ -72,7 +73,7 @@ let idx = Namegen.default_dependent_ident
 
 (* Refining an evar to a product *)
 
-let define_pure_evar_as_product evd evk =
+let define_pure_evar_as_product env evd evk =
   let open Context.Named.Declaration in
   let evi = Evd.find_undefined evd evk in
   let evenv = evar_env evi in
@@ -84,30 +85,31 @@ let define_pure_evar_as_product evd evk =
   let evd1,(dom,u1) =
     new_type_evar evenv evd univ_flexible_alg ~src ~filter:(evar_filter evi)
   in
+  let rdom = Sorts.Relevant in (* TODO relevance *)
   let evd2,rng =
-    let newenv = push_named (LocalAssum (id, dom)) evenv in
+    let newenv = push_named (LocalAssum (make_annot id rdom, dom)) evenv in
     let src = subterm_source evk ~where:Codomain evksrc in
     let filter = Filter.extend 1 (evar_filter evi) in
-      if Sorts.is_prop (ESorts.kind evd1 s) then
+      if Environ.is_impredicative_sort env (ESorts.kind evd1 s) then
        (* Impredicative product, conclusion must fall in [Prop]. *)
         new_evar newenv evd1 concl ~src ~filter
       else
-	let status = univ_flexible_alg in
-	let evd3, (rng, srng) =
+        let status = univ_flexible_alg in
+        let evd3, (rng, srng) =
           new_type_evar newenv evd1 status ~src ~filter
         in
-	let prods = Univ.sup (univ_of_sort u1) (univ_of_sort srng) in
-	let evd3 = Evd.set_leq_sort evenv evd3 (Type prods) (ESorts.kind evd1 s) in
-	  evd3, rng
+        let prods = Univ.sup (univ_of_sort u1) (univ_of_sort srng) in
+        let evd3 = Evd.set_leq_sort evenv evd3 (Sorts.sort_of_univ prods) (ESorts.kind evd1 s) in
+          evd3, rng
   in
-  let prod = mkProd (Name id, dom, subst_var id rng) in
+  let prod = mkProd (make_annot (Name id) rdom, dom, subst_var id rng) in
   let evd3 = Evd.define evk prod evd2 in
     evd3,prod
 
 (* Refine an applied evar to a product and returns its instantiation *)
 
-let define_evar_as_product evd (evk,args) =
-  let evd,prod = define_pure_evar_as_product evd evk in
+let define_evar_as_product env evd (evk,args) =
+  let evd,prod = define_pure_evar_as_product env evd evk in
   (* Quick way to compute the instantiation of evk with args *)
   let na,dom,rng = destProd evd prod in
   let evdom = mkEvar (fst (destEvar evd dom), args) in
@@ -131,16 +133,19 @@ let define_pure_evar_as_lambda env evd evk =
   let typ = Reductionops.whd_all evenv evd (evar_concl evi) in
   let evd1,(na,dom,rng) = match EConstr.kind evd typ with
   | Prod (na,dom,rng) -> (evd,(na,dom,rng))
-  | Evar ev' -> let evd,typ = define_evar_as_product evd ev' in evd,destProd evd typ
+  | Evar ev' -> let evd,typ = define_evar_as_product env evd ev' in evd,destProd evd typ
   | _ -> error_not_product env evd typ in
   let avoid = Environ.ids_of_named_context_val evi.evar_hyps in
   let id =
-    next_name_away_with_default_using_types "x" na avoid (Reductionops.whd_evar evd dom) in
+    map_annot (fun na -> next_name_away_with_default_using_types "x" na avoid
+      (Reductionops.whd_evar evd dom)) na
+  in
   let newenv = push_named (LocalAssum (id, dom)) evenv in
   let filter = Filter.extend 1 (evar_filter evi) in
   let src = subterm_source evk ~where:Body (evar_source evk evd1) in
-  let evd2,body = new_evar newenv evd1 ~src (subst1 (mkVar id) rng) ~filter in
-  let lam = mkLambda (Name id, dom, subst_var id body) in
+  let abstract_arguments = Abstraction.abstract_last evi.evar_abstract_arguments in
+  let evd2,body = new_evar newenv evd1 ~src (subst1 (mkVar id.binder_name) rng) ~filter ~abstract_arguments in
+  let lam = mkLambda (map_annot Name.mk_name id, dom, subst_var id.binder_name body) in
   Evd.define evk lam evd2, lam
 
 let define_evar_as_lambda env evd (evk,args) =
@@ -163,13 +168,12 @@ let rec evar_absorb_arguments env evd (evk,args as ev) = function
 (* Refining an evar to a sort *)
 
 let define_evar_as_sort env evd (ev,args) =
-  let evd, u = new_univ_variable univ_rigid evd in
-  let evi = Evd.find_undefined evd ev in 
-  let s = Type u in
+  let evd, s = new_sort_variable univ_rigid evd in
+  let evi = Evd.find_undefined evd ev in
   let concl = Reductionops.whd_all (evar_env evi) evd evi.evar_concl in
   let sort = destSort evd concl in
   let evd' = Evd.define ev (mkSort s) evd in
-  Evd.set_leq_sort env evd' (Type (Univ.super u)) (ESorts.kind evd' sort), s
+  Evd.set_leq_sort env evd' (Sorts.super s) (ESorts.kind evd' sort), s
 
 (* Propagation of constraints through application and abstraction:
    Given a type constraint on a functional term, returns the type
@@ -180,21 +184,22 @@ let split_tycon ?loc env evd tycon =
   let rec real_split evd c =
     let t = Reductionops.whd_all env evd c in
       match EConstr.kind evd t with
-	| Prod (na,dom,rng) -> evd, (na, dom, rng)
-	| Evar ev (* ev is undefined because of whd_all *) ->
-	    let (evd',prod) = define_evar_as_product evd ev in
-	    let (_,dom,rng) = destProd evd prod in
-	      evd',(Anonymous, dom, rng)
-	| App (c,args) when isEvar evd c ->
-	    let (evd',lam) = define_evar_as_lambda env evd (destEvar evd c) in
-	    real_split evd' (mkApp (lam,args))
-	| _ -> error_not_product ?loc env evd c
+        | Prod (na,dom,rng) -> evd, (na, dom, rng)
+        | Evar ev (* ev is undefined because of whd_all *) ->
+            let (evd',prod) = define_evar_as_product env evd ev in
+            let (na,dom,rng) = destProd evd prod in
+            let anon = {na with binder_name = Anonymous} in
+              evd',(anon, dom, rng)
+        | App (c,args) when isEvar evd c ->
+            let (evd',lam) = define_evar_as_lambda env evd (destEvar evd c) in
+            real_split evd' (mkApp (lam,args))
+        | _ -> error_not_product ?loc env evd c
   in
     match tycon with
-      | None -> evd,(Anonymous,None,None)
+      | None -> evd,(make_annot Anonymous Relevant,None,None)
       | Some c ->
-	  let evd', (n, dom, rng) = real_split evd c in
-	    evd', (n, mk_tycon dom, mk_tycon rng)
+          let evd', (n, dom, rng) = real_split evd c in
+            evd', (n, mk_tycon dom, mk_tycon rng)
 
 let valcon_of_tycon x = x
 let lift_tycon n = Option.map (lift n)
