@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -70,7 +70,7 @@ let map_named_declaration_with_hyploc f hyploc acc decl =
   in
   match decl,hyploc with
   | LocalAssum (id,_), InHypValueOnly ->
-      error_occurrences_error (IncorrectInValueOccurrence id)
+      error_occurrences_error (IncorrectInValueOccurrence id.Context.binder_name)
   | LocalAssum (id,typ), _ ->
       let acc,typ = f acc typ in acc, LocalAssum (id,typ)
   | LocalDef (id,body,typ), InHypTypeOnly ->
@@ -125,7 +125,7 @@ let replace_term_occ_gen_modulo sigma occs like_first test bywhat cl occ t =
          end;
          add_subst t subst; incr pos;
          (* Check nested matching subterms *)
-         if occs != Locus.AllOccurrences && occs != Locus.NoOccurrences then
+         if not (Locusops.is_all_occurrences occs) && occs != Locus.NoOccurrences then
            begin nested := true; ignore (subst_below k t); nested := false end;
          (* Do the effective substitution *)
          Vars.lift k (bywhat ()))
@@ -161,13 +161,13 @@ let make_eq_univs_test env evd c =
     match EConstr.eq_constr_universes_proj env evd c c' with
     | None -> raise (NotUnifiable None)
     | Some cst ->
-	try Evd.add_universe_constraints evd cst
-	with Evd.UniversesDiffer -> raise (NotUnifiable None)
+        try Evd.add_universe_constraints evd cst
+        with Evd.UniversesDiffer -> raise (NotUnifiable None)
     );
   merge_fun = (fun evd _ -> evd);
   testing_state = evd;
   last_found = None
-} 
+}
 
 let subst_closed_term_occ env evd occs c t =
   let test = make_eq_univs_test env evd c in
