@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -10,6 +10,7 @@
 open CErrors
 open Term
 open Constr
+open Context
 open Vars
 open Environ
 open Reduction
@@ -28,7 +29,7 @@ exception Find_at of int
 (* profiling *)
 
 let profiling_enabled = ref false
-    
+
 (* for supported platforms, filename for profiler results *)
 
 let profile_filename = ref "native_compute_profile.data"
@@ -51,8 +52,8 @@ let set_profile_filename fn =
 (* find unused profile filename *)
 let get_available_profile_filename () =
   let profile_filename = get_profile_filename () in
-  let dir = Filename.dirname profile_filename in 
-  let base = Filename.basename profile_filename in 
+  let dir = Filename.dirname profile_filename in
+  let base = Filename.basename profile_filename in
   (* starting with OCaml 4.04, could use Filename.remove_extension and Filename.extension, which
      gets rid of need for exception-handling here
   *)
@@ -64,7 +65,7 @@ let get_available_profile_filename () =
       (nm,ex)
     with Invalid_argument _ -> (base,"")
   in
-  try 
+  try
     (* unlikely race: fn deleted, another process uses fn *)
     Filename.temp_file ~temp_dir:dir (name ^ "_") ext
   with Sys_error s ->
@@ -74,31 +75,33 @@ let get_available_profile_filename () =
 
 let get_profiling_enabled () =
   !profiling_enabled
-    
+
 let set_profiling_enabled b =
   profiling_enabled := b
-    
+
 let invert_tag cst tag reloc_tbl =
   try
     for j = 0 to Array.length reloc_tbl - 1 do
       let tagj,arity = reloc_tbl.(j) in
       if Int.equal tag tagj && (cst && Int.equal arity 0 || not(cst || Int.equal arity 0)) then
-	raise (Find_at j)
+        raise (Find_at j)
       else ()
     done;raise Not_found
   with Find_at j -> (j+1)
 
 let decompose_prod env t =
-  let (name,dom,codom as res) = destProd (whd_all env t) in
-  match name with
-  | Anonymous -> (Name (Id.of_string "x"),dom,codom)
-  | _ -> res
+  let (name,dom,codom) = destProd (whd_all env t) in
+  let name = map_annot (function
+      | Anonymous -> Name (Id.of_string "x")
+      | na -> na) name
+  in
+  (name,dom,codom)
 
 let app_type env c =
   let t = whd_all env c in
   try destApp t with DestKO -> (t,[||])
 
- 
+
 let find_rectype_a env c =
   let (t, l) = app_type env c in
   match kind t with
@@ -107,37 +110,33 @@ let find_rectype_a env c =
 
 (* Instantiate inductives and parameters in constructor type *)
 
-let type_constructor mind mib u typ params =
+let type_constructor mind mib u (ctx, typ) params =
+  let typ = it_mkProd_or_LetIn typ ctx in
   let s = ind_subst mind mib u in
   let ctyp = substl s typ in
   let nparams = Array.length params in
   if Int.equal nparams 0 then ctyp
   else
-    let _,ctyp = decompose_prod_n nparams ctyp in   
+    let _,ctyp = decompose_prod_n nparams ctyp in
     substl (List.rev (Array.to_list params)) ctyp
 
 let construct_of_constr_notnative const env tag (mind, _ as ind) u allargs =
   let mib,mip = lookup_mind_specif env ind in
   let nparams = mib.mind_nparams in
   let params = Array.sub allargs 0 nparams in
-  try
-    if const then
-      let ctyp = type_constructor mind mib u (mip.mind_nf_lc.(0)) params in
-      Retroknowledge.get_vm_decompile_constant_info env.retroknowledge (mkInd ind) tag, ctyp
-    else
-      raise Not_found
-  with Not_found ->
   let i = invert_tag const tag mip.mind_reloc_tbl in
   let ctyp = type_constructor mind mib u (mip.mind_nf_lc.(i-1)) params in
   (mkApp(mkConstructU((ind,i),u), params), ctyp)
- 
+
 
 let construct_of_constr const env sigma tag typ =
   let t, l = app_type env typ in
   match EConstr.kind_upto sigma t with
-  | Ind (ind,u) -> 
+  | Ind (ind,u) ->
       construct_of_constr_notnative const env tag ind u l
-  | _ -> assert false
+  | _ ->
+    assert (Constr.equal t (Typeops.type_of_int env));
+    (mkInt (Uint63.of_int tag), t)
 
 let construct_of_constr_const env sigma tag typ =
   fst (construct_of_constr true env sigma tag typ)
@@ -166,7 +165,7 @@ let build_branches_type env sigma (mind,_ as _ind) mib mip u params p =
       let params = Array.map (lift ndecl) params in
       let dep_cstr = mkApp(mkApp(mkConstructU (cstr,snd ind),params),relargs) in
       mkApp(papp,[|dep_cstr|])
-    in 
+    in
     decl, decl_with_letin, codom
   in Array.mapi build_one_branch mip.mind_nf_lc
 
@@ -175,11 +174,11 @@ let build_case_type p realargs c =
 
 (* normalisation of values *)
 
-let branch_of_switch lvl ans bs = 
+let branch_of_switch lvl ans bs =
   let tbl = ans.asw_reloc in
-  let branch i = 
+  let branch i =
     let tag,arity = tbl.(i) in
-    let ci = 
+    let ci =
       if Int.equal arity 0 then mk_const tag
       else mk_block tag (mk_rels_accu lvl arity) in
     bs ci in
@@ -196,11 +195,11 @@ let get_proj env (ind, proj_arg) =
 let rec nf_val env sigma v typ =
   match kind_of_value v with
   | Vaccu accu -> nf_accu env sigma accu
-  | Vfun f -> 
+  | Vfun f ->
       let lvl = nb_rel env in
-      let name,dom,codom = 
-	try decompose_prod env typ
-	with DestKO ->
+      let name,dom,codom =
+        try decompose_prod env typ
+        with DestKO ->
           CErrors.anomaly
             (Pp.strbrk "Returned a functional value in a type not recognized as a product type.")
       in
@@ -208,6 +207,8 @@ let rec nf_val env sigma v typ =
       let body = nf_val env sigma (f (mk_rel_accu lvl)) codom in
       mkLambda(name,dom,body)
   | Vconst n -> construct_of_constr_const env sigma n typ
+  | Vint64 i -> i |> Uint63.of_int64 |> mkInt
+  | Vfloat64 f -> f |> Float64.of_float |> mkFloat
   | Vblock b ->
       let capp,ctyp = construct_of_constr_block env sigma (block_tag b) typ in
       let args = nf_bargs env sigma b ctyp in
@@ -220,7 +221,7 @@ and nf_type env sigma v =
 
 and nf_type_sort env sigma v =
   match kind_of_value v with
-  | Vaccu accu -> 
+  | Vaccu accu ->
       let t,s = nf_accu_type env sigma accu in
       let s =
         try
@@ -248,12 +249,12 @@ and nf_accu_type env sigma accu =
     mkApp(a,Array.of_list args), t
 
 and nf_args env sigma args t =
-  let aux arg (t,l) = 
+  let aux arg (t,l) =
     let _,dom,codom =
       try decompose_prod env t with
-	DestKO ->
-	CErrors.anomaly
-	  (Pp.strbrk "Returned a functional value in a type not recognized as a product type.")
+        DestKO ->
+        CErrors.anomaly
+          (Pp.strbrk "Returned a functional value in a type not recognized as a product type.")
     in
     let c = nf_val env sigma arg dom in
     (subst1 c codom, c::l)
@@ -267,10 +268,10 @@ and nf_bargs env sigma b t =
   Array.init len
     (fun i ->
       let _,dom,codom =
-	try decompose_prod env !t with
-	  DestKO ->
-	  CErrors.anomaly
-	    (Pp.strbrk "Returned a functional value in a type not recognized as a product type.")
+        try decompose_prod env !t with
+          DestKO ->
+          CErrors.anomaly
+            (Pp.strbrk "Returned a functional value in a type not recognized as a product type.")
       in
       let c = nf_val env sigma (block_field b i) dom in
       t := subst1 c codom; c)
@@ -283,11 +284,13 @@ and nf_atom env sigma atom =
   | Asort s -> mkSort s
   | Avar id -> mkVar id
   | Aprod(n,dom,codom) ->
-      let dom = nf_type env sigma dom in
-      let vn = mk_rel_accu (nb_rel env) in
-      let env = push_rel (LocalAssum (n,dom)) env in
-      let codom = nf_type env sigma (codom vn) in
-      mkProd(n,dom,codom)
+    let dom, sdom = nf_type_sort env sigma dom in
+    let rdom = Sorts.relevance_of_sort sdom in
+    let n = make_annot n rdom in
+    let vn = mk_rel_accu (nb_rel env) in
+    let env = push_rel (LocalAssum (n,dom)) env in
+    let codom = nf_type env sigma (codom vn) in
+    mkProd(n,dom,codom)
   | Ameta (mv,_) -> mkMeta mv
   | Aproj (p, c) ->
       let c = nf_accu env sigma c in
@@ -315,9 +318,9 @@ and nf_atom_type env sigma atom =
       let nparams = mib.mind_nparams in
       let params,realargs = Array.chop nparams allargs in
       let nparamdecls = Context.Rel.length (Inductive.inductive_paramdecls (mib,u)) in
-      let pT = 
+      let pT =
         hnf_prod_applist_assum env nparamdecls
-	  (Inductiveops.type_of_inductive env ind) (Array.to_list params) in
+          (Inductiveops.type_of_inductive env ind) (Array.to_list params) in
       let p = nf_predicate env sigma ind mip params p pT in
       (* Calcul du type des branches *)
       let btypes = build_branches_type env sigma (fst ind) mib mip u params p in
@@ -327,34 +330,40 @@ and nf_atom_type env sigma atom =
        let decl,decl_with_letin,codom = btypes.(i) in
        let b = nf_val (Termops.push_rels_assum decl env) sigma v codom in
         Termops.it_mkLambda_or_LetIn_from_no_LetIn b decl_with_letin
-      in 
+      in
       let branchs = Array.mapi mkbranch bsw in
       let tcase = build_case_type p realargs a in
       let ci = ans.asw_ci in
-      mkCase(ci, p, a, branchs), tcase 
+      mkCase(ci, p, a, branchs), tcase
   | Afix(tt,ft,rp,s) ->
-      let tt = Array.map (fun t -> nf_type env sigma t) tt in
-      let name = Array.map (fun _ -> (Name (Id.of_string "Ffix"))) tt in
+      let tt = Array.map (fun t -> nf_type_sort env sigma t) tt in
+      let tt = Array.map fst tt and rt = Array.map snd tt in
+      let name = Name (Id.of_string "Ffix") in
+      let names = Array.map (fun s -> make_annot name (Sorts.relevance_of_sort s)) rt in
       let lvl = nb_rel env in
       let nbfix = Array.length ft in
       let fargs = mk_rels_accu lvl (Array.length ft) in
-      (* Third argument of the tuple is ignored by push_rec_types *)
-      let env = push_rec_types (name,tt,[||]) env in
+      (* Body argument of the tuple is ignored by push_rec_types *)
+      let env = push_rec_types (names,tt,[||]) env in
       (* We lift here because the types of arguments (in tt) will be evaluated
          in an environment where the fixpoints have been pushed *)
       let norm_body i v = nf_val env sigma (napply v fargs) (lift nbfix tt.(i)) in
       let ft = Array.mapi norm_body ft in
-      mkFix((rp,s),(name,tt,ft)), tt.(s)
+      mkFix((rp,s),(names,tt,ft)), tt.(s)
   | Acofix(tt,ft,s,_) | Acofixe(tt,ft,s,_) ->
-      let tt = Array.map (nf_type env sigma) tt in
-      let name = Array.map (fun _ -> (Name (Id.of_string "Fcofix"))) tt in
+      let tt = Array.map (fun t -> nf_type_sort env sigma t) tt in
+      let tt = Array.map fst tt and rt = Array.map snd tt in
+      let name = Name (Id.of_string "Fcofix") in
       let lvl = nb_rel env in
+      let names = Array.map (fun s -> make_annot name (Sorts.relevance_of_sort s)) rt in
       let fargs = mk_rels_accu lvl (Array.length ft) in
-      let env = push_rec_types (name,tt,[||]) env in
+      let env = push_rec_types (names,tt,[||]) env in
       let ft = Array.mapi (fun i v -> nf_val env sigma (napply v fargs) tt.(i)) ft in
-      mkCoFix(s,(name,tt,ft)), tt.(s)
+      mkCoFix(s,(names,tt,ft)), tt.(s)
   | Aprod(n,dom,codom) ->
       let dom,s1 = nf_type_sort env sigma dom in
+      let r1 = Sorts.relevance_of_sort s1 in
+      let n = make_annot n r1 in
       let vn = mk_rel_accu (nb_rel env) in
       let env = push_rel (LocalAssum (n,dom)) env in
       let codom,s2 = nf_type_sort env sigma (codom vn) in
@@ -384,7 +393,7 @@ and  nf_predicate env sigma ind mip params v pT =
       let k = nb_rel env in
       let vb = f (mk_rel_accu k) in
       let body =
-	nf_predicate (push_rel (LocalAssum (name,dom)) env) sigma ind mip params vb codom in
+        nf_predicate (push_rel (LocalAssum (name,dom)) env) sigma ind mip params vb codom in
       mkLambda(name,dom,body)
     | _ -> nf_type env sigma v
     end
@@ -398,6 +407,8 @@ and  nf_predicate env sigma ind mip params v pT =
       let rargs = Array.init n (fun i -> mkRel (n-i)) in
       let params = if Int.equal n 0 then params else Array.map (lift n) params in
       let dom = mkApp(mkIndU ind,Array.append params rargs) in
+      let r = Inductive.relevance_of_inductive env (fst ind) in
+      let name = make_annot name r in
       let body = nf_type (push_rel (LocalAssum (name,dom)) env) sigma vb in
       mkLambda(name,dom,body)
     | _ -> nf_type env sigma v
@@ -405,20 +416,21 @@ and  nf_predicate env sigma ind mip params v pT =
 and nf_evar env sigma evk args =
   let evi = try Evd.find sigma evk with Not_found -> assert false in
   let hyps = Environ.named_context_of_val (Evd.evar_filtered_hyps evi) in
-  let ty = EConstr.Unsafe.to_constr @@ Evd.evar_concl evi in
+  let ty = EConstr.to_constr ~abort_on_undefined_evars:false sigma @@ Evd.evar_concl evi in
   if List.is_empty hyps then begin
     assert (Int.equal (Array.length args) 0);
     mkEvar (evk, [||]), ty
   end
   else
-    (** Let-bound arguments are present in the evar arguments but not in the
-        type, so we turn the let into a product. *)
+    (* Let-bound arguments are present in the evar arguments but not
+       in the type, so we turn the let into a product. *)
     let hyps = Context.Named.drop_bodies hyps in
     let fold accu d = Term.mkNamedProd_or_LetIn d accu in
     let t = List.fold_left fold ty hyps in
     let ty, args = nf_args env sigma args t in
-    (** nf_args takes arguments in the reverse order but produces them in the
-        correct one, so we have to reverse them again for the evar node *)
+    (* nf_args takes arguments in the reverse order but produces them
+       in the correct one, so we have to reverse them again for the
+       evar node *)
     mkEvar (evk, Array.rev_of_list args), ty
 
 let evars_of_evar_map sigma =
@@ -432,23 +444,23 @@ let start_profiler_linux profile_fn =
   let dev_null = Unix.descr_of_out_channel (open_out_bin "/dev/null") in
   let _ = Feedback.msg_info (Pp.str ("Profiling to file " ^ profile_fn)) in
   let perf = "perf" in
-  let profiler_pid = 
+  let profiler_pid =
     Unix.create_process
       perf
       [|perf; "record"; "-g"; "-o"; profile_fn; "-p"; string_of_int coq_pid |]
       Unix.stdin dev_null dev_null
   in
   (* doesn't seem to be a way to test whether process creation succeeded *)
-  if !Flags.debug then 
+  if !Flags.debug then
     Feedback.msg_debug (Pp.str (Format.sprintf "Native compute profiler started, pid = %d, output to: %s" profiler_pid profile_fn));
   Some profiler_pid
 
 (* kill profiler via SIGINT *)
-let stop_profiler_linux m_pid = 
-  match m_pid with 
+let stop_profiler_linux m_pid =
+  match m_pid with
   | Some pid -> (
     let _ = if !Flags.debug then Feedback.msg_debug (Pp.str "Stopping native code profiler") in
-    try 
+    try
       Unix.kill pid Sys.sigint;
       let _ = Unix.waitpid [] pid in ()
     with Unix.Unix_error (Unix.ESRCH,"kill","") ->
@@ -463,7 +475,7 @@ let start_profiler () =
   | _ ->
      let _ = Feedback.msg_info
        (Pp.str (Format.sprintf "Native_compute profiling not supported on the platform: %s"
-		  (profiler_platform ()))) in
+                  (profiler_platform ()))) in
      None
 
 let stop_profiler m_pid =
@@ -481,25 +493,23 @@ let native_norm env sigma c ty =
   Format.eprintf "Numbers of free variables (named): %i\n" (List.length vl1);
   Format.eprintf "Numbers of free variables (rel): %i\n" (List.length vl2);
   *)
-  let ml_filename, prefix = Nativelib.get_ml_filename () in
-  let code, upd = mk_norm_code env (evars_of_evar_map sigma) prefix c in
-  let profile = get_profiling_enabled () in
-  match Nativelib.compile ml_filename code ~profile:profile with
-    | true, fn ->
-        if !Flags.debug then Feedback.msg_debug (Pp.str "Running norm ...");
-	let profiler_pid = if profile then start_profiler () else None in
-        let t0 = Sys.time () in
-        Nativelib.call_linker ~fatal:true prefix fn (Some upd);
-        let t1 = Sys.time () in
-	if profile then stop_profiler profiler_pid;
-        let time_info = Format.sprintf "Evaluation done in %.5f@." (t1 -. t0) in
-        if !Flags.debug then Feedback.msg_debug (Pp.str time_info);
-        let res = nf_val env sigma !Nativelib.rt1 ty in
-        let t2 = Sys.time () in
-        let time_info = Format.sprintf "Reification done in %.5f@." (t2 -. t1) in
-        if !Flags.debug then Feedback.msg_debug (Pp.str time_info);
-        EConstr.of_constr res
-    | _ -> anomaly (Pp.str "Compilation failure.") 
+    let ml_filename, prefix = Nativelib.get_ml_filename () in
+    let code, upd = mk_norm_code env (evars_of_evar_map sigma) prefix c in
+    let profile = get_profiling_enabled () in
+    let fn = Nativelib.compile ml_filename code ~profile:profile in
+    if !Flags.debug then Feedback.msg_debug (Pp.str "Running norm ...");
+    let profiler_pid = if profile then start_profiler () else None in
+    let t0 = Sys.time () in
+    Nativelib.call_linker ~fatal:true env ~prefix fn (Some upd);
+    let t1 = Sys.time () in
+    if profile then stop_profiler profiler_pid;
+    let time_info = Format.sprintf "Evaluation done in %.5f@." (t1 -. t0) in
+    if !Flags.debug then Feedback.msg_debug (Pp.str time_info);
+    let res = nf_val env sigma !Nativelib.rt1 ty in
+    let t2 = Sys.time () in
+    let time_info = Format.sprintf "Reification done in %.5f@." (t2 -. t1) in
+    if !Flags.debug then Feedback.msg_debug (Pp.str time_info);
+    EConstr.of_constr res
 
 let native_conv_generic pb sigma t =
   Nativeconv.native_conv_gen pb (evars_of_evar_map sigma) t
