@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -10,11 +10,11 @@
 
 open Util
 open Constr
+open Context
 open Pp
 open Names
 open Environ
 open Declarations
-open Globnames
 open Libnames
 open Goptions
 
@@ -23,8 +23,6 @@ open Goptions
     - The "rich" one, that also tries to print the types of the fields.
     The short version used to be the default behavior, but now we print
     types by default. The following option allows changing this.
-    Technically, the environments in this file are either None in
-    the "short" mode or (Some env) in the "rich" one.
 *)
 
 module Tag =
@@ -39,9 +37,11 @@ let tag t s = Pp.tag t s
 let tag_definition s = tag Tag.definition s
 let tag_keyword s = tag Tag.keyword s
 
+type short = OnlyNames | WithContents
+
 let short = ref false
 
-let _ =
+let () =
   declare_bool_option
     { optdepr  = false;
       optname  = "short module printing";
@@ -62,10 +62,10 @@ let keyword s = tag_keyword (str s)
 let get_new_id locals id =
   let rec get_id l id =
     let dir = DirPath.make [id] in
-      if not (Nametab.exists_module dir) then
-	id
+      if not (Nametab.exists_dir dir) then
+        id
       else
-	get_id (Id.Set.add id l) (Namegen.next_ident_away id l)
+        get_id (Id.Set.add id l) (Namegen.next_ident_away id l)
   in
   let avoid = List.fold_left (fun accu (_, id) -> Id.Set.add id accu) Id.Set.empty locals in
     get_id avoid id
@@ -90,9 +90,7 @@ let build_ind_type env mip =
   Inductive.type_of_inductive env mip
 
 let print_one_inductive env sigma mib ((_,i) as ind) =
-  let u = if Declareops.inductive_is_polymorphic mib then 
-      Univ.AUContext.instance (Declareops.inductive_polymorphic_context mib)
-    else Univ.Instance.empty in
+  let u = Univ.make_abstract_instance (Declareops.inductive_polymorphic_context mib) in
   let mip = mib.mind_packets.(i) in
   let params = Inductive.inductive_paramdecls (mib,u) in
   let nparamdecls = Context.Rel.length params in
@@ -111,16 +109,6 @@ let print_one_inductive env sigma mib ((_,i) as ind) =
     str ": " ++ Printer.pr_lconstr_env envpar sigma arity ++ str " :=") ++
   brk(0,2) ++ print_constructors envpar sigma mip.mind_consnames cstrtypes
 
-let instantiate_cumulativity_info cumi =
-  let open Univ in
-  let univs = ACumulativityInfo.univ_context cumi in
-  let expose ctx =
-    let inst = AUContext.instance ctx in
-    let cst = AUContext.instantiate inst ctx in
-    UContext.make (inst, cst)
-  in
-  CumulativityInfo.make (expose univs, ACumulativityInfo.variance cumi)
-
 let print_mutual_inductive env mind mib udecl =
   let inds = List.init (Array.length mib.mind_packets) (fun x -> (mind, x))
   in
@@ -131,47 +119,30 @@ let print_mutual_inductive env mind mib udecl =
     | BiFinite -> "Variant"
     | CoFinite -> "CoInductive"
   in
-  let univs =
-    let open Univ in
-    if Declareops.inductive_is_polymorphic mib then
-      Array.to_list (Instance.to_array
-                       (AUContext.instance (Declareops.inductive_polymorphic_context mib)))
-    else []
+  let bl = UnivNames.universe_binders_with_opt_names
+      (Declareops.inductive_polymorphic_context mib) udecl
   in
-  let bl = UnivNames.universe_binders_with_opt_names (IndRef (mind, 0)) univs udecl in
   let sigma = Evd.from_ctx (UState.of_binders bl) in
-  hov 0 (Printer.pr_polymorphic (Declareops.inductive_is_polymorphic mib) ++
-         Printer.pr_cumulative
-           (Declareops.inductive_is_polymorphic mib) 
-           (Declareops.inductive_is_cumulative mib) ++
-         def keyword ++ spc () ++
+  hov 0 (def keyword ++ spc () ++
          prlist_with_sep (fun () -> fnl () ++ str"  with ")
            (print_one_inductive env sigma mib) inds ++
-         match mib.mind_universes with
-         | Monomorphic_ind _ | Polymorphic_ind _ -> str ""
-         | Cumulative_ind cumi ->
-           Printer.pr_cumulativity_info
-             sigma (instantiate_cumulativity_info cumi))
+         Printer.pr_universes sigma ?variance:mib.mind_variance mib.mind_universes)
 
 let get_fields =
   let rec prodec_rec l subst c =
     match kind c with
     | Prod (na,t,c) ->
-        let id = match na with Name id -> id | Anonymous -> Id.of_string "_" in
+        let id = match na.binder_name with Name id -> id | Anonymous -> Id.of_string "_" in
         prodec_rec ((id,true,Vars.substl subst t)::l) (mkVar id::subst) c
     | LetIn (na,b,_,c) ->
-        let id = match na with Name id -> id | Anonymous -> Id.of_string "_" in
+        let id = match na.binder_name with Name id -> id | Anonymous -> Id.of_string "_" in
         prodec_rec ((id,false,Vars.substl subst b)::l) (mkVar id::subst) c
     | _               -> List.rev l
   in
   prodec_rec [] []
 
 let print_record env mind mib udecl =
-  let u = 
-    if Declareops.inductive_is_polymorphic mib then 
-      Univ.AUContext.instance (Declareops.inductive_polymorphic_context mib)
-    else Univ.Instance.empty 
-  in
+  let u = Univ.make_abstract_instance (Declareops.inductive_polymorphic_context mib) in
   let mip = mib.mind_packets.(0) in
   let params = Inductive.inductive_paramdecls (mib,u) in
   let nparamdecls = Context.Rel.length params in
@@ -181,8 +152,9 @@ let print_record env mind mib udecl =
   let cstrtype = hnf_prod_applist_assum env nparamdecls cstrtypes.(0) args in
   let fields = get_fields cstrtype in
   let envpar = push_rel_context params env in
-  let bl = UnivNames.universe_binders_with_opt_names (IndRef (mind,0))
-      (Array.to_list (Univ.Instance.to_array u)) udecl in
+  let bl = UnivNames.universe_binders_with_opt_names (Declareops.inductive_polymorphic_context mib)
+      udecl
+  in
   let sigma = Evd.from_ctx (UState.of_binders bl) in
   let keyword =
     let open Declarations in
@@ -193,10 +165,6 @@ let print_record env mind mib udecl =
   in
   hov 0 (
     hov 0 (
-      Printer.pr_polymorphic (Declareops.inductive_is_polymorphic mib) ++
-      Printer.pr_cumulative
-        (Declareops.inductive_is_polymorphic mib)
-        (Declareops.inductive_is_cumulative mib) ++
       def keyword ++ spc () ++ Id.print mip.mind_typename ++ brk(1,4) ++
       print_params env sigma params ++
       str ": " ++ Printer.pr_lconstr_env envpar sigma arity ++ brk(1,2) ++
@@ -207,11 +175,7 @@ let print_record env mind mib udecl =
         (fun (id,b,c) ->
           Id.print id ++ str (if b then " : " else " := ") ++
           Printer.pr_lconstr_env envpar sigma c) fields) ++ str" }" ++
-    match mib.mind_universes with
-    | Monomorphic_ind _ | Polymorphic_ind _ -> str ""
-    | Cumulative_ind cumi ->
-      Printer.pr_cumulativity_info
-        sigma (instantiate_cumulativity_info cumi)
+    Printer.pr_universes sigma ?variance:mib.mind_variance mib.mind_universes
   )
 
 let pr_mutual_inductive_body env mind mib udecl =
@@ -241,15 +205,15 @@ let print_kn locals kn =
       pr_qualid qid
   with
       Not_found ->
-	try
-	  print_local_modpath locals kn
-	with
-	    Not_found -> print_modpath locals kn
+        try
+          print_local_modpath locals kn
+        with
+            Not_found -> print_modpath locals kn
 
 let nametab_register_dir obj_mp =
   let id = mk_fake_top () in
   let obj_dir = DirPath.make [id] in
-  Nametab.push_dir (Nametab.Until 1) obj_dir (DirModule { obj_dir; obj_mp; obj_sec = DirPath.empty })
+  Nametab.(push_dir (Until 1) obj_dir (GlobDirRef.DirModule { obj_dir; obj_mp; }))
 
 (** Nota: the [global_reference] we register in the nametab below
     might differ from internal ones, since we cannot recreate here
@@ -266,19 +230,24 @@ let nametab_register_body mp dir (l,body) =
     | SFBmodule _ -> () (* TODO *)
     | SFBmodtype _ -> () (* TODO *)
     | SFBconst _ ->
-      push (Label.to_id l) (ConstRef (Constant.make2 mp l))
+      push (Label.to_id l) (GlobRef.ConstRef (Constant.make2 mp l))
     | SFBmind mib ->
       let mind = MutInd.make2 mp l in
       Array.iteri
-	(fun i mip ->
-	  push mip.mind_typename (IndRef (mind,i));
-	  Array.iteri (fun j id -> push id (ConstructRef ((mind,i),j+1)))
-	    mip.mind_consnames)
-	mib.mind_packets
+        (fun i mip ->
+          push mip.mind_typename (GlobRef.IndRef (mind,i));
+          Array.iteri (fun j id -> push id (GlobRef.ConstructRef ((mind,i),j+1)))
+            mip.mind_consnames)
+        mib.mind_packets
 
-let nametab_register_module_body mp struc =
+type mod_ops =
+  { import_module : export:bool -> ModPath.t -> unit
+  ; process_module_binding : MBId.t -> Declarations.module_alg_expr -> unit
+  }
+
+let nametab_register_module_body ~mod_ops mp struc =
   (* If [mp] is a globally visible module, we simply import it *)
-  try Declaremods.really_import_module mp
+  try mod_ops.import_module ~export:false mp
   with Not_found ->
     (* Otherwise we try to emulate an import by playing with nametab *)
     nametab_register_dir mp;
@@ -288,7 +257,7 @@ let get_typ_expr_alg mtb = match mtb.mod_type_alg with
   | Some (NoFunctor me) -> me
   | _ -> raise Not_found
 
-let nametab_register_modparam mbid mtb =
+let nametab_register_modparam ~mod_ops mbid mtb =
   let id = MBId.to_id mbid in
   match mtb.mod_type with
   | MoreFunctor _ -> id (* functorial param : nothing to register *)
@@ -296,7 +265,7 @@ let nametab_register_modparam mbid mtb =
     (* We first try to use the algebraic type expression if any,
        via a Declaremods function that converts back to module entries *)
     try
-      let () = Declaremods.process_module_binding mbid (get_typ_expr_alg mtb) in
+      let () = mod_ops.process_module_binding mbid (get_typ_expr_alg mtb) in
       id
     with e when CErrors.noncritical e ->
       (* Otherwise, we try to play with the nametab ourselves *)
@@ -308,43 +277,36 @@ let nametab_register_modparam mbid mtb =
       List.iter (nametab_register_body mp dir) struc;
       id
 
-let print_body is_impl env mp (l,body) =
+let print_body is_impl extent env mp (l,body) =
   let name = Label.print l in
   hov 2 (match body with
     | SFBmodule _ -> keyword "Module" ++ spc () ++ name
     | SFBmodtype _ -> keyword "Module Type" ++ spc () ++ name
     | SFBconst cb ->
        let ctx = Declareops.constant_polymorphic_context cb in
-       let u =
-	 if Declareops.constant_is_polymorphic cb then
-            Univ.AUContext.instance ctx
-	 else Univ.Instance.empty
-       in
-       let ctx = Univ.UContext.make (u, Univ.AUContext.instantiate u ctx) in
       (match cb.const_body with
-	| Def _ -> def "Definition" ++ spc ()
-	| OpaqueDef _ when is_impl -> def "Theorem" ++ spc ()
-	| _ -> def "Parameter" ++ spc ()) ++ name ++
-      (match env with
-	  | None -> mt ()
-	  | Some env ->
-	    str " :" ++ spc () ++
-            hov 0 (Printer.pr_ltype_env env (Evd.from_env env)
-		(Vars.subst_instance_constr u
-   		  cb.const_type)) ++
-	    (match cb.const_body with
-	      | Def l when is_impl ->
-		spc () ++
-		hov 2 (str ":= " ++
-                       Printer.pr_lconstr_env env (Evd.from_env env)
-			  (Vars.subst_instance_constr u (Mod_subst.force_constr l)))
-	      | _ -> mt ()) ++ str "." ++
-            Printer.pr_universe_ctx (Evd.from_env env) ctx)
+        | Def _ -> def "Definition" ++ spc ()
+        | OpaqueDef _ when is_impl -> def "Theorem" ++ spc ()
+        | _ -> def "Parameter" ++ spc ()) ++ name ++
+      (match extent with
+         | OnlyNames -> mt ()
+         | WithContents ->
+            let bl = UnivNames.universe_binders_with_opt_names ctx None in
+            let sigma = Evd.from_ctx (UState.of_binders bl) in
+            str " :" ++ spc () ++
+            hov 0 (Printer.pr_ltype_env env sigma cb.const_type) ++
+            (match cb.const_body with
+              | Def l when is_impl ->
+                spc () ++
+                hov 2 (str ":= " ++
+                       Printer.pr_lconstr_env env sigma (Mod_subst.force_constr l))
+              | _ -> mt ()) ++ str "." ++
+            Printer.pr_abstract_universe_ctx sigma ctx)
     | SFBmind mib ->
-      try
-	let env = Option.get env in
+      match extent with
+      | WithContents ->
         pr_mutual_inductive_body env (MutInd.make2 mp l) mib None
-      with e when CErrors.noncritical e ->
+      | OnlyNames ->
         let keyword =
           let open Declarations in
           match mib.mind_finite with
@@ -352,25 +314,24 @@ let print_body is_impl env mp (l,body) =
           | BiFinite -> def "Variant"
           | CoFinite -> def "CoInductive"
         in
-	keyword ++ spc () ++ name)
+        keyword ++ spc () ++ name)
 
-let print_struct is_impl env mp struc =
-  prlist_with_sep spc (print_body is_impl env mp) struc
+let print_struct is_impl extent env mp struc =
+  prlist_with_sep spc (print_body is_impl extent env mp) struc
 
-let print_structure is_type env mp locals struc =
-  let env' = Option.map
-    (Modops.add_structure mp struc Mod_subst.empty_delta_resolver) env in
-  nametab_register_module_body mp struc;
+let print_structure ~mod_ops is_type extent env mp locals struc =
+  let env' = Modops.add_structure mp struc Mod_subst.empty_delta_resolver env in
+  nametab_register_module_body ~mod_ops mp struc;
   let kwd = if is_type then "Sig" else "Struct" in
-  hv 2 (keyword kwd ++ spc () ++ print_struct false env' mp struc ++
-	brk (1,-2) ++ keyword "End")
+  hv 2 (keyword kwd ++ spc () ++ print_struct false extent env' mp struc ++
+        brk (1,-2) ++ keyword "End")
 
 let rec flatten_app mexpr l = match mexpr with
   | MEapply (mexpr, arg) -> flatten_app mexpr (arg::l)
   | MEident mp -> mp::l
   | MEwith _ -> assert false
 
-let rec print_typ_expr env mp locals mty =
+let rec print_typ_expr extent env mp locals mty =
   match mty with
   | MEident kn -> print_kn locals kn
   | MEapply _ ->
@@ -378,21 +339,25 @@ let rec print_typ_expr env mp locals mty =
       let fapp = List.hd lapp in
       let mapp = List.tl lapp in
       hov 3 (str"(" ++ (print_kn locals fapp) ++ spc () ++
-		 prlist_with_sep spc (print_modpath locals) mapp ++ str")")
+                 prlist_with_sep spc (print_modpath locals) mapp ++ str")")
   | MEwith(me,WithDef(idl,(c, _)))->
-      let env' = None in (* TODO: build a proper environment if env <> None *)
       let s = String.concat "." (List.map Id.to_string idl) in
-      (* XXX: What should env and sigma be here? *)
-      let env = Global.env () in
-      let sigma = Evd.from_env env in
-      hov 2 (print_typ_expr env' mp locals me ++ spc() ++ str "with" ++ spc()
-             ++ def "Definition"++ spc() ++ str s ++ spc() ++ str ":="++ spc()
-             ++ Printer.pr_lconstr_env env sigma c)
+      let body = match extent with
+        | WithContents ->
+            let sigma = Evd.from_env env in
+            spc() ++ str ":=" ++ spc() ++ Printer.pr_lconstr_env env sigma c
+        | OnlyNames ->
+            mt() in
+      hov 2 (print_typ_expr extent env mp locals me ++ spc() ++ str "with" ++ spc()
+             ++ def "Definition"++ spc() ++ str s ++ body)
   | MEwith(me,WithMod(idl,mp'))->
       let s = String.concat "." (List.map Id.to_string idl) in
-      hov 2 (print_typ_expr env mp locals me ++ spc() ++ str "with" ++ spc() ++
-             keyword "Module"++ spc() ++ str s ++ spc() ++ str ":="++ spc()
-             ++ print_modpath locals mp')
+      let body = match extent with
+        | WithContents ->
+            spc() ++ str ":="++ spc() ++ print_modpath locals mp'
+        | OnlyNames -> mt () in
+      hov 2 (print_typ_expr extent env mp locals me ++ spc() ++ str "with" ++ spc() ++
+             keyword "Module"++ spc() ++ str s ++ body)
 
 let print_mod_expr env mp locals = function
   | MEident mp -> print_modpath locals mp
@@ -402,90 +367,93 @@ let print_mod_expr env mp locals = function
         (str"(" ++ prlist_with_sep spc (print_modpath locals) lapp ++ str")")
   | MEwith _ -> assert false (* No 'with' syntax for modules *)
 
-let rec print_functor fty fatom is_type env mp locals = function
-  |NoFunctor me -> fatom is_type env mp locals me
-  |MoreFunctor (mbid,mtb1,me2) ->
-      let id = nametab_register_modparam mbid mtb1 in
+let rec print_functor ~mod_ops fty fatom is_type extent env mp locals = function
+  | NoFunctor me -> fatom ~mod_ops is_type extent env mp locals me
+  | MoreFunctor (mbid,mtb1,me2) ->
+      let id = nametab_register_modparam ~mod_ops mbid mtb1 in
       let mp1 = MPbound mbid in
-      let pr_mtb1 = fty env mp1 locals mtb1 in
-      let env' = Option.map (Modops.add_module_type mp1 mtb1) env in
+      let pr_mtb1 = fty ~mod_ops extent env mp1 locals mtb1 in
+      let env' = Modops.add_module_type mp1 mtb1 env in
       let locals' = (mbid, get_new_id locals (MBId.to_id mbid))::locals in
       let kwd = if is_type then "Funsig" else "Functor" in
       hov 2
         (keyword kwd ++ spc () ++
-	 str "(" ++ Id.print id ++ str ":" ++ pr_mtb1 ++ str ")" ++
-	 spc() ++ print_functor fty fatom is_type env' mp locals' me2)
+         str "(" ++ Id.print id ++ str ":" ++ pr_mtb1 ++ str ")" ++
+         spc() ++ print_functor ~mod_ops fty fatom is_type extent env' mp locals' me2)
 
-let rec print_expression x =
-  print_functor
+let rec print_expression ~mod_ops x =
+  print_functor ~mod_ops
     print_modtype
-    (function true -> print_typ_expr | false -> print_mod_expr) x
+    (fun ~mod_ops -> function true -> print_typ_expr | false -> fun _ -> print_mod_expr) x
 
-and print_signature x =
-  print_functor print_modtype print_structure x
+and print_signature ~mod_ops x =
+  print_functor ~mod_ops print_modtype print_structure x
 
-and print_modtype env mp locals mtb = match mtb.mod_type_alg with
-  | Some me -> print_expression true env mp locals me
-  | None -> print_signature true env mp locals mtb.mod_type
+and print_modtype ~mod_ops extent env mp locals mtb = match mtb.mod_type_alg with
+  | Some me -> print_expression ~mod_ops true extent env mp locals me
+  | None -> print_signature ~mod_ops true extent env mp locals mtb.mod_type
 
 let rec printable_body dir =
   let dir = pop_dirpath dir in
     DirPath.is_empty dir ||
     try
+      let open Nametab.GlobDirRef in
       match Nametab.locate_dir (qualid_of_dirpath dir) with
-	  DirOpenModtype _ -> false
-	| DirModule _ | DirOpenModule _ -> printable_body dir
-	| _ -> true
+          DirOpenModtype _ -> false
+        | DirModule _ | DirOpenModule _ -> printable_body dir
+        | _ -> true
     with
-	Not_found -> true
+        Not_found -> true
 
 (** Since we might play with nametab above, we should reset to prior
     state after the printing *)
 
-let print_expression' is_type env mp me =
+let print_expression' ~mod_ops is_type extent env mp me =
   States.with_state_protection
-    (fun e -> print_expression is_type env mp [] e) me
+    (fun e -> print_expression ~mod_ops is_type extent env mp [] e) me
 
-let print_signature' is_type env mp me =
+let print_signature' ~mod_ops is_type extent env mp me =
   States.with_state_protection
-    (fun e -> print_signature is_type env mp [] e) me
+    (fun e -> print_signature ~mod_ops is_type extent env mp [] e) me
 
-let unsafe_print_module env mp with_body mb =
+let unsafe_print_module ~mod_ops extent env mp with_body mb =
   let name = print_modpath [] mp in
   let pr_equals = spc () ++ str ":= " in
   let body = match with_body, mb.mod_expr with
     | false, _
     | true, Abstract -> mt()
-    | _, Algebraic me -> pr_equals ++ print_expression' false env mp me
-    | _, Struct sign -> pr_equals ++ print_signature' false env mp sign
-    | _, FullStruct -> pr_equals ++ print_signature' false env mp mb.mod_type
+    | _, Algebraic me -> pr_equals ++ print_expression' ~mod_ops false extent env mp me
+    | _, Struct sign -> pr_equals ++ print_signature' ~mod_ops false extent env mp sign
+    | _, FullStruct -> pr_equals ++ print_signature' ~mod_ops false extent env mp mb.mod_type
   in
   let modtype = match mb.mod_expr, mb.mod_type_alg with
     | FullStruct, _ -> mt ()
-    | _, Some ty -> brk (1,1) ++ str": " ++ print_expression' true env mp ty
-    | _, _ -> brk (1,1) ++ str": " ++ print_signature' true env mp mb.mod_type
+    | _, Some ty -> brk (1,1) ++ str": " ++ print_expression' ~mod_ops true extent env mp ty
+    | _, _ -> brk (1,1) ++ str": " ++ print_signature' ~mod_ops true extent env mp mb.mod_type
   in
   hv 0 (keyword "Module" ++ spc () ++ name ++ modtype ++ body)
 
 exception ShortPrinting
 
-let print_module with_body mp =
+let print_module ~mod_ops with_body mp =
   let me = Global.lookup_module mp in
   try
     if !short then raise ShortPrinting;
-    unsafe_print_module (Some (Global.env ())) mp with_body me ++ fnl ()
+    unsafe_print_module ~mod_ops WithContents
+      (Global.env ()) mp with_body me ++ fnl ()
   with e when CErrors.noncritical e ->
-    unsafe_print_module None mp with_body me ++ fnl ()
+    unsafe_print_module ~mod_ops OnlyNames
+      (Global.env ()) mp with_body me ++ fnl ()
 
-let print_modtype kn =
+let print_modtype ~mod_ops kn =
   let mtb = Global.lookup_modtype kn in
   let name = print_kn [] kn in
   hv 1
     (keyword "Module Type" ++ spc () ++ name ++ str " =" ++ spc () ++
-     (try
-	if !short then raise ShortPrinting;
-	print_signature' true (Some (Global.env ())) kn mtb.mod_type
-      with e when CErrors.noncritical e ->
-	print_signature' true None kn mtb.mod_type))
-
-
+     try
+      if !short then raise ShortPrinting;
+      print_signature' ~mod_ops true WithContents
+        (Global.env ()) kn mtb.mod_type
+     with e when CErrors.noncritical e ->
+      print_signature' ~mod_ops true OnlyNames
+        (Global.env ()) kn mtb.mod_type)
