@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -15,6 +15,7 @@ open Names
 open Nameops
 open Termops
 open Constr
+open Context
 open Namegen
 open Environ
 open Evd
@@ -50,11 +51,11 @@ let refresh_undefined_univs clenv =
   match EConstr.kind clenv.evd clenv.templval.rebus with
   | Var _ -> clenv, Univ.empty_level_subst
   | App (f, args) when isVar clenv.evd f -> clenv, Univ.empty_level_subst
-  | _ ->  
+  | _ ->
     let evd', subst = Evd.refresh_undefined_universes clenv.evd in
     let map_freelisted f = { f with rebus = subst_univs_level_constr subst f.rebus } in
       { clenv with evd = evd'; templval = map_freelisted clenv.templval;
-	templtyp = map_freelisted clenv.templtyp }, subst
+        templtyp = map_freelisted clenv.templtyp }, subst
 
 let clenv_hnf_constr ce t = hnf_constr (cl_env ce) (cl_sigma ce) t
 
@@ -67,16 +68,16 @@ let clenv_push_prod cl =
   let rec clrec typ = match EConstr.kind cl.evd typ with
     | Cast (t,_,_) -> clrec t
     | Prod (na,t,u) ->
-	let mv = new_meta () in
-	let dep = not (noccurn (cl_sigma cl) 1 u) in
-	let na' = if dep then na else Anonymous in
+        let mv = new_meta () in
+        let dep = not (noccurn (cl_sigma cl) 1 u) in
+        let na' = if dep then na.binder_name else Anonymous in
         let e' = meta_declare mv t ~name:na' cl.evd in
-	let concl = if dep then subst1 (mkMeta mv) u else u in
-	let def = applist (cl.templval.rebus,[mkMeta mv]) in
-	{ templval = mk_freelisted def;
-	  templtyp = mk_freelisted concl;
-	  evd = e';
-	  env = cl.env }
+        let concl = if dep then subst1 (mkMeta mv) u else u in
+        let def = applist (cl.templval.rebus,[mkMeta mv]) in
+        { templval = mk_freelisted def;
+          templtyp = mk_freelisted concl;
+          evd = e';
+          env = cl.env }
     | _ -> raise NotExtensibleClause
   in clrec typ
 
@@ -101,12 +102,12 @@ let clenv_environments evd bound t =
       | (Some 0, _) -> (e, List.rev metas, t)
       | (n, Cast (t,_,_)) -> clrec (e,metas) n t
       | (n, Prod (na,t1,t2)) ->
-	  let mv = new_meta () in
-	  let dep = not (noccurn evd 1 t2) in
-	  let na' = if dep then na else Anonymous in
+          let mv = new_meta () in
+          let dep = not (noccurn evd 1 t2) in
+          let na' = if dep then na.binder_name else Anonymous in
           let e' = meta_declare mv t1 ~name:na' e in
-	  clrec (e', (mkMeta mv)::metas) (Option.map ((+) (-1)) n)
-	    (if dep then (subst1 (mkMeta mv) t2) else t2)
+          clrec (e', (mkMeta mv)::metas) (Option.map ((+) (-1)) n)
+            (if dep then (subst1 (mkMeta mv) t2) else t2)
       | (n, LetIn (na,b,_,t)) -> clrec (e,metas) n (subst1 b t)
       | (n, _) -> (e, List.rev metas, t)
   in
@@ -166,7 +167,7 @@ let clenv_assign mv rhs clenv =
       if not (EConstr.eq_constr clenv.evd (fst (meta_fvalue clenv.evd mv)).rebus rhs) then
         error_incompatible_inst clenv mv
       else
-	clenv
+        clenv
     else
       let st = (Conv,TypeNotProcessed) in
       {clenv with evd = meta_assign mv (rhs_fls.rebus,st) clenv.evd}
@@ -225,8 +226,8 @@ let dependent_closure clenv mvs =
   let rec aux mvs acc =
     Metaset.fold
       (fun mv deps ->
-	let metas_of_meta_type = clenv_metas_in_type_of_meta clenv.evd mv in
-	aux metas_of_meta_type (Metaset.union deps metas_of_meta_type))
+        let metas_of_meta_type = clenv_metas_in_type_of_meta clenv.evd mv in
+        aux metas_of_meta_type (Metaset.union deps metas_of_meta_type))
       mvs acc in
   aux mvs mvs
 
@@ -240,9 +241,9 @@ let clenv_dependent_gen hyps_only ?(iter=true) clenv =
   List.filter
     (fun mv ->
       if hyps_only then
-	Metaset.mem mv deps_in_hyps && not (Metaset.mem mv deps_in_concl)
+        Metaset.mem mv deps_in_hyps && not (Metaset.mem mv deps_in_concl)
       else
-	Metaset.mem mv deps_in_hyps || Metaset.mem mv deps_in_concl)
+        Metaset.mem mv deps_in_hyps || Metaset.mem mv deps_in_concl)
     all_undefined
 
 let clenv_missing ce = clenv_dependent_gen true ce
@@ -277,7 +278,7 @@ let adjust_meta_source evd mv = function
   | loc,Evar_kinds.VarInstance id ->
     let rec match_name c l =
       match EConstr.kind evd c, l with
-      | Lambda (Name id,_,c), a::l when EConstr.eq_constr evd a (mkMeta mv) -> Some id
+      | Lambda ({binder_name=Name id},_,c), a::l when EConstr.eq_constr evd a (mkMeta mv) -> Some id
       | Lambda (_,_,c), a::l -> match_name c l
       | _ -> None in
     (* This is very ad hoc code so that an evar inherits the name of the binder
@@ -324,21 +325,21 @@ let adjust_meta_source evd mv = function
 *)
 
 let clenv_pose_metas_as_evars clenv dep_mvs =
-  let rec fold clenv = function
-  | [] -> clenv
+  let rec fold clenv evs = function
+  | [] -> clenv, evs
   | mv::mvs ->
       let ty = clenv_meta_type clenv mv in
       (* Postpone the evar-ization if dependent on another meta *)
       (* This assumes no cycle in the dependencies - is it correct ? *)
-      if occur_meta clenv.evd ty then fold clenv (mvs@[mv])
+      if occur_meta clenv.evd ty then fold clenv evs (mvs@[mv])
       else
         let src = evar_source_of_meta mv clenv.evd in
         let src = adjust_meta_source clenv.evd mv src in
         let evd = clenv.evd in
-	let (evd, evar) = new_evar (cl_env clenv) evd ~src ty in
-	let clenv = clenv_assign mv evar {clenv with evd=evd} in
-	fold clenv mvs in
-  fold clenv dep_mvs
+        let (evd, evar) = new_evar (cl_env clenv) evd ~src ty in
+        let clenv = clenv_assign mv evar {clenv with evd=evd} in
+        fold clenv (fst (destEvar evd evar) :: evs) mvs in
+  fold clenv [] dep_mvs
 
 (******************************************************************)
 
@@ -414,13 +415,13 @@ let qhyp_eq h1 h2 = match h1, h2 with
 let check_bindings bl =
   match List.duplicates qhyp_eq (List.map (fun {CAst.v=x} -> fst x) bl) with
     | NamedHyp s :: _ ->
-	user_err 
-	  (str "The variable " ++ Id.print s ++
-	   str " occurs more than once in binding list.");
+        user_err
+          (str "The variable " ++ Id.print s ++
+           str " occurs more than once in binding list.");
     | AnonHyp n :: _ ->
-	user_err 
-	  (str "The position " ++ int n ++
-	   str " occurs more than once in binding list.")
+        user_err
+          (str "The position " ++ int n ++
+           str " occurs more than once in binding list.")
     | [] -> ()
 
 let explain_no_such_bound_variable evd id =
@@ -471,7 +472,7 @@ let meta_of_binder clause loc mvs = function
 let error_already_defined b =
   match b with
     | NamedHyp id ->
-        user_err 
+        user_err
           (str "Binder name \"" ++ Id.print id ++
            str"\" already defined with incompatible value.")
     | AnonHyp n ->
@@ -487,12 +488,12 @@ let clenv_unify_binding_type clenv c t u =
     try
       let evd,c = w_coerce_to_type (cl_env clenv) clenv.evd c t u in
       TypeProcessed, { clenv with evd = evd }, c
-    with 
+    with
       | PretypeError (_,_,ActualTypeNotCoercible (_,_,
           (NotClean _ | ConversionFailed _))) as e ->
-	  raise e
+          raise e
       | e when precatchable_exception e ->
-	  TypeNotProcessed, clenv, c
+          TypeNotProcessed, clenv, c
 
 let clenv_assign_binding clenv k c =
   let k_typ = clenv_hnf_constr clenv (clenv_meta_type clenv k) in
@@ -508,12 +509,12 @@ let clenv_match_args bl clenv =
     check_bindings bl;
     List.fold_left
       (fun clenv {CAst.loc;v=(b,c)} ->
-	let k = meta_of_binder clenv loc mvs b in
+        let k = meta_of_binder clenv loc mvs b in
         if meta_defined clenv.evd k then
           if EConstr.eq_constr clenv.evd (fst (meta_fvalue clenv.evd k)).rebus c then clenv
           else error_already_defined b
         else
-	  clenv_assign_binding clenv k c)
+          clenv_assign_binding clenv k c)
       clenv bl
 
 exception NoSuchBinding
@@ -524,7 +525,7 @@ let clenv_constrain_last_binding c clenv =
   clenv_assign_binding clenv k c
 
 let error_not_right_number_missing_arguments n =
-  user_err 
+  user_err
     (strbrk "Not the right number of missing arguments (expected " ++
       int n ++ str ").")
 
@@ -537,14 +538,14 @@ let clenv_constrain_dep_args hyps_only bl clenv =
       List.fold_left2 clenv_assign_binding clenv occlist bl
     else
       if hyps_only then
-	(* Tolerance for compatibility <= 8.3 *)
-	let occlist' = clenv_dependent_gen hyps_only ~iter:false clenv in
-	if Int.equal (List.length occlist') (List.length bl) then
-	  List.fold_left2 clenv_assign_binding clenv occlist' bl
-	else
-	  error_not_right_number_missing_arguments (List.length occlist)
+        (* Tolerance for compatibility <= 8.3 *)
+        let occlist' = clenv_dependent_gen hyps_only ~iter:false clenv in
+        if Int.equal (List.length occlist') (List.length bl) then
+          List.fold_left2 clenv_assign_binding clenv occlist' bl
+        else
+          error_not_right_number_missing_arguments (List.length occlist)
       else
-	error_not_right_number_missing_arguments (List.length occlist)
+        error_not_right_number_missing_arguments (List.length occlist)
 
 (****************************************************************)
 (* Clausal environment for an application *)
@@ -556,7 +557,7 @@ let make_clenv_binding_gen hyps_only n env sigma (c,t) = function
   | ExplicitBindings lbind ->
       let t = rename_bound_vars_as_displayed sigma Id.Set.empty [] t in
       let clause = mk_clenv_from_env env sigma n
-	(c, t)
+        (c, t)
       in clenv_match_args lbind clause
   | NoBindings ->
       mk_clenv_from_env env sigma n (c,t)
@@ -566,7 +567,7 @@ let make_clenv_binding_env_apply env sigma n =
 
 let make_clenv_binding_env env sigma =
   make_clenv_binding_gen false None env sigma
-	
+
 let make_clenv_binding_apply env sigma n = make_clenv_binding_gen true n env sigma
 let make_clenv_binding env sigma = make_clenv_binding_gen false None env sigma
 
@@ -577,7 +578,7 @@ let pr_clenv clenv =
   h 0
     (str"TEMPL: " ++ Termops.Internal.print_constr_env clenv.env clenv.evd clenv.templval.rebus ++
      str" : " ++ Termops.Internal.print_constr_env clenv.env clenv.evd clenv.templtyp.rebus ++ fnl () ++
-     pr_evar_map (Some 2) clenv.evd)
+     pr_evar_map (Some 2) clenv.env clenv.evd)
 
 (****************************************************************)
 (** Evar version of mk_clenv *)
@@ -601,29 +602,36 @@ let make_evar_clause env sigma ?len t =
   | None -> -1
   | Some n -> assert (0 <= n); n
   in
-  (** FIXME: do the renaming online *)
+  (* FIXME: do the renaming online *)
   let t = rename_bound_vars_as_displayed sigma Id.Set.empty [] t in
-  let rec clrec (sigma, holes) n t =
+  let rec clrec (sigma, holes) inst n t =
     if n = 0 then (sigma, holes, t)
     else match EConstr.kind sigma t with
-    | Cast (t, _, _) -> clrec (sigma, holes) n t
+    | Cast (t, _, _) -> clrec (sigma, holes) inst n t
     | Prod (na, t1, t2) ->
-      let store = Typeclasses.set_resolvable Evd.Store.empty false in
-      let (sigma, ev) = new_evar ~store env sigma t1 in
+      (* Share the evar instances as we are living in the same context *)
+      let inst, ctx, args, subst = match inst with
+      | None ->
+        (* Dummy type *)
+        let ctx, _, args, subst = push_rel_context_to_named_context env sigma mkProp in
+        Some (ctx, args, subst), ctx, args, subst
+      | Some (ctx, args, subst) -> inst, ctx, args, subst
+      in
+      let (sigma, ev) = new_evar_instance ~typeclass_candidate:false ctx sigma (csubst_subst subst t1) args in
       let dep = not (noccurn sigma 1 t2) in
       let hole = {
         hole_evar = ev;
         hole_type = t1;
         hole_deps = dep;
         (* We fix it later *)
-        hole_name = na;
+        hole_name = na.binder_name;
       } in
       let t2 = if dep then subst1 ev t2 else t2 in
-      clrec (sigma, hole :: holes) (pred n) t2
-    | LetIn (na, b, _, t) -> clrec (sigma, holes) n (subst1 b t)
+      clrec (sigma, hole :: holes) inst (pred n) t2
+    | LetIn (na, b, _, t) -> clrec (sigma, holes) inst n (subst1 b t)
     | _ -> (sigma, holes, t)
   in
-  let (sigma, holes, t) = clrec (sigma, []) bound t in
+  let (sigma, holes, t) = clrec (sigma, []) None bound t in
   let holes = List.rev holes in
   let clause = { cl_concl = t; cl_holes = holes } in
   (sigma, clause)
@@ -651,7 +659,7 @@ let evar_with_name holes id =
   | [] -> explain_no_such_bound_variable holes id
   | [h] -> h.hole_evar
   | _ ->
-    user_err 
+    user_err
       (str "Binder name \"" ++ Id.print id ++
         str "\" occurs more than once in clause.")
 
@@ -670,7 +678,7 @@ let define_with_type sigma env ev c =
   let t = Retyping.get_type_of env sigma ev in
   let ty = Retyping.get_type_of env sigma c in
   let j = Environ.make_judge c ty in
-  let (sigma, j) = Coercion.inh_conv_coerce_to true env sigma j t in
+  let (sigma, j, _trace) = Coercion.inh_conv_coerce_to ~program_mode:false true env sigma j t in
   let (ev, _) = destEvar sigma ev in
   let sigma = Evd.define ev j.Environ.uj_val sigma in
   sigma
@@ -681,7 +689,7 @@ let solve_evar_clause env sigma hyp_only clause = function
   let open EConstr in
   let fold holes h =
     if h.hole_deps then
-      (** Some subsequent term uses the hole *)
+      (* Some subsequent term uses the hole *)
       let (ev, _) = destEvar sigma h.hole_evar in
       let is_dep hole = occur_evar sigma ev hole.hole_type in
       let in_hyp = List.exists is_dep holes in
@@ -690,7 +698,7 @@ let solve_evar_clause env sigma hyp_only clause = function
       let h = { h with hole_deps = dep } in
       h :: holes
     else
-      (** The hole does not occur anywhere *)
+      (* The hole does not occur anywhere *)
       h :: holes
   in
   let holes = List.fold_left fold [] (List.rev clause.cl_holes) in
