@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -20,7 +20,6 @@ open Environ
 open Reductionops
 open Inductiveops
 open Typing
-open Proof_type
 open Type_errors
 open Retyping
 
@@ -62,13 +61,14 @@ let is_unification_error = function
 
 let catchable_exception = function
   | CErrors.UserError _ | TypeError _
-  | Notation.NumeralNotationError _
+  | Proof.OpenProof _
+  (* abstract will call close_proof inside a tactic *)
   | RefinerError _ | Indrec.RecursionSchemeError _
   | Nametab.GlobalizationError _
   (* reduction errors *)
   | Tacred.ReductionTacticError _ -> true
   (* unification and typing errors *)
-  | PretypeError(_,_, e) -> is_unification_error e || is_typing_error e 
+  | PretypeError(_,_, e) -> is_unification_error e || is_typing_error e
   | _ -> false
 
 let error_no_such_hypothesis env sigma id = raise (RefinerError (env, sigma, NoSuchHyp id))
@@ -78,14 +78,6 @@ let error_no_such_hypothesis env sigma id = raise (RefinerError (env, sigma, NoS
 let check = ref false
 let with_check = Flags.with_option check
 
-(* [apply_to_hyp sign id f] splits [sign] into [tail::[id,_,_]::head] and
-   returns [tail::(f head (id,_,_) (rev tail))] *)
-let apply_to_hyp env sigma check sign id f =
-  try apply_to_hyp sign id f
-  with Hyp_not_found ->
-    if check then error_no_such_hypothesis env sigma id
-    else sign
-
 let check_typability env sigma c =
   if !check then let _ = unsafe_type_of env sigma (EConstr.of_constr c) in ()
 
@@ -161,12 +153,14 @@ let reorder_context env sigma sign ord =
   step ord ords sign mt_q []
 
 let reorder_val_context env sigma sign ord =
+match ord with
+| [] | [_] ->
+  (* Single variable-free definitions need not be reordered *)
+  sign
+| _ :: _ :: _ ->
   let open EConstr in
   val_of_named_context (reorder_context env sigma (named_context_of_val sign) ord)
 
-
-
-
 let check_decl_position env sigma sign d =
   let open EConstr in
   let x = NamedDecl.get_id d in
@@ -213,15 +207,15 @@ let split_sign env sigma hfrom hto l =
     | [] -> error_no_such_hypothesis env sigma hfrom
     | d :: right ->
         let hyp = NamedDecl.get_id d in
-      	if Id.equal hyp hfrom then
-	  (left,right,d, toleft || move_location_eq hto MoveLast)
-      	else
+        if Id.equal hyp hfrom then
+          (left,right,d, toleft || move_location_eq hto MoveLast)
+        else
           let is_toleft = match hto with
           | MoveAfter h' | MoveBefore h' -> Id.equal hyp h'
           | _ -> false
           in
-	  splitrec (d::left) (toleft || is_toleft)
-	    right
+          splitrec (d::left) (toleft || is_toleft)
+            right
   in
     splitrec [] false l
 
@@ -230,8 +224,7 @@ let hyp_of_move_location = function
   | MoveBefore id -> id
   | _ -> assert false
 
-let move_hyp sigma toleft (left,declfrom,right) hto =
-  let env = Global.env() in
+let move_hyp env sigma toleft (left,declfrom,right) hto =
   let test_dep d d2 =
     if toleft
     then occur_var_in_decl env sigma (NamedDecl.get_id d2) d
@@ -239,29 +232,29 @@ let move_hyp sigma toleft (left,declfrom,right) hto =
   in
   let rec moverec first middle = function
     | [] ->
-	if match hto with MoveFirst | MoveLast -> false | _ -> true then
+        if match hto with MoveFirst | MoveLast -> false | _ -> true then
           error_no_such_hypothesis env sigma (hyp_of_move_location hto);
-	List.rev first @ List.rev middle
+        List.rev first @ List.rev middle
     | d :: _ as right when move_location_eq hto (MoveBefore (NamedDecl.get_id d)) ->
-	List.rev first @ List.rev middle @ right
+        List.rev first @ List.rev middle @ right
     | d :: right ->
         let hyp = NamedDecl.get_id d in
-	let (first',middle') =
-      	  if List.exists (test_dep d) middle then
-	    if not (move_location_eq hto (MoveAfter hyp)) then
-	      (first, d::middle)
+        let (first',middle') =
+          if List.exists (test_dep d) middle then
+            if not (move_location_eq hto (MoveAfter hyp)) then
+              (first, d::middle)
             else
-	      user_err ~hdr:"move_hyp" (str "Cannot move " ++ Id.print (NamedDecl.get_id declfrom) ++
+              user_err ~hdr:"move_hyp" (str "Cannot move " ++ Id.print (NamedDecl.get_id declfrom) ++
                 pr_move_location Id.print hto ++
-	        str (if toleft then ": it occurs in the type of " else ": it depends on ")
-	        ++ Id.print hyp ++ str ".")
+                str (if toleft then ": it occurs in the type of " else ": it depends on ")
+                ++ Id.print hyp ++ str ".")
           else
-	    (d::first, middle)
-	in
-      	if move_location_eq hto (MoveAfter hyp) then
-	  List.rev first' @ List.rev middle' @ right
-      	else
-	  moverec first' middle' right
+            (d::first, middle)
+        in
+        if move_location_eq hto (MoveAfter hyp) then
+          List.rev first' @ List.rev middle' @ right
+        else
+          moverec first' middle' right
   in
   let open EConstr in
   if toleft then
@@ -272,7 +265,7 @@ let move_hyp sigma toleft (left,declfrom,right) hto =
   else
     let right =
       List.fold_right push_named_context_val
-	(moverec [] [declfrom] right) empty_named_context_val in
+        (moverec [] [declfrom] right) empty_named_context_val in
     List.fold_left (fun sign d -> push_named_context_val d sign)
       right left
 
@@ -280,11 +273,11 @@ let move_hyp_in_named_context env sigma hfrom hto sign =
   let open EConstr in
   let (left,right,declfrom,toleft) =
     split_sign env sigma hfrom hto (named_context_of_val sign) in
-  move_hyp sigma toleft (left,declfrom,right) hto
+  move_hyp env sigma toleft (left,declfrom,right) hto
 
-let insert_decl_in_named_context sigma decl hto sign =
+let insert_decl_in_named_context env sigma decl hto sign =
   let open EConstr in
-  move_hyp sigma false ([],decl,named_context_of_val sign) hto
+  move_hyp env sigma false ([],decl,named_context_of_val sign) hto
 
 (**********************************************************************)
 
@@ -335,7 +328,7 @@ exception Stop of EConstr.t list
 let meta_free_prefix sigma a =
   try
     let a = Array.map EConstr.of_constr a in
-    let _ = Array.fold_left (fun acc a -> 
+    let _ = Array.fold_left (fun acc a ->
       if occur_meta sigma a then raise (Stop acc)
       else a :: acc) [] a
     in a
@@ -351,7 +344,7 @@ let rec mk_refgoals sigma goal goalacc conclty trm =
   let env = Goal.V82.env sigma goal in
   let hyps = Goal.V82.hyps sigma goal in
   let mk_goal hyps concl =
-    Goal.V82.mk_goal sigma hyps concl (Goal.V82.extra sigma goal)
+    Goal.V82.mk_goal sigma hyps concl
   in
     if (not !check) && not (occur_meta sigma (EConstr.of_constr trm)) then
       let t'ty = Retyping.get_type_of env sigma (EConstr.of_constr trm) in
@@ -362,69 +355,69 @@ let rec mk_refgoals sigma goal goalacc conclty trm =
       match kind trm with
       | Meta _ ->
         let conclty = nf_betaiota env sigma (EConstr.of_constr conclty) in
-	  if !check && occur_meta sigma conclty then
+          if !check && occur_meta sigma conclty then
             raise (RefinerError (env, sigma, MetaInType conclty));
-	  let (gl,ev,sigma) = mk_goal hyps conclty in
-	  let ev = EConstr.Unsafe.to_constr ev in
-	  let conclty = EConstr.Unsafe.to_constr conclty in
-	  gl::goalacc, conclty, sigma, ev
+          let (gl,ev,sigma) = mk_goal hyps conclty in
+          let ev = EConstr.Unsafe.to_constr ev in
+          let conclty = EConstr.Unsafe.to_constr conclty in
+          gl::goalacc, conclty, sigma, ev
 
       | Cast (t,k, ty) ->
-	check_typability env sigma ty;
+        check_typability env sigma ty;
         let sigma = check_conv_leq_goal env sigma trm ty conclty in
-	let res = mk_refgoals sigma goal goalacc ty t in
-	(** we keep the casts (in particular VMcast and NATIVEcast) except
-	    when they are annotating metas *)
-	if isMeta t then begin
-	  assert (k != VMcast && k != NATIVEcast);
-	  res
-	end else
-	  let (gls,cty,sigma,ans) = res in
+        let res = mk_refgoals sigma goal goalacc ty t in
+        (* we keep the casts (in particular VMcast and NATIVEcast) except
+           when they are annotating metas *)
+        if isMeta t then begin
+          assert (k != VMcast && k != NATIVEcast);
+          res
+        end else
+          let (gls,cty,sigma,ans) = res in
           let ans = if ans == t then trm else mkCast(ans,k,ty) in
-	  (gls,cty,sigma,ans)
+          (gls,cty,sigma,ans)
 
       | App (f,l) ->
-	let (acc',hdty,sigma,applicand) =
-	  if is_template_polymorphic env sigma (EConstr.of_constr f) then
-	    let ty = 
-	      (* Template polymorphism of definitions and inductive types *)
-	      let firstmeta = Array.findi (fun i x -> occur_meta sigma (EConstr.of_constr x)) l in
-	      let args, _ = Option.cata (fun i -> CArray.chop i l) (l, [||]) firstmeta in
-	        type_of_global_reference_knowing_parameters env sigma (EConstr.of_constr f) (Array.map EConstr.of_constr args)
-	    in
-	    let ty = EConstr.Unsafe.to_constr ty in
-	      goalacc, ty, sigma, f
-	  else
-	    mk_hdgoals sigma goal goalacc f
-	in
-	let ((acc'',conclty',sigma), args) = mk_arggoals sigma goal acc' hdty l in
+        let (acc',hdty,sigma,applicand) =
+          if Termops.is_template_polymorphic_ind env sigma (EConstr.of_constr f) then
+            let ty =
+              (* Template polymorphism of definitions and inductive types *)
+              let firstmeta = Array.findi (fun i x -> occur_meta sigma (EConstr.of_constr x)) l in
+              let args, _ = Option.cata (fun i -> CArray.chop i l) (l, [||]) firstmeta in
+                type_of_global_reference_knowing_parameters env sigma (EConstr.of_constr f) (Array.map EConstr.of_constr args)
+            in
+            let ty = EConstr.Unsafe.to_constr ty in
+              goalacc, ty, sigma, f
+          else
+            mk_hdgoals sigma goal goalacc f
+        in
+        let ((acc'',conclty',sigma), args) = mk_arggoals sigma goal acc' hdty l in
         let sigma = check_conv_leq_goal env sigma trm conclty' conclty in
         let ans = if applicand == f && args == l then trm else mkApp (applicand, args) in
         (acc'',conclty',sigma, ans)
 
       | Proj (p,c) ->
-	let (acc',cty,sigma,c') = mk_hdgoals sigma goal goalacc c in
-	let c = mkProj (p, c') in
-	let ty = get_type_of env sigma (EConstr.of_constr c) in
-	let ty = EConstr.Unsafe.to_constr ty in
-	  (acc',ty,sigma,c)
+        let (acc',cty,sigma,c') = mk_hdgoals sigma goal goalacc c in
+        let c = mkProj (p, c') in
+        let ty = get_type_of env sigma (EConstr.of_constr c) in
+        let ty = EConstr.Unsafe.to_constr ty in
+          (acc',ty,sigma,c)
 
       | Case (ci,p,c,lf) ->
-	let (acc',lbrty,conclty',sigma,p',c') = mk_casegoals sigma goal goalacc p c in
-	let sigma = check_conv_leq_goal env sigma trm conclty' conclty in
+        let (acc',lbrty,conclty',sigma,p',c') = mk_casegoals sigma goal goalacc p c in
+        let sigma = check_conv_leq_goal env sigma trm conclty' conclty in
         let (acc'',sigma,rbranches) = treat_case sigma goal ci lbrty lf acc' in
         let lf' = Array.rev_of_list rbranches in
         let ans =
           if p' == p && c' == c && Array.equal (==) lf' lf then trm
           else mkCase (ci,p',c',lf')
         in
-	(acc'',conclty',sigma, ans)
+        (acc'',conclty',sigma, ans)
 
       | _ ->
-	if occur_meta sigma (EConstr.of_constr trm) then
-	  anomaly (Pp.str "refiner called with a meta in non app/case subterm.");
-	let (sigma, t'ty) = goal_type_of env sigma trm in
-	let sigma = check_conv_leq_goal env sigma trm t'ty conclty in
+        if occur_meta sigma (EConstr.of_constr trm) then
+          anomaly (Pp.str "refiner called with a meta in non app/case subterm.");
+        let (sigma, t'ty) = goal_type_of env sigma trm in
+        let sigma = check_conv_leq_goal env sigma trm t'ty conclty in
           (goalacc,t'ty,sigma, trm)
 
 (* Same as mkREFGOALS but without knowing the type of the term. Therefore,
@@ -433,53 +426,53 @@ let rec mk_refgoals sigma goal goalacc conclty trm =
 and mk_hdgoals sigma goal goalacc trm =
   let env = Goal.V82.env sigma goal in
   let hyps = Goal.V82.hyps sigma goal in
-  let mk_goal hyps concl = 
-    Goal.V82.mk_goal sigma hyps concl (Goal.V82.extra sigma goal) in
+  let mk_goal hyps concl =
+    Goal.V82.mk_goal sigma hyps concl in
   match kind trm with
     | Cast (c,_, ty) when isMeta c ->
-	check_typability env sigma ty;
+        check_typability env sigma ty;
         let (gl,ev,sigma) = mk_goal hyps (nf_betaiota env sigma (EConstr.of_constr ty)) in
-	let ev = EConstr.Unsafe.to_constr ev in
-	gl::goalacc,ty,sigma,ev
+        let ev = EConstr.Unsafe.to_constr ev in
+        gl::goalacc,ty,sigma,ev
 
     | Cast (t,_, ty) ->
-	check_typability env sigma ty;
-	mk_refgoals sigma goal goalacc ty t
+        check_typability env sigma ty;
+        mk_refgoals sigma goal goalacc ty t
 
     | App (f,l) ->
-	let (acc',hdty,sigma,applicand) =
-	  if is_template_polymorphic env sigma (EConstr.of_constr f)
-	  then
-	    let l' = meta_free_prefix sigma l in
-	   (goalacc,EConstr.Unsafe.to_constr (type_of_global_reference_knowing_parameters env sigma (EConstr.of_constr f) l'),sigma,f)
-	  else mk_hdgoals sigma goal goalacc f
-	in
-	let ((acc'',conclty',sigma), args) = mk_arggoals sigma goal acc' hdty l in
+        let (acc',hdty,sigma,applicand) =
+          if Termops.is_template_polymorphic_ind env sigma (EConstr.of_constr f)
+          then
+            let l' = meta_free_prefix sigma l in
+           (goalacc,EConstr.Unsafe.to_constr (type_of_global_reference_knowing_parameters env sigma (EConstr.of_constr f) l'),sigma,f)
+          else mk_hdgoals sigma goal goalacc f
+        in
+        let ((acc'',conclty',sigma), args) = mk_arggoals sigma goal acc' hdty l in
         let ans = if applicand == f && args == l then trm else mkApp (applicand, args) in
-	(acc'',conclty',sigma, ans)
+        (acc'',conclty',sigma, ans)
 
     | Case (ci,p,c,lf) ->
-	let (acc',lbrty,conclty',sigma,p',c') = mk_casegoals sigma goal goalacc p c in
+        let (acc',lbrty,conclty',sigma,p',c') = mk_casegoals sigma goal goalacc p c in
         let (acc'',sigma,rbranches) = treat_case sigma goal ci lbrty lf acc' in
-	let lf' = Array.rev_of_list rbranches in
-	let ans =
+        let lf' = Array.rev_of_list rbranches in
+        let ans =
           if p' == p && c' == c && Array.equal (==) lf' lf then trm
           else mkCase (ci,p',c',lf')
-	in
-	(acc'',conclty',sigma, ans)
+        in
+        (acc'',conclty',sigma, ans)
 
     | Proj (p,c) ->
          let (acc',cty,sigma,c') = mk_hdgoals sigma goal goalacc c in
-	 let c = mkProj (p, c') in
+         let c = mkProj (p, c') in
          let ty = get_type_of env sigma (EConstr.of_constr c) in
          let ty = EConstr.Unsafe.to_constr ty in
-	   (acc',ty,sigma,c)
+           (acc',ty,sigma,c)
 
     | _ ->
-	if !check && occur_meta sigma (EConstr.of_constr trm) then
-	  anomaly (Pp.str "refine called with a dependent meta.");
+        if !check && occur_meta sigma (EConstr.of_constr trm) then
+          anomaly (Pp.str "refine called with a dependent meta.");
         let (sigma, ty) = goal_type_of env sigma trm in
-	goalacc, ty, sigma, trm
+        goalacc, ty, sigma, trm
 
 and mk_arggoals sigma goal goalacc funty allargs =
   let foldmap (goalacc, funty, sigma) harg =
@@ -557,25 +550,25 @@ and treat_case sigma goal ci lbrty lf acc' =
           (lacc,sigma,fi::bacc))
     (acc',sigma,[]) lbrty lf ci.ci_pp_info.cstr_tags
 
-let convert_hyp check sign sigma d =
+let convert_hyp ~check ~reorder env sigma d =
   let id = NamedDecl.get_id d in
   let b = NamedDecl.get_value d in
-  let env = Global.env () in
-  let reorder = ref [] in
-  let sign' =
-    apply_to_hyp env sigma check sign id
-      (fun _ d' _ ->
-        let c = Option.map EConstr.of_constr (NamedDecl.get_value d') in
-        let env = Global.env_of_context sign in
-        if check && not (is_conv env sigma (NamedDecl.get_type d) (EConstr.of_constr (NamedDecl.get_type d'))) then
-	  user_err ~hdr:"Logic.convert_hyp"
-            (str "Incorrect change of the type of " ++ Id.print id ++ str ".");
-        if check && not (Option.equal (is_conv env sigma) b c) then
-	  user_err ~hdr:"Logic.convert_hyp"
-            (str "Incorrect change of the body of "++ Id.print id ++ str ".");
-       if check then reorder := check_decl_position env sigma sign d;
-       map_named_decl EConstr.Unsafe.to_constr d) in
-  reorder_val_context env sigma sign' !reorder
+  let sign = Environ.named_context_val env in
+  match lookup_named_ctxt id sign with
+  | exception Not_found ->
+    if check then error_no_such_hypothesis env sigma id
+    else sign
+  | d' ->
+    let c = Option.map EConstr.of_constr (NamedDecl.get_value d') in
+    if check && not (is_conv env sigma (NamedDecl.get_type d) (EConstr.of_constr (NamedDecl.get_type d'))) then
+      user_err ~hdr:"Logic.convert_hyp"
+        (str "Incorrect change of the type of " ++ Id.print id ++ str ".");
+    if check && not (Option.equal (is_conv env sigma) b c) then
+      user_err ~hdr:"Logic.convert_hyp"
+        (str "Incorrect change of the body of "++ Id.print id ++ str ".");
+    let sign' = apply_to_hyp sign id (fun _ _ _ -> EConstr.Unsafe.to_named_decl d) in
+    if reorder then reorder_val_context env sigma sign' (check_decl_position env sigma sign d)
+    else sign'
 
 (************************************************************************)
 (************************************************************************)
@@ -584,12 +577,15 @@ let convert_hyp check sign sigma d =
 let prim_refiner r sigma goal =
   let env = Goal.V82.env sigma goal in
   let cl = Goal.V82.concl sigma goal in
-  match r with
-    (* Logical rules *)
-    | Refine c ->
-        let cl = EConstr.Unsafe.to_constr cl in
-        check_meta_variables env sigma c;
-	let (sgl,cl',sigma,oterm) = mk_refgoals sigma goal [] cl c in
-	let sgl = List.rev sgl in
-	let sigma = Goal.V82.partial_solution sigma goal (EConstr.of_constr oterm) in
-	  (sgl, sigma)
+  let cl = EConstr.Unsafe.to_constr cl in
+  check_meta_variables env sigma r;
+  let (sgl,cl',sigma,oterm) = mk_refgoals sigma goal [] cl r in
+  let sgl = List.rev sgl in
+  let sigma = Goal.V82.partial_solution env sigma goal (EConstr.of_constr oterm) in
+  (sgl, sigma)
+
+let prim_refiner ~check r sigma goal =
+  if check then
+    with_check (prim_refiner r sigma) goal
+  else
+    prim_refiner r sigma goal
