@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -13,27 +13,20 @@
 open Names
 open Constr
 open Evd
-open Proof_type
 
-(** This suppresses check done in [prim_refiner] for the tactic given in
-   argument; works by side-effect *)
-
-val with_check    : tactic -> tactic
-
-(** [without_check] respectively means:\\
-  [Intro]: no check that the name does not exist\\
-  [Intro_after]: no check that the name does not exist and that variables in
+(** [check] respectively means:\\
+  [Intro]: check that the name does not exist\\
+  [Intro_after]: check that the name does not exist and that variables in
      its type does not escape their scope\\
-  [Intro_replacing]: no check that the name does not exist and that
+  [Intro_replacing]: check that the name does not exist and that
      variables in its type does not escape their scope\\
   [Convert_hyp]:
-  no check that the name exist and that its type is convertible\\
+  check that the name exist and that its type is convertible\\
 *)
 
 (** The primitive refiner. *)
 
-val prim_refiner : prim_rule -> evar_map -> goal -> goal list * evar_map
-
+val prim_refiner : check:bool -> constr -> evar_map -> Goal.goal -> Goal.goal list * evar_map
 
 (** {6 Refiner errors. } *)
 
@@ -69,12 +62,12 @@ type 'id move_location =
 val pr_move_location :
   ('a -> Pp.t) -> 'a move_location -> Pp.t
 
-val convert_hyp : bool -> Environ.named_context_val -> evar_map ->
+val convert_hyp : check:bool -> reorder:bool -> Environ.env -> evar_map ->
   EConstr.named_declaration -> Environ.named_context_val
 
 val move_hyp_in_named_context : Environ.env -> Evd.evar_map -> Id.t -> Id.t move_location ->
   Environ.named_context_val -> Environ.named_context_val
 
-val insert_decl_in_named_context : Evd.evar_map ->
+val insert_decl_in_named_context : Environ.env -> Evd.evar_map ->
   EConstr.named_declaration -> Id.t move_location ->
   Environ.named_context_val -> Environ.named_context_val
