@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -15,11 +15,8 @@ open Environ
 open Reductionops
 open Evd
 open Typing
-open Redexpr
 open Tacred
-open Proof_type
 open Logic
-open Refiner
 open Context.Named.Declaration
 
 module NamedDecl = Context.Named.Declaration
@@ -30,14 +27,7 @@ let re_sig it  gc = { it = it; sigma = gc; }
 (* Operations for handling terms under a local typing context *)
 (**************************************************************)
 
-type 'a sigma   = 'a Evd.sigma;;
-type tactic     = Proof_type.tactic;;
-
-[@@@ocaml.warning "-3"]
-let unpackage = Refiner.unpackage
-let repackage = Refiner.repackage
-let apply_sig_tac = Refiner.apply_sig_tac
-[@@@ocaml.warning "+3"]
+type tactic = Proofview.V82.tac
 
 let sig_it   = Refiner.sig_it
 let project  = Refiner.project
@@ -75,16 +65,8 @@ let pf_ids_set_of_hyps gls =
 let pf_get_new_id id gls =
   next_ident_away id (pf_ids_set_of_hyps gls)
 
-let pf_global gls id = EConstr.of_constr (UnivGen.constr_of_global (Constrintern.construct_reference (pf_hyps gls) id))
-
-let pf_reduction_of_red_expr gls re c =
-  let (redfun, _) = reduction_of_red_expr (pf_env gls) re in
-  let sigma = project gls in
-  redfun (pf_env gls) sigma c
-
 let pf_apply f gls = f (pf_env gls) (project gls)
-let pf_eapply f gls x = 
-  on_sig gls (fun evm -> f (pf_env gls) evm x)
+let pf_eapply f gls x = on_sig gls (fun evm -> f (pf_env gls) evm x)
 let pf_reduce = pf_apply
 let pf_e_reduce = pf_apply
 
@@ -99,28 +81,12 @@ let pf_type_of                   = pf_reduce type_of
 let pf_get_type_of               = pf_reduce Retyping.get_type_of
 
 let pf_conv_x gl                = pf_reduce test_conversion gl Reduction.CONV
-let pf_conv_x_leq gl            = pf_reduce test_conversion gl Reduction.CUMUL
 let pf_const_value              = pf_reduce (fun env _ c -> EConstr.of_constr (constant_value_in env c))
 
 let pf_reduce_to_quantified_ind = pf_reduce reduce_to_quantified_ind
 let pf_reduce_to_atomic_ind     = pf_reduce reduce_to_atomic_ind
-
 let pf_hnf_type_of gls          = pf_get_type_of gls %> pf_whd_all gls
 
-(********************************************)
-(* Definition of the most primitive tactics *)
-(********************************************)
-
-let refiner = refiner
-
-let refine_no_check c gl =
-  let c = EConstr.Unsafe.to_constr c in
-  refiner (Refine c) gl
-
-(* Versions with consistency checks *)
-
-let refine c           = with_check (refine_no_check c)
-
 (* Pretty-printers *)
 
 open Pp
@@ -134,11 +100,7 @@ let db_pr_goal sigma g =
                    str" "  ++ pc) ++ fnl ()
 
 let pr_gls gls =
-  hov 0 (pr_evar_map (Some 2) (sig_sig gls) ++ fnl () ++ db_pr_goal (project gls) (sig_it gls))
-
-let pr_glls glls =
-  hov 0 (pr_evar_map (Some 2) (sig_sig glls) ++ fnl () ++
-         prlist_with_sep fnl (db_pr_goal (project glls)) (sig_it glls))
+  hov 0 (pr_evar_map (Some 2) (pf_env gls) (sig_sig gls) ++ fnl () ++ db_pr_goal (project gls) (sig_it gls))
 
 (* Variants of [Tacmach] functions built with the new proof engine *)
 module New = struct
@@ -152,11 +114,6 @@ module New = struct
   let of_old f gl =
     f { Evd.it = Proofview.Goal.goal gl ; sigma = project gl; }
 
-  let pf_global id gl =
-    (** We only check for the existence of an [id] in [hyps] *)
-    let hyps = Proofview.Goal.hyps gl in
-    Constrintern.construct_reference hyps id
-
   let pf_env = Proofview.Goal.env
   let pf_concl = Proofview.Goal.concl
 
@@ -169,12 +126,12 @@ module New = struct
   let pf_conv_x gl t1 t2 = pf_apply is_conv gl t1 t2
 
   let pf_ids_of_hyps gl =
-    (** We only get the identifiers in [hyps] *)
+    (* We only get the identifiers in [hyps] *)
     let hyps = Proofview.Goal.hyps gl in
     ids_of_named_context hyps
 
   let pf_ids_set_of_hyps gl =
-    (** We only get the identifiers in [hyps] *)
+    (* We only get the identifiers in [hyps] *)
     let env = Proofview.Goal.env gl in
     Environ.ids_of_named_context_val (Environ.named_context_val env)
 
@@ -198,7 +155,7 @@ module New = struct
     let env = Proofview.Goal.env gl in
     let sign = Environ.named_context env in
     List.map (function LocalAssum (id,x)
-                     | LocalDef (id,_,x) -> id, EConstr.of_constr x)
+                     | LocalDef (id,_,x) -> id.Context.binder_name, EConstr.of_constr x)
              sign
 
   let pf_last_hyp gl =
@@ -206,7 +163,7 @@ module New = struct
     List.hd hyps
 
   let pf_nf_concl (gl : Proofview.Goal.t) =
-    (** We normalize the conclusion just after *)
+    (* We normalize the conclusion just after *)
     let concl = Proofview.Goal.concl gl in
     let sigma = project gl in
     nf_evar sigma concl
@@ -222,9 +179,7 @@ module New = struct
   let pf_hnf_type_of gl t =
     pf_whd_all gl (pf_get_type_of gl t)
 
-  let pf_whd_all gl t = pf_apply whd_all gl t
   let pf_compute gl t = pf_apply compute gl t
 
   let pf_nf_evar gl t = nf_evar (project gl) t
-
 end
