@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -60,7 +60,7 @@ module Make(T : Task) () = struct
   type request = Request of T.request
 
   type more_data =
-    | MoreDataUnivLevel of UnivGen.universe_id list
+    | MoreDataUnivLevel of UnivGen.univ_unique_id list
 
   let slave_respond (Request r) =
     let res = T.perform r in
@@ -115,26 +115,48 @@ module Make(T : Task) () = struct
   type process = Worker.process
   type extra = (T.task * cancel_switch) TQueue.t
 
-  let spawn id =
+  let spawn id priority =
     let name = Printf.sprintf "%s:%d" !T.name id in
     let proc, ic, oc =
+      (* Filter arguments for slaves. *)
       let rec set_slave_opt = function
         | [] -> !async_proofs_flags_for_workers @
                 ["-worker-id"; name;
                  "-async-proofs-worker-priority";
-                   CoqworkmgrApi.(string_of_priority !async_proofs_worker_priority)]
-        | ("-emacs"|"-emacs-U"|"-batch")::tl -> set_slave_opt tl
-        | ("-async-proofs" |"-vio2vo"
-          |"-load-vernac-source" |"-l" |"-load-vernac-source-verbose" |"-lv"
-          |"-compile" |"-compile-verbose"
-          |"-async-proofs-worker-priority" |"-worker-id") :: _ :: tl ->
+                 CoqworkmgrApi.(string_of_priority priority)]
+        (* Options to discard: 0 arguments *)
+        | "-emacs"::tl ->
           set_slave_opt tl
-        | x::tl -> x :: set_slave_opt tl in
+        (* Options to discard: 1 argument *)
+        | ( "-async-proofs" | "-vio2vo" | "-o"
+          | "-load-vernac-source" | "-l" | "-load-vernac-source-verbose" | "-lv"
+          | "-compile" | "-compile-verbose"
+          | "-async-proofs-cache" | "-async-proofs-j" | "-async-proofs-tac-j"
+          | "-async-proofs-private-flags" | "-async-proofs-tactic-error-resilience"
+          | "-async-proofs-command-error-resilience" | "-async-proofs-delegation-threshold"
+          | "-async-proofs-worker-priority" | "-worker-id") :: _ :: tl ->
+          set_slave_opt tl
+        (* We need to pass some options with one argument *)
+        | ( "-I" | "-include" | "-top" | "-topfile" | "-coqlib" | "-exclude-dir" | "-compat"
+          | "-load-ml-object" | "-load-ml-source" | "-require" | "-w" | "-color" | "-init-file"
+          | "-profile-ltac-cutoff" | "-main-channel" | "-control-channel" | "-mangle-names" | "-set" | "-unset"
+          | "-diffs" | "-mangle-name" | "-dump-glob" | "-bytecode-compiler" | "-native-compiler" as x) :: a :: tl ->
+          x :: a :: set_slave_opt tl
+        (* We need to pass some options with two arguments *)
+        | ( "-R" | "-Q" as x) :: a1 :: a2 :: tl ->
+          x :: a1 :: a2 :: set_slave_opt tl
+        (* Finally we pass all options starting in '-'; check this is safe w.r.t the weird vio* option set *)
+        | x :: tl when x.[0] = '-' ->
+          x :: set_slave_opt tl
+        (* We assume this is a file, filter out *)
+        | _ :: tl ->
+          set_slave_opt tl
+      in
       let args =
         Array.of_list (set_slave_opt (List.tl (Array.to_list Sys.argv))) in
       let env = Array.append (T.extra_env ()) (Unix.environment ()) in
-    let worker_name = System.get_toplevel_path ("coq" ^ !T.name) in
-    Worker.spawn ~env worker_name args in
+      let worker_name = System.get_toplevel_path ("coq" ^ !T.name) in
+      Worker.spawn ~env worker_name args in
     name, proc, CThread.prepare_in_channel_for_thread_friendly_io ic, oc
 
   let manager cpanel (id, proc, ic, oc) =
@@ -190,7 +212,7 @@ module Make(T : Task) () = struct
         let () = TQueue.broadcast queue in
         Worker.kill proc
     in
-    let _ = Thread.create kill_if () in
+    let _ = CThread.create kill_if () in
 
     try while true do
       report_status ~id "Idle";
@@ -240,7 +262,7 @@ module Make(T : Task) () = struct
     cleaner : Thread.t option;
   }
 
-  let create size =
+  let create size priority =
     let cleaner queue =
       while true do
         try ignore(TQueue.pop ~picky:(fun (_,cancelled) -> !cancelled) queue)
@@ -248,9 +270,9 @@ module Make(T : Task) () = struct
       done in
     let queue = TQueue.create () in
     {
-      active = Pool.create queue ~size;
+      active = Pool.create queue ~size priority;
       queue;
-      cleaner = if size > 0 then Some (Thread.create cleaner queue) else None;
+      cleaner = if size > 0 then Some (CThread.create cleaner queue) else None;
     }
 
   let destroy { active; queue } =
@@ -307,10 +329,12 @@ module Make(T : Task) () = struct
   let main_loop () =
     (* We pass feedback to master *)
     let slave_feeder oc fb =
-      Marshal.to_channel oc (RespFeedback (debug_with_pid fb)) []; flush oc in
+      Control.protect_sigalrm (fun () ->
+          Marshal.to_channel oc (RespFeedback (debug_with_pid fb)) []; flush oc) ()
+    in
     ignore (Feedback.add_feeder (fun x -> slave_feeder (Option.get !slave_oc) x));
     (* We ask master to allocate universe identifiers *)
-    UnivGen.set_remote_new_univ_id (bufferize (fun () ->
+    UnivGen.set_remote_new_univ_id (bufferize @@ Control.protect_sigalrm (fun () ->
       marshal_response (Option.get !slave_oc) RespGetCounterNewUnivLevel;
       match unmarshal_more_data (Option.get !slave_ic) with
       | MoreDataUnivLevel l -> l));
@@ -325,7 +349,7 @@ module Make(T : Task) () = struct
         let response = slave_respond request in
         report_status "Idle";
         marshal_response (Option.get !slave_oc) response;
-        CEphemeron.clear ()
+        CEphemeron.clean ()
       with
       | MarshalError s ->
         stm_pr_err Pp.(prlist str ["Fatal marshal error: "; s]); flush_all (); exit 2
@@ -345,8 +369,8 @@ module Make(T : Task) () = struct
      (TQueue.wait_until_n_are_waiting_then_snapshot
        (Pool.n_workers active) queue)
 
-  let with_n_workers n f =
-    let q = create n in
+  let with_n_workers n priority f =
+    let q = create n priority in
     try let rc = f q in destroy q; rc
     with e -> let e = CErrors.push e in destroy q; iraise e
 
