@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -15,7 +15,6 @@ open Names
 open Constr
 open Termops
 open EConstr
-open Proof_type
 open Tacticals
 open Tacmach
 open Evd
@@ -29,7 +28,7 @@ open Locusops
 open Hints
 open Proofview.Notations
 
-let eauto_unif_flags = auto_flags_of_state full_transparent_state
+let eauto_unif_flags = auto_flags_of_state TransparentState.full
 
 let e_give_exact ?(flags=eauto_unif_flags) c =
   Proofview.Goal.enter begin fun gl ->
@@ -114,7 +113,7 @@ let priority l = List.map snd (List.filter (fun (pr,_) -> Int.equal pr 0) l)
 
 let unify_e_resolve poly flags (c,clenv) =
   Proofview.Goal.enter begin fun gl ->
-      let clenv', c = connect_hint_clenv poly c clenv gl in
+      let clenv', c = connect_hint_clenv ~poly c clenv gl in
       let clenv' = clenv_unique_resolver ~flags clenv' gl in
       Proofview.tclTHEN
         (Proofview.Unsafe.tclEVARUNIVCONTEXT (Evd.evar_universe_context clenv'.evd))
@@ -132,7 +131,7 @@ let hintmap_of sigma secvars hdc concl =
 
 let e_exact poly flags (c,clenv) =
   Proofview.Goal.enter begin fun gl ->
-    let clenv', c = connect_hint_clenv poly c clenv gl in
+    let clenv', c = connect_hint_clenv ~poly c clenv gl in
     Tacticals.New.tclTHEN
     (Proofview.Unsafe.tclEVARUNIVCONTEXT (Evd.evar_universe_context clenv'.evd))
     (e_give_exact c)
@@ -151,15 +150,15 @@ let rec e_trivial_fail_db db_list local_db =
     (Tacticals.New.tclTHEN Tactics.intro next) ::
     (List.map fst (e_trivial_resolve (Tacmach.New.pf_env gl) (Tacmach.New.project gl) db_list local_db secvars (Tacmach.New.pf_concl gl)))
   in
-  Tacticals.New.tclFIRST (List.map Tacticals.New.tclCOMPLETE tacl)
+  Tacticals.New.tclSOLVE tacl
   end
 
 and e_my_find_search env sigma db_list local_db secvars hdc concl =
   let hint_of_db = hintmap_of sigma secvars hdc concl in
   let hintl =
       List.map_append (fun db ->
-	let flags = auto_flags_of_state (Hint_db.transparent_state db) in
-	  List.map (fun x -> flags, x) (hint_of_db db)) (local_db::db_list)
+        let flags = auto_flags_of_state (Hint_db.transparent_state db) in
+          List.map (fun x -> flags, x) (hint_of_db db)) (local_db::db_list)
   in
   let tac_of_hint =
     fun (st, {pri = b; pat = p; code = t; poly = poly}) ->
@@ -169,7 +168,7 @@ and e_my_find_search env sigma db_list local_db secvars hdc concl =
       in
       (b,
         let tac = function
-        | Res_pf (term,cl) -> unify_resolve poly st (term,cl)
+        | Res_pf (term,cl) -> unify_resolve ~poly st (term,cl)
         | ERes_pf (term,cl) -> unify_e_resolve poly st (term,cl)
         | Give_exact (c,cl) -> e_exact poly st (c,cl)
         | Res_pf_THEN_trivial_fail (term,cl) ->
@@ -203,7 +202,7 @@ let find_first_goal gls =
 type search_state = {
   priority : int;
   depth : int; (*r depth of search before failing *)
-  tacres : goal list sigma;
+  tacres : Goal.goal list sigma;
   last_tactic : Pp.t Lazy.t;
   dblist : hint_db list;
   localdb :  hint_db list;
@@ -231,12 +230,12 @@ module SearchProblem = struct
     let rec aux = function
       | [] -> []
       | (tac, cost, pptac) :: tacl ->
-	  try
-	    let lgls = apply_tac_list (Proofview.V82.of_tactic tac) glls in
+          try
+            let lgls = apply_tac_list (Proofview.V82.of_tactic tac) glls in
 (* 	    let gl = Proof_trees.db_pr_goal (List.hd (sig_it glls)) in *)
 (* 	      msg (hov 1 (pptac ++ str" gives: \n" ++ pr_goals lgls ++ str"\n")); *)
-	      (lgls, cost, pptac) :: aux tacl
-	  with e when CErrors.noncritical e ->
+              (lgls, cost, pptac) :: aux tacl
+          with e when CErrors.noncritical e ->
             let e = CErrors.push e in
             Refiner.catch_failerror e; aux tacl
     in aux l
@@ -266,60 +265,60 @@ module SearchProblem = struct
       let assumption_tacs =
         let tacs = List.map map_assum hyps in
         let l = filter_tactics s.tacres tacs in
-	List.map (fun (res, cost, pp) -> { depth = s.depth; priority = cost; tacres = res;
-				    last_tactic = pp; dblist = s.dblist;
-				    localdb = List.tl s.localdb;
-				    prev = ps; local_lemmas = s.local_lemmas}) l
+        List.map (fun (res, cost, pp) -> { depth = s.depth; priority = cost; tacres = res;
+                                    last_tactic = pp; dblist = s.dblist;
+                                    localdb = List.tl s.localdb;
+                                    prev = ps; local_lemmas = s.local_lemmas}) l
       in
       let intro_tac =
         let l = filter_tactics s.tacres [Tactics.intro, (-1), lazy (str "intro")] in
-	List.map
-	  (fun (lgls, cost, pp) ->
-	     let g' = first_goal lgls in
-	     let hintl =
-	       make_resolve_hyp (pf_env g') (project g') (pf_last_hyp g')
-	     in
+        List.map
+          (fun (lgls, cost, pp) ->
+             let g' = first_goal lgls in
+             let hintl =
+               make_resolve_hyp (pf_env g') (project g') (pf_last_hyp g')
+             in
              let ldb = Hint_db.add_list (pf_env g') (project g')
-		  hintl (List.hd s.localdb) in
-	     { depth = s.depth; priority = cost; tacres = lgls;
-	       last_tactic = pp; dblist = s.dblist;
-	       localdb = ldb :: List.tl s.localdb; prev = ps;
+                  hintl (List.hd s.localdb) in
+             { depth = s.depth; priority = cost; tacres = lgls;
+               last_tactic = pp; dblist = s.dblist;
+               localdb = ldb :: List.tl s.localdb; prev = ps;
                local_lemmas = s.local_lemmas})
-	  l
+          l
       in
       let rec_tacs =
-	let l =
+        let l =
           let concl = Reductionops.nf_evar (project g) (pf_concl g) in
-	  filter_tactics s.tacres
+          filter_tactics s.tacres
                          (e_possible_resolve (pf_env g) (project g) s.dblist (List.hd s.localdb) secvars concl)
-	in
-	List.map
-	  (fun (lgls, cost, pp) ->
-	     let nbgl' = List.length (sig_it lgls) in
-	     if nbgl' < nbgl then
-	       { depth = s.depth; priority = cost; tacres = lgls; last_tactic = pp;
+        in
+        List.map
+          (fun (lgls, cost, pp) ->
+             let nbgl' = List.length (sig_it lgls) in
+             if nbgl' < nbgl then
+               { depth = s.depth; priority = cost; tacres = lgls; last_tactic = pp;
                   prev = ps; dblist = s.dblist; localdb = List.tl s.localdb;
                   local_lemmas = s.local_lemmas }
-	     else
-	       let newlocal = 
-		 let hyps = pf_hyps g in
-		   List.map (fun gl ->
-		     let gls = {Evd.it = gl; sigma = lgls.Evd.sigma } in
-		     let hyps' = pf_hyps gls in
-		       if hyps' == hyps then List.hd s.localdb
-		       else make_local_hint_db (pf_env gls) (project gls) ~ts:full_transparent_state true s.local_lemmas)
-		     (List.firstn ((nbgl'-nbgl) + 1) (sig_it lgls))
-	       in
-		 { depth = pred s.depth; priority = cost; tacres = lgls;
-		   dblist = s.dblist; last_tactic = pp; prev = ps;
-		   localdb = newlocal @ List.tl s.localdb;
+             else
+               let newlocal =
+                 let hyps = pf_hyps g in
+                   List.map (fun gl ->
+                     let gls = {Evd.it = gl; sigma = lgls.Evd.sigma } in
+                     let hyps' = pf_hyps gls in
+                       if hyps' == hyps then List.hd s.localdb
+                       else make_local_hint_db (pf_env gls) (project gls) ~ts:TransparentState.full true s.local_lemmas)
+                     (List.firstn ((nbgl'-nbgl) + 1) (sig_it lgls))
+               in
+                 { depth = pred s.depth; priority = cost; tacres = lgls;
+                   dblist = s.dblist; last_tactic = pp; prev = ps;
+                   localdb = newlocal @ List.tl s.localdb;
                    local_lemmas = s.local_lemmas })
-	  l
+          l
       in
       List.sort compare (assumption_tacs @ intro_tac @ rec_tacs)
 
   let pp s = hov 0 (str " depth=" ++ int s.depth ++ spc () ++
-		      (Lazy.force s.last_tactic))
+                      (Lazy.force s.last_tactic))
 
 end
 
@@ -330,21 +329,21 @@ module Search = Explore.Make(SearchProblem)
 let global_debug_eauto = ref false
 let global_info_eauto = ref false
 
-let _ =
-  Goptions.declare_bool_option
-    { Goptions.optdepr  = false;
-      Goptions.optname  = "Debug Eauto";
-      Goptions.optkey   = ["Debug";"Eauto"];
-      Goptions.optread  = (fun () -> !global_debug_eauto);
-      Goptions.optwrite = (:=) global_debug_eauto }
+let () =
+  Goptions.(declare_bool_option
+    { optdepr  = false;
+      optname  = "Debug Eauto";
+      optkey   = ["Debug";"Eauto"];
+      optread  = (fun () -> !global_debug_eauto);
+      optwrite = (:=) global_debug_eauto })
 
-let _ =
-  Goptions.declare_bool_option
-    { Goptions.optdepr  = false;
-      Goptions.optname  = "Info Eauto";
-      Goptions.optkey   = ["Info";"Eauto"];
-      Goptions.optread  = (fun () -> !global_info_eauto);
-      Goptions.optwrite = (:=) global_info_eauto }
+let () =
+  Goptions.(declare_bool_option
+    { optdepr  = false;
+      optname  = "Info Eauto";
+      optkey   = ["Info";"Eauto"];
+      optread  = (fun () -> !global_info_eauto);
+      optwrite = (:=) global_info_eauto })
 
 let mk_eauto_dbg d =
   if d == Debug || !global_debug_eauto then Debug
@@ -352,25 +351,25 @@ let mk_eauto_dbg d =
   else Off
 
 let pr_info_nop = function
-  | Info -> Feedback.msg_info (str "idtac.")
+  | Info -> Feedback.msg_notice (str "idtac.")
   | _ -> ()
 
 let pr_dbg_header = function
   | Off -> ()
-  | Debug -> Feedback.msg_debug (str "(* debug eauto: *)")
-  | Info  -> Feedback.msg_info (str "(* info eauto: *)")
+  | Debug -> Feedback.msg_notice (str "(* debug eauto: *)")
+  | Info  -> Feedback.msg_notice (str "(* info eauto: *)")
 
 let pr_info dbg s =
   if dbg != Info then ()
   else
     let rec loop s =
       match s.prev with
-	| Unknown | Init -> s.depth
-	| State sp ->
-	  let mindepth = loop sp in
-	  let indent = String.make (mindepth - sp.depth) ' ' in
-	  Feedback.msg_info (str indent ++ Lazy.force s.last_tactic ++ str ".");
-	  mindepth
+        | Unknown | Init -> s.depth
+        | State sp ->
+          let mindepth = loop sp in
+          let indent = String.make (mindepth - sp.depth) ' ' in
+          Feedback.msg_notice (str indent ++ Lazy.force s.last_tactic ++ str ".");
+          mindepth
     in
     ignore (loop s)
 
@@ -388,7 +387,7 @@ let make_initial_state dbg n gl dblist localdb lems =
   }
 
 let e_search_auto debug (in_depth,p) lems db_list gl =
-  let local_db = make_local_hint_db (pf_env gl) (project gl) ~ts:full_transparent_state true lems in
+  let local_db = make_local_hint_db (pf_env gl) (project gl) ~ts:TransparentState.full true lems in
   let d = mk_eauto_dbg debug in
   let tac = match in_depth,d with
     | (true,Debug) -> Search.debug_depth_first
@@ -409,7 +408,7 @@ let e_search_auto debug (in_depth,p) lems db_list gl =
 (* let e_search_auto = CProfile.profile5 e_search_auto_key e_search_auto *)
 
 let eauto_with_bases ?(debug=Off) np lems db_list =
-  tclTRY (e_search_auto debug np lems db_list)
+  Hints.wrap_hint_warning (Proofview.V82.tactic (tclTRY (e_search_auto debug np lems db_list)))
 
 let eauto ?(debug=Off) np lems dbnames =
   let db_list = make_db_list dbnames in
@@ -420,8 +419,8 @@ let full_eauto ?(debug=Off) n lems gl =
   tclTRY (e_search_auto debug n lems db_list) gl
 
 let gen_eauto ?(debug=Off) np lems = function
-  | None -> Proofview.V82.tactic (full_eauto ~debug np lems)
-  | Some l -> Proofview.V82.tactic (eauto ~debug np lems l)
+  | None -> Hints.wrap_hint_warning (Proofview.V82.tactic (full_eauto ~debug np lems))
+  | Some l -> Hints.wrap_hint_warning (Proofview.V82.tactic (eauto ~debug np lems l))
 
 let make_depth = function
   | None -> !default_search_depth
@@ -434,15 +433,15 @@ let make_dimension n = function
 let cons a l = a :: l
 
 let autounfolds db occs cls gl =
-  let unfolds = List.concat (List.map (fun dbname -> 
-    let db = try searchtable_map dbname 
+  let unfolds = List.concat (List.map (fun dbname ->
+    let db = try searchtable_map dbname
       with Not_found -> user_err ~hdr:"autounfold" (str "Unknown database " ++ str dbname)
     in
     let (ids, csts) = Hint_db.unfolds db in
     let hyps = pf_ids_of_hyps gl in
     let ids = Id.Set.filter (fun id -> List.mem id hyps) ids in
       Cset.fold (fun cst -> cons (AllOccurrences, EvalConstRef cst)) csts
-	(Id.Set.fold (fun id -> cons (AllOccurrences, EvalVarRef id)) ids [])) db)
+        (Id.Set.fold (fun id -> cons (AllOccurrences, EvalVarRef id)) ids [])) db)
   in Proofview.V82.of_tactic (unfold_option unfolds cls) gl
 
 let autounfold db cls =
@@ -465,36 +464,36 @@ let autounfold_tac db cls =
   autounfold dbs cls
 
 let unfold_head env sigma (ids, csts) c =
-  let rec aux c = 
+  let rec aux c =
     match EConstr.kind sigma c with
     | Var id when Id.Set.mem id ids ->
-	(match Environ.named_body id env with
-	| Some b -> true, EConstr.of_constr b
-	| None -> false, c)
+        (match Environ.named_body id env with
+        | Some b -> true, EConstr.of_constr b
+        | None -> false, c)
     | Const (cst, u) when Cset.mem cst csts ->
         let u = EInstance.kind sigma u in
-	true, EConstr.of_constr (Environ.constant_value_in env (cst, u))
+        true, EConstr.of_constr (Environ.constant_value_in env (cst, u))
     | App (f, args) ->
-	(match aux f with
-	| true, f' -> true, Reductionops.whd_betaiota sigma (mkApp (f', args))
-	| false, _ -> 
-	    let done_, args' = 
-	      Array.fold_left_i (fun i (done_, acc) arg -> 
-		if done_ then done_, arg :: acc 
-		else match aux arg with
-		| true, arg' -> true, arg' :: acc
-		| false, arg' -> false, arg :: acc)
-		(false, []) args
-	    in 
-	      if done_ then true, mkApp (f, Array.of_list (List.rev args'))
-	      else false, c)
-    | _ -> 
-	let done_ = ref false in
-	let c' = EConstr.map sigma (fun c -> 
-	  if !done_ then c else 
-	    let x, c' = aux c in
-	      done_ := x; c') c
-	in !done_, c'
+        (match aux f with
+        | true, f' -> true, Reductionops.whd_betaiota sigma (mkApp (f', args))
+        | false, _ ->
+            let done_, args' =
+              Array.fold_left_i (fun i (done_, acc) arg ->
+                if done_ then done_, arg :: acc
+                else match aux arg with
+                | true, arg' -> true, arg' :: acc
+                | false, arg' -> false, arg :: acc)
+                (false, []) args
+            in
+              if done_ then true, mkApp (f, Array.of_list (List.rev args'))
+              else false, c)
+    | _ ->
+        let done_ = ref false in
+        let c' = EConstr.map sigma (fun c ->
+          if !done_ then c else
+            let x, c' = aux c in
+              done_ := x; c') c
+        in !done_, c'
   in aux c
 
 let autounfold_one db cl =
@@ -503,19 +502,19 @@ let autounfold_one db cl =
   let sigma = Tacmach.New.project gl in
   let concl = Proofview.Goal.concl gl in
   let st =
-    List.fold_left (fun (i,c) dbname -> 
-      let db = try searchtable_map dbname 
-	with Not_found -> user_err ~hdr:"autounfold" (str "Unknown database " ++ str dbname)
+    List.fold_left (fun (i,c) dbname ->
+      let db = try searchtable_map dbname
+        with Not_found -> user_err ~hdr:"autounfold" (str "Unknown database " ++ str dbname)
       in
       let (ids, csts) = Hint_db.unfolds db in
-	(Id.Set.union ids i, Cset.union csts c)) (Id.Set.empty, Cset.empty) db
+        (Id.Set.union ids i, Cset.union csts c)) (Id.Set.empty, Cset.empty) db
   in
-  let did, c' = unfold_head env sigma st 
-    (match cl with Some (id, _) -> Tacmach.New.pf_get_hyp_typ id gl | None -> concl) 
+  let did, c' = unfold_head env sigma st
+    (match cl with Some (id, _) -> Tacmach.New.pf_get_hyp_typ id gl | None -> concl)
   in
     if did then
       match cl with
-      | Some hyp -> change_in_hyp None (make_change_arg c') hyp
-      | None -> convert_concl_no_check c' DEFAULTcast
+      | Some hyp -> change_in_hyp ~check:true None (make_change_arg c') hyp
+      | None -> convert_concl ~check:false c' DEFAULTcast
     else Tacticals.New.tclFAIL 0 (str "Nothing to unfold")
   end
