@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -66,8 +66,8 @@ and general_decompose_aux recognizer id =
   elimHypThen
     (introElimAssumsThen
        (fun bas ->
-	  tclTHEN (clear [id])
-	    (tclMAP (general_decompose_on_hyp recognizer)
+          tclTHEN (clear [id])
+            (tclMAP (general_decompose_on_hyp recognizer)
                (ids_of_named_context bas.Tacticals.assums))))
     id
 
@@ -81,13 +81,14 @@ let up_to_delta = ref false (* true *)
 let general_decompose recognizer c =
   Proofview.Goal.enter begin fun gl ->
   let type_of = pf_unsafe_type_of gl in
+  let env = pf_env gl in
   let sigma = project gl in
   let typc = type_of c in
   tclTHENS (cut typc)
     [ tclTHEN (intro_using tmphyp_name)
          (onLastHypId
-	    (ifOnHyp (recognizer sigma) (general_decompose_aux (recognizer sigma))
-	      (fun id -> clear [id])));
+            (ifOnHyp (recognizer env sigma) (general_decompose_aux (recognizer env sigma))
+              (fun id -> clear [id])));
        exact_no_check c ]
   end
 
@@ -105,17 +106,17 @@ let head_in indl t gl =
 let decompose_these c l =
   Proofview.Goal.enter begin fun gl ->
   let indl = List.map (fun x -> x, Univ.Instance.empty) l in
-  general_decompose (fun sigma (_,t) -> head_in indl t gl) c
+  general_decompose (fun env sigma (_,t) -> head_in indl t gl) c
   end
 
 let decompose_and c =
   general_decompose
-    (fun sigma (_,t) -> is_record sigma t)
+    (fun env sigma (_,t) -> is_record env sigma t)
     c
 
 let decompose_or c =
   general_decompose
-    (fun sigma (_,t) -> is_disjunction sigma t)
+    (fun env sigma (_,t) -> is_disjunction env sigma t)
     c
 
 let h_decompose l c = decompose_these c l
@@ -135,22 +136,22 @@ let induction_trailer abs_i abs_j bargs =
     (onLastHypId
        (fun id ->
           Proofview.Goal.enter begin fun gl ->
-	  let idty = pf_unsafe_type_of gl (mkVar id) in
-	  let fvty = global_vars (pf_env gl) (project gl) idty in
-	  let possible_bring_hyps =
-	    (List.tl (nLastDecls gl (abs_j - abs_i))) @ bargs.Tacticals.assums
+          let idty = pf_unsafe_type_of gl (mkVar id) in
+          let fvty = global_vars (pf_env gl) (project gl) idty in
+          let possible_bring_hyps =
+            (List.tl (nLastDecls gl (abs_j - abs_i))) @ bargs.Tacticals.assums
           in
-	  let (hyps,_) =
+          let (hyps,_) =
             List.fold_left
-	      (fun (bring_ids,leave_ids) d ->
+              (fun (bring_ids,leave_ids) d ->
                  let cid = NamedDecl.get_id d in
                  if not (List.mem cid leave_ids)
                  then (d::bring_ids,leave_ids)
                  else (bring_ids,cid::leave_ids))
-	      ([],fvty) possible_bring_hyps
-	  in
+              ([],fvty) possible_bring_hyps
+          in
           let ids = List.rev (ids_of_named_context hyps) in
-	  (tclTHENLIST
+          (tclTHENLIST
             [revert ids; simple_elimination (mkVar id)])
           end
           ))
@@ -166,7 +167,7 @@ let double_ind h1 h2 =
   abs >>= fun (abs_i,abs_j) ->
   (tclTHEN (tclDO abs_i intro)
      (onLastHypId
-       	(fun id ->
+        (fun id ->
            elimination_then
              (introElimAssumsThen (induction_trailer abs_i abs_j)) (mkVar id))))
   end
