@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -30,7 +30,7 @@ let optimize_non_type_induction_scheme kind dep sort _ ind =
   if check_scheme kind ind then
     (* in case the inductive has a type elimination, generates only one
        induction scheme, the other ones share the same code with the
-       apropriate type *)
+       appropriate type *)
     let cte, eff = find_scheme kind ind in
     let sigma, cte = Evd.fresh_constant_instance env sigma cte in
     let c = mkConstU cte in
@@ -38,12 +38,12 @@ let optimize_non_type_induction_scheme kind dep sort _ ind =
     let (mib,mip) = Global.lookup_inductive ind in
     let npars =
       (* if a constructor of [ind] contains a recursive call, the scheme
-	 is generalized only wrt recursively uniform parameters *)
+         is generalized only wrt recursively uniform parameters *)
       if (Inductiveops.mis_is_recursive_subset [snd ind] mip.mind_recargs)
       then
-	mib.mind_nparams_rec
+        mib.mind_nparams_rec
       else
-	mib.mind_nparams in
+        mib.mind_nparams in
     let sigma, sort = Evd.fresh_sort_in_family sigma sort in
     let sigma, t', c' = weaken_sort_scheme env sigma false sort npars c t in
     let sigma = Evd.minimize_universes sigma in
@@ -51,7 +51,7 @@ let optimize_non_type_induction_scheme kind dep sort _ ind =
   else
     let sigma, pind = Evd.fresh_inductive_instance env sigma ind in
     let sigma, c = build_induction_scheme env sigma pind dep sort in
-      (c, Evd.evar_universe_context sigma), Safe_typing.empty_private_constants
+      (c, Evd.evar_universe_context sigma), Evd.empty_side_effects
 
 let build_induction_scheme_in_type dep sort ind =
   let env = Global.env () in
@@ -59,18 +59,18 @@ let build_induction_scheme_in_type dep sort ind =
   let sigma, pind = Evd.fresh_inductive_instance env sigma ind in
   let sigma, c = build_induction_scheme env sigma pind dep sort in
     c, Evd.evar_universe_context sigma
- 
+
 let rect_scheme_kind_from_type =
   declare_individual_scheme_object "_rect_nodep"
-    (fun _ x -> build_induction_scheme_in_type false InType x, Safe_typing.empty_private_constants)
+    (fun _ x -> build_induction_scheme_in_type false InType x, Evd.empty_side_effects)
 
 let rect_scheme_kind_from_prop =
   declare_individual_scheme_object "_rect" ~aux:"_rect_from_prop"
-    (fun _ x -> build_induction_scheme_in_type false InType x, Safe_typing.empty_private_constants)
+    (fun _ x -> build_induction_scheme_in_type false InType x, Evd.empty_side_effects)
 
 let rect_dep_scheme_kind_from_type =
   declare_individual_scheme_object "_rect" ~aux:"_rect_from_type"
-    (fun _ x -> build_induction_scheme_in_type true InType x, Safe_typing.empty_private_constants)
+    (fun _ x -> build_induction_scheme_in_type true InType x, Evd.empty_side_effects)
 
 let rec_scheme_kind_from_type =
   declare_individual_scheme_object "_rec_nodep" ~aux:"_rec_nodep_from_type"
@@ -88,14 +88,37 @@ let ind_scheme_kind_from_type =
   declare_individual_scheme_object "_ind_nodep"
   (optimize_non_type_induction_scheme rec_scheme_kind_from_type false InProp)
 
+let sind_scheme_kind_from_type =
+  declare_individual_scheme_object "_sind_nodep"
+  (fun _ x -> build_induction_scheme_in_type false InSProp x, Evd.empty_side_effects)
+
 let ind_dep_scheme_kind_from_type =
   declare_individual_scheme_object "_ind" ~aux:"_ind_from_type"
   (optimize_non_type_induction_scheme rec_dep_scheme_kind_from_type true InProp)
 
+let sind_dep_scheme_kind_from_type =
+  declare_individual_scheme_object "_sind" ~aux:"_sind_from_type"
+  (fun _ x -> build_induction_scheme_in_type true InSProp x, Evd.empty_side_effects)
+
 let ind_scheme_kind_from_prop =
   declare_individual_scheme_object "_ind" ~aux:"_ind_from_prop"
   (optimize_non_type_induction_scheme rec_scheme_kind_from_prop false InProp)
 
+let sind_scheme_kind_from_prop =
+  declare_individual_scheme_object "_sind" ~aux:"_sind_from_prop"
+  (fun _ x -> build_induction_scheme_in_type false InSProp x, Evd.empty_side_effects)
+
+let nondep_elim_scheme from_kind to_kind =
+  match from_kind, to_kind with
+  | InProp, InProp  -> ind_scheme_kind_from_prop
+  | InProp, InSProp -> sind_scheme_kind_from_prop
+  | InProp, InSet   -> rec_scheme_kind_from_prop
+  | InProp, InType  -> rect_scheme_kind_from_prop
+  | _     , InProp  -> ind_scheme_kind_from_type
+  | _     , InSProp -> sind_scheme_kind_from_type
+  | _     , InSet   -> rec_scheme_kind_from_type
+  | _     , InType  -> rect_scheme_kind_from_type
+
 (* Case analysis *)
 
 let build_case_analysis_scheme_in_type dep sort ind =
@@ -107,24 +130,24 @@ let build_case_analysis_scheme_in_type dep sort ind =
 
 let case_scheme_kind_from_type =
   declare_individual_scheme_object "_case_nodep"
-  (fun _ x -> build_case_analysis_scheme_in_type false InType x, Safe_typing.empty_private_constants)
+  (fun _ x -> build_case_analysis_scheme_in_type false InType x, Evd.empty_side_effects)
 
 let case_scheme_kind_from_prop =
   declare_individual_scheme_object "_case" ~aux:"_case_from_prop"
-  (fun _ x -> build_case_analysis_scheme_in_type false InType x, Safe_typing.empty_private_constants)
+  (fun _ x -> build_case_analysis_scheme_in_type false InType x, Evd.empty_side_effects)
 
 let case_dep_scheme_kind_from_type =
   declare_individual_scheme_object "_case" ~aux:"_case_from_type"
-  (fun _ x -> build_case_analysis_scheme_in_type true InType x, Safe_typing.empty_private_constants)
+  (fun _ x -> build_case_analysis_scheme_in_type true InType x, Evd.empty_side_effects)
 
 let case_dep_scheme_kind_from_type_in_prop =
   declare_individual_scheme_object "_casep_dep"
-  (fun _ x -> build_case_analysis_scheme_in_type true InProp x, Safe_typing.empty_private_constants)
+  (fun _ x -> build_case_analysis_scheme_in_type true InProp x, Evd.empty_side_effects)
 
 let case_dep_scheme_kind_from_prop =
   declare_individual_scheme_object "_case_dep"
-  (fun _ x -> build_case_analysis_scheme_in_type true InType x, Safe_typing.empty_private_constants)
+  (fun _ x -> build_case_analysis_scheme_in_type true InType x, Evd.empty_side_effects)
 
 let case_dep_scheme_kind_from_prop_in_prop =
   declare_individual_scheme_object "_casep"
-  (fun _ x -> build_case_analysis_scheme_in_type true InProp x, Safe_typing.empty_private_constants)
+  (fun _ x -> build_case_analysis_scheme_in_type true InProp x, Evd.empty_side_effects)
