@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -42,7 +42,7 @@
 
    One may wonder whether these extensions are worth to be done
    regarding the price we have to pay and regarding the rare
-   situations where they are needed. However, I believe it meets a 
+   situations where they are needed. However, I believe it meets a
    natural expectation of the user.
 *)
 
@@ -51,6 +51,7 @@ open Util
 open Names
 open Term
 open Constr
+open Context
 open Vars
 open Declarations
 open Environ
@@ -66,9 +67,9 @@ module RelDecl = Context.Rel.Declaration
 
 let hid = Id.of_string "H"
 let xid = Id.of_string "X"
-let default_id_of_sort = function InProp | InSet -> hid | InType -> xid
+let default_id_of_sort = function InSProp | InProp | InSet -> hid | InType -> xid
 let fresh env id = next_global_ident_away id Id.Set.empty
-let with_context_set ctx (b, ctx') = 
+let with_context_set ctx (b, ctx') =
   (b, Univ.ContextSet.union ctx ctx')
 
 let build_dependent_inductive ind (mib,mip) =
@@ -80,14 +81,14 @@ let build_dependent_inductive ind (mib,mip) =
 
 let named_hd env t na = named_hd env (Evd.from_env env) (EConstr.of_constr t) na
 let name_assumption env = function
-| LocalAssum (na,t) -> LocalAssum (named_hd env t na, t)
-| LocalDef (na,c,t) -> LocalDef (named_hd env c na, c, t)
+| LocalAssum (na,t) -> LocalAssum (map_annot (named_hd env t) na, t)
+| LocalDef (na,c,t) -> LocalDef (map_annot (named_hd env c) na, c, t)
 
 let name_context env hyps =
   snd
     (List.fold_left
        (fun (env,hyps) d ->
-	  let d' = name_assumption env d in (push_rel d' env, d' :: hyps))
+          let d' = name_assumption env d in (push_rel d' env, d' :: hyps))
        (env,[]) (List.rev hyps))
 
 let my_it_mkLambda_or_LetIn s c = it_mkLambda_or_LetIn c s
@@ -99,9 +100,9 @@ let my_it_mkLambda_or_LetIn_name s c =
 
 let get_coq_eq ctx =
   try
-    let eq = Globnames.destIndRef Coqlib.glob_eq in
+    let eq = Globnames.destIndRef (Coqlib.lib_ref "core.eq.type") in
     (* Do not force the lazy if they are not defined *)
-    let eq, ctx = with_context_set ctx 
+    let eq, ctx = with_context_set ctx
       (UnivGen.fresh_inductive_instance (Global.env ()) eq) in
       mkIndU eq, mkConstructUi (eq,1), ctx
   with Not_found ->
@@ -138,7 +139,7 @@ let get_sym_eq_data env (ind,u) =
   let realsign,_ = List.chop mip.mind_nrealdecls arityctxt in
   if List.exists is_local_def realsign then
     error "Inductive equalities with local definitions in arity not supported.";
-  let constrsign,ccl = decompose_prod_assum mip.mind_nf_lc.(0) in
+  let constrsign,ccl = mip.mind_nf_lc.(0) in
   let _,constrargs = decompose_app ccl in
   if not (Int.equal (Context.Rel.length constrsign) (Context.Rel.length mib.mind_params_ctxt)) then
     error "Constructor must have no arguments"; (* This can be relaxed... *)
@@ -173,7 +174,7 @@ let get_non_sym_eq_data env (ind,u) =
   let realsign,_ = List.chop mip.mind_nrealdecls arityctxt in
   if List.exists is_local_def realsign then
     error "Inductive equalities with local definitions in arity not supported";
-  let constrsign,ccl = decompose_prod_assum mip.mind_nf_lc.(0) in
+  let constrsign,ccl = mip.mind_nf_lc.(0) in
   let _,constrargs = decompose_app ccl in
   if not (Int.equal (Context.Rel.length constrsign) (Context.Rel.length mib.mind_params_ctxt)) then
     error "Constructor must have no arguments";
@@ -202,21 +203,24 @@ let build_sym_scheme env ind =
     get_sym_eq_data env indu in
   let cstr n =
     mkApp (mkConstructUi(indu,1),Context.Rel.to_extended_vect mkRel n mib.mind_params_ctxt) in
-  let varH = fresh env (default_id_of_sort (snd (mind_arity mip))) in
+  let inds = snd (mind_arity mip) in
+  let varH = fresh env (default_id_of_sort inds) in
   let applied_ind = build_dependent_inductive indu specif in
+  let indr = Sorts.relevance_of_sort_family inds in
   let realsign_ind =
-    name_context env ((LocalAssum (Name varH,applied_ind))::realsign) in
-  let ci = make_case_info (Global.env()) ind RegularStyle in
-  let c = 
+    name_context env ((LocalAssum (make_annot (Name varH) indr,applied_ind))::realsign) in
+  let rci = Sorts.Relevant in (* TODO relevance *)
+  let ci = make_case_info (Global.env()) ind rci RegularStyle in
+  let c =
   (my_it_mkLambda_or_LetIn paramsctxt
   (my_it_mkLambda_or_LetIn_name realsign_ind
   (mkCase (ci,
      my_it_mkLambda_or_LetIn_name
        (lift_rel_context (nrealargs+1) realsign_ind)
        (mkApp (mkIndU indu,Array.concat
-	  [Context.Rel.to_extended_vect mkRel (3*nrealargs+2) paramsctxt1;
-	   rel_vect 1 nrealargs;
-	   rel_vect (2*nrealargs+2) nrealargs])),
+          [Context.Rel.to_extended_vect mkRel (3*nrealargs+2) paramsctxt1;
+           rel_vect 1 nrealargs;
+           rel_vect (2*nrealargs+2) nrealargs])),
      mkRel 1 (* varH *),
        [|cstr (nrealargs+1)|]))))
   in c, UState.of_context_set ctx
@@ -225,7 +229,7 @@ let sym_scheme_kind =
   declare_individual_scheme_object "_sym_internal"
   (fun _ ind ->
     let c, ctx = build_sym_scheme (Global.env() (* side-effect! *)) ind in
-      (c, ctx), Safe_typing.empty_private_constants)
+      (c, ctx), Evd.empty_side_effects)
 
 (**********************************************************************)
 (* Build the involutivity of symmetry for an inductive type           *)
@@ -243,9 +247,9 @@ let sym_scheme_kind =
 (*                                                                    *)
 (**********************************************************************)
 
-let const_of_scheme kind env ind ctx = 
+let const_of_scheme kind env ind ctx =
   let sym_scheme, eff = (find_scheme kind ind) in
-  let sym, ctx = with_context_set ctx 
+  let sym, ctx = with_context_set ctx
     (UnivGen.fresh_constant_instance (Global.env()) sym_scheme) in
     mkConstU sym, ctx, eff
 
@@ -256,7 +260,9 @@ let build_sym_involutive_scheme env ind =
   let eq,eqrefl,ctx = get_coq_eq ctx in
   let sym, ctx, eff = const_of_scheme sym_scheme_kind env ind ctx in
   let cstr n = mkApp (mkConstructUi (indu,1),Context.Rel.to_extended_vect mkRel n paramsctxt) in
-  let varH = fresh env (default_id_of_sort (snd (mind_arity mip))) in
+  let inds = snd (mind_arity mip) in
+  let indr = Sorts.relevance_of_sort_family inds in
+  let varH = fresh env (default_id_of_sort inds) in
   let applied_ind = build_dependent_inductive indu specif in
   let applied_ind_C =
     mkApp
@@ -264,32 +270,33 @@ let build_sym_involutive_scheme env ind =
          (Context.Rel.to_extended_vect mkRel (nrealargs+1) mib.mind_params_ctxt)
          (rel_vect (nrealargs+1) nrealargs)) in
   let realsign_ind =
-    name_context env ((LocalAssum (Name varH,applied_ind))::realsign) in
-  let ci = make_case_info (Global.env()) ind RegularStyle in
-  let c = 
+    name_context env ((LocalAssum (make_annot (Name varH) indr,applied_ind))::realsign) in
+  let rci = Sorts.Relevant in (* TODO relevance *)
+  let ci = make_case_info (Global.env()) ind rci RegularStyle in
+  let c =
     (my_it_mkLambda_or_LetIn paramsctxt
      (my_it_mkLambda_or_LetIn_name realsign_ind
       (mkCase (ci,
-	       my_it_mkLambda_or_LetIn_name
-	       (lift_rel_context (nrealargs+1) realsign_ind)
-	       (mkApp (eq,[|
-	       mkApp
-	       (mkIndU indu, Array.concat
-	       [Context.Rel.to_extended_vect mkRel (3*nrealargs+2) paramsctxt1;
-		rel_vect (2*nrealargs+2) nrealargs;
-		rel_vect 1 nrealargs]);
+               my_it_mkLambda_or_LetIn_name
+               (lift_rel_context (nrealargs+1) realsign_ind)
+               (mkApp (eq,[|
+               mkApp
+               (mkIndU indu, Array.concat
+               [Context.Rel.to_extended_vect mkRel (3*nrealargs+2) paramsctxt1;
+                rel_vect (2*nrealargs+2) nrealargs;
+                rel_vect 1 nrealargs]);
                mkApp (sym,Array.concat
-	       [Context.Rel.to_extended_vect mkRel (3*nrealargs+2) paramsctxt1;
-		rel_vect 1 nrealargs;
-		rel_vect (2*nrealargs+2) nrealargs;
-		[|mkApp (sym,Array.concat
-		[Context.Rel.to_extended_vect mkRel (3*nrealargs+2) paramsctxt1;
-		 rel_vect (2*nrealargs+2) nrealargs;
-		 rel_vect 1 nrealargs;
-		 [|mkRel 1|]])|]]);
-	       mkRel 1|])),
-	       mkRel 1 (* varH *),
-	       [|mkApp(eqrefl,[|applied_ind_C;cstr (nrealargs+1)|])|]))))
+               [Context.Rel.to_extended_vect mkRel (3*nrealargs+2) paramsctxt1;
+                rel_vect 1 nrealargs;
+                rel_vect (2*nrealargs+2) nrealargs;
+                [|mkApp (sym,Array.concat
+                [Context.Rel.to_extended_vect mkRel (3*nrealargs+2) paramsctxt1;
+                 rel_vect (2*nrealargs+2) nrealargs;
+                 rel_vect 1 nrealargs;
+                 [|mkRel 1|]])|]]);
+               mkRel 1|])),
+               mkRel 1 (* varH *),
+               [|mkApp(eqrefl,[|applied_ind_C;cstr (nrealargs+1)|])|]))))
   in (c, UState.of_context_set ctx), eff
 
 let sym_involutive_scheme_kind =
@@ -368,7 +375,9 @@ let build_l2r_rew_scheme dep env ind kind =
     mkApp (mkConstructUi(indu,1),
       Array.concat [Context.Rel.to_extended_vect mkRel n paramsctxt1;
                     rel_vect p nrealargs]) in
-  let varH = fresh env (default_id_of_sort (snd (mind_arity mip))) in
+  let inds = snd (mind_arity mip) in
+  let indr = Sorts.relevance_of_sort_family inds in
+  let varH = fresh env (default_id_of_sort inds) in
   let varHC = fresh env (Id.of_string "HC") in
   let varP = fresh env (Id.of_string "P") in
   let applied_ind = build_dependent_inductive indu specif in
@@ -384,9 +393,9 @@ let build_l2r_rew_scheme dep env ind kind =
         rel_vect 0 nrealargs]) in
   let realsign_P = lift_rel_context nrealargs realsign in
   let realsign_ind_P =
-    name_context env ((LocalAssum (Name varH,applied_ind_P))::realsign_P) in
+    name_context env ((LocalAssum (make_annot (Name varH) indr,applied_ind_P))::realsign_P) in
   let realsign_ind_G =
-    name_context env ((LocalAssum (Name varH,applied_ind_G))::
+    name_context env ((LocalAssum (make_annot (Name varH) indr,applied_ind_G))::
                       lift_rel_context (nrealargs+3) realsign) in
   let applied_sym_C n =
      mkApp(sym,
@@ -396,12 +405,13 @@ let build_l2r_rew_scheme dep env ind kind =
        Array.concat [Context.Rel.to_extended_vect mkRel (nrealargs*3+4) paramsctxt1;
                      rel_vect (nrealargs+4) nrealargs;
                      rel_vect 1 nrealargs;
-		     [|mkRel 1|]]) in
+                     [|mkRel 1|]]) in
   let s, ctx' = UnivGen.fresh_sort_in_family kind in
   let ctx = Univ.ContextSet.union ctx ctx' in
   let s = mkSort s in
-  let ci = make_case_info (Global.env()) ind RegularStyle in
-  let cieq = make_case_info (Global.env()) (fst (destInd eq)) RegularStyle in
+  let rci = Sorts.Relevant in (* TODO relevance *)
+  let ci = make_case_info (Global.env()) ind rci RegularStyle in
+  let cieq = make_case_info (Global.env()) (fst (destInd eq)) rci RegularStyle in
   let applied_PC =
     mkApp (mkVar varP,Array.append (Context.Rel.to_extended_vect mkRel 1 realsign)
            (if dep then [|cstr (2*nrealargs+1) 1|] else [||])) in
@@ -413,30 +423,30 @@ let build_l2r_rew_scheme dep env ind kind =
            (if dep then [|mkRel 2|] else [||])) in
   let applied_sym_sym =
          mkApp (sym,Array.concat
-	   [Context.Rel.to_extended_vect mkRel (2*nrealargs+4) paramsctxt1;
+           [Context.Rel.to_extended_vect mkRel (2*nrealargs+4) paramsctxt1;
             rel_vect 4 nrealargs;
             rel_vect (nrealargs+4) nrealargs;
             [|mkApp (sym,Array.concat
-	      [Context.Rel.to_extended_vect mkRel (2*nrealargs+4) paramsctxt1;
-	       rel_vect (nrealargs+4) nrealargs;
-	       rel_vect 4 nrealargs;
-	       [|mkRel 2|]])|]]) in
+              [Context.Rel.to_extended_vect mkRel (2*nrealargs+4) paramsctxt1;
+               rel_vect (nrealargs+4) nrealargs;
+               rel_vect 4 nrealargs;
+               [|mkRel 2|]])|]]) in
   let main_body =
     mkCase (ci,
       my_it_mkLambda_or_LetIn_name realsign_ind_G applied_PG,
       applied_sym_C 3,
       [|mkVar varHC|]) in
-  let c = 
+  let c =
   (my_it_mkLambda_or_LetIn paramsctxt
   (my_it_mkLambda_or_LetIn_name realsign
-  (mkNamedLambda varP
+  (mkNamedLambda (make_annot varP indr)
     (my_it_mkProd_or_LetIn (if dep then realsign_ind_P else realsign_P) s)
-  (mkNamedLambda varHC applied_PC
-  (mkNamedLambda varH (lift 2 applied_ind)
+  (mkNamedLambda (make_annot varHC indr) applied_PC
+  (mkNamedLambda (make_annot varH indr) (lift 2 applied_ind)
   (if dep then (* we need a coercion *)
      mkCase (cieq,
-       mkLambda (Name varH,lift 3 applied_ind,
-         mkLambda (Anonymous,
+       mkLambda (make_annot (Name varH) indr,lift 3 applied_ind,
+         mkLambda (make_annot Anonymous indr,
                    mkApp (eq,[|lift 4 applied_ind;applied_sym_sym;mkRel 1|]),
                    applied_PR)),
        mkApp (sym_involutive,
@@ -445,7 +455,7 @@ let build_l2r_rew_scheme dep env ind kind =
    else
      main_body))))))
   in (c, UState.of_context_set ctx),
-      Safe_typing.concat_private eff' eff
+      Evd.concat_side_effects eff' eff
 
 (**********************************************************************)
 (* Build the left-to-right rewriting lemma for hypotheses associated  *)
@@ -481,7 +491,9 @@ let build_l2r_forward_rew_scheme dep env ind kind =
     mkApp (mkConstructUi(indu,1),
       Array.concat [Context.Rel.to_extended_vect mkRel n paramsctxt1;
                     rel_vect p nrealargs]) in
-  let varH = fresh env (default_id_of_sort (snd (mind_arity mip))) in
+  let inds = snd (mind_arity mip) in
+  let indr = Sorts.relevance_of_sort_family inds in
+  let varH = fresh env (default_id_of_sort inds) in
   let varHC = fresh env (Id.of_string "HC") in
   let varP = fresh env (Id.of_string "P") in
   let applied_ind = build_dependent_inductive indu specif in
@@ -497,13 +509,14 @@ let build_l2r_forward_rew_scheme dep env ind kind =
         rel_vect (2*nrealargs+1) nrealargs]) in
   let realsign_P n = lift_rel_context (nrealargs*n+n) realsign in
   let realsign_ind =
-    name_context env ((LocalAssum (Name varH,applied_ind))::realsign) in
+    name_context env ((LocalAssum (make_annot (Name varH) indr,applied_ind))::realsign) in
   let realsign_ind_P n aP =
-    name_context env ((LocalAssum (Name varH,aP))::realsign_P n) in
+    name_context env ((LocalAssum (make_annot (Name varH) indr,aP))::realsign_P n) in
   let s, ctx' = UnivGen.fresh_sort_in_family kind in
   let ctx = Univ.ContextSet.union ctx ctx' in
   let s = mkSort s in
-  let ci = make_case_info (Global.env()) ind RegularStyle in
+  let rci = Sorts.Relevant in
+  let ci = make_case_info (Global.env()) ind rci RegularStyle in
   let applied_PC =
     mkApp (mkVar varP,Array.append
            (rel_vect (nrealargs*2+3) nrealargs)
@@ -512,27 +525,27 @@ let build_l2r_forward_rew_scheme dep env ind kind =
     mkApp (mkVar varP,Array.append
            (rel_vect (nrealargs+2) nrealargs)
            (if dep then [|cstr (2*nrealargs+2) (nrealargs+2)|]
-	    else [||])) in
+            else [||])) in
   let applied_PG =
     mkApp (mkVar varP,Array.append (rel_vect 3 nrealargs)
            (if dep then [|cstr (3*nrealargs+4) 3|] else [||])) in
-  let c = 
+  let c =
   (my_it_mkLambda_or_LetIn paramsctxt
   (my_it_mkLambda_or_LetIn_name realsign
-  (mkNamedLambda varH applied_ind
+  (mkNamedLambda (make_annot varH indr) applied_ind
   (mkCase (ci,
      my_it_mkLambda_or_LetIn_name
        (lift_rel_context (nrealargs+1) realsign_ind)
-       (mkNamedProd varP
+       (mkNamedProd (make_annot varP indr)
          (my_it_mkProd_or_LetIn
-	   (if dep then realsign_ind_P 2 applied_ind_P else realsign_P 2) s)
-       (mkNamedProd varHC applied_PC applied_PG)),
+           (if dep then realsign_ind_P 2 applied_ind_P else realsign_P 2) s)
+       (mkNamedProd (make_annot varHC indr) applied_PC applied_PG)),
      (mkVar varH),
-     [|mkNamedLambda varP
+     [|mkNamedLambda (make_annot varP indr)
         (my_it_mkProd_or_LetIn
-	  (if dep then realsign_ind_P 1 applied_ind_P' else realsign_P 2) s)
-      (mkNamedLambda varHC applied_PC'
-	(mkVar varHC))|])))))
+          (if dep then realsign_ind_P 1 applied_ind_P' else realsign_P 2) s)
+      (mkNamedLambda (make_annot varHC indr) applied_PC'
+        (mkVar varHC))|])))))
   in c, UState.of_context_set ctx
 
 (**********************************************************************)
@@ -565,46 +578,49 @@ let build_l2r_forward_rew_scheme dep env ind kind =
 (* statement but no need for symmetry of the equality.                *)
 (**********************************************************************)
 
-let build_r2l_forward_rew_scheme dep env ind kind = 
+let build_r2l_forward_rew_scheme dep env ind kind =
   let (ind,u as indu), ctx = UnivGen.fresh_inductive_instance env ind in
   let ((mib,mip as specif),constrargs,realsign,paramsctxt,nrealargs) =
     get_non_sym_eq_data env indu in
   let cstr n =
     mkApp (mkConstructUi(indu,1),Context.Rel.to_extended_vect mkRel n mib.mind_params_ctxt) in
   let constrargs_cstr = constrargs@[cstr 0] in
-  let varH = fresh env (default_id_of_sort (snd (mind_arity mip))) in
+  let inds = snd (mind_arity mip) in
+  let indr = Sorts.relevance_of_sort_family inds in
+  let varH = fresh env (default_id_of_sort inds) in
   let varHC = fresh env (Id.of_string "HC") in
   let varP = fresh env (Id.of_string "P") in
   let applied_ind = build_dependent_inductive indu specif in
   let realsign_ind =
-    name_context env ((LocalAssum (Name varH,applied_ind))::realsign) in
+    name_context env ((LocalAssum (make_annot (Name varH) indr,applied_ind))::realsign) in
   let s, ctx' = UnivGen.fresh_sort_in_family kind in
   let ctx = Univ.ContextSet.union ctx ctx' in
   let s = mkSort s in
-  let ci = make_case_info (Global.env()) ind RegularStyle in
+  let rci = Sorts.Relevant in (* TODO relevance *)
+  let ci = make_case_info (Global.env()) ind rci RegularStyle in
   let applied_PC =
     applist (mkVar varP,if dep then constrargs_cstr else constrargs) in
   let applied_PG =
     mkApp (mkVar varP,
            if dep then Context.Rel.to_extended_vect mkRel 0 realsign_ind
-	   else Context.Rel.to_extended_vect mkRel 1 realsign) in
-  let c = 
+           else Context.Rel.to_extended_vect mkRel 1 realsign) in
+  let c =
   (my_it_mkLambda_or_LetIn paramsctxt
   (my_it_mkLambda_or_LetIn_name realsign_ind
-  (mkNamedLambda varP
+  (mkNamedLambda (make_annot varP indr)
     (my_it_mkProd_or_LetIn (lift_rel_context (nrealargs+1)
                              (if dep then realsign_ind else realsign)) s)
-  (mkNamedLambda varHC (lift 1 applied_PG)
+  (mkNamedLambda (make_annot varHC indr) (lift 1 applied_PG)
   (mkApp
     (mkCase (ci,
        my_it_mkLambda_or_LetIn_name
          (lift_rel_context (nrealargs+3) realsign_ind)
-         (mkArrow applied_PG (lift (2*nrealargs+5) applied_PC)),
+         (mkArrow applied_PG indr (lift (2*nrealargs+5) applied_PC)),
        mkRel 3 (* varH *),
        [|mkLambda
-          (Name varHC,
-	   lift (nrealargs+3) applied_PC,
-	   mkRel 1)|]),
+          (make_annot (Name varHC) indr,
+           lift (nrealargs+3) applied_PC,
+           mkRel 1)|]),
     [|mkVar varHC|]))))))
   in c, UState.of_context_set ctx
 
@@ -632,14 +648,14 @@ let fix_r2l_forward_rew_scheme (c, ctx') =
   let ctx,_ = decompose_prod_assum t in
   match ctx with
   | hp :: p :: ind :: indargs ->
-     let c' = 
+     let c' =
       my_it_mkLambda_or_LetIn indargs
         (mkLambda_or_LetIn (RelDecl.map_constr (liftn (-1) 1) p)
-	  (mkLambda_or_LetIn (RelDecl.map_constr (liftn (-1) 2) hp)
-	    (mkLambda_or_LetIn (RelDecl.map_constr (lift 2) ind)
+          (mkLambda_or_LetIn (RelDecl.map_constr (liftn (-1) 2) hp)
+            (mkLambda_or_LetIn (RelDecl.map_constr (lift 2) ind)
               (EConstr.Unsafe.to_constr (Reductionops.whd_beta sigma
-		(EConstr.of_constr (applist (c,
-	          Context.Rel.to_extended_list mkRel 3 indargs @ [mkRel 1;mkRel 3;mkRel 2]))))))))
+                (EConstr.of_constr (applist (c,
+                  Context.Rel.to_extended_list mkRel 3 indargs @ [mkRel 1;mkRel 3;mkRel 2]))))))))
       in c', ctx'
   | _ -> anomaly (Pp.str "Ill-formed non-dependent left-to-right rewriting scheme.")
 
@@ -663,18 +679,13 @@ let fix_r2l_forward_rew_scheme (c, ctx') =
 (*          (H:I q1..qm a1..an),                                      *)
 (*          P b1..bn C -> P a1..an H                                  *)
 (**********************************************************************)
- 
+
 let build_r2l_rew_scheme dep env ind k =
   let sigma = Evd.from_env env in
   let (sigma, indu) = Evd.fresh_inductive_instance env sigma ind in
   let (sigma, c) = build_case_analysis_scheme env sigma indu dep k in
     c, Evd.evar_universe_context sigma
 
-let build_l2r_rew_scheme = build_l2r_rew_scheme
-let build_l2r_forward_rew_scheme = build_l2r_forward_rew_scheme
-let build_r2l_rew_scheme = build_r2l_rew_scheme
-let build_r2l_forward_rew_scheme = build_r2l_forward_rew_scheme
-
 (**********************************************************************)
 (* Register the rewriting schemes                                     *)
 (**********************************************************************)
@@ -697,7 +708,7 @@ let rew_l2r_dep_scheme_kind =
 (**********************************************************************)
 let rew_r2l_dep_scheme_kind =
   declare_individual_scheme_object "_rew_dep"
-  (fun _ ind -> build_r2l_rew_scheme true (Global.env()) ind InType,Safe_typing.empty_private_constants)
+  (fun _ ind -> build_r2l_rew_scheme true (Global.env()) ind InType,Evd.empty_side_effects)
 
 (**********************************************************************)
 (* Dependent rewrite from right-to-left in hypotheses                 *)
@@ -707,7 +718,7 @@ let rew_r2l_dep_scheme_kind =
 (**********************************************************************)
 let rew_r2l_forward_dep_scheme_kind =
   declare_individual_scheme_object "_rew_fwd_dep"
-  (fun _ ind -> build_r2l_forward_rew_scheme true (Global.env()) ind InType,Safe_typing.empty_private_constants)
+  (fun _ ind -> build_r2l_forward_rew_scheme true (Global.env()) ind InType,Evd.empty_side_effects)
 
 (**********************************************************************)
 (* Dependent rewrite from left-to-right in hypotheses                 *)
@@ -717,7 +728,7 @@ let rew_r2l_forward_dep_scheme_kind =
 (**********************************************************************)
 let rew_l2r_forward_dep_scheme_kind =
   declare_individual_scheme_object "_rew_fwd_r_dep"
-  (fun _ ind -> build_l2r_forward_rew_scheme true (Global.env()) ind InType,Safe_typing.empty_private_constants)
+  (fun _ ind -> build_l2r_forward_rew_scheme true (Global.env()) ind InType,Evd.empty_side_effects)
 
 (**********************************************************************)
 (* Non-dependent rewrite from either left-to-right in conclusion or   *)
@@ -731,7 +742,7 @@ let rew_l2r_forward_dep_scheme_kind =
 let rew_l2r_scheme_kind =
   declare_individual_scheme_object "_rew_r"
   (fun _ ind -> fix_r2l_forward_rew_scheme
-     (build_r2l_forward_rew_scheme false (Global.env()) ind InType), Safe_typing.empty_private_constants)
+     (build_r2l_forward_rew_scheme false (Global.env()) ind InType), Evd.empty_side_effects)
 
 (**********************************************************************)
 (* Non-dependent rewrite from either right-to-left in conclusion or   *)
@@ -741,7 +752,7 @@ let rew_l2r_scheme_kind =
 (**********************************************************************)
 let rew_r2l_scheme_kind =
   declare_individual_scheme_object "_rew"
-  (fun _ ind -> build_r2l_rew_scheme false (Global.env()) ind InType, Safe_typing.empty_private_constants)
+  (fun _ ind -> build_r2l_rew_scheme false (Global.env()) ind InType, Evd.empty_side_effects)
 
 (* End of rewriting schemes *)
 
@@ -761,7 +772,7 @@ let rew_r2l_scheme_kind =
 (* TODO: extend it to types with more than one index *)
 
 let build_congr env (eq,refl,ctx) ind =
-  let (ind,u as indu), ctx = with_context_set ctx 
+  let (ind,u as indu), ctx = with_context_set ctx
     (UnivGen.fresh_inductive_instance env ind) in
   let (mib,mip) = lookup_mind_specif env ind in
   if not (Int.equal (Array.length mib.mind_packets) 1) || not (Int.equal (Array.length mip.mind_nf_lc) 1) then
@@ -775,8 +786,11 @@ let build_congr env (eq,refl,ctx) ind =
   if List.exists is_local_def realsign then
     error "Inductive equalities with local definitions in arity not supported.";
   let env_with_arity = push_rel_context arityctxt env in
-  let ty = RelDecl.get_type (lookup_rel (mip.mind_nrealargs - i + 1) env_with_arity) in
-  let constrsign,ccl = decompose_prod_assum mip.mind_nf_lc.(0) in
+  let ty, tyr =
+    let decl = lookup_rel (mip.mind_nrealargs - i + 1) env_with_arity in
+    RelDecl.get_type decl, RelDecl.get_relevance decl
+  in
+  let constrsign,ccl = mip.mind_nf_lc.(0) in
   let _,constrargs = decompose_app ccl in
   if not (Int.equal (Context.Rel.length constrsign) (Context.Rel.length mib.mind_params_ctxt)) then
     error "Constructor must have no arguments";
@@ -784,41 +798,42 @@ let build_congr env (eq,refl,ctx) ind =
   let varB = fresh env (Id.of_string "B") in
   let varH = fresh env (Id.of_string "H") in
   let varf = fresh env (Id.of_string "f") in
-  let ci = make_case_info (Global.env()) ind RegularStyle in
-  let uni, ctx = UnivGen.extend_context (UnivGen.new_global_univ ()) ctx in
+  let rci = Sorts.Relevant in (* TODO relevance *)
+  let ci = make_case_info (Global.env()) ind rci RegularStyle in
+  let uni, ctx = Univ.extend_in_context_set (UnivGen.new_global_univ ()) ctx in
   let ctx = (fst ctx, Univ.enforce_leq uni (univ_of_eq env eq) (snd ctx)) in
-  let c = 
+  let c =
   my_it_mkLambda_or_LetIn paramsctxt
-     (mkNamedLambda varB (mkSort (Type uni))
-     (mkNamedLambda varf (mkArrow (lift 1 ty) (mkVar varB))
+     (mkNamedLambda (make_annot varB Sorts.Relevant) (mkType uni)
+     (mkNamedLambda (make_annot varf Sorts.Relevant) (mkArrow (lift 1 ty) tyr (mkVar varB))
      (my_it_mkLambda_or_LetIn_name (lift_rel_context 2 realsign)
-     (mkNamedLambda varH
+     (mkNamedLambda (make_annot varH Sorts.Relevant)
         (applist
            (mkIndU indu,
-	    Context.Rel.to_extended_list mkRel (mip.mind_nrealargs+2) paramsctxt @
-	    Context.Rel.to_extended_list mkRel 0 realsign))
+            Context.Rel.to_extended_list mkRel (mip.mind_nrealargs+2) paramsctxt @
+            Context.Rel.to_extended_list mkRel 0 realsign))
      (mkCase (ci,
        my_it_mkLambda_or_LetIn_name
-	 (lift_rel_context (mip.mind_nrealargs+3) realsign)
+         (lift_rel_context (mip.mind_nrealargs+3) realsign)
          (mkLambda
-           (Anonymous,
+           (make_annot Anonymous Sorts.Relevant,
             applist
              (mkIndU indu,
-	        Context.Rel.to_extended_list mkRel (2*mip.mind_nrealdecls+3)
-		  paramsctxt
-	        @ Context.Rel.to_extended_list mkRel 0 realsign),
+                Context.Rel.to_extended_list mkRel (2*mip.mind_nrealdecls+3)
+                  paramsctxt
+                @ Context.Rel.to_extended_list mkRel 0 realsign),
             mkApp (eq,
-	      [|mkVar varB;
+              [|mkVar varB;
                 mkApp (mkVar varf, [|lift (2*mip.mind_nrealdecls+4) b|]);
-		mkApp (mkVar varf, [|mkRel (mip.mind_nrealargs - i + 2)|])|]))),
+                mkApp (mkVar varf, [|mkRel (mip.mind_nrealargs - i + 2)|])|]))),
        mkVar varH,
        [|mkApp (refl,
           [|mkVar varB;
-	    mkApp (mkVar varf, [|lift (mip.mind_nrealargs+3) b|])|])|]))))))
+            mkApp (mkVar varf, [|lift (mip.mind_nrealargs+3) b|])|])|]))))))
   in c, UState.of_context_set ctx
 
 let congr_scheme_kind = declare_individual_scheme_object "_congr"
   (fun _ ind ->
      (* May fail if equality is not defined *)
    build_congr (Global.env()) (get_coq_eq Univ.ContextSet.empty) ind,
-   Safe_typing.empty_private_constants)
+   Evd.empty_side_effects)
