@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -35,7 +35,7 @@ open Coqlib
    contained in the arguments of the application *)
 
 (** I implemented the following functions which test whether a term [t]
-   is an inductive but non-recursive type, a general conjuction, a
+   is an inductive but non-recursive type, a general conjunction, a
    general disjunction, or a type with no constructors.
 
    They are more general than matching with [or_term], [and_term], etc,
@@ -43,8 +43,8 @@ open Coqlib
    also work on ad-hoc disjunctions introduced by the user.
    (Eduardo, 6/8/97). *)
 
-type 'a matching_function = evar_map -> constr -> 'a option
-type testing_function = evar_map -> constr -> bool
+type 'a matching_function = Environ.env -> evar_map -> constr -> 'a option
+type testing_function = Environ.env -> evar_map -> constr -> bool
 
 val match_with_non_recursive_type : (constr * constr list) matching_function
 val is_non_recursive_type         : testing_function
@@ -83,10 +83,10 @@ val is_inductive_equality  : inductive -> bool
 val match_with_equality_type : (constr * constr list) matching_function
 val is_equality_type       : testing_function
 
-val match_with_nottype     : Environ.env -> (constr * constr) matching_function
-val is_nottype             : Environ.env -> testing_function
+val match_with_nottype     : (constr * constr) matching_function
+val is_nottype             : testing_function
 
-val match_with_forall_term    : (Name.t * constr * constr) matching_function
+val match_with_forall_term    : (Name.t Context.binder_annot * constr * constr) matching_function
 val is_forall_term            : testing_function
 
 val match_with_imp_term    : (constr * constr) matching_function
@@ -120,7 +120,7 @@ val match_with_equation:
 
 (***** Destructing patterns bound to some theory *)
 
-(** Match terms [eq A t u], [identity A t u] or [JMeq A t A u] 
+(** Match terms [eq A t u], [identity A t u] or [JMeq A t A u]
    Returns associated lemmas and [A,t,u] or fails PatternMatchingFailure *)
 val find_eq_data_decompose : Proofview.Goal.t -> constr ->
       coq_eq_data * EInstance.t * (types * constr * constr)
@@ -132,7 +132,7 @@ val find_this_eq_data_decompose : Proofview.Goal.t -> constr ->
 (** A variant that returns more informative structure on the equality found *)
 val find_eq_data : evar_map -> constr -> coq_eq_data * EInstance.t * equation_kind
 
-(** Match a term of the form [(existT A P t p)] 
+(** Match a term of the form [(existT A P t p)]
    Returns associated lemmas and [A,P,t,p] *)
 val find_sigma_data_decompose : Environ.env -> evar_map -> constr ->
   coq_sigma_data * (EInstance.t * constr * constr * constr * constr)
