@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -15,20 +15,18 @@ open Names
 open Termops
 open Environ
 open Constr
+open Context
 open EConstr
 open Vars
 open Namegen
 open Evd
 open Printer
 open Reductionops
-open Entries
 open Inductiveops
 open Tacmach.New
 open Clenv
-open Declare
 open Tacticals.New
 open Tactics
-open Decl_kinds
 open Context.Named.Declaration
 
 module NamedDecl = Context.Named.Declaration
@@ -97,7 +95,7 @@ let get_local_sign sign =
   in
   List.fold_right add_local lid nil_sign
 *)
-(* returs the identifier of lid that was the latest declared in sign.
+(* returns the identifier of lid that was the latest declared in sign.
  * (i.e. is the identifier id of lid such that
  * sign_length (sign_prefix id sign) > sign_length (sign_prefix id' sign) >
  * for any id'<>id in lid).
@@ -107,11 +105,11 @@ let max_prefix_sign lid sign =
   let rec max_rec (resid,prefix) = function
     | [] -> (resid,prefix)
     | (id::l) ->
-	let pre = sign_prefix id sign in
-	if sign_length pre > sign_length prefix then
+        let pre = sign_prefix id sign in
+        if sign_length pre > sign_length prefix then
           max_rec (id,pre) l
         else
-	  max_rec (resid,prefix) l
+          max_rec (resid,prefix) l
   in
   match lid with
     | [] -> nil_sign
@@ -120,13 +118,13 @@ let max_prefix_sign lid sign =
 let rec add_prods_sign env sigma t =
   match EConstr.kind sigma (whd_all env sigma t) with
     | Prod (na,c1,b) ->
-	let id = id_of_name_using_hdchar env sigma t na in
-	let b'= subst1 (mkVar id) b in
-        add_prods_sign (push_named (LocalAssum (id,c1)) env) sigma b'
+        let id = id_of_name_using_hdchar env sigma t na.binder_name in
+        let b'= subst1 (mkVar id) b in
+        add_prods_sign (push_named (LocalAssum ({na with binder_name=id},c1)) env) sigma b'
     | LetIn (na,c1,t1,b) ->
-	let id = id_of_name_using_hdchar env sigma t na in
-	let b'= subst1 (mkVar id) b in
-        add_prods_sign (push_named (LocalDef (id,c1,t1)) env) sigma b'
+        let id = id_of_name_using_hdchar env sigma t na.binder_name in
+        let b'= subst1 (mkVar id) b in
+        add_prods_sign (push_named (LocalDef ({na with binder_name=id},c1,t1)) env) sigma b'
     | _ -> (env,t)
 
 (* [dep_option] indicates whether the inversion lemma is dependent or not.
@@ -149,31 +147,32 @@ let compute_first_inversion_scheme env sigma ind sort dep_option =
   let pty,goal =
     if dep_option  then
       let pty = make_arity env sigma true indf sort in
+      let r = relevance_of_inductive_type env ind in
       let goal =
-	mkProd
-	  (Anonymous, mkAppliedInd ind, applist(mkVar p,realargs@[mkRel 1]))
+        mkProd
+          (make_annot Anonymous r, mkAppliedInd ind, applist(mkVar p,realargs@[mkRel 1]))
       in
       pty,goal
     else
       let i = mkAppliedInd ind in
       let ivars = global_vars env sigma i in
       let revargs,ownsign =
-	fold_named_context
-	  (fun env d (revargs,hyps) ->
+        fold_named_context
+          (fun env d (revargs,hyps) ->
             let d = map_named_decl EConstr.of_constr d in
              let id = NamedDecl.get_id d in
              if Id.List.mem id ivars then
-	       ((mkVar id)::revargs, Context.Named.add d hyps)
-	     else
-	       (revargs,hyps))
+               ((mkVar id)::revargs, Context.Named.add d hyps)
+             else
+               (revargs,hyps))
           env ~init:([],[])
       in
       let pty = it_mkNamedProd_or_LetIn (mkSort sort) ownsign in
-      let goal = mkArrow i (applist(mkVar p, List.rev revargs)) in
+      let goal = mkArrow i Sorts.Relevant (applist(mkVar p, List.rev revargs)) in
       (pty,goal)
   in
   let npty = nf_all env sigma pty in
-  let extenv = push_named (LocalAssum (p,npty)) env in
+  let extenv = push_named (LocalAssum (make_annot p Sorts.Relevant,npty)) env in
   extenv, goal
 
 (* [inversion_scheme sign I]
@@ -183,7 +182,7 @@ let compute_first_inversion_scheme env sigma ind sort dep_option =
    scheme on sort [sort]. Depending on the value of [dep_option] it will
    build a dependent lemma or a non-dependent one *)
 
-let inversion_scheme env sigma t sort dep_option inv_op =
+let inversion_scheme ~name ~poly env sigma t sort dep_option inv_op =
   let (env,i) = add_prods_sign env sigma t in
   let ind =
     try find_rectype env sigma i
@@ -201,11 +200,8 @@ let inversion_scheme env sigma t sort dep_option inv_op =
     user_err ~hdr:"lemma_inversion"
     (str"Computed inversion goal was not closed in initial signature.");
   *)
-  let pf = Proof.start (Evd.from_ctx (evar_universe_context sigma)) [invEnv,invGoal] in
-  let pf =
-    fst (Proof.run_tactic env (
-      tclTHEN intro (onLastHypId inv_op)) pf)
-  in
+  let pf = Proof.start ~name ~poly (Evd.from_ctx (evar_universe_context sigma)) [invEnv,invGoal] in
+  let pf, _, () = Proof.run_tactic env (tclTHEN intro (onLastHypId inv_op)) pf in
   let pfterm = List.hd (Proof.partial_proof pf) in
   let global_named_context = Global.named_context_val () in
   let ownSign = ref begin
@@ -213,20 +209,20 @@ let inversion_scheme env sigma t sort dep_option inv_op =
       (fun env d sign ->
         let d = map_named_decl EConstr.of_constr d in
          if mem_named_context_val (NamedDecl.get_id d) global_named_context then sign
-	 else Context.Named.add d sign)
+         else Context.Named.add d sign)
       invEnv ~init:Context.Named.empty
   end in
   let avoid = ref Id.Set.empty in
-  let _,_,_,_,sigma = Proof.proof pf in
+  let Proof.{sigma} = Proof.data pf in
   let sigma = Evd.minimize_universes sigma in
   let rec fill_holes c =
     match EConstr.kind sigma c with
     | Evar (e,args) ->
-	let h = next_ident_away (Id.of_string "H") !avoid in
-	let ty,inst = Evarutil.generalize_evar_over_rels sigma (e,args) in
-	avoid := Id.Set.add h !avoid;
-	ownSign := Context.Named.add (LocalAssum (h,ty)) !ownSign;
-	applist (mkVar h, inst)
+        let h = next_ident_away (Id.of_string "H") !avoid in
+        let ty,inst = Evarutil.generalize_evar_over_rels sigma (e,args) in
+        avoid := Id.Set.add h !avoid;
+        ownSign := Context.Named.add (LocalAssum (make_annot h Sorts.Relevant,ty)) !ownSign;
+        applist (mkVar h, inst)
     | _ -> EConstr.map sigma fill_holes c
   in
   let c = fill_holes pfterm in
@@ -236,12 +232,10 @@ let inversion_scheme env sigma t sort dep_option inv_op =
   p, sigma
 
 let add_inversion_lemma ~poly name env sigma t sort dep inv_op =
-  let invProof, sigma = inversion_scheme env sigma t sort dep inv_op in
-  let univs =
-    Evd.const_univ_entry ~poly sigma
-  in
-  let entry = definition_entry ~univs invProof in
-  let _ = declare_constant name (DefinitionEntry entry, IsProof Lemma) in
+  let invProof, sigma = inversion_scheme ~name ~poly env sigma t sort dep inv_op in
+  let univs = Evd.univ_entry ~poly sigma in
+  let entry = Declare.definition_entry ~univs invProof in
+  let _ : Names.Constant.t = Declare.declare_constant ~name ~kind:Decls.(IsProof Lemma) (Declare.DefinitionEntry entry) in
   ()
 
 (* inv_op = Inv (derives de complete inv. lemma)
@@ -250,13 +244,13 @@ let add_inversion_lemma ~poly name env sigma t sort dep inv_op =
 let add_inversion_lemma_exn ~poly na com comsort bool tac =
   let env = Global.env () in
   let sigma = Evd.from_env env in
-  let sigma, c = Constrintern.interp_type_evars env sigma com in
+  let sigma, c = Constrintern.interp_type_evars ~program_mode:false env sigma com in
   let sigma, sort = Evd.fresh_sort_in_family ~rigid:univ_rigid sigma comsort in
   try
     add_inversion_lemma ~poly na env sigma c sort bool tac
   with
     |   UserError (Some "Case analysis",s) -> (* Reference to Indrec *)
-	  user_err ~hdr:"Inv needs Nodep Prop Set" s
+          user_err ~hdr:"Inv needs Nodep Prop Set" s
 
 (* ================================= *)
 (* Applying a given inversion lemma  *)
@@ -270,12 +264,12 @@ let lemInv id c =
     Clenvtac.res_pf clause ~flags:(Unification.elim_flags ()) ~with_evars:false
   with
     | NoSuchBinding ->
-	user_err 
-	  (hov 0 (pr_econstr_env (pf_env gls) (project gls) c ++ spc () ++ str "does not refer to an inversion lemma."))
+        user_err
+          (hov 0 (pr_econstr_env (pf_env gls) (project gls) c ++ spc () ++ str "does not refer to an inversion lemma."))
     | UserError (a,b) ->
-	 user_err ~hdr:"LemInv"
-	   (str "Cannot refine current goal with the lemma " ++
-	      pr_leconstr_env (pf_env gls) (project gls) c)
+         user_err ~hdr:"LemInv"
+           (str "Cannot refine current goal with the lemma " ++
+              pr_leconstr_env (pf_env gls) (project gls) c)
   end
 
 let lemInv_gen id c = try_intros_until (fun id -> lemInv id c) id
