@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -20,12 +20,15 @@ open Tacmach
 open Clenv
 open Tactypes
 
+module RelDecl = Context.Rel.Declaration
 module NamedDecl = Context.Named.Declaration
 
 (************************************************************************)
 (* Tacticals re-exported from the Refiner module                        *)
 (************************************************************************)
 
+type tactic = Proofview.V82.tac
+
 let tclIDTAC         = Refiner.tclIDTAC
 let tclIDTAC_MESSAGE = Refiner.tclIDTAC_MESSAGE
 let tclORELSE0       = Refiner.tclORELSE0
@@ -40,12 +43,8 @@ let tclTHENS         = Refiner.tclTHENS
 let tclTHENSV        = Refiner.tclTHENSV
 let tclTHENSFIRSTn   = Refiner.tclTHENSFIRSTn
 let tclTHENSLASTn    = Refiner.tclTHENSLASTn
-let tclTHENFIRSTn    = Refiner.tclTHENFIRSTn
-let tclTHENLASTn     = Refiner.tclTHENLASTn
 let tclREPEAT        = Refiner.tclREPEAT
-let tclREPEAT_MAIN   = Refiner.tclREPEAT_MAIN
 let tclFIRST         = Refiner.tclFIRST
-let tclSOLVE         = Refiner.tclSOLVE
 let tclTRY           = Refiner.tclTRY
 let tclCOMPLETE      = Refiner.tclCOMPLETE
 let tclAT_LEAST_ONCE = Refiner.tclAT_LEAST_ONCE
@@ -55,14 +54,6 @@ let tclDO            = Refiner.tclDO
 let tclPROGRESS      = Refiner.tclPROGRESS
 let tclSHOWHYPS      = Refiner.tclSHOWHYPS
 let tclTHENTRY       = Refiner.tclTHENTRY
-let tclIFTHENELSE    = Refiner.tclIFTHENELSE
-let tclIFTHENSELSE   = Refiner.tclIFTHENSELSE
-let tclIFTHENSVELSE   = Refiner.tclIFTHENSVELSE
-let tclIFTHENTRYELSEMUST = Refiner.tclIFTHENTRYELSEMUST
-
-(* Synonyms *)
-
-let tclTHENSEQ       = tclTHENLIST
 
 (************************************************************************)
 (* Tacticals applying on hypotheses                                     *)
@@ -175,7 +166,7 @@ let check_or_and_pattern_size ?loc check_and names branchsigns =
     user_err ?loc  (str "Expects " ++ msg p1 p2 ++ str ".") in
   let errn n =
     user_err ?loc  (str "Expects a disjunctive pattern with " ++ int n
-	++ str " branches.") in
+        ++ str " branches.") in
   let err1' p1 p2 =
     user_err ?loc  (strbrk "Expects a disjunctive pattern with 1 branch or " ++ msg p1 p2 ++ str ".") in
   let errforthcoming ?loc =
@@ -225,8 +216,8 @@ let compute_induction_names = compute_induction_names_gen true
 (* Compute the let-in signature of case analysis or standard induction scheme *)
 let compute_constructor_signatures ~rec_flag ((_,k as ity),u) =
   let rec analrec c recargs =
-    match Constr.kind c, recargs with
-    | Prod (_,_,c), recarg::rest ->
+    match c, recargs with
+    | RelDecl.LocalAssum _ :: c, recarg::rest ->
         let rest = analrec c rest in
         begin match Declareops.dest_recarg recarg with
         | Norec | Imbr _  -> true :: rest
@@ -234,14 +225,13 @@ let compute_constructor_signatures ~rec_flag ((_,k as ity),u) =
             if rec_flag && Int.equal j k then true :: true :: rest
             else true :: rest
         end
-    | LetIn (_,_,_,c), rest -> false :: analrec c rest
-    | _, [] -> []
+    | RelDecl.LocalDef _ :: c, rest -> false :: analrec c rest
+    | [], [] -> []
     | _ -> anomaly (Pp.str "compute_constructor_signatures.")
   in
   let (mib,mip) = Global.lookup_inductive ity in
-  let n = mib.mind_nparams in
-  let lc =
-    Array.map (fun c -> snd (Term.decompose_prod_n_assum n c)) mip.mind_nf_lc in
+  let map (ctx, _) = List.skipn (Context.Rel.length mib.mind_params_ctxt) (List.rev ctx) in
+  let lc = Array.map map mip.mind_nf_lc in
   let lrecargs = Declareops.dest_subterms mip.mind_recargs in
   Array.map2 analrec lc lrecargs
 
@@ -256,7 +246,7 @@ let elimination_sort_of_clause = function
   | Some id -> elimination_sort_of_hyp id
 
 
-let pf_with_evars glsev k gls = 
+let pf_with_evars glsev k gls =
   let evd, a = glsev gls in
     tclTHEN (Refiner.tclEVARS evd) (k a) gls
 
@@ -421,7 +411,7 @@ module New = struct
 
   let tclTRY t =
     tclORELSE0 t (tclUNIT ())
-  
+
   let tclTRYb t =
     tclORELSE0L (t <*> tclUNIT true) (tclUNIT false)
 
@@ -483,7 +473,7 @@ module New = struct
       ) <*>
         tclUNIT res
 
-  (* Try the first thats solves the current goal *)
+  (* Try the first that solves the current goal *)
   let tclSOLVE tacl = tclFIRST (List.map tclCOMPLETE tacl)
 
   let tclPROGRESS t =
@@ -508,7 +498,7 @@ module New = struct
       | Evd.Evar_empty -> Some (evk,evi)
       | Evd.Evar_defined c -> match Constr.kind (EConstr.Unsafe.to_constr c) with
         | Evar (evk,l) -> is_undefined_up_to_restriction sigma evk
-        | _ -> 
+        | _ ->
           (* We make the assumption that there is no way to refine an
             evar remaining after typing from the initial term given to
             apply/elim and co tactics, is it correct? *)
@@ -556,7 +546,8 @@ module New = struct
     Proofview.tclOR
       (Proofview.tclTIMEOUT n t)
       begin function (e, info) -> match e with
-        | Proofview.Timeout as e -> Proofview.tclZERO (Refiner.FailError (0,lazy (CErrors.print e)))
+        | Logic_monad.Tac_Timeout as e ->
+          Proofview.tclZERO (Refiner.FailError (0,lazy (CErrors.print e)))
         | e -> Proofview.tclZERO ~info e
       end
 
@@ -574,13 +565,13 @@ module New = struct
     with Failure _ -> CErrors.user_err Pp.(str "Not enough hypotheses in the goal.")
 
   let nthHypId m gl =
-    (** We only use [id] *)
+    (* We only use [id] *)
     nthDecl m gl |> NamedDecl.get_id
-  let nthHyp m gl = 
+  let nthHyp m gl =
     mkVar (nthHypId m gl)
 
   let onNthHypId m tac =
-    Proofview.Goal.enter begin fun gl -> tac (nthHypId m gl) end 
+    Proofview.Goal.enter begin fun gl -> tac (nthHypId m gl) end
   let onNthHyp m tac =
     Proofview.Goal.enter begin fun gl -> tac (nthHyp m gl) end
 
@@ -653,12 +644,12 @@ module New = struct
       match EConstr.kind elimclause.evd p with
       | Meta p -> p
       | _ ->
-	  let name_elim =
-	    match EConstr.kind sigma elim with
+          let name_elim =
+            match EConstr.kind sigma elim with
             | Const _ | Var _ -> str " " ++ Printer.pr_econstr_env (pf_env gl) sigma elim
             | _ -> mt ()
-	  in
-	  user_err ~hdr:"Tacticals.general_elim_then_using"
+          in
+          user_err ~hdr:"Tacticals.general_elim_then_using"
             (str "The elimination combinator " ++ name_elim ++ str " is unknown.")
     in
     let elimclause' = clenv_fchain ~with_univs:false indmv elimclause indclause in
@@ -690,12 +681,12 @@ module New = struct
     end) end
 
   let elimination_sort_of_goal gl =
-    (** Retyping will expand evars anyway. *)
+    (* Retyping will expand evars anyway. *)
     let c = Proofview.Goal.concl gl in
     pf_apply Retyping.get_sort_family_of gl c
 
   let elimination_sort_of_hyp id gl =
-    (** Retyping will expand evars anyway. *)
+    (* Retyping will expand evars anyway. *)
     let c = pf_get_hyp_typ id gl in
     pf_apply Retyping.get_sort_family_of gl c
 
@@ -706,7 +697,8 @@ module New = struct
   (* computing the case/elim combinators *)
 
   let gl_make_elim ind = begin fun gl ->
-    let gr = Indrec.lookup_eliminator (fst ind) (elimination_sort_of_goal gl) in
+    let env = Proofview.Goal.env gl in
+    let gr = Indrec.lookup_eliminator env (fst ind) (elimination_sort_of_goal gl) in
     let (sigma, c) = pf_apply Evd.fresh_global gl gr in
     (sigma, c)
   end
@@ -741,7 +733,7 @@ module New = struct
     tac branches
     end
 
-  let case_on_ba tac ba = 
+  let case_on_ba tac ba =
     Proofview.Goal.enter begin fun gl ->
     let branches = make_elim_branch_assumptions ba (Proofview.Goal.hyps gl) in
     tac branches
