@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -44,6 +44,8 @@ struct
     | DCase   of case_info * 't * 't * 't array
     | DFix    of int array * int * 't array * 't array
     | DCoFix  of int * 't array * 't array
+    | DInt    of Uint63.t
+    | DFloat  of Float64.t
 
     (* special constructors only inside the left-hand side of DCtx or
        DApp. Used to encode lists of foralls/letins/apps as contexts *)
@@ -61,9 +63,11 @@ struct
     | DCase (_,t1,t2,ta) -> str "case"
     | DFix _ -> str "fix"
     | DCoFix _ -> str "cofix"
+    | DInt _ -> str "INT"
+    | DFloat _ -> str "FLOAT"
     | DCons ((t,dopt),tl) -> f t ++ (match dopt with
-	  Some t' -> str ":=" ++ f t'
-	| None -> str "") ++ spc() ++ str "::" ++ spc() ++ f tl
+          Some t' -> str ":=" ++ f t'
+        | None -> str "") ++ spc() ++ str "::" ++ spc() ++ f tl
     | DNil -> str "[]"
 
   (*
@@ -72,15 +76,15 @@ struct
    *)
 
   let map f = function
-    | (DRel | DSort | DNil | DRef _) as c -> c
+    | (DRel | DSort | DNil | DRef _ | DInt _ | DFloat _) as c -> c
     | DCtx (ctx,c) -> DCtx (f ctx, f c)
     | DLambda (t,c) -> DLambda (f t, f c)
     | DApp (t,u) -> DApp (f t,f u)
     | DCase (ci,p,c,bl) -> DCase (ci, f p, f c, Array.map f bl)
     | DFix (ia,i,ta,ca) ->
-	DFix (ia,i,Array.map f ta,Array.map f ca)
+        DFix (ia,i,Array.map f ta,Array.map f ca)
     | DCoFix(i,ta,ca) ->
-	DCoFix (i,Array.map f ta,Array.map f ca)
+        DCoFix (i,Array.map f ta,Array.map f ca)
     | DCons ((t,topt),u) -> DCons ((f t,Option.map f topt), f u)
 
   let compare_ci ci1 ci2 =
@@ -100,7 +104,7 @@ struct
   | DRel, _ -> -1 | _, DRel -> 1
   | DSort, DSort -> 0
   | DSort, _ -> -1 | _, DSort -> 1
-  | DRef gr1, DRef gr2 -> RefOrdered.compare gr1 gr2
+  | DRef gr1, DRef gr2 -> GlobRef.Ordered.compare gr1 gr2
   | DRef _, _ -> -1 | _, DRef _ -> 1
 
   | DCtx (tl1, tr1), DCtx (tl2, tr2)
@@ -145,6 +149,14 @@ struct
     else c
   | DCoFix _, _ -> -1 | _, DCoFix _ -> 1
 
+  | DInt i1, DInt i2 -> Uint63.compare i1 i2
+
+  | DInt _, _ -> -1 | _, DInt _ -> 1
+
+  | DFloat f1, DFloat f2 -> Float64.total_compare f1 f2
+
+  | DFloat _, _ -> -1 | _, DFloat _ -> 1
+
   | DCons ((t1, ot1), u1), DCons ((t2, ot2), u2) ->
      let c = cmp t1 t2 in
      if Int.equal c 0 then
@@ -157,19 +169,19 @@ struct
   | DNil, DNil -> 0
 
   let fold f acc = function
-    | (DRel | DNil | DSort | DRef _) -> acc
+    | (DRel | DNil | DSort | DRef _ | DInt _ | DFloat _) -> acc
     | DCtx (ctx,c) -> f (f acc ctx) c
     | DLambda (t,c) -> f (f acc t) c
     | DApp (t,u) -> f (f acc t) u
     | DCase (ci,p,c,bl) -> Array.fold_left f (f (f acc p) c) bl
     | DFix (ia,i,ta,ca) ->
-	Array.fold_left f (Array.fold_left f acc ta) ca
+        Array.fold_left f (Array.fold_left f acc ta) ca
     | DCoFix(i,ta,ca) ->
-	Array.fold_left f (Array.fold_left f acc ta) ca
+        Array.fold_left f (Array.fold_left f acc ta) ca
     | DCons ((t,topt),u) -> f (Option.fold_left f (f acc t) topt) u
 
   let choose f = function
-    | (DRel | DSort | DNil | DRef _) -> invalid_arg "choose"
+    | (DRel | DSort | DNil | DRef _ | DInt _ | DFloat _) -> invalid_arg "choose"
     | DCtx (ctx,c) -> f ctx
     | DLambda (t,c) -> f t
     | DApp (t,u) -> f u
@@ -185,44 +197,46 @@ struct
     if not (Int.equal (compare dummy_cmp (head c1) (head c2)) 0)
     then invalid_arg "fold2:compare" else
       match c1,c2 with
-	| (DRel, DRel | DNil, DNil | DSort, DSort | DRef _, DRef _) -> acc
-	| (DCtx (c1,t1), DCtx (c2,t2)
-	  | DApp (c1,t1), DApp (c2,t2)
-	  | DLambda (c1,t1), DLambda (c2,t2)) -> f (f acc c1 c2) t1 t2
-	| DCase (ci,p1,c1,bl1),DCase (_,p2,c2,bl2) ->
-	    Array.fold_left2 f (f (f acc p1 p2) c1 c2) bl1 bl2
-	| DFix (ia,i,ta1,ca1), DFix (_,_,ta2,ca2) ->
-	    Array.fold_left2 f (Array.fold_left2 f acc ta1 ta2) ca1 ca2
-	| DCoFix(i,ta1,ca1), DCoFix(_,ta2,ca2) ->
-	    Array.fold_left2 f (Array.fold_left2 f acc ta1 ta2) ca1 ca2
-	| DCons ((t1,topt1),u1), DCons ((t2,topt2),u2) ->
-	    f (Option.fold_left2 f (f acc t1 t2) topt1 topt2) u1 u2
+        | (DRel, DRel | DNil, DNil | DSort, DSort | DRef _, DRef _
+           | DInt _, DInt _ | DFloat _, DFloat _) -> acc
+        | (DCtx (c1,t1), DCtx (c2,t2)
+          | DApp (c1,t1), DApp (c2,t2)
+          | DLambda (c1,t1), DLambda (c2,t2)) -> f (f acc c1 c2) t1 t2
+        | DCase (ci,p1,c1,bl1),DCase (_,p2,c2,bl2) ->
+            Array.fold_left2 f (f (f acc p1 p2) c1 c2) bl1 bl2
+        | DFix (ia,i,ta1,ca1), DFix (_,_,ta2,ca2) ->
+            Array.fold_left2 f (Array.fold_left2 f acc ta1 ta2) ca1 ca2
+        | DCoFix(i,ta1,ca1), DCoFix(_,ta2,ca2) ->
+            Array.fold_left2 f (Array.fold_left2 f acc ta1 ta2) ca1 ca2
+        | DCons ((t1,topt1),u1), DCons ((t2,topt2),u2) ->
+            f (Option.fold_left2 f (f acc t1 t2) topt1 topt2) u1 u2
         | (DRel | DNil | DSort | DRef _ | DCtx _ | DApp _ | DLambda _ | DCase _
-           | DFix _ | DCoFix _ | DCons _), _ -> assert false
+           | DFix _ | DCoFix _ | DCons _ | DInt _ | DFloat _), _ -> assert false
 
   let map2 (f:'a -> 'b -> 'c) (c1:'a t) (c2:'b t) : 'c t =
     let head w = map (fun _ -> ()) w in
     if not (Int.equal (compare dummy_cmp (head c1) (head c2)) 0)
     then invalid_arg "map2_t:compare" else
       match c1,c2 with
-	| (DRel, DRel | DSort, DSort | DNil, DNil | DRef _, DRef _) as cc ->
-	    let (c,_) = cc in c
-	| DCtx (c1,t1), DCtx (c2,t2) -> DCtx (f c1 c2, f t1 t2)
-	| DLambda (t1,c1), DLambda (t2,c2) -> DLambda (f t1 t2, f c1 c2)
-	| DApp (t1,u1), DApp (t2,u2) -> DApp (f t1 t2,f u1 u2)
-	| DCase (ci,p1,c1,bl1), DCase (_,p2,c2,bl2) ->
-	    DCase (ci, f p1 p2, f c1 c2, Array.map2 f bl1 bl2)
-	| DFix (ia,i,ta1,ca1), DFix (_,_,ta2,ca2) ->
-	    DFix (ia,i,Array.map2 f ta1 ta2,Array.map2 f ca1 ca2)
-	| DCoFix (i,ta1,ca1), DCoFix (_,ta2,ca2) ->
-	    DCoFix (i,Array.map2 f ta1 ta2,Array.map2 f ca1 ca2)
-	| DCons ((t1,topt1),u1), DCons ((t2,topt2),u2) ->
-	    DCons ((f t1 t2,Option.lift2 f topt1 topt2), f u1 u2)
+        | (DRel, DRel | DSort, DSort | DNil, DNil | DRef _, DRef _
+           | DInt _, DInt _ | DFloat _, DFloat _) as cc ->
+            let (c,_) = cc in c
+        | DCtx (c1,t1), DCtx (c2,t2) -> DCtx (f c1 c2, f t1 t2)
+        | DLambda (t1,c1), DLambda (t2,c2) -> DLambda (f t1 t2, f c1 c2)
+        | DApp (t1,u1), DApp (t2,u2) -> DApp (f t1 t2,f u1 u2)
+        | DCase (ci,p1,c1,bl1), DCase (_,p2,c2,bl2) ->
+            DCase (ci, f p1 p2, f c1 c2, Array.map2 f bl1 bl2)
+        | DFix (ia,i,ta1,ca1), DFix (_,_,ta2,ca2) ->
+            DFix (ia,i,Array.map2 f ta1 ta2,Array.map2 f ca1 ca2)
+        | DCoFix (i,ta1,ca1), DCoFix (_,ta2,ca2) ->
+            DCoFix (i,Array.map2 f ta1 ta2,Array.map2 f ca1 ca2)
+        | DCons ((t1,topt1),u1), DCons ((t2,topt2),u2) ->
+            DCons ((f t1 t2,Option.lift2 f topt1 topt2), f u1 u2)
         | (DRel | DNil | DSort | DRef _ | DCtx _ | DApp _ | DLambda _ | DCase _
-           | DFix _ | DCoFix _ | DCons _), _ -> assert false
+           | DFix _ | DCoFix _ | DCons _ | DInt _ | DFloat _), _ -> assert false
 
   let terminal = function
-    | (DRel | DSort | DNil | DRef _) -> true
+    | (DRel | DSort | DNil | DRef _ | DInt _ | DFloat _) -> true
     | DLambda _ | DApp _ | DCase _ | DFix _ | DCoFix _ | DCtx _ | DCons _ ->
       false
 
@@ -260,8 +274,8 @@ module Make =
 struct
 
   module TDnet : Dnet.S with type ident=Ident.t
-			and  type 'a structure = 'a DTerm.t
-			and  type meta = int
+                        and  type 'a structure = 'a DTerm.t
+                        and  type meta = int
     = Dnet.Make(DTerm)(Ident)(Int)
 
   type t = TDnet.t
@@ -281,7 +295,8 @@ struct
   open TDnet
 
   let pat_of_constr c : term_pattern =
-    (** To each evar we associate a unique identifier. *)
+    let open GlobRef in
+    (* To each evar we associate a unique identifier. *)
     let metas = ref Evar.Map.empty in
     let rec pat_of_constr c = match Constr.kind c with
     | Rel _          -> Term DRel
@@ -308,13 +323,15 @@ struct
       Term(DCoFix(i,Array.map pat_of_constr ta,Array.map pat_of_constr ca))
     | Cast (c,_,_)   -> pat_of_constr c
     | Lambda (_,t,c) -> Term(DLambda (pat_of_constr t, pat_of_constr c))
-    | (Prod (_,_,_) | LetIn(_,_,_,_))   ->
+    | (Prod _ | LetIn _)   ->
       let (ctx,c) = ctx_of_constr (Term DNil) c in Term (DCtx (ctx,c))
     | App (f,ca)     ->
       Array.fold_left (fun c a -> Term (DApp (c,a)))
         (pat_of_constr f) (Array.map pat_of_constr ca)
-    | Proj (p,c) -> 
+    | Proj (p,c) ->
         Term (DApp (Term (DRef (ConstRef (Projection.constant p))), pat_of_constr c))
+    | Int i -> Term (DInt i)
+    | Float f -> Term (DFloat f)
 
     and ctx_of_constr ctx c = match Constr.kind c with
     | Prod (_,t,c)   -> ctx_of_constr (Term(DCons((pat_of_constr t,None),ctx))) c
@@ -375,16 +392,16 @@ struct
     let dpat = under_prod (empty_ctx dpat) in
     TDnet.Idset.fold
       (fun id acc ->
-	 let c_id = Opt.reduce (Ident.constr_of id) in
-	 let c_id = EConstr.of_constr c_id in
-	 let (ctx,wc) =
-	   try Termops.align_prod_letin Evd.empty whole_c c_id (** FIXME *)
-	   with Invalid_argument _ -> [],c_id in
-	 let wc,whole_c = if Opt.direction then whole_c,wc else wc,whole_c in
-	 try
+         let c_id = Opt.reduce (Ident.constr_of id) in
+         let c_id = EConstr.of_constr c_id in
+         let (ctx,wc) =
+           try Termops.align_prod_letin Evd.empty whole_c c_id (* FIXME *)
+           with Invalid_argument _ -> [],c_id in
+         let wc,whole_c = if Opt.direction then whole_c,wc else wc,whole_c in
+         try
           let _ = Termops.filtering Evd.empty ctx Reduction.CUMUL wc whole_c in
           id :: acc
-	 with Termops.CannotFilter -> (* msgnl(str"recon "++Termops.print_constr_env (Global.env()) wc); *) acc
+         with Termops.CannotFilter -> (* msgnl(str"recon "++Termops.print_constr_env (Global.env()) wc); *) acc
       ) (TDnet.find_match dpat dn) []
 
   (*
