@@ -11,7 +11,7 @@ mkdir -p theories/sub
 
 cp ../../template/theories/sub/testsub.v theories/sub
 cp ../../template/theories/test.v theories
-cp ../../template/src/test.ml4 src
+cp ../../template/src/test.mlg src
 cp ../../template/src/test_aux.mli src
 cp ../../template/src/test.mli src
 cp ../../template/src/test_plugin.mlpack src
