@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -34,7 +34,7 @@ let rec print_prefix_list sep = function
 let usage_coq_makefile () =
   output_string stderr "Usage summary:\
 \n\
-\ncoq_makefile .... [file.v] ... [file.ml[i4]?] ... [file.ml{lib,pack}]\
+\ncoq_makefile .... [file.v] ... [file.ml[ig]?] ... [file.ml{lib,pack}]\
 \n  ... [any] ... [-extra[-phony] result dependencies command]\
 \n  ... [-I dir] ... [-R physicalpath logicalpath]\
 \n  ... [-Q physicalpath logicalpath] ... [VARIABLE = value]\
@@ -45,8 +45,8 @@ let usage_coq_makefile () =
 \nFull list of options:\
 \n\
 \n[file.v]: Coq file to be compiled\
-\n[file.ml[i4]?]: Objective Caml file to be compiled\
-\n[file.ml{lib,pack}]: ocamlbuild file that describes a Objective Caml\
+\n[file.ml[ig]?]: Objective Caml file to be compiled\
+\n[file.ml{lib,pack}]: ocamlbuild-style file that describes a Objective Caml\
 \n  library/module\
 \n[any] : subdirectory that should be \"made\" and has a Makefile itself\
 \n  to do so. Very fragile and discouraged.\
@@ -122,16 +122,18 @@ let read_whole_file s =
 
 let quote s = if String.contains s ' ' || CString.is_empty s then "'" ^ s ^ "'" else s
 
-let generate_makefile oc conf_file local_file args project =
+let generate_makefile oc conf_file local_file dep_file args project =
+  let coqlib = Envars.coqlib () in
   let makefile_template =
-    let template = "/tools/CoqMakefile.in" in
-    Envars.coqlib () ^ template in
+    let template = Filename.concat "tools" "CoqMakefile.in" in
+    Filename.concat coqlib template in
   let s = read_whole_file makefile_template in
   let s = List.fold_left
     (* We use global_substitute to avoid running into backslash issues due to \1 etc. *)
     (fun s (k,v) -> Str.global_substitute (Str.regexp_string k) (fun _ -> v) s) s
     [ "@CONF_FILE@", conf_file;
       "@LOCAL_FILE@", local_file;
+      "@DEP_FILE@", dep_file;
       "@COQ_VERSION@", Coq_config.version;
       "@PROJECT_FILE@", (Option.default "" project.project_file);
       "@COQ_MAKEFILE_INVOCATION@",String.concat " " (List.map quote args);
@@ -179,7 +181,7 @@ let generate_conf_subdirs oc sds =
   iter (fprintf oc "clean::\n\tcd \"%s\" && $(MAKE) clean\n") sds;
   iter (fprintf oc "archclean::\n\tcd \"%s\" && $(MAKE) archclean\n") sds;
   iter (fprintf oc "install-extra::\n\tcd \"%s\" && $(MAKE) install\n") sds
-   
+
 
 let generate_conf_includes oc { ml_includes; r_includes; q_includes } =
   section oc "Path directives (-I, -R, -Q).";
@@ -218,7 +220,7 @@ let generate_conf_coq_config oc =
 ;;
 
 let generate_conf_files oc
-  { v_files; mli_files; ml4_files; ml_files; mllib_files; mlpack_files; }
+  { v_files; mli_files; mlg_files; ml_files; mllib_files; mlpack_files; }
 =
   let module S = String in
   let map = map_sourced_list in
@@ -226,7 +228,7 @@ let generate_conf_files oc
   fprintf oc "COQMF_VFILES = %s\n"      (S.concat " " (map quote v_files));
   fprintf oc "COQMF_MLIFILES = %s\n"    (S.concat " " (map quote mli_files));
   fprintf oc "COQMF_MLFILES = %s\n"     (S.concat " " (map quote ml_files));
-  fprintf oc "COQMF_ML4FILES = %s\n"    (S.concat " " (map quote ml4_files));
+  fprintf oc "COQMF_MLGFILES = %s\n"    (S.concat " " (map quote mlg_files));
   fprintf oc "COQMF_MLPACKFILES = %s\n" (S.concat " " (map quote mlpack_files));
   fprintf oc "COQMF_MLLIBFILES = %s\n"  (S.concat " " (map quote mllib_files));
   let cmdline_vfiles = filter_cmdline v_files in
@@ -248,7 +250,7 @@ let rec logic_gcd acc = function
 
 let generate_conf_doc oc { defs; q_includes; r_includes } =
   let includes = List.map (forget_source > snd) (q_includes @ r_includes) in
-  let logpaths = List.map (CString.split '.') includes in
+  let logpaths = List.map (String.split_on_char '.') includes in
   let gcd = logic_gcd [] logpaths in
   let root =
     if gcd = [] then
@@ -260,7 +262,7 @@ let generate_conf_doc oc { defs; q_includes; r_includes } =
          eprintf "Warning: in %s\n" destination;
          destination
       end else "$(INSTALLDEFAULTROOT)"
-    else String.concat "/" gcd in
+    else String.concat Filename.dir_sep gcd in
   Printf.fprintf oc "COQMF_INSTALLCOQDOCROOT = %s\n" (quote root)
 
 let generate_conf_defs oc { defs; extra_args } =
@@ -283,7 +285,7 @@ let generate_conf oc project args  =
 
 let ensure_root_dir
   ({ ml_includes; r_includes; q_includes;
-     v_files; ml_files; mli_files; ml4_files;
+     v_files; ml_files; mli_files; mlg_files;
      mllib_files; mlpack_files } as project)
   =
   let exists f = List.exists (forget_source > f) in
@@ -293,8 +295,8 @@ let ensure_root_dir
   || exists (fun ({ canonical_path = x },_) -> is_prefix x here) r_includes
   || exists (fun ({ canonical_path = x },_) -> is_prefix x here) q_includes
   || (not_tops v_files &&
-      not_tops mli_files && not_tops ml4_files && not_tops ml_files &&
-      not_tops mllib_files && not_tops mlpack_files)
+      not_tops mli_files && not_tops mlg_files &&
+      not_tops ml_files && not_tops mllib_files && not_tops mlpack_files)
   then
     project
   else
@@ -305,7 +307,7 @@ let ensure_root_dir
         r_includes = source (here_path, "Top") :: r_includes }
 ;;
 
-let warn_install_at_root_directory 
+let warn_install_at_root_directory
   ({ q_includes; r_includes; } as project)
 =
   let open CList in
@@ -328,11 +330,11 @@ let check_overlapping_include { q_includes; r_includes } =
   let aux = function
     | [] -> ()
     | {thing = { path; canonical_path }, _} :: l ->
-	if not (is_prefix pwd canonical_path) then
-	  eprintf "Warning: %s (used in -R or -Q) is not a subdirectory of the current directory\n\n" path;
+        if not (is_prefix pwd canonical_path) then
+          eprintf "Warning: %s (used in -R or -Q) is not a subdirectory of the current directory\n\n" path;
         List.iter (fun {thing={ path = p; canonical_path = cp }, _} ->
-	  if is_prefix canonical_path cp  || is_prefix cp canonical_path then
-	    eprintf "Warning: %s and %s overlap (used in -R or -Q)\n\n"
+          if is_prefix canonical_path cp  || is_prefix cp canonical_path then
+            eprintf "Warning: %s and %s overlap (used in -R or -Q)\n\n"
               path p) l
   in
     aux (q_includes @ r_includes)
@@ -343,15 +345,13 @@ let chop_prefix p f =
   let len_f = String.length f in
   String.sub f len_p (len_f - len_p)
 
-let clean_path p =
-  Str.global_replace (Str.regexp_string "//") "/" p
-
 let destination_of { ml_includes; q_includes; r_includes; } file =
   let file_dir = CUnix.canonical_path_name (Filename.dirname file) in
   let includes = q_includes @ r_includes in
   let mk_destination logic canonical_path =
-    clean_path (physical_dir_of_logical_dir logic ^ "/" ^
-                chop_prefix canonical_path file_dir ^ "/") in
+    Filename.concat
+      (physical_dir_of_logical_dir logic)
+      (chop_prefix canonical_path file_dir) in
   let candidates =
     CList.map_filter (fun {thing={ canonical_path }, logic} ->
       if is_prefix canonical_path file_dir then
@@ -368,8 +368,9 @@ let destination_of { ml_includes; q_includes; r_includes; } file =
      with
         | [{thing={ canonical_path }, logic}], {thing={ canonical_path = p }} ->
             let destination =
-              clean_path (physical_dir_of_logical_dir logic ^ "/" ^
-                          chop_prefix p file_dir ^ "/") in
+              Filename.concat
+                (physical_dir_of_logical_dir logic)
+                (chop_prefix p file_dir) in
             Printf.printf "%s" (quote destination)
         | _ -> () (* skip *)
         | exception Not_found -> () (* skip *)
@@ -378,8 +379,8 @@ let destination_of { ml_includes; q_includes; r_includes; } file =
   | _ -> assert false
 
 let share_prefix s1 s2 =
-  let s1 = CString.split '.' s1 in
-  let s2 = CString.split '.' s2 in
+  let s1 = String.split_on_char '.' s1 in
+  let s2 = String.split_on_char '.' s2 in
   match s1, s2 with
   | x :: _ , y :: _ -> x = y
   | _ -> false
@@ -395,8 +396,9 @@ let _ =
     | "-destination-of" :: tgt :: rest -> Some tgt, rest
     | _ -> None, args in
 
-  let project = 
-    try cmdline_args_to_project ~curdir:Filename.current_dir_name args
+  let project =
+    let warning_fn x = Format.eprintf "%s@\n%!" x in
+    try cmdline_args_to_project ~warning_fn ~curdir:Filename.current_dir_name args
     with Parsing_error s -> prerr_endline s; usage_coq_makefile () in
 
   if only_destination <> None then begin
@@ -411,6 +413,7 @@ let _ =
 
   let conf_file = Option.default "CoqMakefile" project.makefile ^ ".conf" in
   let local_file = Option.default "CoqMakefile" project.makefile ^ ".local" in
+  let dep_file = "." ^ Option.default "CoqMakefile" project.makefile ^ ".d" in
 
   if project.extra_targets <> [] then begin
     eprintf "Warning: -extra and -extra-phony are deprecated.\n";
@@ -423,7 +426,7 @@ let _ =
   end;
 
   let project = ensure_root_dir project in
-  
+
   if project.install_kind <> (Some CoqProject_file.NoInstall) then begin
     warn_install_at_root_directory project;
   end;
@@ -431,9 +434,9 @@ let _ =
   check_overlapping_include project;
 
   Envars.set_coqlib ~fail:(fun x -> Printf.eprintf "Error: %s\n" x; exit 1);
-  
+
   let ocm = Option.cata open_out stdout project.makefile in
-  generate_makefile ocm conf_file local_file (prog :: args) project;
+  generate_makefile ocm conf_file local_file dep_file (prog :: args) project;
   close_out ocm;
   let occ = open_out conf_file in
   generate_conf occ project (prog :: args);
