@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -41,7 +41,7 @@ let warning_mult suf iter =
       if (Filename.dirname (file_name f d))
         <> (Filename.dirname (file_name f d')) then begin
           coqdep_warning "the file %s is defined twice!" (f ^ suf)
-	end
+        end
     with Not_found -> () end;
     Hashtbl.add tab f d
   in
@@ -56,20 +56,20 @@ let sort () =
       let cin = open_in (file ^ ".v") in
       let lb = Lexing.from_channel cin in
       try
-	while true do
-	  match coq_action lb with
-	    | Require (from, sl) ->
-		List.iter
-		  (fun s ->
+        while true do
+          match coq_action lb with
+            | Require (from, sl) ->
+                List.iter
+                  (fun s ->
                     match search_v_known ?from s with
                     | None -> ()
                     | Some f -> loop f)
-		sl
-	    | _ -> ()
-	done
+                sl
+            | _ -> ()
+        done
       with Fin_fichier ->
-	close_in cin;
-	printf "%s%s " file !suffixe
+        close_in cin;
+        printf "%s%s " file !suffixe
     end
   in
   List.iter (fun (name,_) -> loop name) !vAccu
@@ -85,18 +85,18 @@ let mL_dep_list b f =
       let chan = open_in f in
       let buf = Lexing.from_channel chan in
       try
-	while true do
-	  let (Use_module str) = caml_action buf in
-	  if str = b then begin
+        while true do
+          let (Use_module str) = caml_action buf in
+          if str = b then begin
             coqdep_warning "in file %s the notation %s. is useless !\n" f b
-	  end else
+          end else
             if not (List.mem str !deja_vu) then addQueue deja_vu str
-      	done; []
+        done; []
       with Fin_fichier -> begin
-	close_in chan;
-	let rl = List.rev !deja_vu in
-      	Hashtbl.add dep_tab f rl;
-	rl
+        close_in chan;
+        let rl = List.rev !deja_vu in
+        Hashtbl.add dep_tab f rl;
+        rl
       end
     with Sys_error _ -> []
 
@@ -116,36 +116,36 @@ let traite_Declare f =
   let decl_list = ref ([] : string list) in
   let rec treat = function
     | s :: ll ->
-	let s' = basename_noext s in
-	(match search_ml_known s with
-	   | Some mldir when not (List.mem s' !decl_list) ->
-	       let fullname = file_name s' mldir in
-	       let depl = mL_dep_list s (fullname ^ ".ml") in
-	       treat depl;
-	       decl_list := s :: !decl_list
-	   | _ -> ());
-	treat ll
+        let s' = basename_noext s in
+        (match search_ml_known s with
+           | Some mldir when not (List.mem s' !decl_list) ->
+               let fullname = file_name s' mldir in
+               let depl = mL_dep_list s (fullname ^ ".ml") in
+               treat depl;
+               decl_list := s :: !decl_list
+           | _ -> ());
+        treat ll
     | [] -> ()
   in
     try
       let chan = open_in f in
       let buf = Lexing.from_channel chan in
-	begin try
-	    while true do
-      	      let tok = coq_action buf in
-      		(match tok with
-		  | Declare sl ->
-		      decl_list := [];
-		      treat sl;
-		      decl_list := List.rev !decl_list;
-		      if !option_D then
-			affiche_Declare f !decl_list
-		      else if !decl_list <> sl then
-			warning_Declare f !decl_list
-		  | _ -> ())
-	    done
-	  with Fin_fichier -> () end;
-	close_in chan
+        begin try
+            while true do
+              let tok = coq_action buf in
+                (match tok with
+                  | Declare sl ->
+                      decl_list := [];
+                      treat sl;
+                      decl_list := List.rev !decl_list;
+                      if !option_D then
+                        affiche_Declare f !decl_list
+                      else if !decl_list <> sl then
+                        warning_Declare f !decl_list
+                  | _ -> ())
+            done
+          with Fin_fichier -> () end;
+        close_in chan
     with Sys_error _ -> ()
 
 let declare_dependencies () =
@@ -233,7 +233,7 @@ struct
             let rem = NSet.fold push next rem in
             aux rem seen
           | Some false ->
-            (** The path we took encountered x -> y but not the one in seen *)
+            (* The path we took encountered x -> y but not the one in seen *)
             if through then aux (NMap.add n true rem) (NMap.add n true seen)
             else aux rem seen
           | Some true -> aux rem seen
@@ -246,7 +246,7 @@ struct
       else
         let x = NSet.choose rem in
         let rem = NSet.remove x rem in
-        if NSet.mem x seen then 
+        if NSet.mem x seen then
           aux rem seen
         else
           let seen = NSet.add x seen in
@@ -297,7 +297,7 @@ module DAG = DAG(struct type t = string let compare = compare end)
 (** TODO: we should share this code with Coqdep_common *)
 module VData = struct
   type t = string list option * string list
-  let compare = Pervasives.compare
+  let compare = Util.pervasives_compare
 end
 
 module VCache = Set.Make(VData)
@@ -357,7 +357,7 @@ let treat_coq_file chan =
           | None -> acc
           | Some file_str -> (canonize file_str, ".v") :: acc
         else acc
-      | AddLoadPath _ | AddRecLoadPath _ -> acc (** TODO *)
+      | AddLoadPath _ | AddRecLoadPath _ -> acc (* TODO *)
       in
       loop acc
   in
@@ -454,7 +454,8 @@ let usage () =
   eprintf "  -I dir : add (non recursively) dir to ocaml path\n";
   eprintf "  -R dir -as logname : add and import dir recursively to coq load path under logical name logname\n"; (* deprecate? *)
   eprintf "  -R dir logname : add and import dir recursively to coq load path under logical name logname\n";
-  eprintf "  -Q dir logname : add (recusively) and open (non recursively) dir to coq load path under logical name logname\n";
+  eprintf "  -Q dir logname : add (recursively) and open (non recursively) dir to coq load path under logical name logname\n";
+  eprintf "  -vos : also output dependencies about .vos files\n";
   eprintf "  -dumpgraph f : print a dot dependency graph in file 'f'\n";
   eprintf "  -dumpgraphbox f : print a dot dependency graph box in file 'f'\n";
   eprintf "  -exclude-dir dir : skip subdirectories named 'dir' during -R/-Q search\n";
@@ -473,7 +474,8 @@ let add_r_include path l = add_rec_dir_import add_known path (split_period l)
 let treat_coqproject f =
   let open CoqProject_file in
   let iter_sourced f = List.iter (fun {thing} -> f thing) in
-  let project = read_project_file f in
+  let warning_fn x = coqdep_warning "%s" x in
+  let project = read_project_file ~warning_fn f in
   iter_sourced (fun { path } -> add_caml_dir path) project.ml_includes;
   iter_sourced (fun ({ path }, l) -> add_q_include path l) project.q_includes;
   iter_sourced (fun ({ path }, l) -> add_r_include path l) project.r_includes;
@@ -485,6 +487,7 @@ let rec parse = function
   | "-w" :: ll -> option_w := true; parse ll
   | "-boot" :: ll -> option_boot := true; parse ll
   | "-sort" :: ll -> option_sort := true; parse ll
+  | "-vos" :: ll -> write_vos := true; parse ll
   | ("-noglob" | "-no-glob") :: ll -> option_noglob := true; parse ll
   | "-f" :: f :: ll -> treat_coqproject f; parse ll
   | "-I" :: r :: ll -> add_caml_dir r; parse ll
@@ -496,7 +499,7 @@ let rec parse = function
   | "-dumpgraphbox" :: f :: ll -> option_dump := Some (true, f); parse ll
   | "-exclude-dir" :: r :: ll -> System.exclude_directory r; parse ll
   | "-exclude-dir" :: [] -> usage ()
-  | "-coqlib" :: r :: ll -> Flags.coqlib_spec := true; Flags.coqlib := r; parse ll
+  | "-coqlib" :: r :: ll -> Envars.set_user_coqlib r; parse ll
   | "-coqlib" :: [] -> usage ()
   | "-suffix" :: s :: ll -> suffixe := s ; parse ll
   | "-suffix" :: [] -> usage ()
@@ -528,7 +531,13 @@ let coqdep () =
     add_rec_dir_import add_known "plugins" ["Coq"];
     add_rec_dir_import (fun _ -> add_caml_known) "theories" ["Coq"];
     add_rec_dir_import (fun _ -> add_caml_known) "plugins" ["Coq"];
+    let user = "user-contrib" in
+    if Sys.file_exists user then begin
+      add_rec_dir_no_import add_known user [];
+      add_rec_dir_no_import (fun _ -> add_caml_known) user [];
+    end;
   end else begin
+    (* option_boot is actually always false in this branch *)
     Envars.set_coqlib ~fail:(fun msg -> raise (CoqlibError msg));
     let coqlib = Envars.coqlib () in
     add_rec_dir_import add_coqlib_known (coqlib//"theories") ["Coq"];
@@ -561,4 +570,5 @@ let _ =
   try
     coqdep ()
   with CoqlibError msg ->
-    eprintf "*** Error: %s@\n%!" msg
+    eprintf "*** Error: %s@\n%!" msg;
+    exit 1
