@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -24,6 +24,9 @@ val option_c : bool ref
 val option_noglob : bool ref
 val option_boot : bool ref
 
+val write_vos : bool ref
+(** output vos and vok dependencies *)
+
 type dynlink = Opt | Byte | Both | No | Variable
 
 val option_dynlink : dynlink ref
