@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -79,14 +79,17 @@ let use_suffix prefix suffix =
 
 (** A weaker analog of the function in Envars *)
 
+let getenv_else s dft = try Sys.getenv s with Not_found -> dft ()
+
 let guess_coqlib () =
+  getenv_else "COQLIB" (fun () ->
   let file = "theories/Init/Prelude.vo" in
   let coqbin = normalize_path (Filename.dirname Sys.executable_name) in
   let prefix = Filename.dirname coqbin in
   let coqlib = use_suffix prefix Coq_config.coqlibsuffix in
   if Sys.file_exists (coqlib / file) then coqlib else
   if not Coq_config.local && Sys.file_exists (Coq_config.coqlib / file)
-  then Coq_config.coqlib else prefix
+  then Coq_config.coqlib else prefix)
 
 let header_trailer = ref true
 let header_file = ref ""
