@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -11,4 +11,4 @@
 module W = AsyncTaskQueue.MakeWorker(Stm.ProofTask) ()
 
 let () =
-  WorkerLoop.start ~init:W.init_stdout ~loop:W.main_loop
+  WorkerLoop.start ~init:W.init_stdout ~loop:W.main_loop "coqproofworker"
