@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -33,24 +33,24 @@ open Pcoq
 let constr_level = string_of_int
 
 let default_levels =
-  [200,Extend.RightA,false;
-   100,Extend.RightA,false;
-   99,Extend.RightA,true;
-   90,Extend.RightA,true;
-   10,Extend.LeftA,false;
-   9,Extend.RightA,false;
-   8,Extend.RightA,true;
-   1,Extend.LeftA,false;
-   0,Extend.RightA,false]
+  [200,Gramlib.Gramext.RightA,false;
+   100,Gramlib.Gramext.RightA,false;
+   99,Gramlib.Gramext.RightA,true;
+   90,Gramlib.Gramext.RightA,true;
+   10,Gramlib.Gramext.LeftA,false;
+   9,Gramlib.Gramext.RightA,false;
+   8,Gramlib.Gramext.RightA,true;
+   1,Gramlib.Gramext.LeftA,false;
+   0,Gramlib.Gramext.RightA,false]
 
 let default_pattern_levels =
-  [200,Extend.RightA,true;
-   100,Extend.RightA,false;
-   99,Extend.RightA,true;
-   90,Extend.RightA,true;
-   10,Extend.LeftA,false;
-   1,Extend.LeftA,false;
-   0,Extend.RightA,false]
+  [200,Gramlib.Gramext.RightA,true;
+   100,Gramlib.Gramext.RightA,false;
+   99,Gramlib.Gramext.RightA,true;
+   90,Gramlib.Gramext.RightA,true;
+   10,Gramlib.Gramext.LeftA,false;
+   1,Gramlib.Gramext.LeftA,false;
+   0,Gramlib.Gramext.RightA,false]
 
 let default_constr_levels = (default_levels, default_pattern_levels)
 
@@ -70,28 +70,28 @@ let save_levels levels custom lev =
 (* first LeftA, then RightA and NoneA together *)
 
 let admissible_assoc = function
-  | Extend.LeftA, Some (Extend.RightA | Extend.NonA) -> false
-  | Extend.RightA, Some Extend.LeftA -> false
+  | Gramlib.Gramext.LeftA, Some (Gramlib.Gramext.RightA | Gramlib.Gramext.NonA) -> false
+  | Gramlib.Gramext.RightA, Some Gramlib.Gramext.LeftA -> false
   | _ -> true
 
 let create_assoc = function
-  | None -> Extend.RightA
+  | None -> Gramlib.Gramext.RightA
   | Some a -> a
 
 let error_level_assoc p current expected =
   let open Pp in
   let pr_assoc = function
-    | Extend.LeftA -> str "left"
-    | Extend.RightA -> str "right"
-    | Extend.NonA -> str "non" in
-  user_err 
+    | Gramlib.Gramext.LeftA -> str "left"
+    | Gramlib.Gramext.RightA -> str "right"
+    | Gramlib.Gramext.NonA -> str "non" in
+  user_err
     (str "Level " ++ int p ++ str " is already declared " ++
      pr_assoc current ++ str " associative while it is now expected to be " ++
      pr_assoc expected ++ str " associative.")
 
 let create_pos = function
-  | None -> Extend.First
-  | Some lev -> Extend.After (constr_level lev)
+  | None -> Gramlib.Gramext.First
+  | Some lev -> Gramlib.Gramext.After (constr_level lev)
 
 let find_position_gen current ensure assoc lev =
   match lev with
@@ -104,30 +104,30 @@ let find_position_gen current ensure assoc lev =
         | (p,_,_ as pa)::l when p > n -> pa :: add_level (Some p) l
         | (p,a,reinit)::l when Int.equal p n ->
             if reinit then
-	      let a' = create_assoc assoc in
+              let a' = create_assoc assoc in
               (init := Some (a',create_pos q); (p,a',false)::l)
-	    else if admissible_assoc (a,assoc) then
-	      raise Exit
+            else if admissible_assoc (a,assoc) then
+              raise Exit
             else
-	      error_level_assoc p a (Option.get assoc)
-	| l -> after := q; (n,create_assoc assoc,ensure)::l
+              error_level_assoc p a (Option.get assoc)
+        | l -> after := q; (n,create_assoc assoc,ensure)::l
       in
       try
-	let updated = add_level None current in
-	let assoc = create_assoc assoc in
+        let updated = add_level None current in
+        let assoc = create_assoc assoc in
         begin match !init with
         | None ->
-	  (* Create the entry *)
-	   updated, (Some (create_pos !after), Some assoc, Some (constr_level n), None)
+          (* Create the entry *)
+           updated, (Some (create_pos !after), Some assoc, Some (constr_level n), None)
         | _ ->
-	  (* The reinit flag has been updated *)
-	   updated, (Some (Extend.Level (constr_level n)), None, None, !init)
+          (* The reinit flag has been updated *)
+           updated, (Some (Gramlib.Gramext.Level (constr_level n)), None, None, !init)
         end
       with
-	  (* Nothing has changed *)
+          (* Nothing has changed *)
           Exit ->
-	    (* Just inherit the existing associativity and name (None) *)
-	    current, (Some (Extend.Level (constr_level n)), None, None, None)
+            (* Just inherit the existing associativity and name (None) *)
+            current, (Some (Gramlib.Gramext.Level (constr_level n)), None, None, None)
 
 let rec list_mem_assoc_triple x = function
   | [] -> false
@@ -187,15 +187,18 @@ let find_position accu custom forpat assoc level =
 (* Binding constr entry keys to entries                               *)
 
 (* Camlp5 levels do not treat NonA: use RightA with a NEXT on the left *)
-let camlp5_assoc = function
-  | Some NonA | Some RightA -> RightA
-  | None | Some LeftA -> LeftA
+let camlp5_assoc =
+  let open Gramlib.Gramext in function
+    | Some NonA | Some RightA -> RightA
+    | None | Some LeftA -> LeftA
 
-let assoc_eq al ar = match al, ar with
-| NonA, NonA
-| RightA, RightA
-| LeftA, LeftA -> true
-| _, _ -> false
+let assoc_eq al ar =
+  let open Gramlib.Gramext in
+  match al, ar with
+  | NonA, NonA
+  | RightA, RightA
+  | LeftA, LeftA -> true
+  | _, _ -> false
 
 (* [adjust_level assoc from prod] where [assoc] and [from] are the name
    and associativity of the level where to add the rule; the meaning of
@@ -205,7 +208,7 @@ let assoc_eq al ar = match al, ar with
      Some None = NEXT
      Some (Some (n,cur)) = constr LEVEL n
          s.t. if [cur] is set then [n] is the same as the [from] level *)
-let adjust_level assoc from = function
+let adjust_level assoc from = let open Gramlib.Gramext in function
 (* Associativity is None means force the level *)
   | (NumLevel n,BorderProd (_,None)) -> Some (Some (n,true))
 (* Compute production name on the right side *)
@@ -242,12 +245,12 @@ type prod_info = production_level * production_position
 type (_, _) entry =
 | TTName : ('self, lname) entry
 | TTReference : ('self, qualid) entry
-| TTBigint : ('self, Constrexpr.raw_natural_number) entry
+| TTBigint : ('self, string) entry
 | TTConstr : notation_entry * prod_info * 'r target -> ('r, 'r) entry
-| TTConstrList : prod_info * Tok.t list * 'r target -> ('r, 'r list) entry
+| TTConstrList : notation_entry * prod_info * string Tok.p list * 'r target -> ('r, 'r list) entry
 | TTPattern : int -> ('self, cases_pattern_expr) entry
 | TTOpenBinderList : ('self, local_binder_expr list) entry
-| TTClosedBinderList : Tok.t list -> ('self, local_binder_expr list list) entry
+| TTClosedBinderList : string Tok.p list -> ('self, local_binder_expr list list) entry
 
 type _ any_entry = TTAny : ('s, 'r) entry -> 's any_entry
 
@@ -275,6 +278,10 @@ let find_custom_entry s =
   try (find_custom_entry constr_custom_entry sc, find_custom_entry pattern_custom_entry sp)
   with Not_found -> user_err Pp.(str "Undeclared custom entry: " ++ str s ++ str ".")
 
+let exists_custom_entry s = match find_custom_entry s with
+| _ -> true
+| exception _ -> false
+
 let locality_of_custom_entry s = String.Set.mem s !custom_entry_locality
 
 (* This computes the name of the level where to add a new rule *)
@@ -316,41 +323,49 @@ let is_binder_level from e = match e with
 let make_sep_rules = function
   | [tk] -> Atoken tk
   | tkl ->
-  let rec mkrule : Tok.t list -> string rules = function
-  | [] -> Rules ({ norec_rule = Stop }, fun _ -> (* dropped anyway: *) "")
+  let rec mkrule : 'a Tok.p list -> 'a rules = function
+  | [] -> Rules (Stop, fun _ -> (* dropped anyway: *) "")
   | tkn :: rem ->
-    let Rules ({ norec_rule = r }, f) = mkrule rem in
-    let r = { norec_rule = Next (r, Atoken tkn) } in
+    let Rules (r, f) = mkrule rem in
+    let r = NextNoRec (r, Atoken tkn) in
     Rules (r, fun _ -> f)
   in
   let r = mkrule (List.rev tkl) in
   Arules [r]
 
-let symbol_of_target : type s. _ -> _ -> _ -> _ -> s target -> (s, s) symbol = fun custom p assoc from forpat ->
-  if custom = InConstrEntry && is_binder_level from p then Aentryl (target_entry InConstrEntry forpat, "200")
-  else if is_self from p then Aself
+type ('s, 'a) mayrec_symbol =
+| MayRecNo : ('s, norec, 'a) symbol -> ('s, 'a) mayrec_symbol
+| MayRecMay : ('s, mayrec, 'a) symbol -> ('s, 'a) mayrec_symbol
+
+let symbol_of_target : type s. _ -> _ -> _ -> _ -> s target -> (s, s) mayrec_symbol = fun custom p assoc from forpat ->
+  if custom = InConstrEntry && is_binder_level from p then MayRecNo (Aentryl (target_entry InConstrEntry forpat, "200"))
+  else if is_self from p then MayRecMay Aself
   else
     let g = target_entry custom forpat in
     let lev = adjust_level assoc from p in
     begin match lev with
-    | None -> Aentry g
-    | Some None -> Anext
-    | Some (Some (lev, cur)) -> Aentryl (g, string_of_int lev)
+    | None -> MayRecNo (Aentry g)
+    | Some None -> MayRecMay Anext
+    | Some (Some (lev, cur)) -> MayRecNo (Aentryl (g, string_of_int lev))
     end
 
-let symbol_of_entry : type s r. _ -> _ -> (s, r) entry -> (s, r) symbol = fun assoc from typ -> match typ with
+let symbol_of_entry : type s r. _ -> _ -> (s, r) entry -> (s, r) mayrec_symbol = fun assoc from typ -> match typ with
 | TTConstr (s, p, forpat) -> symbol_of_target s p assoc from forpat
-| TTConstrList (typ', [], forpat) ->
-  Alist1 (symbol_of_target InConstrEntry typ' assoc from forpat)
-| TTConstrList (typ', tkl, forpat) ->
-  Alist1sep (symbol_of_target InConstrEntry typ' assoc from forpat, make_sep_rules tkl)
-| TTPattern p -> Aentryl (Constr.pattern, string_of_int p)
-| TTClosedBinderList [] -> Alist1 (Aentry Constr.binder)
-| TTClosedBinderList tkl -> Alist1sep (Aentry Constr.binder, make_sep_rules tkl)
-| TTName -> Aentry Prim.name
-| TTOpenBinderList -> Aentry Constr.open_binders
-| TTBigint -> Aentry Prim.bigint
-| TTReference -> Aentry Constr.global
+| TTConstrList (s, typ', [], forpat) ->
+  begin match symbol_of_target s typ' assoc from forpat with
+  | MayRecNo s -> MayRecNo (Alist1 s)
+  | MayRecMay s -> MayRecMay (Alist1 s) end
+| TTConstrList (s, typ', tkl, forpat) ->
+  begin match symbol_of_target s typ' assoc from forpat with
+  | MayRecNo s -> MayRecNo (Alist1sep (s, make_sep_rules tkl))
+  | MayRecMay s -> MayRecMay (Alist1sep (s, make_sep_rules tkl)) end
+| TTPattern p -> MayRecNo (Aentryl (Constr.pattern, string_of_int p))
+| TTClosedBinderList [] -> MayRecNo (Alist1 (Aentry Constr.binder))
+| TTClosedBinderList tkl -> MayRecNo (Alist1sep (Aentry Constr.binder, make_sep_rules tkl))
+| TTName -> MayRecNo (Aentry Prim.name)
+| TTOpenBinderList -> MayRecNo (Aentry Constr.open_binders)
+| TTBigint -> MayRecNo (Aentry Prim.bigint)
+| TTReference -> MayRecNo (Aentry Constr.global)
 
 let interp_entry forpat e = match e with
 | ETProdName -> TTAny TTName
@@ -358,7 +373,7 @@ let interp_entry forpat e = match e with
 | ETProdBigint -> TTAny TTBigint
 | ETProdConstr (s,p) -> TTAny (TTConstr (s, p, forpat))
 | ETProdPattern p -> TTAny (TTPattern p)
-| ETProdConstrList (p, tkl) -> TTAny (TTConstrList (p, tkl, forpat))
+| ETProdConstrList (s, p, tkl) -> TTAny (TTConstrList (s, p, tkl, forpat))
 | ETProdBinderList ETBinderOpen -> TTAny TTOpenBinderList
 | ETProdBinderList (ETBinderClosed tkl) -> TTAny (TTClosedBinderList tkl)
 
@@ -392,8 +407,8 @@ match e with
 | TTClosedBinderList _ -> { subst with binderlists = List.flatten v :: subst.binderlists }
 | TTBigint ->
   begin match forpat with
-  | ForConstr ->  push_constr subst (CAst.make @@ CPrim (Numeral (v,true)))
-  | ForPattern -> push_constr subst (CAst.make @@ CPatPrim (Numeral (v,true)))
+  | ForConstr ->  push_constr subst (CAst.make @@ CPrim (Numeral (SPlus,NumTok.int v)))
+  | ForPattern -> push_constr subst (CAst.make @@ CPatPrim (Numeral (SPlus,NumTok.int v)))
   end
 | TTReference ->
   begin match forpat with
@@ -403,8 +418,8 @@ match e with
 | TTConstrList _ -> { subst with constrlists = v :: subst.constrlists }
 
 type (_, _) ty_symbol =
-| TyTerm : Tok.t -> ('s, string) ty_symbol
-| TyNonTerm : 's target * ('s, 'a) entry * ('s, 'a) symbol * bool -> ('s, 'a) ty_symbol
+| TyTerm : string Tok.p -> ('s, string) ty_symbol
+| TyNonTerm : 's target * ('s, 'a) entry * ('s, 'a) mayrec_symbol * bool -> ('s, 'a) ty_symbol
 
 type ('self, _, 'r) ty_rule =
 | TyStop : ('self, 'r, 'r) ty_rule
@@ -439,13 +454,25 @@ let rec ty_eval : type s a. (s, a, Loc.t -> s) ty_rule -> s gen_eval -> s env ->
         let constrlist, tail = List.chop (n - p) heads in
         constrlist :: env.constrlists, tail @ constrs
     in
-    ty_eval rem f { env with constrs; constrlists; } 
+    ty_eval rem f { env with constrs; constrlists; }
 
-let rec ty_erase : type s a r. (s, a, r) ty_rule -> (s, a, r) Extend.rule = function
-| TyStop -> Stop
+type ('s, 'a, 'r) mayrec_rule =
+| MayRecRNo : ('s, Extend.norec, 'a, 'r) Extend.rule -> ('s, 'a, 'r) mayrec_rule
+| MayRecRMay : ('s, Extend.mayrec, 'a, 'r) Extend.rule -> ('s, 'a, 'r) mayrec_rule
+
+let rec ty_erase : type s a r. (s, a, r) ty_rule -> (s, a, r) mayrec_rule = function
+| TyStop -> MayRecRNo Stop
 | TyMark (_, _, _, r) -> ty_erase r
-| TyNext (rem, TyTerm tok) -> Next (ty_erase rem, Atoken tok)
-| TyNext (rem, TyNonTerm (_, _, s, _)) -> Next (ty_erase rem, s)
+| TyNext (rem, TyTerm tok) ->
+   begin match ty_erase rem with
+   | MayRecRNo rem -> MayRecRMay (Next (rem, Atoken tok))
+   | MayRecRMay rem -> MayRecRMay (Next (rem, Atoken tok)) end
+| TyNext (rem, TyNonTerm (_, _, s, _)) ->
+   begin match ty_erase rem, s with
+   | MayRecRNo rem, MayRecNo s -> MayRecRMay (Next (rem, s))
+   | MayRecRNo rem, MayRecMay s -> MayRecRMay (Next (rem, s))
+   | MayRecRMay rem, MayRecNo s -> MayRecRMay (Next (rem, s))
+   | MayRecRMay rem, MayRecMay s -> MayRecRMay (Next (rem, s)) end
 
 type ('self, 'r) any_ty_rule =
 | AnyTyRule : ('self, 'act, Loc.t -> 'r) ty_rule -> ('self, 'r) any_ty_rule
@@ -482,7 +509,7 @@ let rec pure_sublevels' custom assoc from forpat level = function
    let rem = pure_sublevels' custom assoc from forpat level rem in
    let push where p rem =
      match symbol_of_target custom p assoc from forpat with
-     | Aentryl (_,i) when level <> Some (int_of_string i) -> (where,int_of_string i) :: rem
+     | MayRecNo (Aentryl (_,i)) when level <> Some (int_of_string i) -> (where,int_of_string i) :: rem
      | _ -> rem in
    (match e with
    | ETProdPattern i -> push InConstrEntry (NumLevel i,InternalProd) rem
@@ -504,7 +531,6 @@ let extend_constr state forpat ng =
   let (entry, level) = interp_constr_entry_key custom forpat n in
   let fold (accu, state) pt =
     let AnyTyRule r = make_ty_rule assoc n forpat pt in
-    let symbs = ty_erase r in
     let pure_sublevels = pure_sublevels' custom assoc n forpat level pt in
     let isforpat = target_to_bool forpat in
     let needed_levels, state = register_empty_levels state isforpat pure_sublevels in
@@ -512,7 +538,11 @@ let extend_constr state forpat ng =
     let empty_rules = List.map (prepare_empty_levels forpat) needed_levels in
     let empty = { constrs = []; constrlists = []; binders = []; binderlists = [] } in
     let act = ty_eval r (make_act forpat ng.notgram_notation) empty in
-    let rule = (name, p4assoc, [Rule (symbs, act)]) in
+    let rule =
+      let r = match ty_erase r with
+        | MayRecRNo symbs -> Rule (symbs, act)
+        | MayRecRMay symbs -> Rule (symbs, act) in
+      name, p4assoc, [r] in
     let r = ExtendRule (entry, reinit, (pos, [rule])) in
     (accu @ empty_rules @ [r], state)
   in
@@ -520,6 +550,15 @@ let extend_constr state forpat ng =
 
 let constr_levels = GramState.field ()
 
+let is_disjunctive_pattern_rule ng =
+  String.is_sub "( _ | " (snd ng.notgram_notation) 0
+
+let warn_disj_pattern_notation =
+  let open Pp in
+  let pp ng = str "Use of " ++ Notation.pr_notation ng.notgram_notation ++
+              str " Notation is deprecated as it is inconsistent with pattern syntax." in
+  CWarnings.create ~name:"disj-pattern-notation" ~category:"notation" ~default:CWarnings.Disabled pp
+
 let extend_constr_notation ng state =
   let levels = match GramState.get state constr_levels with
   | None -> String.Map.add "constr" default_constr_levels String.Map.empty
@@ -527,8 +566,13 @@ let extend_constr_notation ng state =
   in
   (* Add the notation in constr *)
   let (r, levels) = extend_constr levels ForConstr ng in
-  (* Add the notation in cases_pattern *)
-  let (r', levels) = extend_constr levels ForPattern ng in
+  (* Add the notation in cases_pattern, unless it would disrupt *)
+  (* parsing nested disjunctive patterns. *)
+  let (r', levels) =
+    if is_disjunctive_pattern_rule ng then begin
+       warn_disj_pattern_notation ng;
+       ([], levels)
+    end else extend_constr levels ForPattern ng in
   let state = GramState.set state constr_levels levels in
   (r @ r', state)
 
