@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -21,19 +21,15 @@ open CErrors
 open Util
 open Names
 open Declarations
-open Entries
 open Term
 open Constr
 open Inductive
-open Decl_kinds
 open Indrec
 open Declare
 open Libnames
-open Globnames
 open Goptions
 open Nameops
 open Termops
-open Nametab
 open Smartlocate
 open Vernacexpr
 open Ind_tables
@@ -45,7 +41,7 @@ open Context.Rel.Declaration
 (* Flags governing automatic synthesis of schemes *)
 
 let elim_flag = ref true
-let _ =
+let () =
   declare_bool_option
     { optdepr  = false;
       optname  = "automatic declaration of induction schemes";
@@ -54,7 +50,7 @@ let _ =
       optwrite = (fun b -> elim_flag := b) }
 
 let bifinite_elim_flag = ref false
-let _ =
+let () =
   declare_bool_option
     { optdepr  = false;
       optname  = "automatic declaration of induction schemes for non-recursive types";
@@ -63,7 +59,7 @@ let _ =
       optwrite = (fun b -> bifinite_elim_flag := b) }
 
 let case_flag = ref false
-let _ =
+let () =
   declare_bool_option
     { optdepr  = false;
       optname  = "automatic declaration of case analysis schemes";
@@ -72,7 +68,7 @@ let _ =
       optwrite = (fun b -> case_flag := b) }
 
 let eq_flag = ref false
-let _ =
+let () =
   declare_bool_option
     { optdepr  = false;
       optname  = "automatic declaration of boolean equality";
@@ -82,8 +78,8 @@ let _ =
 
 let is_eq_flag () = !eq_flag
 
-let eq_dec_flag = ref false 
-let _ =
+let eq_dec_flag = ref false
+let () =
   declare_bool_option
     { optdepr  = false;
       optname  = "automatic declaration of decidable equality";
@@ -92,7 +88,7 @@ let _ =
       optwrite = (fun b -> eq_dec_flag := b) }
 
 let rewriting_flag = ref false
-let _ =
+let () =
   declare_bool_option
     { optdepr  = false;
       optname  ="automatic declaration of rewriting schemes for equality types";
@@ -102,25 +98,12 @@ let _ =
 
 (* Util *)
 
-let define id internal ctx c t =
-  let f = declare_constant ~internal in
-  let univs =
-    if Flags.is_universe_polymorphism ()
-    then Polymorphic_const_entry (Evd.to_universe_context ctx)
-    else Monomorphic_const_entry (Evd.universe_context_set ctx)
-  in
-  let kn = f id
-    (DefinitionEntry
-      { const_entry_body = c;
-        const_entry_secctx = None;
-        const_entry_type = t;
-	const_entry_universes = univs;
-        const_entry_opaque = false;
-        const_entry_inline_code = false;
-        const_entry_feedback = None;
-      },
-      Decl_kinds.IsDefinition Scheme) in
-  definition_message id;
+let define ~poly name sigma c types =
+  let f = declare_constant ~kind:Decls.(IsDefinition Scheme) in
+  let univs = Evd.univ_entry ~poly sigma in
+  let entry = Declare.definition_entry ~univs ?types c in
+  let kn = f ~name (DefinitionEntry entry) in
+  definition_message name;
   kn
 
 (* Boolean equality *)
@@ -135,38 +118,40 @@ let alarm what internal msg =
   | InternalTacticRequest ->
     (if debug then
       Feedback.msg_debug
-	(hov 0 msg ++ fnl () ++ what ++ str " not defined.")); None
+        (hov 0 msg ++ fnl () ++ what ++ str " not defined.")); None
   | _ -> Some msg
 
 let try_declare_scheme what f internal names kn =
   try f internal names kn
   with e ->
   let e = CErrors.push e in
-  let msg = match fst e with
+  let rec extract_exn = function Logic_monad.TacticFailure e -> extract_exn e | e -> e in
+  let msg = match extract_exn (fst e) with
     | ParameterWithoutEquality cst ->
-	alarm what internal
-	  (str "Boolean equality not found for parameter " ++ Printer.pr_global cst ++
-	   str".")
+        alarm what internal
+          (str "Boolean equality not found for parameter " ++ Printer.pr_global cst ++
+           str".")
     | InductiveWithProduct ->
-	alarm what internal
-	  (str "Unable to decide equality of functional arguments.")
+        alarm what internal
+          (str "Unable to decide equality of functional arguments.")
     | InductiveWithSort ->
-	alarm what internal
-	  (str "Unable to decide equality of type arguments.")
+        alarm what internal
+          (str "Unable to decide equality of type arguments.")
     | NonSingletonProp ind ->
-	alarm what internal
-	  (str "Cannot extract computational content from proposition " ++
-	   quote (Printer.pr_inductive (Global.env()) ind) ++ str ".")
+        alarm what internal
+          (str "Cannot extract computational content from proposition " ++
+           quote (Printer.pr_inductive (Global.env()) ind) ++ str ".")
     | EqNotFound (ind',ind) ->
-	alarm what internal
-	  (str "Boolean equality on " ++
-	   quote (Printer.pr_inductive (Global.env()) ind') ++
-	   strbrk " is missing.")
+        alarm what internal
+          (str "Boolean equality on " ++
+           quote (Printer.pr_inductive (Global.env()) ind') ++
+           strbrk " is missing.")
     | UndefinedCst s ->
-	alarm what internal
-	  (strbrk "Required constant " ++ str s ++ str " undefined.")
-    | AlreadyDeclared msg ->
-        alarm what internal (msg ++ str ".")
+        alarm what internal
+          (strbrk "Required constant " ++ str s ++ str " undefined.")
+    | AlreadyDeclared (kind, id) as exn ->
+      let msg = CErrors.print exn in
+      alarm what internal msg
     | DecidabilityMutualNotSupported ->
         alarm what internal
           (str "Decidability lemma for mutual inductive types not supported.")
@@ -176,9 +161,17 @@ let try_declare_scheme what f internal names kn =
     | NoDecidabilityCoInductive ->
          alarm what internal
            (str "Scheme Equality is only for inductive types.")
+    | DecidabilityIndicesNotSupported ->
+         alarm what internal
+           (str "Inductive types with annotations not supported.")
+    | ConstructorWithNonParametricInductiveType ind ->
+         alarm what internal
+           (strbrk "Unsupported constructor with an argument whose type is a non-parametric inductive type." ++
+            strbrk " Type " ++ quote (Printer.pr_inductive (Global.env()) ind) ++
+            str " is applied to an argument which is not a variable.")
     | e when CErrors.noncritical e ->
         alarm what internal
-	  (str "Unexpected error during scheme creation: " ++ CErrors.print e)
+          (str "Unexpected error during scheme creation: " ++ CErrors.print e)
     | _ -> iraise e
   in
   match msg with
@@ -212,11 +205,11 @@ let declare_one_case_analysis_scheme ind =
     else if not (Inductiveops.has_dependent_elim mib) then
       case_scheme_kind_from_type
     else case_dep_scheme_kind_from_type in
-  let kelim = elim_sorts (mib,mip) in
+  let kelim = elim_sort (mib,mip) in
     (* in case the inductive has a type elimination, generates only one
        induction scheme, the other ones share the same code with the
-       apropriate type *)
-  if Sorts.List.mem InType kelim then
+       appropriate type *)
+  if Sorts.family_leq InType kelim then
     ignore (define_individual_scheme dep UserAutomaticRequest None ind)
 
 (* Induction/recursion schemes *)
@@ -224,30 +217,37 @@ let declare_one_case_analysis_scheme ind =
 let kinds_from_prop =
   [InType,rect_scheme_kind_from_prop;
    InProp,ind_scheme_kind_from_prop;
-   InSet,rec_scheme_kind_from_prop]
+   InSet,rec_scheme_kind_from_prop;
+   InSProp,sind_scheme_kind_from_prop]
 
 let kinds_from_type =
   [InType,rect_dep_scheme_kind_from_type;
    InProp,ind_dep_scheme_kind_from_type;
-   InSet,rec_dep_scheme_kind_from_type]
+   InSet,rec_dep_scheme_kind_from_type;
+   InSProp,sind_dep_scheme_kind_from_type]
 
 let nondep_kinds_from_type =
   [InType,rect_scheme_kind_from_type;
    InProp,ind_scheme_kind_from_type;
-   InSet,rec_scheme_kind_from_type]
+   InSet,rec_scheme_kind_from_type;
+   InSProp,sind_scheme_kind_from_type]
 
 let declare_one_induction_scheme ind =
   let (mib,mip) = Global.lookup_inductive ind in
   let kind = inductive_sort_family mip in
   let from_prop = kind == InProp in
   let depelim = Inductiveops.has_dependent_elim mib in
-  let kelim = elim_sorts (mib,mip) in
+  let kelim = Inductiveops.sorts_below (elim_sort (mib,mip)) in
+  let kelim = if Global.sprop_allowed () then kelim
+    else List.filter (fun s -> s <> InSProp) kelim
+  in
   let elims =
     List.map_filter (fun (sort,kind) ->
-      if Sorts.List.mem sort kelim then Some kind else None)
+        if List.mem_f Sorts.family_equal sort kelim then Some kind else None)
       (if from_prop then kinds_from_prop
        else if depelim then kinds_from_type
-       else nondep_kinds_from_type) in
+       else nondep_kinds_from_type)
+  in
   List.iter (fun kind -> ignore (define_individual_scheme kind UserAutomaticRequest None ind))
     elims
 
@@ -303,7 +303,9 @@ let warn_cannot_build_congruence =
           strbrk "Cannot build congruence scheme because eq is not found")
 
 let declare_congr_scheme ind =
-  if Hipattern.is_equality_type Evd.empty (EConstr.of_constr (mkInd ind)) (** FIXME *) then begin
+  let env = Global.env () in
+  let sigma = Evd.from_env env in
+  if Hipattern.is_equality_type env sigma (EConstr.of_constr (mkInd ind)) (* FIXME *) then begin
     if
       try Coqlib.check_required_library Coqlib.logic_module_name; true
       with e when CErrors.noncritical e -> false
@@ -321,11 +323,10 @@ let declare_sym_scheme ind =
 (* Scheme command *)
 
 let smart_global_inductive y = smart_global_inductive y
-let rec split_scheme l =
- let env = Global.env() in
+let rec split_scheme env l =
  match l with
   | [] -> [],[]
-  | (Some id,t)::q -> let l1,l2 = split_scheme q in
+  | (Some id,t)::q -> let l1,l2 = split_scheme env q in
     ( match t with
       | InductionScheme (x,y,z) -> ((id,x,smart_global_inductive y,z)::l1),l2
       | CaseScheme (x,y,z) -> ((id,x,smart_global_inductive y,z)::l1),l2
@@ -336,7 +337,7 @@ let rec split_scheme l =
 requested
 *)
   | (None,t)::q ->
-      let l1,l2 = split_scheme q in
+      let l1,l2 = split_scheme env q in
       let names inds recs isdep y z =
         let ind = smart_global_inductive y in
         let sort_of_ind = inductive_sort_family (snd (lookup_mind_specif env ind)) in
@@ -344,76 +345,86 @@ requested
           match sort_of_ind with
           | InProp ->
               if isdep then (match z with
+              | InSProp -> inds ^ "s_dep"
               | InProp -> inds ^ "_dep"
               | InSet  -> recs ^ "_dep"
               | InType -> recs ^ "t_dep")
               else ( match z with
+              | InSProp -> inds ^ "s"
               | InProp -> inds
               | InSet -> recs
               | InType -> recs ^ "t" )
           | _ ->
               if isdep then (match z with
+              | InSProp -> inds ^ "s"
               | InProp -> inds
               | InSet -> recs
               | InType -> recs ^ "t" )
               else (match z with
+              | InSProp -> inds ^ "s_nodep"
               | InProp -> inds ^ "_nodep"
               | InSet -> recs ^ "_nodep"
               | InType -> recs ^ "t_nodep")
         ) in
-        let newid = add_suffix (basename_of_global (IndRef ind)) suffix in
+        let newid = add_suffix (Nametab.basename_of_global (GlobRef.IndRef ind)) suffix in
         let newref = CAst.make newid in
           ((newref,isdep,ind,z)::l1),l2
       in
-	match t with
-	| CaseScheme (x,y,z) -> names "_case" "_case" x y z
-	| InductionScheme (x,y,z) -> names "_ind" "_rec" x y z
-	| EqualityScheme  x -> l1,((None,smart_global_inductive x)::l2)
+        match t with
+        | CaseScheme (x,y,z) -> names "_case" "_case" x y z
+        | InductionScheme (x,y,z) -> names "_ind" "_rec" x y z
+        | EqualityScheme  x -> l1,((None,smart_global_inductive x)::l2)
 
 let do_mutual_induction_scheme ?(force_mutual=false) lnamedepindsort =
   let lrecnames = List.map (fun ({CAst.v},_,_,_) -> v) lnamedepindsort
   and env0 = Global.env() in
   let sigma, lrecspec, _ =
     List.fold_right
-      (fun (_,dep,ind,sort) (evd, l, inst) -> 
+      (fun (_,dep,ind,sort) (evd, l, inst) ->
        let evd, indu, inst =
-	 match inst with
-	 | None ->
-	    let _, ctx = Global.type_of_global_in_context env0 (IndRef ind) in
+         match inst with
+         | None ->
+            let _, ctx = Typeops.type_of_global_in_context env0 (GlobRef.IndRef ind) in
             let u, ctx = UnivGen.fresh_instance_from ctx None in
             let evd = Evd.from_ctx (UState.of_context_set ctx) in
-	      evd, (ind,u), Some u
-	 | Some ui -> evd, (ind, ui), inst
+              evd, (ind,u), Some u
+         | Some ui -> evd, (ind, ui), inst
        in
           (evd, (indu,dep,sort) :: l, inst))
     lnamedepindsort (Evd.from_env env0,[],None)
   in
   let sigma, listdecl = Indrec.build_mutual_induction_scheme env0 sigma ~force_mutual lrecspec in
+  let poly =
+    (* NB: build_mutual_induction_scheme forces nonempty list of mutual inductives
+       (force_mutual is about the generated schemes) *)
+    let _,_,ind,_ = List.hd lnamedepindsort in
+    Global.is_polymorphic (GlobRef.IndRef ind)
+  in
   let declare decl fi lrecref =
     let decltype = Retyping.get_type_of env0 sigma (EConstr.of_constr decl) in
     let decltype = EConstr.to_constr sigma decltype in
-    let proof_output = Future.from_val ((decl,Univ.ContextSet.empty),Safe_typing.empty_private_constants) in
-    let cst = define fi UserIndividualRequest sigma proof_output (Some decltype) in
-    ConstRef cst :: lrecref
+    let cst = define ~poly fi sigma decl (Some decltype) in
+    GlobRef.ConstRef cst :: lrecref
   in
   let _ = List.fold_right2 declare listdecl lrecnames [] in
   fixpoint_message None lrecnames
 
-let get_common_underlying_mutual_inductive = function
+let get_common_underlying_mutual_inductive env = function
   | [] -> assert false
   | (id,(mind,i as ind))::l as all ->
       match List.filter (fun (_,(mind',_)) -> not (MutInd.equal mind mind')) l with
       | (_,ind')::_ ->
-	  raise (RecursionSchemeError (NotMutualInScheme (ind,ind')))
+          raise (RecursionSchemeError (env, NotMutualInScheme (ind,ind')))
       | [] ->
-	  if not (List.distinct_f Int.compare (List.map snd (List.map snd all)))
+          if not (List.distinct_f Int.compare (List.map snd (List.map snd all)))
           then user_err Pp.(str "A type occurs twice");
-	  mind,
-	  List.map_filter
+          mind,
+          List.map_filter
             (function (Some id,(_,i)) -> Some (i,id.CAst.v) | (None,_) -> None) all
 
 let do_scheme l =
-  let ischeme,escheme = split_scheme l in
+  let env = Global.env() in
+  let ischeme,escheme = split_scheme env l in
 (* we want 1 kind of scheme at a time so we check if the user
 tried to declare different schemes at once *)
     if not (List.is_empty ischeme) && not (List.is_empty escheme)
@@ -422,9 +433,9 @@ tried to declare different schemes at once *)
     else (
       if not (List.is_empty ischeme) then do_mutual_induction_scheme ischeme
       else
-	let mind,l = get_common_underlying_mutual_inductive escheme in
-	declare_beq_scheme_with l mind;
-	declare_eq_decidability_scheme_with l mind
+        let mind,l = get_common_underlying_mutual_inductive env escheme in
+        declare_beq_scheme_with l mind;
+        declare_eq_decidability_scheme_with l mind
     )
 
 (**********************************************************************)
@@ -442,31 +453,48 @@ let fold_left' f = function
     [] -> invalid_arg "fold_left'"
   | hd :: tl -> List.fold_left f hd tl
 
-let mk_coq_and sigma = Evarutil.new_global sigma (Coqlib.build_coq_and ())
-let mk_coq_conj sigma = Evarutil.new_global sigma (Coqlib.build_coq_conj ())
+let mk_coq_and sigma = Evarutil.new_global sigma (Coqlib.lib_ref "core.and.type")
+let mk_coq_conj sigma = Evarutil.new_global sigma (Coqlib.lib_ref "core.and.conj")
+
+let mk_coq_prod sigma = Evarutil.new_global sigma (Coqlib.lib_ref "core.prod.type")
+let mk_coq_pair sigma = Evarutil.new_global sigma (Coqlib.lib_ref "core.prod.intro")
 
 let build_combined_scheme env schemes =
-  let evdref = ref (Evd.from_env env) in
-  let defs = List.map (fun cst ->
-    let evd, c = Evd.fresh_constant_instance env !evdref cst in
-    evdref := evd; (c, Typeops.type_of_constant_in env c)) schemes in
+  let sigma = Evd.from_env env in
+  let sigma, defs = List.fold_left_map (fun sigma cst ->
+    let sigma, c = Evd.fresh_constant_instance env sigma cst in
+    sigma, (c, Typeops.type_of_constant_in env c)) sigma schemes in
   let find_inductive ty =
     let (ctx, arity) = decompose_prod ty in
     let (_, last) = List.hd ctx in
       match Constr.kind last with
-	| App (ind, args) ->
-	    let ind = destInd ind in
-	    let (_,spec) = Inductive.lookup_mind_specif env (fst ind) in
-	      ctx, ind, spec.mind_nrealargs
-	| _ -> ctx, destInd last, 0
+        | App (ind, args) ->
+            let ind = destInd ind in
+            let (_,spec) = Inductive.lookup_mind_specif env (fst ind) in
+              ctx, ind, spec.mind_nrealargs
+        | _ -> ctx, destInd last, 0
   in
   let (c, t) = List.hd defs in
   let ctx, ind, nargs = find_inductive t in
+  (* We check if ALL the predicates are in Prop, if so we use propositional
+     conjunction '/\', otherwise we use the simple product '*'.
+  *)
+  let inprop =
+    let inprop (_,t) =
+      Retyping.get_sort_family_of env sigma (EConstr.of_constr t)
+      == Sorts.InProp
+    in
+    List.for_all inprop defs
+  in
+  let mk_and, mk_conj =
+    if inprop
+    then (mk_coq_and, mk_coq_conj)
+    else (mk_coq_prod, mk_coq_pair)
+  in
   (* Number of clauses, including the predicates quantification *)
-  let prods = nb_prod !evdref (EConstr.of_constr t) - (nargs + 1) in
-  let sigma, coqand  = mk_coq_and !evdref in
-  let sigma, coqconj = mk_coq_conj sigma in
-  let () = evdref := sigma in
+  let prods = nb_prod sigma (EConstr.of_constr t) - (nargs + 1) in
+  let sigma, coqand  = mk_and sigma in
+  let sigma, coqconj = mk_conj sigma in
   let relargs = rel_vect 0 prods in
   let concls = List.rev_map
     (fun (cst, t) ->
@@ -475,15 +503,16 @@ let build_combined_scheme env schemes =
   let concl_bod, concl_typ =
     fold_left'
       (fun (accb, acct) (cst, x) ->
-	mkApp (EConstr.to_constr !evdref coqconj, [| x; acct; cst; accb |]),
-	mkApp (EConstr.to_constr !evdref coqand, [| x; acct |])) concls
+        mkApp (EConstr.to_constr sigma coqconj, [| x; acct; cst; accb |]),
+        mkApp (EConstr.to_constr sigma coqand, [| x; acct |])) concls
   in
   let ctx, _ =
     list_split_rev_at prods
       (List.rev_map (fun (x, y) -> LocalAssum (x, y)) ctx) in
   let typ = List.fold_left (fun d c -> Term.mkProd_wo_LetIn c d) concl_typ ctx in
   let body = it_mkLambda_or_LetIn concl_bod ctx in
-  (!evdref, body, typ)
+  let sigma = Typing.check env sigma (EConstr.of_constr body) (EConstr.of_constr typ) in
+  (sigma, body, typ)
 
 let do_combined_scheme name schemes =
   let open CAst in
@@ -495,8 +524,14 @@ let do_combined_scheme name schemes =
       schemes
   in
   let sigma,body,typ = build_combined_scheme (Global.env ()) csts in
-  let proof_output = Future.from_val ((body,Univ.ContextSet.empty),Safe_typing.empty_private_constants) in
-  ignore (define name.v UserIndividualRequest sigma proof_output (Some typ));
+  (* It is possible for the constants to have different universe
+     polymorphism from each other, however that is only when the user
+     manually defined at least one of them (as Scheme would pick the
+     polymorphism of the inductive block). In that case if they want
+     some other polymorphism they can also manually define the
+     combined scheme. *)
+  let poly = Global.is_polymorphic (GlobRef.ConstRef (List.hd csts)) in
+  ignore (define ~poly name.v sigma body (Some typ));
   fixpoint_message None [name.v]
 
 (**********************************************************************)
@@ -507,7 +542,7 @@ let declare_default_schemes kn =
   let mib = Global.lookup_mind kn in
   let n = Array.length mib.mind_packets in
   if !elim_flag && (mib.mind_finite <> Declarations.BiFinite || !bifinite_elim_flag)
-     && mib.mind_typing_flags.check_guarded then
+     && mib.mind_typing_flags.check_positive then
     declare_induction_schemes kn;
   if !case_flag then map_inductive_block declare_one_case_analysis_scheme kn n;
   if is_eq_flag() then try_declare_beq_scheme kn;
