@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -23,7 +23,6 @@ open Libobject
 open Constrintern
 open Vernacexpr
 open Libnames
-open Tok
 open Notation
 open Nameops
 
@@ -33,11 +32,9 @@ open Nameops
 let cache_token (_,s) = CLexer.add_keyword s
 
 let inToken : string -> obj =
-  declare_object {(default_object "TOKEN") with
-       open_function = (fun i o -> if Int.equal i 1 then cache_token o);
-       cache_function = cache_token;
-       subst_function = Libobject.ident_subst_function;
-       classify_function = (fun o -> Substitute o)}
+  declare_object @@ global_object_nodischarge "TOKEN"
+    ~cache:cache_token
+    ~subst:(Some Libobject.ident_subst_function)
 
 let add_token_obj s = Lib.add_anonymous_leaf (inToken s)
 
@@ -53,12 +50,12 @@ let pr_entry e =
   str (Buffer.contents entry_buf)
 
 let pr_registered_grammar name =
-  let gram = try Some (Pcoq.find_grammars_by_name name) with Not_found -> None in
+  let gram = Pcoq.find_grammars_by_name name in
   match gram with
-  | None -> user_err Pp.(str "Unknown or unprintable grammar entry.")
-  | Some entries ->
+  | [] -> user_err Pp.(str "Unknown or unprintable grammar entry.")
+  | entries ->
     let pr_one (Pcoq.AnyEntry e) =
-      str "Entry " ++ str (Pcoq.Gram.Entry.name e) ++ str " is" ++ fnl () ++
+      str "Entry " ++ str (Pcoq.Entry.name e) ++ str " is" ++ fnl () ++
       pr_entry e
     in
     prlist pr_one entries
@@ -88,6 +85,8 @@ let pr_grammar = function
       pr_entry Pvernac.Vernac_.gallina_ext
   | name -> pr_registered_grammar name
 
+let pr_custom_grammar name = pr_registered_grammar ("constr:"^name)
+
 (**********************************************************************)
 (* Parse a format (every terminal starting with a letter or a single
    quote (except a single quote alone) must be quoted) *)
@@ -132,37 +131,37 @@ let parse_format ({CAst.loc;v=str} : lstring) =
       (* Parse " // " *)
       | '/' when i+1 < len && str.[i+1] == '/' ->
           (* We discard the useless n spaces... *)
-	  push_token (make_loc (i-n) (i+1)) (UnpCut PpFnl)
+          push_token (make_loc (i-n) (i+1)) (UnpCut PpFnl)
             (parse_token 1 (close_quotation i (i+2)))
       (* Parse " .. / .. " *)
       | '/' when i+1 < len ->
-	  let p = spaces 0 (i+1) in
-	  push_token (make_loc (i-n) (i+p)) (UnpCut (PpBrk (n,p)))
+          let p = spaces 0 (i+1) in
+          push_token (make_loc (i-n) (i+p)) (UnpCut (PpBrk (n,p)))
             (parse_token 1 (close_quotation i (i+p+1)))
       | c ->
       (* The spaces are real spaces *)
       push_white i n (match c with
       | '[' ->
-	  if i+1 < len then match str.[i+1] with
-	    (* Parse " [h .. ",  *)
-	    | 'h' when i+1 <= len && str.[i+2] == 'v' ->
-		  (parse_box i (fun n -> PpHVB n) (i+3))
-		(* Parse " [v .. ",  *)
-	    | 'v' ->
-		    parse_box i (fun n -> PpVB n) (i+2)
-		(* Parse " [ .. ",  *)
-	    | ' ' | '\'' ->
-		parse_box i (fun n -> PpHOVB n) (i+1)
-	    | _ -> user_err ?loc:(make_loc i i) Pp.(str "\"v\", \"hv\", \" \" expected after \"[\" in format.")
-	  else user_err ?loc:(make_loc i i) Pp.(str "\"v\", \"hv\" or \" \" expected after \"[\" in format.")
+          if i+1 < len then match str.[i+1] with
+            (* Parse " [h .. ",  *)
+            | 'h' when i+1 <= len && str.[i+2] == 'v' ->
+                  (parse_box i (fun n -> PpHVB n) (i+3))
+                (* Parse " [v .. ",  *)
+            | 'v' ->
+                    parse_box i (fun n -> PpVB n) (i+2)
+                (* Parse " [ .. ",  *)
+            | ' ' | '\'' ->
+                parse_box i (fun n -> PpHOVB n) (i+1)
+            | _ -> user_err ?loc:(make_loc i i) Pp.(str "\"v\", \"hv\", \" \" expected after \"[\" in format.")
+          else user_err ?loc:(make_loc i i) Pp.(str "\"v\", \"hv\" or \" \" expected after \"[\" in format.")
       (* Parse "]"  *)
       | ']' ->
-	  ((i,[]) :: parse_token 1 (close_quotation i (i+1)))
+          ((i,[]) :: parse_token 1 (close_quotation i (i+1)))
       (* Parse a non formatting token *)
       | c ->
-	  let n = nonspaces true 0 i in
-	  push_token (make_loc i (i+n-1)) (UnpTerminal (String.sub str (i-1) (n+2)))
-	    (parse_token 1 (close_quotation i (i+n))))
+          let n = nonspaces true 0 i in
+          push_token (make_loc i (i+n-1)) (UnpTerminal (String.sub str (i-1) (n+2)))
+            (parse_token 1 (close_quotation i (i+n))))
     else
       if Int.equal n 0 then []
       else user_err ?loc:(make_loc (len-n) len) Pp.(str "Ending spaces non part of a format annotation.")
@@ -175,7 +174,7 @@ let parse_format ({CAst.loc;v=str} : lstring) =
     if i < len then match str.[i] with
       (* Parse a ' *)
       |	'\'' when i+1 >= len || str.[i+1] == ' ' ->
-	  push_white (i-n) (n-k) (push_token (make_loc i (i+1)) (UnpTerminal "'") (parse_token 1 (i+1)))
+          push_white (i-n) (n-k) (push_token (make_loc i (i+1)) (UnpTerminal "'") (parse_token 1 (i+1)))
       (* Parse the beginning of a quoted expression *)
       |	'\'' ->
           parse_quoted (n-k) (i+1)
@@ -253,7 +252,7 @@ let quote_notation_token x =
 let is_numeral symbs =
   match List.filter (function Break _ -> false | _ -> true) symbs with
   | ([Terminal "-"; Terminal x] | [Terminal x]) ->
-      (try let _ = Bigint.of_string x in true with Failure _ -> false)
+      NumTok.of_string x <> None
   | _ ->
       false
 
@@ -262,11 +261,11 @@ let rec get_notation_vars onlyprint = function
   | NonTerminal id :: sl ->
       let vars = get_notation_vars onlyprint sl in
       if Id.equal id ldots_var then vars else
-	(* don't check for nonlinearity if printing only, see Bug 5526 *)
-	if not onlyprint && Id.List.mem id vars then 
-	  user_err ~hdr:"Metasyntax.get_notation_vars"
+        (* don't check for nonlinearity if printing only, see Bug 5526 *)
+        if not onlyprint && Id.List.mem id vars then
+          user_err ~hdr:"Metasyntax.get_notation_vars"
             (str "Variable " ++ Id.print id ++ str " occurs more than once.")
-	else id::vars
+        else id::vars
   | (Terminal _ | Break _) :: sl -> get_notation_vars onlyprint sl
   | SProdList _ :: _ -> assert false
 
@@ -287,7 +286,7 @@ let pr_notation_entry = function
   | InConstrEntry -> str "constr"
   | InCustomEntry s -> str "custom " ++ str s
 
-let prec_assoc = function
+let prec_assoc = let open Gramlib.Gramext in function
   | RightA -> (L,E)
   | LeftA -> (E,L)
   | NonA -> (L,L)
@@ -394,23 +393,23 @@ let make_hunks etyps symbols from =
   let vars,typs = List.split etyps in
   let rec make b = function
     | NonTerminal m :: prods ->
-	let i = index_id m vars in
+        let i = index_id m vars in
         let u = unparsing_metavar i from typs in
         if is_next_non_terminal b prods then
           (None, u) :: add_break_if_none 1 b (make b prods)
-	else
+        else
           (None, u) :: make_with_space b prods
     | Terminal s :: prods
          when (* true to simulate presence of non-terminal *) b || List.exists is_non_terminal prods ->
         if (is_comma s || is_operator s) then
           (* Always a breakable space after comma or separator *)
           (None, UnpTerminal s) :: add_break_if_none 1 b (make b prods)
-	else if is_right_bracket s && is_next_terminal prods then
+        else if is_right_bracket s && is_next_terminal prods then
           (* Always no space after right bracked, but possibly a break *)
           (None, UnpTerminal s) :: add_break_if_none 0 b (make b prods)
         else if is_left_bracket s  && is_next_non_terminal b prods then
           (None, UnpTerminal s) :: make b prods
-	else if not (is_next_break prods) then
+        else if not (is_next_break prods) then
           (* Add rigid space, no break, unless user asked for something *)
           (None, UnpTerminal (s^" ")) :: make b prods
         else
@@ -427,20 +426,20 @@ let make_hunks etyps symbols from =
         add_break n (make b prods)
 
     | SProdList (m,sl) :: prods ->
-	let i = index_id m vars in
-	let typ = List.nth typs (i-1) in
-	let _,prec = precedence_of_entry_type from typ in
+        let i = index_id m vars in
+        let typ = List.nth typs (i-1) in
+        let _,prec = precedence_of_entry_type from typ in
         let sl' =
           (* If no separator: add a break *)
-	  if List.is_empty sl then add_break 1 []
+          if List.is_empty sl then add_break 1 []
           (* We add NonTerminal for simulation but remove it afterwards *)
           else make true sl in
-	let hunk = match typ with
-	  | ETConstr _ -> UnpListMetaVar (i,prec,List.map snd sl')
-	  | ETBinder isopen ->
-	      check_open_binder isopen sl m;
-	      UnpBinderListMetaVar (i,isopen,List.map snd sl')
-	  | _ -> assert false in
+        let hunk = match typ with
+          | ETConstr _ -> UnpListMetaVar (i,prec,List.map snd sl')
+          | ETBinder isopen ->
+              check_open_binder isopen sl m;
+              UnpBinderListMetaVar (i,isopen,List.map snd sl')
+          | _ -> assert false in
         (None, hunk) :: make_with_space b prods
 
     | [] -> []
@@ -520,7 +519,7 @@ let read_recursive_format sl fmt =
     let sl = skip_var_in_recursive_format fmt in
     try split_format_at_ldots [] sl with Exit -> error_not_same ?loc:(fst (List.last (if sl = [] then fmt else sl))) () in
   let rec get_tail = function
-    | (loc,a) :: sepfmt, (_,b) :: fmt when Pervasives.(=) a b -> get_tail (sepfmt, fmt) (* FIXME *)
+    | (loc,a) :: sepfmt, (_,b) :: fmt when (=) a b -> get_tail (sepfmt, fmt) (* FIXME *)
     | [], tail -> skip_var_in_recursive_format tail
     | (loc,_) :: _, ([] | (_,UnpTerminal _) :: _)-> error_not_same ?loc ()
     | _, (loc,_)::_ -> error_not_same ?loc () in
@@ -553,11 +552,11 @@ let hunks_of_format (from,(vars,typs)) symfmt =
       if not (List.is_empty sl) then error_format ?loc:(find_prod_list_loc loc_slfmt fmt) ();
       let symbs, l = aux (symbs,rfmt) in
       let hunk = match typ with
-	| ETConstr _ -> UnpListMetaVar (i,prec,slfmt)
-	| ETBinder isopen ->
-	    check_open_binder isopen sl m;
-	    UnpBinderListMetaVar (i,isopen,slfmt)
-	| _ -> assert false in
+        | ETConstr _ -> UnpListMetaVar (i,prec,slfmt)
+        | ETBinder isopen ->
+            check_open_binder isopen sl m;
+            UnpBinderListMetaVar (i,isopen,slfmt)
+        | _ -> assert false in
       symbs, hunk :: l
   | symbs, [] -> symbs, []
   | Break _ :: symbs, fmt -> warn_format_break (); aux (symbs,fmt)
@@ -577,20 +576,20 @@ let is_not_small_constr = function
   | _ -> false
 
 let rec define_keywords_aux = function
-  | GramConstrNonTerminal(e,Some _) as n1 :: GramConstrTerminal(IDENT k) :: l
+  | GramConstrNonTerminal(e,Some _) as n1 :: GramConstrTerminal(Tok.PIDENT (Some k)) :: l
       when is_not_small_constr e ->
       Flags.if_verbose Feedback.msg_info (str "Identifier '" ++ str k ++ str "' now a keyword");
       CLexer.add_keyword k;
-      n1 :: GramConstrTerminal(KEYWORD k) :: define_keywords_aux l
+      n1 :: GramConstrTerminal(Tok.PKEYWORD k) :: define_keywords_aux l
   | n :: l -> n :: define_keywords_aux l
   | [] -> []
 
   (* Ensure that IDENT articulation terminal symbols are keywords *)
 let define_keywords = function
-  | GramConstrTerminal(IDENT k)::l ->
+  | GramConstrTerminal(Tok.PIDENT (Some k))::l ->
       Flags.if_verbose Feedback.msg_info (str "Identifier '" ++ str k ++ str "' now a keyword");
       CLexer.add_keyword k;
-      GramConstrTerminal(KEYWORD k) :: define_keywords_aux l
+      GramConstrTerminal(Tok.PKEYWORD k) :: define_keywords_aux l
   | l -> define_keywords_aux l
 
 let distribute a ll = List.map (fun l -> a @ l) ll
@@ -612,7 +611,7 @@ let expand_list_rule s typ tkl x n p ll =
   else if Int.equal i (p+n) then
     let hds =
       GramConstrListMark (p+n,true,p) :: hds
-      @	[GramConstrNonTerminal (ETProdConstrList (typ,tkl), Some x)] in
+      @	[GramConstrNonTerminal (ETProdConstrList (s, typ,tkl), Some x)] in
     distribute hds ll
   else
     distribute (GramConstrListMark (i+1,false,p) :: hds @ [main]) ll @
@@ -657,7 +656,7 @@ let make_production etyps symbols =
           (List.map (function Terminal s -> [CLexer.terminal s]
             | Break _ -> []
             | _ -> anomaly (Pp.str "Found a non terminal token in recursive notation separator.")) sl) in
-	match List.assoc x etyps with
+        match List.assoc x etyps with
         | ETConstr (s,_,typ) ->
             let p,l' = include_possible_similar_trailing_pattern typ etyps sl l in
             expand_list_rule s typ tkl x 1 p (aux l')
@@ -685,7 +684,7 @@ let border = function
   | (_,(ETConstr(_,_,(_,BorderProd (_,a))))) :: _ -> a
   | _ -> None
 
-let recompute_assoc typs =
+let recompute_assoc typs = let open Gramlib.Gramext in
   match border typs, border (List.rev typs) with
     | Some LeftA, Some RightA -> assert false
     | Some LeftA, _ -> Some LeftA
@@ -713,7 +712,7 @@ let pr_level ntn (from,fromlevel,args,typs) =
   prlist_with_sep pr_comma (pr_arg_level fromlevel) (List.combine args typs)
 
 let error_incompatible_level ntn oldprec prec =
-  user_err 
+  user_err
     (str "Notation " ++ pr_notation ntn ++ str " is already defined" ++ spc() ++
     pr_level ntn oldprec ++
     spc() ++ str "while it is now required to be" ++ spc() ++
@@ -733,13 +732,8 @@ type syntax_extension = {
   synext_notgram : notation_grammar;
   synext_unparsing : unparsing list;
   synext_extra : (string * string) list;
-  synext_compat : Flags.compat_version option;
 }
 
-let is_active_compat = function
-| None -> true
-| Some v -> 0 <= Flags.version_compare v !Flags.compat_version
-
 type syntax_extension_obj = locality_flag * syntax_extension
 
 let check_and_extend_constr_grammar ntn rule =
@@ -760,7 +754,7 @@ let cache_one_syntax_extension se =
     let oldprec = Notgram_ops.level_of_notation ~onlyprint ntn in
     if not (Notgram_ops.level_eq prec oldprec) then error_incompatible_level ntn oldprec prec;
   with Not_found ->
-    if is_active_compat se.synext_compat then begin
+    begin
       (* Reserve the notation level *)
       Notgram_ops.declare_notation_level ntn prec ~onlyprint;
       (* Declare the parsing rule *)
@@ -802,7 +796,7 @@ let inSyntaxExtension : syntax_extension_obj -> obj =
 module NotationMods = struct
 
 type notation_modifier = {
-  assoc         : gram_assoc option;
+  assoc         : Gramlib.Gramext.g_assoc option;
   level         : int option;
   custom        : notation_entry;
   etyps         : (Id.t * simple_constr_prod_entry_key) list;
@@ -835,17 +829,17 @@ let interp_modifiers modl = let open NotationMods in
   let rec interp subtyps acc = function
     | [] -> subtyps, acc
     | SetEntryType (s,typ) :: l ->
-	let id = Id.of_string s in
-	if Id.List.mem_assoc id acc.etyps then
-	  user_err ~hdr:"Metasyntax.interp_modifiers"
+        let id = Id.of_string s in
+        if Id.List.mem_assoc id acc.etyps then
+          user_err ~hdr:"Metasyntax.interp_modifiers"
             (str s ++ str " is already assigned to an entry or constr level.");
         interp subtyps { acc with etyps = (id,typ) :: acc.etyps; } l
     | SetItemLevel ([],bko,n) :: l ->
         interp subtyps acc l
     | SetItemLevel (s::idl,bko,n) :: l ->
-	let id = Id.of_string s in
-	if Id.List.mem_assoc id acc.etyps then
-	  user_err ~hdr:"Metasyntax.interp_modifiers"
+        let id = Id.of_string s in
+        if Id.List.mem_assoc id acc.etyps then
+          user_err ~hdr:"Metasyntax.interp_modifiers"
             (str s ++ str " is already assigned to an entry or constr level.");
         interp ((id,bko,n)::subtyps) acc (SetItemLevel (idl,bko,n)::l)
     | SetLevel n :: l ->
@@ -877,7 +871,7 @@ let interp_modifiers modl = let open NotationMods in
            user_err (str "Entry is already assigned to custom " ++ str s ++ (match acc.level with None -> mt () | Some lev -> str " at level " ++ int lev) ++ str ".");
         interp subtyps { acc with custom = InCustomEntry s; level = n } l
     | SetAssoc a :: l ->
-	if not (Option.is_empty acc.assoc) then user_err Pp.(str "An associativity is given more than once.");
+        if not (Option.is_empty acc.assoc) then user_err Pp.(str "An associativity is given more than once.");
         interp subtyps { acc with assoc = Some a; } l
      | SetOnlyParsing :: l ->
         interp subtyps { acc with only_parsing = true; } l
@@ -886,7 +880,7 @@ let interp_modifiers modl = let open NotationMods in
     | SetCompatVersion v :: l ->
         interp subtyps { acc with compat = Some v; } l
     | SetFormat ("text",s) :: l ->
-	if not (Option.is_empty acc.format) then user_err Pp.(str "A format is given more than once.");
+        if not (Option.is_empty acc.format) then user_err Pp.(str "A format is given more than once.");
         interp subtyps { acc with format = Some s; } l
     | SetFormat (k,s) :: l ->
         interp subtyps { acc with extra = (k,s.CAst.v)::acc.extra; } l
@@ -935,10 +929,6 @@ let is_only_printing mods =
   let test = function SetOnlyPrinting -> true | _ -> false in
   List.exists test mods
 
-let get_compat_version mods =
-  let test = function SetCompatVersion v -> Some v | _ -> None in
-  try Some (List.find_map test mods) with Not_found -> None
-
 (* Compute precedences from modifiers (or find default ones) *)
 
 let set_entry_type from etyps (x,typ) =
@@ -963,11 +953,11 @@ let join_auxiliary_recursive_types recvars etyps =
     | None, None -> typs
     | Some _, None -> typs
     | None, Some ytyp -> (x,ytyp)::typs
-    | Some xtyp, Some ytyp when Pervasives.(=) xtyp ytyp -> typs (* FIXME *)
+    | Some xtyp, Some ytyp when (=) xtyp ytyp -> typs (* FIXME *)
     | Some xtyp, Some ytyp ->
-	user_err 
-	  (strbrk "In " ++ Id.print x ++ str " .. " ++ Id.print y ++
-	   strbrk ", both ends have incompatible types."))
+        user_err
+          (strbrk "In " ++ Id.print x ++ str " .. " ++ Id.print y ++
+           strbrk ", both ends have incompatible types."))
     recvars etyps
 
 let internalization_type_of_entry_type = function
@@ -1113,7 +1103,7 @@ let find_precedence custom lev etyps symbols onlyprint =
         | (ETIdent | ETBigint | ETGlobal), _ ->
             begin match lev with
             | None ->
-	      ([Feedback.msg_info ?loc:None ,strbrk "Setting notation at level 0."],0)
+              ([Feedback.msg_info ?loc:None ,strbrk "Setting notation at level 0."],0)
             | Some 0 ->
               ([],0)
             | _ ->
@@ -1125,12 +1115,12 @@ let find_precedence custom lev etyps symbols onlyprint =
               user_err Pp.(str "Need an explicit level.")
             else [],Option.get lev
       with Not_found ->
-	if Option.is_empty lev then
-	  user_err Pp.(str "A left-recursive notation must have an explicit level.")
-	else [],Option.get lev)
+        if Option.is_empty lev then
+          user_err Pp.(str "A left-recursive notation must have an explicit level.")
+        else [],Option.get lev)
   | Some (Terminal _) when last_is_terminal () ->
       if Option.is_empty lev then
-	([Feedback.msg_info ?loc:None ,strbrk "Setting notation at level 0."], 0)
+        ([Feedback.msg_info ?loc:None ,strbrk "Setting notation at level 0."], 0)
       else [],Option.get lev
   | Some _ ->
       if Option.is_empty lev then user_err Pp.(str "Cannot determine the level.");
@@ -1156,7 +1146,7 @@ let remove_curly_brackets l =
             let br',next' = skip_break [] l' in
             (match next' with
               | Terminal "}" as t2 :: l'' ->
-		  if deb && List.is_empty l'' then [t1;x;t2] else begin
+                  if deb && List.is_empty l'' then [t1;x;t2] else begin
                     check_curly_brackets_notation_exists ();
                     x :: aux false l''
                   end
@@ -1178,7 +1168,7 @@ module SynData = struct
     (* Fields coming from the vernac-level modifiers *)
     only_parsing  : bool;
     only_printing : bool;
-    compat        : Flags.compat_version option;
+    deprecation   : Deprecation.t option;
     format        : lstring option;
     extra         : (string * string) list;
 
@@ -1223,14 +1213,34 @@ let check_locality_compatibility local custom i_typs =
                     strbrk " which is local."))
       (List.uniquize allcustoms)
 
-let compute_syntax_data local df modifiers =
+let warn_deprecated_compat =
+  CWarnings.create ~name:"deprecated-compat" ~category:"deprecated"
+    (fun () -> Pp.(str"The \"compat\" modifier is deprecated." ++ spc () ++
+                   str"Please use the \"deprecated\" attributed instead."))
+
+(* Returns the new deprecation and the onlyparsing status. This should be
+removed together with the compat syntax modifier. *)
+let merge_compat_deprecation compat deprecation =
+  match compat, deprecation with
+  | Some Flags.Current, _ -> deprecation, true
+  | Some _, Some _ ->
+    CErrors.user_err Pp.(str"The \"compat\" modifier cannot be used with the \"deprecated\" attribute."
+                        ++ spc () ++ str"Please use only the latter.")
+  | Some v, None ->
+    warn_deprecated_compat ();
+    Some (Deprecation.make ~since:(Flags.pr_version v) ()), true
+  | None, Some _ -> deprecation, true
+  | None, None -> deprecation, false
+
+let compute_syntax_data ~local deprecation df modifiers =
   let open SynData in
   let open NotationMods in
   let mods = interp_modifiers modifiers in
   let onlyprint = mods.only_printing in
   let onlyparse = mods.only_parsing in
+  let deprecation, _ = merge_compat_deprecation mods.compat deprecation in
   if onlyprint && onlyparse then user_err (str "A notation cannot be both 'only printing' and 'only parsing'.");
-  let assoc = Option.append mods.assoc (Some NonA) in
+  let assoc = Option.append mods.assoc (Some Gramlib.Gramext.NonA) in
   let (recvars,mainvars,symbols) = analyze_notation_tokens ~onlyprint df in
   let _ = check_useless_entry_types recvars mainvars mods.etyps in
   let _ = check_binder_type recvars mods.etyps in
@@ -1266,7 +1276,7 @@ let compute_syntax_data local df modifiers =
 
     only_parsing  = mods.only_parsing;
     only_printing = mods.only_printing;
-    compat        = mods.compat;
+    deprecation;
     format        = mods.format;
     extra         = mods.extra;
 
@@ -1282,9 +1292,9 @@ let compute_syntax_data local df modifiers =
     not_data    = sy_fulldata;
   }
 
-let compute_pure_syntax_data local df mods =
+let compute_pure_syntax_data ~local df mods =
   let open SynData in
-  let sd = compute_syntax_data local df mods in
+  let sd = compute_syntax_data ~local None df mods in
   let msgs =
     if sd.only_parsing then
       (Feedback.msg_warning ?loc:None,
@@ -1302,23 +1312,33 @@ type notation_obj = {
   notobj_coercion : entry_coercion_kind option;
   notobj_onlyparse : bool;
   notobj_onlyprint : bool;
-  notobj_compat : Flags.compat_version option;
+  notobj_deprecation : Deprecation.t option;
   notobj_notation : notation * notation_location;
 }
 
-let load_notation _ (_, nobj) =
-  Option.iter Notation.declare_scope nobj.notobj_scope
+let load_notation_common silently_define_scope_if_undefined _ (_, nobj) =
+  (* When the default shall be to require that a scope already exists *)
+  (* the call to ensure_scope will have to be removed *)
+  if silently_define_scope_if_undefined then
+    (* Don't warn if the scope is not defined: *)
+    (* there was already a warning at "cache" time *)
+    Option.iter Notation.declare_scope nobj.notobj_scope
+  else
+    Option.iter Notation.ensure_scope nobj.notobj_scope
+
+let load_notation =
+  load_notation_common true
 
 let open_notation i (_, nobj) =
   let scope = nobj.notobj_scope in
   let (ntn, df) = nobj.notobj_notation in
   let pat = nobj.notobj_interp in
   let onlyprint = nobj.notobj_onlyprint  in
+  let deprecation = nobj.notobj_deprecation in
   let fresh = not (Notation.exists_notation_in_scope scope ntn onlyprint pat) in
-  let active = is_active_compat nobj.notobj_compat in
-  if Int.equal i 1 && fresh && active then begin
+  if Int.equal i 1 && fresh then begin
     (* Declare the interpretation *)
-    let () = Notation.declare_notation_interpretation ntn scope pat df ~onlyprint in
+    let () = Notation.declare_notation_interpretation ntn scope pat df ~onlyprint deprecation in
     (* Declare the uninterpretation *)
     if not nobj.notobj_onlyparse then
       Notation.declare_uninterpretation (NotationRule (scope, ntn)) pat;
@@ -1331,7 +1351,7 @@ let open_notation i (_, nobj) =
   end
 
 let cache_notation o =
-  load_notation 1 o;
+  load_notation_common false 1 o;
   open_notation 1 o
 
 let subst_notation (subst, nobj) =
@@ -1351,7 +1371,7 @@ let inNotation : notation_obj -> obj =
 (**********************************************************************)
 
 let with_lib_stk_protection f x =
-  let fs = Lib.freeze ~marshallable:`No in
+  let fs = Lib.freeze ~marshallable:false in
   try let a = f x in Lib.unfreeze fs; a
   with reraise ->
     let reraise = CErrors.push reraise in
@@ -1379,7 +1399,6 @@ let recover_notation_syntax ntn =
       synext_notgram = pa_rule;
       synext_unparsing = pp_rule;
       synext_extra = pp_extra_rules;
-      synext_compat = None;
     }
   with Not_found ->
     raise NoSyntaxRule
@@ -1428,7 +1447,6 @@ let make_syntax_rules (sd : SynData.syn_data) = let open SynData in
     synext_notgram  = { notgram_onlyprinting = sd.only_printing; notgram_rules = pa_rule };
     synext_unparsing = pp_rule;
     synext_extra  = sd.extra;
-    synext_compat = sd.compat;
   }
 
 (**********************************************************************)
@@ -1438,9 +1456,9 @@ let to_map l =
   let fold accu (x, v) = Id.Map.add x v accu in
   List.fold_left fold Id.Map.empty l
 
-let add_notation_in_scope local df env c mods scope =
+let add_notation_in_scope ~local deprecation df env c mods scope =
   let open SynData in
-  let sd = compute_syntax_data local df mods in
+  let sd = compute_syntax_data ~local deprecation df mods in
   (* Prepare the interpretation *)
   (* Prepare the parsing and printing rules *)
   let sy_rules = make_syntax_rules sd in
@@ -1457,11 +1475,11 @@ let add_notation_in_scope local df env c mods scope =
     notobj_local = local;
     notobj_scope = scope;
     notobj_interp = (List.map_filter map i_vars, ac);
-    (** Order is important here! *)
+    (* Order is important here! *)
     notobj_onlyparse = onlyparse;
     notobj_coercion = coe;
     notobj_onlyprint = sd.only_printing;
-    notobj_compat = sd.compat;
+    notobj_deprecation = sd.deprecation;
     notobj_notation = sd.info;
   } in
   (* Ready to change the global state *)
@@ -1470,13 +1488,13 @@ let add_notation_in_scope local df env c mods scope =
   Lib.add_anonymous_leaf (inNotation notation);
   sd.info
 
-let add_notation_interpretation_core local df env ?(impls=empty_internalization_env) c scope onlyparse onlyprint compat =
+let add_notation_interpretation_core ~local df env ?(impls=empty_internalization_env) c scope onlyparse onlyprint deprecation =
   let (recvars,mainvars,symbs) = analyze_notation_tokens ~onlyprint df in
   (* Recover types of variables and pa/pp rules; redeclare them if needed *)
   let level, i_typs, onlyprint = if not (is_numeral symbs) then begin
     let sy = recover_notation_syntax (make_notation_key InConstrEntrySomeLevel symbs) in
     let () = Lib.add_anonymous_leaf (inSyntaxExtension (local,sy)) in
-    (** If the only printing flag has been explicitly requested, put it back *)
+    (* If the only printing flag has been explicitly requested, put it back *)
     let onlyprint = onlyprint || sy.synext_notgram.notgram_onlyprinting in
     let _,_,_,typs = sy.synext_level in
     Some sy.synext_level, typs, onlyprint
@@ -1497,11 +1515,11 @@ let add_notation_interpretation_core local df env ?(impls=empty_internalization_
     notobj_local = local;
     notobj_scope = scope;
     notobj_interp = (List.map_filter map i_vars, ac);
-    (** Order is important here! *)
+    (* Order is important here! *)
     notobj_onlyparse = onlyparse;
     notobj_coercion = coe;
     notobj_onlyprint = onlyprint;
-    notobj_compat = compat;
+    notobj_deprecation = deprecation;
     notobj_notation = df';
   } in
   Lib.add_anonymous_leaf (inNotation notation);
@@ -1509,46 +1527,45 @@ let add_notation_interpretation_core local df env ?(impls=empty_internalization_
 
 (* Notations without interpretation (Reserved Notation) *)
 
-let add_syntax_extension local ({CAst.loc;v=df},mods) = let open SynData in
-  let psd = compute_pure_syntax_data local df mods in
-  let sy_rules = make_syntax_rules {psd with compat = None} in
+let add_syntax_extension ~local ({CAst.loc;v=df},mods) = let open SynData in
+  let psd = compute_pure_syntax_data ~local df mods in
+  let sy_rules = make_syntax_rules {psd with deprecation = None} in
   Flags.if_verbose (List.iter (fun (f,x) -> f x)) psd.msgs;
   Lib.add_anonymous_leaf (inSyntaxExtension(local,sy_rules))
 
 (* Notations with only interpretation *)
 
 let add_notation_interpretation env ({CAst.loc;v=df},c,sc) =
-  let df' = add_notation_interpretation_core false df env c sc false false None in
+  let df' = add_notation_interpretation_core ~local:false df env c sc false false None in
   Dumpglob.dump_notation (loc,df') sc true
 
 let set_notation_for_interpretation env impls ({CAst.v=df},c,sc) =
   (try ignore
-    (Flags.silently (fun () -> add_notation_interpretation_core false df env ~impls c sc false false None) ());
+    (Flags.silently (fun () -> add_notation_interpretation_core ~local:false df env ~impls c sc false false None) ());
   with NoSyntaxRule ->
     user_err Pp.(str "Parsing rule for this notation has to be previously declared."));
   Option.iter (fun sc -> Notation.open_close_scope (false,true,sc)) sc
 
 (* Main entry point *)
 
-let add_notation local env c ({CAst.loc;v=df},modifiers) sc =
+let add_notation ~local deprecation env c ({CAst.loc;v=df},modifiers) sc =
   let df' =
    if no_syntax_modifiers modifiers then
     (* No syntax data: try to rely on a previously declared rule *)
     let onlyparse = is_only_parsing modifiers in
     let onlyprint = is_only_printing modifiers in
-    let compat = get_compat_version modifiers in
-    try add_notation_interpretation_core local df env c sc onlyparse onlyprint compat
+    try add_notation_interpretation_core ~local df env c sc onlyparse onlyprint deprecation
     with NoSyntaxRule ->
       (* Try to determine a default syntax rule *)
-      add_notation_in_scope local df env c modifiers sc
+      add_notation_in_scope ~local deprecation df env c modifiers sc
    else
     (* Declare both syntax and interpretation *)
-    add_notation_in_scope local df env c modifiers sc
+    add_notation_in_scope ~local deprecation df env c modifiers sc
   in
   Dumpglob.dump_notation (loc,df') sc true
 
 let add_notation_extra_printing_rule df k v =
-  let notk = 
+  let notk =
     let _,_, symbs = analyze_notation_tokens ~onlyprint:true df in
     make_notation_key InConstrEntrySomeLevel symbs in
   add_notation_extra_printing_rule notk k v
@@ -1557,7 +1574,7 @@ let add_notation_extra_printing_rule df k v =
 
 let inject_var x = CAst.make @@ CRef (qualid_of_ident x,None)
 
-let add_infix local env ({CAst.loc;v=inf},modifiers) pr sc =
+let add_infix ~local deprecation env ({CAst.loc;v=inf},modifiers) pr sc =
   check_infix_modifiers modifiers;
   (* check the precedence *)
   let vars = names_of_constr_expr pr in
@@ -1566,55 +1583,75 @@ let add_infix local env ({CAst.loc;v=inf},modifiers) pr sc =
   let metas = [inject_var x; inject_var y] in
   let c = mkAppC (pr,metas) in
   let df = CAst.make ?loc @@ Id.to_string x ^" "^(quote_notation_token inf)^" "^Id.to_string y in
-  add_notation local env c (df,modifiers) sc
+  add_notation ~local deprecation env c (df,modifiers) sc
 
 (**********************************************************************)
-(* Delimiters and classes bound to scopes                             *)
+(* Scopes, delimiters and classes bound to scopes                     *)
 
 type scope_command =
-  | ScopeDelim of string
+  | ScopeDeclare
+  | ScopeDelimAdd of string
+  | ScopeDelimRemove
   | ScopeClasses of scope_class list
-  | ScopeRemove
 
-let load_scope_command _ (_,(scope,dlm)) =
-  Notation.declare_scope scope
+let load_scope_command_common silently_define_scope_if_undefined _ (_,(local,scope,o)) =
+  let declare_scope_if_needed =
+    if silently_define_scope_if_undefined then Notation.declare_scope
+    else Notation.ensure_scope in
+  match o with
+  | ScopeDeclare -> Notation.declare_scope scope
+  (* When the default shall be to require that a scope already exists *)
+  (* the call to declare_scope_if_needed will have to be removed below *)
+  | ScopeDelimAdd dlm -> declare_scope_if_needed scope
+  | ScopeDelimRemove -> declare_scope_if_needed scope
+  | ScopeClasses cl -> declare_scope_if_needed scope
 
-let open_scope_command i (_,(scope,o)) =
+let load_scope_command =
+  load_scope_command_common true
+
+let open_scope_command i (_,(local,scope,o)) =
   if Int.equal i 1 then
     match o with
-    | ScopeDelim dlm -> Notation.declare_delimiters scope dlm
+    | ScopeDeclare -> ()
+    | ScopeDelimAdd dlm -> Notation.declare_delimiters scope dlm
+    | ScopeDelimRemove -> Notation.remove_delimiters scope
     | ScopeClasses cl -> List.iter (Notation.declare_scope_class scope) cl
-    | ScopeRemove -> Notation.remove_delimiters scope
 
 let cache_scope_command o =
-  load_scope_command 1 o;
+  load_scope_command_common false 1 o;
   open_scope_command 1 o
 
-let subst_scope_command (subst,(scope,o as x)) = match o with
+let subst_scope_command (subst,(local,scope,o as x)) = match o with
   | ScopeClasses cl ->
       let cl' = List.map_filter (subst_scope_class subst) cl in
       let cl' =
         if List.for_all2eq (==) cl cl' then cl
         else cl' in
-      scope, ScopeClasses cl'
+      local, scope, ScopeClasses cl'
   | _ -> x
 
-let inScopeCommand : scope_name * scope_command -> obj =
+let classify_scope_command (local, _, _ as o) =
+  if local then Dispose else Substitute o
+
+let inScopeCommand : locality_flag * scope_name * scope_command -> obj =
   declare_object {(default_object "DELIMITERS") with
       cache_function = cache_scope_command;
       open_function = open_scope_command;
       load_function = load_scope_command;
       subst_function = subst_scope_command;
-      classify_function = (fun obj -> Substitute obj)}
+      classify_function = classify_scope_command}
 
-let add_delimiters scope key =
-  Lib.add_anonymous_leaf (inScopeCommand(scope,ScopeDelim key))
+let declare_scope local scope =
+  Lib.add_anonymous_leaf (inScopeCommand(local,scope,ScopeDeclare))
 
-let remove_delimiters scope =
-  Lib.add_anonymous_leaf (inScopeCommand(scope,ScopeRemove))
+let add_delimiters local scope key =
+  Lib.add_anonymous_leaf (inScopeCommand(local,scope,ScopeDelimAdd key))
 
-let add_class_scope scope cl =
-  Lib.add_anonymous_leaf (inScopeCommand(scope,ScopeClasses cl))
+let remove_delimiters local scope =
+  Lib.add_anonymous_leaf (inScopeCommand(local,scope,ScopeDelimRemove))
+
+let add_class_scope local scope cl =
+  Lib.add_anonymous_leaf (inScopeCommand(local,scope,ScopeClasses cl))
 
 (* Check if abbreviation to a name and avoid early insertion of
    maximal implicit arguments *)
@@ -1622,7 +1659,7 @@ let try_interp_name_alias = function
   | [], { CAst.v = CRef (ref,_) } -> intern_reference ref
   | _ -> raise Not_found
 
-let add_syntactic_definition env ident (vars,c) local onlyparse =
+let add_syntactic_definition ~local deprecation env ident (vars,c) compat =
   let vars,reversibility,pat =
     try [], APrioriReversible, NRef (try_interp_name_alias (vars,c))
     with Not_found ->
@@ -1636,19 +1673,23 @@ let add_syntactic_definition env ident (vars,c) local onlyparse =
       let map id = let (_,sc) = Id.Map.find id nvars in (id, sc) in
       List.map map vars, reversibility, pat
   in
-  let onlyparse = match onlyparse with
-    | None when fst (printability None false reversibility pat) -> Some Flags.Current
-    | p -> p
-  in
-  Syntax_def.declare_syntactic_definition local ident onlyparse (vars,pat)
+  let deprecation, onlyparsing = merge_compat_deprecation compat deprecation in
+  let onlyparsing = onlyparsing || fst (printability None false reversibility pat) in
+  Syntax_def.declare_syntactic_definition ~local deprecation ident ~onlyparsing (vars,pat)
 
 (**********************************************************************)
 (* Declaration of custom entry                                        *)
 
+let warn_custom_entry =
+  CWarnings.create ~name:"custom-entry-overriden" ~category:"parsing"
+         (fun s ->
+          strbrk "Custom entry " ++ str s ++ strbrk " has been overriden.")
+
 let load_custom_entry _ _ = ()
 
 let open_custom_entry _ (_,(local,s)) =
-  Egramcoq.create_custom_entry ~local s
+  if Egramcoq.exists_custom_entry s then warn_custom_entry s
+  else Egramcoq.create_custom_entry ~local s
 
 let cache_custom_entry o =
   load_custom_entry 1 o;
@@ -1668,4 +1709,7 @@ let inCustomEntry : locality_flag * string -> obj =
       classify_function = classify_custom_entry}
 
 let declare_custom_entry local s =
-  Lib.add_anonymous_leaf (inCustomEntry (local,s))
+  if Egramcoq.exists_custom_entry s then
+    user_err Pp.(str "Custom entry " ++ str s ++ str " already exists")
+  else
+    Lib.add_anonymous_leaf (inCustomEntry (local,s))
