@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -69,9 +69,6 @@ type kind_load =
 (* Must be always initialized *)
 let load = ref WithoutTop
 
-(* Are we in a native version of Coq? *)
-let is_native = Dynlink.is_native
-
 (* Sets and initializes a toplevel (if any) *)
 let set_top toplevel = load :=
   WithTop toplevel;
@@ -89,7 +86,7 @@ let is_ocaml_top () =
     |_ -> false
 
 (* Tests if we can load ML files *)
-let has_dynlink = Coq_config.has_natdynlink || not is_native
+let has_dynlink = Coq_config.has_natdynlink || not Sys.(backend_type = Native)
 
 (* Runs the toplevel loop of Ocaml *)
 let ocaml_toploop () =
@@ -130,7 +127,7 @@ let ml_load s =
     | WithoutTop ->
         try
           Dynlink.loadfile s; s
-	with Dynlink.Error a ->
+        with Dynlink.Error a ->
           user_err ~hdr:"Mltop.load_object"
             (strbrk "while loading " ++ str s ++
              strbrk ": " ++ str (Dynlink.error_message a))
@@ -149,8 +146,8 @@ let dir_ml_use s =
     | WithTop t -> t.use_file s
     | _ ->
        let moreinfo =
-	 if Dynlink.is_native then " Loading ML code works only in bytecode."
-	 else ""
+         if Sys.(backend_type = Native) then " Loading ML code works only in bytecode."
+         else ""
        in
       user_err ~hdr:"Mltop.dir_ml_use" (str "Could not load ML code." ++ str moreinfo)
 
@@ -162,75 +159,9 @@ let add_ml_dir s =
     | _ -> ()
 
 (* For Rec Add ML Path (-R) *)
-let add_rec_ml_dir unix_path =
-  List.iter (fun (lp,_) -> add_ml_dir lp) (all_subdirs ~unix_path)
-
-(* Adding files to Coq and ML loadpath *)
-
-let warn_cannot_use_directory =
-  CWarnings.create ~name:"cannot-use-directory" ~category:"filesystem"
-         (fun d ->
-          str "Directory " ++ str d ++
-            strbrk " cannot be used as a Coq identifier (skipped)")
-
-let convert_string d =
-  try Names.Id.of_string d
-  with UserError _ ->
-    let d = Unicode.escaped_if_non_utf8 d in
-    warn_cannot_use_directory d;
-    raise Exit
-
-let warn_cannot_open_path =
-  CWarnings.create ~name:"cannot-open-path" ~category:"filesystem"
-      (fun unix_path -> str "Cannot open " ++ str unix_path)
-
-type add_ml = AddNoML | AddTopML | AddRecML
-
-type vo_path_spec = {
-  unix_path : string;
-  coq_path  : Names.DirPath.t;
-  implicit  : bool;
-  has_ml    : add_ml;
-}
-
-type coq_path_spec =
-  | VoPath of vo_path_spec
-  | MlPath of string
-
-type coq_path = {
-  path_spec: coq_path_spec;
-  recursive: bool;
-}
-
-let add_vo_path ~recursive lp =
-  let unix_path = lp.unix_path in
-  let implicit = lp.implicit in
-  if exists_dir unix_path then
-    let dirs = if recursive then all_subdirs ~unix_path else [] in
-    let prefix = Names.DirPath.repr lp.coq_path in
-    let convert_dirs (lp, cp) =
-      try
-        let path = List.rev_map convert_string cp @ prefix in
-        Some (lp, Names.DirPath.make path)
-      with Exit -> None
-    in
-    let dirs = List.map_filter convert_dirs dirs in
-    let () = match lp.has_ml with
-      | AddNoML -> ()
-      | AddTopML -> add_ml_dir unix_path
-      | AddRecML -> List.iter (fun (lp,_) -> add_ml_dir lp) dirs; add_ml_dir unix_path in
-    let add (path, dir) =
-      Loadpath.add_load_path path ~implicit dir in
-    let () = List.iter add dirs in
-    Loadpath.add_load_path unix_path ~implicit lp.coq_path
-  else
-    warn_cannot_open_path unix_path
-
-let add_coq_path { recursive; path_spec } = match path_spec with
-  | VoPath lp ->
-    add_vo_path ~recursive lp
-  | MlPath dir ->
-    if recursive then add_rec_ml_dir dir else add_ml_dir dir
+let add_ml_dir ~recursive unix_path =
+  let dirs = if recursive then (all_subdirs ~unix_path) else [unix_path,[]] in
+  List.iter (fun (lp,_) -> add_ml_dir lp) dirs
 
 (* convertit un nom quelconque en nom de fichier ou de module *)
 let mod_of_name name =
@@ -257,7 +188,8 @@ let file_of_name name =
        str"Loadpath: " ++ str(String.concat ":" !coq_mlpath_copy)) in
   if not (Filename.is_relative name) then
     if Sys.file_exists name then name else fail name
-  else if is_native then
+  else if Sys.(backend_type = Native) then
+    (* XXX: Dynlink.adapt_filename does the same? *)
     let name = match suffix with
       | Some ((".cmo"|".cma") as suffix) ->
           (Filename.chop_suffix name suffix) ^ ".cmxs"
@@ -396,7 +328,7 @@ let unfreeze_ml_modules x =
 
 let _ =
   Summary.declare_ml_modules_summary
-    { Summary.freeze_function = (fun _ -> get_loaded_modules ());
+    { Summary.freeze_function = (fun ~marshallable -> get_loaded_modules ());
       Summary.unfreeze_function = unfreeze_ml_modules;
       Summary.init_function = reset_loaded_modules }
 
