@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -14,26 +14,35 @@ open Constrexpr
 
 val primitive_flag : bool ref
 
+type projection_flags = {
+  pf_subclass: bool;
+  pf_canonical: bool;
+}
+
 val declare_projections :
   inductive ->
-  Entries.constant_universes_entry ->
-  ?kind:Decl_kinds.definition_object_kind ->
+  Entries.universes_entry ->
+  ?kind:Decls.definition_object_kind ->
   Id.t ->
-  bool list ->
-  UnivNames.universe_binders ->
+  projection_flags list ->
   Impargs.manual_implicits list ->
   Constr.rel_context ->
-    (Name.t * bool) list * Constant.t option list
+    Recordops.proj_kind list * Constant.t option list
 
-val definition_structure :
-  inductive_kind -> Decl_kinds.cumulative_inductive_flag -> Decl_kinds.polymorphic ->
-  Declarations.recursivity_kind ->
-  (coercion_flag *
-  Names.lident *
-  universe_decl_expr option *
-  local_binder_expr list *
-  (local_decl_expr with_instance with_priority with_notation) list *
-  Id.t * constr_expr option) list ->
-  GlobRef.t list
+val declare_structure_entry : Recordops.struc_tuple -> unit
+
+val definition_structure
+  :  universe_decl_expr option
+  -> inductive_kind
+  -> template:bool option
+  -> cumulative:bool
+  -> poly:bool
+  -> Declarations.recursivity_kind
+  -> (coercion_flag *
+      Names.lident *
+      local_binder_expr list *
+      (local_decl_expr * record_field_attr) list *
+      Id.t * constr_expr option) list
+  -> GlobRef.t list
 
 val declare_existing_class : GlobRef.t -> unit
