@@ -1,6 +1,6 @@
 (************************************************************************)
 (*         *   The Coq Proof Assistant / The Coq Development Team       *)
-(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2018       *)
+(*  v      *   INRIA, CNRS and contributors - Copyright 1999-2019       *)
 (* <O___,, *       (see CREDITS file for the list of authors)           *)
 (*   \VV/  **************************************************************)
 (*    //   *    This file is distributed under the terms of the         *)
@@ -13,47 +13,26 @@
 
 open Vernacexpr
 
-let rec under_control = function
-  | VernacExpr (_, c) -> c
-  | VernacRedirect (_,{CAst.v=c})
-  | VernacTime (_,{CAst.v=c})
-  | VernacFail c
-  | VernacTimeout (_,c) -> under_control c
-
-let rec has_Fail = function
-  | VernacExpr _ -> false
-  | VernacRedirect (_,{CAst.v=c})
-  | VernacTime (_,{CAst.v=c})
-  | VernacTimeout (_,c) -> has_Fail c
-  | VernacFail _ -> true
+(* Does this vernacular involve Fail? *)
+let has_Fail { CAst.v } = List.mem ControlFail v.control
 
 (* Navigation commands are allowed in a coqtop session but not in a .v file *)
-let is_navigation_vernac_expr = function
+let is_navigation_vernac = function
   | VernacResetInitial
   | VernacResetName _
-  | VernacBackTo _
   | VernacBack _ -> true
   | _ -> false
 
-let is_navigation_vernac c =
-  is_navigation_vernac_expr (under_control c)
-
-let rec is_deep_navigation_vernac = function
-  | VernacTime (_,{CAst.v=c}) -> is_deep_navigation_vernac c
-  | VernacRedirect (_, {CAst.v=c})
-  | VernacTimeout (_,c) | VernacFail c -> is_navigation_vernac c
-  | VernacExpr _ -> false
-
 (* NB: Reset is now allowed again as asked by A. Chlipala *)
 let is_reset = function
-  | VernacExpr ( _, VernacResetInitial)
-  | VernacExpr (_, VernacResetName _) -> true
+  | VernacResetInitial
+  | VernacResetName _ -> true
   | _ -> false
 
-let is_debug cmd = match under_control cmd with
+let is_debug = function
   | VernacSetOption (_, ["Ltac";"Debug"], _) -> true
   | _ -> false
 
-let is_undo cmd = match under_control cmd with
+let is_undo = function
   | VernacUndo _ | VernacUndoTo _ -> true
   | _ -> false
